//------------------------------------------------------------------------------
function HttpRequester( pFct, pFctError )
{
	//------------------------------------------------------------------------------
	// PRIVATE ATTRIBUTES
	//------------------------------------------------------------------------------	
	var _oXMLHTTP            = null ; // THE "XMLHttpRequest" object
	var _sRequestContent     = null ; // The last request content
	var _sRequestContentType = null ; // The last request content type
	var _fctProcessResponse  = null ; // External function for response processing
	var _fctError            = null ; // External function for error processing 
	
	if ( pFct != null )
	{
		_fctProcessResponse = pFct ;
	}
	if ( pFctError != null )
	{
		_fctError = pFctError ;
	}
	
	//--- Instance init 	
    if (window.ActiveXObject) 
	{
    	//--- Internet Explorer
    	_oXMLHTTP = new ActiveXObject("Microsoft.XMLHTTP");
    }
    else if (window.XMLHttpRequest)
    {
    	//--- Mozilla & Safari
    	_oXMLHTTP = new XMLHttpRequest();
    }

	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================

	//------------------------------------------------------------------------------
	// AJAX response processing : build the response and call the callBackFunction 
	//------------------------------------------------------------------------------
	function _processAjaxResponse() // Called for each state change (1,2,3 & 4)
	{
		//--- If new state is "COMPLETE" ...
		if ( _oXMLHTTP.readyState == 4 )
		{
			if ( _oXMLHTTP.status == 200 )
			{
				//--- Run the callback if it exists
				if ( _fctProcessResponse != null )
				{
					_fctProcessResponse( _oXMLHTTP ) ;
				}
			}
			else
			{
				if ( _fctError != null )
				{
					_fctError( _oXMLHTTP ) ;
				}
				else
				{
					fwkError ( "HTTP error : " + _oXMLHTTP.status + " " + _oXMLHTTP.statusText ); 
				}
			}
		}
	}
	//------------------------------------------------------------------------------
	// Send HttpRequest to the server 
	//------------------------------------------------------------------------------
	function _sendRequest ( sHttpMethod, sURL, bAsync, sContent, sContentType ) // RET : void 
	{
		if ( _oXMLHTTP == null )
    	{
			fwkError ( "_sendRequest() : XMLHttpRequest object is null !" ); return ;
    	}
    	//--- Check parameters 
		if ( sHttpMethod == null )
		{
			fwkError ( "_sendRequest() : HTTP Method is null ! " ); return ;
		}
		if ( sURL == null ) 
		{
			fwkError ( "_sendRequest() : URL is null ! " ); return ;
		}
		if ( bAsync == null )
		{
			bAsync = false ;
		}		

		_sRequestContent = sContent ;
		_sRequestContentType = sContentType ;
		
		//--- (1) Setup the HTTP Request Method and URL 
	    _oXMLHTTP.open (sHttpMethod, sURL, bAsync);  // false = SYNCHRONOUS  (true = ASYNCHRONOUS)	    
		if ( sContent != null )
		{
			if ( sContentType != null )
			{
		    	_oXMLHTTP.setRequestHeader("Content-Type", sContentType );
		    }
		}

		//--- (2) Configure the callback method (if ASYNCHRONOUS) 
	    if ( bAsync )
	    {
			_oXMLHTTP.onreadystatechange = _processAjaxResponse ;
		}

		//--- (3) Send the request to the server 
	    _oXMLHTTP.send (sContent); // Send the content (via HTTP)
	    
	    //--- (4) if this request is not ASYNCHRONOUS => call the method to process the response
	    if ( ! bAsync )
	    {
			_processAjaxResponse() ;
		}
	}
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================

	//------------------------------------------------------------------------------
	// Get the last request content (request body)
	//------------------------------------------------------------------------------
	this.getRequestContent = function() 
	{
		return _sRequestContent ;
	}
	//------------------------------------------------------------------------------
	// Get the last request content (request body)
	//------------------------------------------------------------------------------
	this.getRequestContentType = function() 
	{
		return _sRequestContentType ;
	}
	
	//------------------------------------------------------------------------------
	// Get the last HTTP status 
	//------------------------------------------------------------------------------
	this.getHttpStatus = function() 
	{
		return _oXMLHTTP.status ;
	}
	//------------------------------------------------------------------------------
	// Get the last response as text 
	//------------------------------------------------------------------------------
	this.getResponseText = function() 
	{
		return _oXMLHTTP.responseText ;
	}
	//------------------------------------------------------------------------------
	// Get the last response as XML 
	//------------------------------------------------------------------------------
	this.getResponseXML = function() 
	{
		return _oXMLHTTP.responseXML ;
	}


	//------------------------------------------------------------------------------
	// SYNCHRONOUS "GET"
	//------------------------------------------------------------------------------
	this.httpGetSync = function ( sUrl, sContent, sContentType )
	{
		_sendRequest("GET", sUrl, false, sContent, sContentType);
	}
	//------------------------------------------------------------------------------
	// ASYNCHRONOUS "GET"
	//------------------------------------------------------------------------------
	this.httpGetAsync = function ( sUrl, sContent, sContentType )
	{
		_sendRequest("GET", sUrl, true, sContent, sContentType);
	}
	//------------------------------------------------------------------------------
	// SYNCHRONOUS "POST"
	//------------------------------------------------------------------------------
	this.httpPostSync = function ( sUrl, sContent, sContentType )
	{
		_sendRequest("POST", sUrl, false, sContent, sContentType);
	}
	//------------------------------------------------------------------------------
	// ASYNCHRONOUS "POST"
	//------------------------------------------------------------------------------
	this.httpPostAsync = function ( sUrl, sContent, sContentType )
	{
		_sendRequest("POST", sUrl, true, sContent, sContentType);
	}
	//------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------
