//------------------------------------------------------------------------------
// fwkKeyboard.js 
// Keyborad class : keyboard management
//------------------------------------------------------------------------------
// Uses : Browser class
//------------------------------------------------------------------------------
// Version : 1.0 ( 26/10/2005 )
// Author  : Laurent Gurin 
//------------------------------------------------------------------------------
// Mozilla/Firefox behavior :
// Standard Keys : 1) onKeyPress + 2) onKeyDown ( no order ?,  2 distinct events )
// Fct Keys      : 1) onKeyDown + 2) onKeyPress ( F1..F12, Esc, Tab, Arrows, End, Home, Del )
// CAPS, Ctrl, Alt : onKeyDown ( NO onKeyPress )
//------------------------------------------------------------------------------
// Internet Explorer behavior :
// Standard Keys : onKeyPress ONLY ( NO onKeyDown )
// Fct Keys      : onKeyDown  ONLY ( NO onKeyPress ) ( F1..F12, Tab, )
// ESC Key       : 1) onKeyDown + 2) onKeyPress
// CAPS, Ctrl, Alt, Arrows, End, Home, Del : onKeyDown ( NO onKeyPress )
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
function KeyAction(oFct, iType, bStop)
{
	this.oFunction = oFct ;
	this.iEvtType  = iType ;
	this.bStop     = bStop ;
}

//------------------------------------------------------------------------------
// STATIC CONSTANTS
//------------------------------------------------------------------------------
Keyboard.EVT_KEY_PRESS = 1 ;
Keyboard.EVT_KEY_DOWN  = 2 ;

//--- KEY CODES :
Keyboard.ESC = 27 ;
Keyboard.DEL = 46 ;

Keyboard.CTRL = 17 ;
Keyboard.ALT  = 18 ;
Keyboard.INS  = 45 ;
Keyboard.BACKSPACE = 8 ;
Keyboard.SHIFT = 16 ;

Keyboard.ARROW_UP    = 38 ;
Keyboard.ARROW_DOWN  = 40 ;
Keyboard.ARROW_LEFT  = 37 ;
Keyboard.ARROW_RIGHT = 39 ;

Keyboard.HOME      = 36 ;
Keyboard.END       = 35 ;
Keyboard.PAGE_UP   = 33 ;
Keyboard.PAGE_DOWN = 34 ;

Keyboard.F1  = 112 ;
Keyboard.F2  = 113 ;
Keyboard.F3  = 114 ;
Keyboard.F4  = 115 ;
Keyboard.F5  = 116 ;
Keyboard.F6  = 117 ;
Keyboard.F7  = 118 ;
Keyboard.F8  = 119 ;
Keyboard.F9  = 120 ;
Keyboard.F10 = 121 ;
Keyboard.F11 = 122 ;
Keyboard.F12 = 123 ;

//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
function Keyboard()
{
	//------------------------------------------------------------------------------
	// CONSTANTS
	//------------------------------------------------------------------------------

	//------------------------------------------------------------------------------
	// ATTRIBUTES
	//------------------------------------------------------------------------------
	//this.browser = Browser.getBrowser();
	this.aKeyActions = new Array();
	//------------------------------------------------------------------------------
	// METHODS
	//------------------------------------------------------------------------------
	this.desc = function()
	{
		//return ( "keyboard object for browser " + this.browser.getType() ) ;
		return ( "keyboard object : " + this.aKeyActions + " key(s) mapped") ;
	}
	//------------------------------------------------------------------------------
	this.setKeyAction = function( iKey, oFunction, iEventType )
	{
		//alert ( "setKeyAction : key = " + iKey ) ;
		if ( iKey      == null ) alert ( "ERROR: keyAction() : KEY parameter is null !" );
		if ( oFunction == null ) 
		{
			this.aKeyActions[iKey] = new KeyAction ( null, iEventType, true );
		}
		else
		{		
			if ( ! oFunction instanceof Function ) 
			{
				alert ( "ERROR: keyAction() : ACTION is not a function !" );
			}
			this.aKeyActions[iKey] = new KeyAction ( oFunction, iEventType, true );
		}
	}
	//------------------------------------------------------------------------------
	this.getNbKeyActions = function() // INT
	{
		return this.aKeyActions.length ;
	}
	//------------------------------------------------------------------------------
	this.showKeyActions = function() // VOID
	{
		var action = null ;
		var s = "Key Actions : \n" ;
		for ( var i=0 ; i < this.aKeyActions.length ; i++ )
		{
			action = this.aKeyActions[i];
			if ( action != null )
			{
				s += i + " -> " + action + "\n" ;
			}
		}
		alert(s);
	}
	//------------------------------------------------------------------------------
	this.keyPress = function(evt)
	{
		//alert ( " keyPress " );
		if ( window.event )
		{
			//iKeyCode = event.keyCode ;
			this.processKeyCode(window.event.keyCode, Keyboard.EVT_KEY_PRESS );
		}
		else
		{
			if ( evt ) // Mozilla - Firefox 
			{
				this.processKeyCode(evt.keyCode, Keyboard.EVT_KEY_PRESS, evt);
			}
		}
	}
	//------------------------------------------------------------------------------
	this.keyDown = function(evt)
	{
		//alert ( " keyDown " );
		if ( window.event )
		{
			this.processKeyCode(window.event.keyCode, Keyboard.EVT_KEY_DOWN );
		}
		else
		{
			if ( evt ) // Mozilla - Firefox 
			{
				this.processKeyCode(evt.keyCode, Keyboard.EVT_KEY_DOWN, evt);
			}
		}
		return false ;
	}
	//------------------------------------------------------------------------------
	this.processKeyCode = function(iKeyCode, iEvtType, evt )
	{
		//alert("processKeyCode("+iKeyCode+","+iEvtType+")"  );
		//--- Is there an action associated with this KeyCode ?
		var oAction = this.aKeyActions[iKeyCode] ;
		if ( oAction != null )
		{
			//--- 1) STOP PROPAGATION
			if ( window.event )
			{
			    window.event.cancelBubble = true;
			    window.event.returnValue  = false;
			    window.event.keyCode      = -1 ;
			}
			else
			{
				if ( evt ) // Mozilla - Firefox 
				{
				    evt.stopPropagation();
				    evt.preventDefault();
				    // evt.preventBubble(); // Sans effet
				    // evt.cancelBubble = true ; // Sans effet
				    //evt.keyCode = 0 ; // Impossible (Read-Only)
				}
			}
			//--- 2) CALL THE ACTION FUNCTION
			if ( oAction.iEvtType == iEvtType )
			{
				if ( oAction.oFunction != null )
				{
					oAction.oFunction();
				}
			}
		}
	/***
		else
		{
			alert ( "No specific action for the key " + iKeyCode ) ;
		}
	***/
		//alert("END OF processKeyCode("+iKeyCode+") evt.processed = " + evt.processed );
	}
	//------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------
// Static properties
//------------------------------------------------------------------------------
Keyboard.keyboard = new Keyboard() ; 
//------------------------------------------------------------------------------
// Static methods
//------------------------------------------------------------------------------
Keyboard.getKeyboard = function ()
{
	return Keyboard.keyboard ;
}
//------------------------------------------------------------------------------
Keyboard.onKeyPress = function (evt)
{
	//alert ( "Keyboard.onKeyPress " );
	Keyboard.keyboard.keyPress(evt) ;
	//return false ;
}
//------------------------------------------------------------------------------
Keyboard.onKeyDown = function (evt)
{
	//alert ( "Keyboard.onKeyDown " );
	Keyboard.keyboard.keyDown(evt) ;
	//return false ;
}
//------------------------------------------------------------------------------
Keyboard.onHelp = function () // Method to catch the IE "onHelp" event ( F1 key )
{
	window.event.returnValue = false;
	alert("fwkOnHelp");
}
//------------------------------------------------------------------------------
// METHODE : Permet de catcher l'vnement standard sur la touche 'F1'
/**
function document.onhelp()
{
	window.event.returnValue = false;
}
**/		
//------------------------------------------------------------------------------
{
	var browser = Browser.getBrowser() ;
	if ( browser.isInternetExplorer() )
	{
		//alert ( "Keyboard Config for IE" ) ;
		document.onhelp = Keyboard.onHelp ;
		// Only the KeyDown is used 
		document.onkeydown = Keyboard.onKeyDown ;
	}
	if ( browser.isMozilla() )
	{
		//alert ( "Keyboard Config for Mozilla" ) ;
		document.onkeydown  = Keyboard.onKeyDown ;
		document.onkeypress = Keyboard.onKeyPress ;	
		//document.onkeypress = Keyboard.keyboard.keyPress ; // NOT OK ( the instance method doesn't access attributes !) 
	}
}
