//------------------------------------------------------------------------------
// ListView_class.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 13/11/2006 )
// Author  : Laurent Gurin 
// Changes : none (creation)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// CLASS DEFINITION
//------------------------------------------------------------------------------
function ListView( sListId, sListName, sItemName, oActions, fResponseProcessor )
{
	//==============================================================================
	// STANDARD ATTRIBUTES & METHODS
	//==============================================================================
	var _sClassName = "ListView" ;
	var _bTrace = false ;
	//------------------------------------------------------------------------------
	function _trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE : [" + _sClassName + "] \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	function _error ( sMsg ) 
	{
		alert ( "ERROR : [" + _sClassName + "] \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	
	//--- Check constructor parameters 
	if ( arguments.length < 4 )
	{
		_error ( "constructor : 4 or 5 arguments expected ( " + arguments.length + " arguments )" ) ;
	}
	
	//==============================================================================
	// PRIVATE ATTRIBUTES
	//==============================================================================
	var _sListId             = sListId ; 
	var _sListName           = sListName ; 
	var _sItemName           = sItemName ; 
	var _oActions            = oActions ;
	var _fResponseProcessor  = fResponseProcessor ;
	
	var _sSelectedRowColor   = "D2CDAC" ; // Default selected color
	var _sOriginalRowColor   = null ;
	
	var _oSelectedRow        = null ; // the row <TR> currently selected
	
	var _eventSelectRow      = null ;
	var _eventInsertRow      = null ;
	var _eventAddRow         = null ;
	var _eventDeleteRow      = null ;
	var _eventClearList      = null ;
	var _eventSortList       = null ;
	
	
	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================
	function _chekEvent( oFunction )
	{
		if ( oFunction == null )
		{
			_error ( "setEvent : no FUNCTION argument ! ");
			return null ;
		}
		if ( typeof oFunction != "function" )
		{
			_error ( "setEvent : the argument is not a function ! ");
			return null ;
		}
		return oFunction ;
	}
	//------------------------------------------------------------------------------
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================
	//------------------------------------------------------------------------------
	// Setters
	//------------------------------------------------------------------------------
	this.setSelectedRowColor = function ( sColor ) 
	{
		_sSelectedRowColor = sColor ;
	}
	//------------------------------------------------------------------------------
	this.setEventSelectRow = function ( oFunction ) 
	{		
		_eventSelectRow = _chekEvent( oFunction );
	}
	//------------------------------------------------------------------------------
	this.setEventInsertRow = function ( oFunction ) 
	{
		_eventInsertRow = _chekEvent( oFunction );
	}
	//------------------------------------------------------------------------------
	this.setEventAddRow = function ( oFunction ) 
	{
		_eventAddRow = _chekEvent( oFunction );
	}
	//------------------------------------------------------------------------------
	this.setEventDeleteRow = function ( oFunction ) 
	{
		_eventDeleteRow = _chekEvent( oFunction );
	}
	//------------------------------------------------------------------------------
	this.setEventClearList = function ( oFunction ) 
	{
		_eventClearList = _chekEvent( oFunction );
	}
	//------------------------------------------------------------------------------
	this.setEventSortList = function ( oFunction ) 
	{
		_eventSortList = _chekEvent( oFunction );
	}
	//------------------------------------------------------------------------------
	
	//------------------------------------------------------------------------------
	// Getters
	//------------------------------------------------------------------------------
	this.getListId = function() 
	{
		return _sListId ;
	}
	//------------------------------------------------------------------------------
	this.getListName = function() 
	{
		return _sListName ;
	}
	//------------------------------------------------------------------------------
	this.getItemName = function() 
	{
		return _sItemName ;
	}
	//------------------------------------------------------------------------------
	this.getActions = function() 
	{
		return _oActions ;
	}
	//------------------------------------------------------------------------------
	this.getResponseProcessor = function() 
	{
		return _fResponseProcessor ;
	}
	//------------------------------------------------------------------------------
	this.getSelectedRow = function () 
	{
		return _oSelectedRow ;
	}
	//------------------------------------------------------------------------------
	this.getSelectedRowIndex = function () 
	{
		if ( _oSelectedRow != null )
		{
			return _oSelectedRow.rowIndex ;
		}
		return -1 ;
	}
	//------------------------------------------------------------------------------
	this.getSelectedRowColor = function () 
	{
		return _sSelectedRowColor ;
	}
	//------------------------------------------------------------------------------
	this.show = function () 
	{
		var sActions = ( _oActions != null ) ? "set (not null)" : "not set (null)"  ;
		var sResponseProcessor = ( _fResponseProcessor != null ) ? "set (not null)" : "not set (null)"  ;
		
		var sMsg = "ListView instance : " ;
		sMsg += "\n . List Id   : " + _sListId ;
		sMsg += "\n . List Name : " + _sListName ;
		sMsg += "\n . Item Name : " + _sItemName  ;
		sMsg += "\n . Selected row color : " + _sSelectedRowColor ; 
		sMsg += "\n . Actions   : " + sActions ;
		sMsg += "\n . Response Processor : " + sResponseProcessor ;
		alert( sMsg ); 
	}
	//------------------------------------------------------------------------------
	this.highlightRow = function(oTR) 
	{
		if ( oTR == null ) { _error ( "highlightRow : TR argument is null ! "); return; }
		//--- Restore the original color of the current selected row 
		if ( _oSelectedRow != null && _sOriginalRowColor != null )
		{
			// oTBody.selectedRow.style.backgroundColor = oTBody.originalRowColor ;  //--- Normal color
			_oSelectedRow.style.backgroundColor = _sOriginalRowColor ;
		}
		//--- Highlight the given row 
		//oTBody.originalRowColor = oTR.style.backgroundColor ;
		//oTBody.selectedRow = oTR ;
		//oTBody.selectedRow.style.backgroundColor = oListView.getSelectedRowColor() ; //--- Selected color	
		_sOriginalRowColor = oTR.style.backgroundColor ;
		_oSelectedRow = oTR ;
		_oSelectedRow.style.backgroundColor = _sSelectedRowColor ;		
	}
	//------------------------------------------------------------------------------
		
	//------------------------------------------------------------------------------
	// LIST MANAGEMENT METHODS
	//------------------------------------------------------------------------------
	//this.doSelect = function(iOldRowIndex, iNewRowIndex) 
	this.doSelect = function(iNewRowIndex) 
	{
		var iOldRowIndex = this.getSelectedRowIndex() ;
		var bOkToContinue = true ;
		if ( _eventSelectRow != null )
		{
			bOkToContinue = _eventSelectRow( _sListName, _sItemName, iOldRowIndex, iNewRowIndex );
		}
		else 
		{
			if ( _oActions != null )
			{
				var response = _oActions.doListSelect ( _sListName, _sItemName, iOldRowIndex, iNewRowIndex ) ;
				if ( _fResponseProcessor != null )
				{
					bOkToContinue = _fResponseProcessor(response);			
				}
			}
		}
		return bOkToContinue ;
	}
	//------------------------------------------------------------------------------
	this.doInsert = function(iNewRowIndex) 
	{
		var iOldRowIndex = this.getSelectedRowIndex() ;
		var bOkToContinue = true ;
		if ( _eventInsertRow != null )
		{
			bOkToContinue = _eventInsertRow( _sListName, _sItemName, iOldRowIndex, iNewRowIndex );
		}
		else 
		{
			if ( _oActions != null )
			{
				var response = _oActions.doListInsert ( _sListName, _sItemName, iOldRowIndex, iNewRowIndex ) ;
				if ( _fResponseProcessor != null )
				{
					bOkToContinue = _fResponseProcessor(response);			
				}
			}
		}
		return bOkToContinue ;
	}
	//------------------------------------------------------------------------------
	this.doAdd = function() 
	{
		var iOldRowIndex = this.getSelectedRowIndex() ;
		var bOkToContinue = true ;
		if ( _eventAddRow != null )
		{
			bOkToContinue = _eventAddRow( _sListName, _sItemName, iOldRowIndex );
		}
		else 
		{
			if ( _oActions != null )
			{
				var response = _oActions.doListAdd ( _sListName, _sItemName, iOldRowIndex ) ;
				if ( _fResponseProcessor != null )
				{
					bOkToContinue = _fResponseProcessor(response);			
				}
			}
		}
		return bOkToContinue ;
	}
	//------------------------------------------------------------------------------
	this.doDelete = function(iRowIndexToDelete) 
	{
		var iOldRowIndex = this.getSelectedRowIndex() ;
		var bOkToContinue = true ;
		if ( _eventDeleteRow != null )
		{
			bOkToContinue = _eventDeleteRow( _sListName, _sItemName, iOldRowIndex, iRowIndexToDelete );
		}
		else 
		{
			if ( _oActions != null )
			{
				var response = _oActions.doListRemove ( _sListName, _sItemName, iOldRowIndex, iRowIndexToDelete ) ;
				if ( _fResponseProcessor != null )
				{
					bOkToContinue = _fResponseProcessor(response);			
				}
			}
		}
		return bOkToContinue ;
	}
	//------------------------------------------------------------------------------
	this.doClear = function() 
	{
		var iOldRowIndex = this.getSelectedRowIndex() ;
		var bOkToContinue = true ;
		if ( _eventClearList != null )
		{
			bOkToContinue = _eventClearList( _sListName, _sItemName, iOldRowIndex );
		}
		else 
		{
			if ( _oActions != null )
			{
				var response = _oActions.doListClear ( _sListName, _sItemName, iOldRowIndex ) ;
				if ( _fResponseProcessor != null )
				{
					bOkToContinue = _fResponseProcessor(response);			
				}
			}
		}
		return bOkToContinue ;
	}
	//------------------------------------------------------------------------------
	this.doSort = function( sField, sOrder ) 
	{
		var iOldRowIndex = this.getSelectedRowIndex() ;
		var bOkToContinue = true ;
		if ( _eventSortList != null )
		{
			bOkToContinue = _eventSortList( _sListName, _sItemName, iOldRowIndex, sField, sOrder );
		}
		else 
		{
			if ( _oActions != null )
			{
				var response = _oActions.doListSort ( _sListName, _sItemName, iOldRowIndex, sField, sOrder ) ;
				if ( _fResponseProcessor != null )
				{
					bOkToContinue = _fResponseProcessor(response);			
				}
			}
		}
		return bOkToContinue ;
	}
	//------------------------------------------------------------------------------
}
