//------------------------------------------------------------------------------
// Logger_class.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 16/11/2007 )
// Author  : Laurent Gurin 
// Changes : none (creation)
//------------------------------------------------------------------------------

function Logger(sLoggerName)
{
	//------------------------------------------------------------------------------
	// PRIVATE ATTRIBUTES
	//------------------------------------------------------------------------------
	var _iCount         = 0 ;
	var _sName          = ( sLoggerName != null ? sLoggerName : "logger" ) ;
	var _sConsoleId     = _sName + "_console" ;
	var _oConsole       = null ;
	var _sConsoleBodyId = _sName + "_console_body" ;
	var _oConsoleBody   = null ;
		
	//------------------------------------------------------------------------------
	// PRIVATE METHODS
	//------------------------------------------------------------------------------
	function getConsole() 
	{
		if ( _oConsole == null )
		{
			_oConsole = document.getElementById( _sConsoleId );
			if ( _oConsole == null )
			{
				alert("ERROR in Logger : Cannot get logger console ( ID = " + _sConsoleId + ")" );
				return null ;
			}
		}
		return _oConsole ;
	}
	//------------------------------------------------------------------------------
	function getConsoleBody() 
	{
		if ( _oConsoleBody == null )
		{
			_oConsoleBody = document.getElementById( _sConsoleBodyId );
			if ( _oConsoleBody == null )
			{
				alert("ERROR in Logger : Cannot get logger console body ( ID = " + _sConsoleBodyId + ")" );
				return null ;
			}
		}
		return _oConsoleBody ;
	}
	//------------------------------------------------------------------------------
	function createSeverity(sLevel) 
	{
		var oDIV = document.createElement("span");
		oDIV.style.marginLeft="2px";
		oDIV.style.paddingLeft="4px";
		oDIV.style.paddingRight="4px";
		oDIV.style.marginRight="2px";		
		// different colors for different severities
		if (sLevel == "DEBUG") {
			oDIV.style.backgroundColor="#CCCCCC";
		} else if (sLevel == "INFO") {
			oDIV.style.backgroundColor="#CCFFFF";
		} else if (sLevel == "WARN") {
			oDIV.style.backgroundColor="yellow";
		} else if (sLevel == "ERROR") {
			oDIV.style.backgroundColor="red";
		}		
		oDIV.appendChild(document.createTextNode("."));
		return oDIV ;
	}
	//------------------------------------------------------------------------------
	function createMessageRow(sLevel,sMsg) 
	{
		_iCount++;
		var oDivMsgRow = document.createElement("div");
		oDivMsgRow.style.borderBottom="1px solid #AAA";
		oDivMsgRow.style.verticalAlign="top";
		
		oDivMsgRow.appendChild( createSeverity(sLevel) );
		oDivMsgRow.appendChild( document.createTextNode("" + _iCount + " [" + sLevel+"] "+sMsg));
		
		return oDivMsgRow ;
	}
	//------------------------------------------------------------------------------
	function logMsg(sLevel,sMsg) 
	{
		var oConsoleBody = getConsoleBody();
		if ( oConsoleBody != null )
		{
			oConsoleBody.appendChild(createMessageRow(sLevel,sMsg));
		}
 	}
	//------------------------------------------------------------------------------
	function setConsoleDisplayStyle(s) 
	{
		var console = getConsole();
		if ( console != null )
		{
			console.style.display = s ;
		}
	}
	
	//------------------------------------------------------------------------------
	// PUBLIC METHODS
	//------------------------------------------------------------------------------
	this.debug = function(sMsg) 
	{
		logMsg("DEBUG",sMsg);
	}
	//------------------------------------------------------------------------------
	this.info = function(sMsg) 
	{
		logMsg("INFO",sMsg);
	}
	//------------------------------------------------------------------------------
	this.warning = function(sMsg) 
	{
		logMsg("WARN",sMsg);
	}
	//------------------------------------------------------------------------------
	this.error = function(sMsg) 
	{
		logMsg("ERROR",sMsg);
	}
	//------------------------------------------------------------------------------
	this.show = function() 
	{
		setConsoleDisplayStyle("block") ;
	}
	//------------------------------------------------------------------------------
	this.hide = function() 
	{
		setConsoleDisplayStyle("none") ;
	}
	//------------------------------------------------------------------------------
	this.clear = function() 
	{
		var oConsoleBody = getConsoleBody();
		if ( oConsoleBody != null )
		{
			oConsoleBody.innerHTML = "";
			_iCount = 0 ;
		}
	}
	//------------------------------------------------------------------------------
	this.eval = function(sId) 
	{
		if ( sId != null )
		{
			var oField = document.getElementById( sId );
			if ( oField != null )
			{
				if ( oField.value != null )
				{
					var result = eval(oField.value);
					alert ( "result = " + result) ;
				}
			}
			else
			{
				alert("ERROR in Logger.eval : No field with ID '" + sId + "'" );
			}
		}
		else
		{
			alert("ERROR in Logger.eval : Field ID parameter is null" );
		}
	}
	//------------------------------------------------------------------------------
	this.desc = function() 
	{
		alert("Logger object : " 
		+ "\n . name = " + _sName 
		+ "\n . console id = " + _sConsoleId 
		+ "\n . console body id  = " + _sConsoleBodyId 
		) ;
	}
	//------------------------------------------------------------------------------
	
}

function fwkCreateLoggerConsole(sLoggerName)
{
	var sName = ( sLoggerName != null ? sLoggerName : "logger" ) ;
	var s = 
	  "<div id='" + sName + "_console' " 
	+ "style='font-family:arial; color:black; font-size:12px; line-height:normal; letter-spacing: normal; "
	+ "width:600px; display:none; position:absolute; top:20px; left:20px;" 
	+ "z-index:10000; border : 1px solid black ;' >"
	
	+ "<table style='align:center; width:100%; background-color:#3300FF; border:none; color:black; cursor:default'  >"
	+ "<tr>"
	+ "<td style='text-align:center; width:100%; color:#FFFFFF; font-family:arial; font-size:14px; font-weight: bold;' >Javascript logger console</td>"
	+ "<td style='background-color:silver; font-family:arial; font-size:14px; font-weight: bold; ' onclick='" + sName + ".hide();'>&nbsp;X&nbsp;</td>"
	+ "</tr>"
	+ "</table>"

	+ "<table style='align:center; width:100%; color:black; background-color:#FFFFCC; border:1px solid black; font-family:Tahoma;  '  >"
	+ "<tr style='background-color:grey; font-size:12px;' >"
	+ "<td style='background-color:#DDDDDD;cursor:default;border:1px solid #AAAAAA;' onclick='" + sName + ".clear();'>&nbsp;&nbsp;Clear&nbsp;&nbsp;</td>"
	+ "<td style='background-color:#FFFFCC; width:100%;' >&nbsp;</td>"
	+ "<td style='text-align:center; width:100%;' ><input id='"+sName+"_eval_input' style='width:400px; '/></td>"
	+ "<td style='background-color:#DDDDDD;cursor:default;border:1px solid #AAAAAA;' onclick=\"" +sName+".eval('"+sName+"_eval_input');\">&nbsp;&nbsp;Eval&nbsp;&nbsp;</td>"
	+ "</tr>"
	+ "</table>"	
	
	+ "<div id='" + sName + "_console_body' style='height:240px; overflow:auto; background-color:#FFFFFF; ' ></div>"
	
	+ "<div "
	+ " style='height:12px; padding:2px; border-top:1px solid black; background-color:#FFFFCC;'>&nbsp;</div>"	
	+ "</div>"
	
	+ "</div>" ;
	
	document.write(s);
	return sName ;
}

function fwkCreateLogger(sLoggerName)
{
	var sName = ( sLoggerName != null ? sLoggerName : "logger" ) ;
	fwkCreateLoggerConsole( sName );
	var logger = new Logger( sName );	
	window[sName] = logger ;
}
