//------------------------------------------------------------------------------
// ScreenActionsXHR class : XHR object wrapper used by the ScreenActions class
// Author : Laurent GUERIN 
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// OBJECT 
//------------------------------------------------------------------------------
function ScreenActionsXHR( sDefaultUrlParam )
{
	//------------------------------------------------------------------------------
	// PRIVATE ATTRIBUTES
	//------------------------------------------------------------------------------
	var _bTrace          = false ;
	var _sDefaultUrl     = null ;
	var _oSyncXHR        = null ; // THE "XMLHttpRequest" object for SYNCHRONOUS calls
	var _oAsyncXHR       = null ; // THE "XMLHttpRequest" object for ASYNCHRONOUS calls
	
	var _sRequestContent = null ; // The last request content
	var _oScreenResponse = null ; // The last response object
	var _fCallback       = null ; // The current callback function for Async calls
	
	if ( sDefaultUrlParam != null )
	{
		_sDefaultUrl = sDefaultUrlParam ;
	}
	
	//--- Instance init 	
    if (window.ActiveXObject) 
	{
    	//--- Internet Explorer
    	_oSyncXHR  = new ActiveXObject("Microsoft.XMLHTTP");
    	_oAsyncXHR = new ActiveXObject("Microsoft.XMLHTTP");
    }
    else if (window.XMLHttpRequest)
    {
    	//--- Mozilla & Safari
    	_oSyncXHR  = new XMLHttpRequest();
    	_oAsyncXHR = new XMLHttpRequest();
    }

	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================

	//------------------------------------------------------------------------------
	// 
	//------------------------------------------------------------------------------
	function _trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE from 'ScreenActionsXHR' : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	function _error ( sMsg ) 
	{
		alert ( "ERROR in 'ScreenActionsXHR' object : \n\n" + sMsg ) ;
	}
	
	//------------------------------------------------------------------------------
	// Build the response object 
	//------------------------------------------------------------------------------
	function _buildScreenResponse (oXHR) 
	{
		_trace( "_buildScreenResponse() : \n" + oXHR.responseText ); // Show the response 
		
        var sTelosysRedirect = oXHR.getResponseHeader("Telosys-Redirect");  // v 0.9.9
        // NB : if not found Firefox returns NULL and IE returns ""
		if ( sTelosysRedirect != null && sTelosysRedirect != "" )
		{
			//--- Redirection required (the content received is not an XML response)
			_trace("_buildScreenResponse() : Telosys Redirection : '" + sTelosysRedirect + "' " );
			fwkLoadPage( sTelosysRedirect );
			return null ; // never reach this line ( only if there's an error in fwkLoadPage )
		}
		else
		{		
			_oScreenResponse = new ScreenResponse(oXHR);
			return _oScreenResponse ;
		}
	}
	//------------------------------------------------------------------------------
	// CALLBACK METHOD USED WITH 'ASYNCHRONUS' CALL
	//------------------------------------------------------------------------------
	function _asyncCallback() // Called for each state change (1,2,3 & 4)
	{
		_trace ("_asyncCallback() : readyState = " + _oAsyncXHR.readyState );
		//--- If new state is "COMPLETE" ...
		if ( _oAsyncXHR.readyState == 4 )
		{
			//--- Build the ScreenResponse object
			var response = _buildScreenResponse(_oAsyncXHR);
			//--- Run the callback if it exists
			if ( _fCallback != null )
			{
				_trace ("_asyncCallback() : call application callback function..." );
				_fCallback(response) ;
			}
		}
	}
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================
	this.setTrace = function ( p ) 
	{
		_bTrace = p ;
	}

	//------------------------------------------------------------------------------
	// Get the Xml String of the request
	//------------------------------------------------------------------------------
	this.getRequest = function() 
	{
		return _sRequestContent ;
	}

	//------------------------------------------------------------------------------
	// Get the Screen Response object
	//------------------------------------------------------------------------------
	this.getResponse = function() 
	{
		return _oScreenResponse ;
	}

	//------------------------------------------------------------------------------
	// "POST" SYNCHRONOUS
	//------------------------------------------------------------------------------
	this.postSync = function ( sContent, sUrlParam )
	{
		var sURL = _sDefaultUrl ;
		if ( sUrlParam != null )
		{
			sURL = sUrlParam ;
		}

		if ( _oSyncXHR == null )
    	{
			_error ( "postSync : XMLHttpRequest object is null !" );
			return ;
    	}

		if ( sContent != null )
		{
			//_sRequestContent = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\r\n" + sContent ;
			_sRequestContent = sContent ;
		}
		else
		{
			_sRequestContent = null ;
		}
		
		//--- (1) Setup the HTTP Request Method and URL 
	    _oSyncXHR.open ("POST", sURL, false);  // false = SYNCHRONOUS CALL
	    
	    //_oSyncXHR.setRequestHeader("Content-Type", "text/xml;charset=ISO-8859-1");
	    _oSyncXHR.setRequestHeader("Content-Type", "text/xml");

		//--- (2) Synchronous => No callback method (nothing to do "onreadystatechange" is undefined => do not use)
		// _oSyncXHR.onreadystatechange = null  ; // DON'T DO THAT !
		
		//--- (3) Send the request to the server 
	    _oSyncXHR.send (_sRequestContent); // Send the content ( SYNCHRONOUS => continue ... )

		//--- (4) Response received
		_buildScreenResponse( _oSyncXHR );
	}
	
	//------------------------------------------------------------------------------
	// "POST" ASYNCHRONOUS ( with callback function )
	//------------------------------------------------------------------------------
	this.postAsync = function ( sContent, callbackFunction, sUrlParam )
	{
		var sURL = _sDefaultUrl ;
		if ( sUrlParam != null )
		{
			sURL = sUrlParam ;
		}
		if ( callbackFunction == null )
		{
			_error ("postAsync : callback function is NULL !");
			return ;
		}
		_fCallback = callbackFunction ;
		
		if ( _oAsyncXHR == null )
    	{
			_error ( "postAsync : XMLHttpRequest object is null !" );
			return ;
    	}

		if ( sContent != null )
		{
			//_sRequestContent = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\r\n" + sContent ;
			_sRequestContent = sContent ;
		}
		else
		{
			_sRequestContent = null ;
		}
		
		//--- (1) Setup the HTTP Request Method and URL 
	    _oAsyncXHR.open ("POST", sURL, true);  // true = ASYNCHRONOUS CALL
	    
	    //_oAsyncXHR.setRequestHeader("Content-Type", "text/xml;charset=ISO-8859-1");
	    _oAsyncXHR.setRequestHeader("Content-Type", "text/xml");

		//--- (2) Set the callback method
		_oAsyncXHR.onreadystatechange = _asyncCallback ;

		//--- (3) Send the request to the server 
	    _oAsyncXHR.send (_sRequestContent); // Send the content (via HTTP)

	}

	//------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------
//                              T H E    E N D
//------------------------------------------------------------------------------
