//------------------------------------------------------------------------------
// fwkScreenRequest.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 12/12/2005 )
// Author  : Laurent Gurin 
// Changes : none (creation)
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// CLASS DEFINITION
//------------------------------------------------------------------------------
function ScreenRequest( )
{
	//==============================================================================
	// PRIVATE ATTRIBUTES
	//==============================================================================
	var _bTrace  = false ;
	var _sEOL    = TelosysConst.getRequestEndOfLine() ;
	
	var _sScreenName = null ;
	var _sScreenId   = null ;
	var _sAction     = null ;
	var _sProcedure  = null ;
	var _sElements   = null ;
	
	var _sFlagGet    = null ;
	var _sFlagSet    = null ;
	var _sFlagClear  = null ;
	
	var _oParameters = new Object();
	var _oViews      = null ;

	var _sDataTags   = null ;
	
	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================
	function _trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	function _error ( sMsg ) 
	{
		alert ( "ERROR in 'ScreenRequest' object : \n\n" + sMsg ) ;
	}
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================
	this.setScreenName = function ( s ) 
	{
		_sScreenName = s ;
	}
	this.getScreenName = function() 
	{
		return _sScreenName ;
	}
	//------------------------------------------------------------------------------
	this.setScreenId = function ( s ) 
	{
		_sScreenId = s ;
	}
	this.getScreenId = function() 
	{
		return _sScreenId ;
	}
	//------------------------------------------------------------------------------
	this.setAction = function ( s ) 
	{
		_sAction = s ;
	}
	this.getAction = function() 
	{
		return _sAction ;
	}
	//------------------------------------------------------------------------------
	this.setProcedure = function ( s ) 
	{
		_sProcedure = s ;
	}
	this.getProcedure = function() 
	{
		return _sProcedure ;
	}
	//------------------------------------------------------------------------------
	this.setElements = function ( s )
	{
		_sElements = s ;
	}
	this.getElements = function()
	{
		return _sElements ;
	}
	//------------------------------------------------------------------------------
	this.setFlagGet = function ( s ) 
	{
		_sFlagGet = s ;
	}
	this.getFlagGet = function() 
	{
		return _sFlagGet ;
	}
	//------------------------------------------------------------------------------
	this.setFlagSet = function ( s ) 
	{
		_sFlagSet = s ;
	}
	this.getFlagSet = function () 
	{
		return _sFlagSet ;
	}
	//------------------------------------------------------------------------------
	this.setFlagClear = function ( s ) 
	{
		_sFlagClear = s ;
	}
	this.getFlagClear = function() 
	{
		return _sFlagClear ;
	}
	
	//------------------------------------------------------------------------------
	// Parameters
	//------------------------------------------------------------------------------
	this.setParameter = function ( sName, sValue ) 
	{		
		_trace ( "setParameter : " + sName + "=" + sValue ) ;
		if ( sName == null )
		{
			_error ( "setParameter() : name = null" );
			return ;
		}
		if ( sValue == null )
		{
			_error ( "setParameter() : value = null" );
			return ;
		}
		if ( _oParameters == null ) { _oParameters = new Object(); }
		_oParameters[sName] = sValue ;
	}
	//------------------------------------------------------------------------------
	this.clearParameters = function () 
	{		
		_trace ( "clearParameters " ) ;
		_oParameters = new Object(); ;
	}
	//------------------------------------------------------------------------------
	this.showParameters = function () 
	{
		if ( _oParameters != null ) 
		{
			var s = "PARAMETERS : \n" ;
			for ( p	in _oParameters )
			{
				s += p + "=" + _oParameters[p] + "\n" ;
			}
			alert( s ); 
		}
		else
		{
			alert( "NO PARAMETERS" ); 
		}
	}
	//------------------------------------------------------------------------------
	// Views
	//------------------------------------------------------------------------------
	this.setView = function ( sName, sRenderer ) 
	{		
		_trace ( "setView : view='" + sName + "' renderer='" + sRenderer +"'") ;
		if ( sName == null )
		{
			_error ( "setView() : name = null" );
			return ;
		}
		if ( sRenderer == null )
		{
			_error ( "setView() : renderer = null" );
			return ;
		}
		if ( _oViews == null ) { _oViews = new Object(); }
		_oViews[sName] = sRenderer ;
	}
	//------------------------------------------------------------------------------
	this.clearViews = function () 
	{		
		_trace ( "clearViews" ) ;
		_oViews = null ;
	}
	//------------------------------------------------------------------------------
	this.showViews = function () 
	{
		if ( _oViews != null ) 
		{
			var s = "VIEWS : \n" ;
			for ( v	in _oViews )
			{
				s += " name='" + v + "' renderer='" + _oViews[v] + "' \n" ;
			}
			alert( s ); 
		}
		else
		{
			alert( "NO VIEWS" ); 
		}
	}

	//------------------------------------------------------------------------------
	// Data XML Tags ( to be inserted between <data> and </data> )
	//------------------------------------------------------------------------------
	this.setData = function ( s ) 
	{
		_trace ( "setData() : \n" + s ) ;
		_sDataTags = s ;
	}
	//------------------------------------------------------------------------------
	this.clearData = function ( s ) 
	{
		_trace ( "clearData()" ) ;
		_sDataTags = null ;
	}
	//------------------------------------------------------------------------------
	this.showData = function () 
	{
		alert ( "DATA : \n" + _sDataTags ) ;
	}
		
	//------------------------------------------------------------------------------
	// Build and return the XML Request
	//------------------------------------------------------------------------------
	this.getXmlRequest = function () 
	{
		var sXml = "<request " 
			+ " screen-name=\"" + _sScreenName + "\" " 
			+ " screen-id=\""   + _sScreenId   + "\" " 
			+ " action=\""      + _sAction     + "\" " ;
		if ( _sAction.toLowerCase() == "exec" ) 
		{
			if ( _sProcedure != null ) 
			{
				sXml += " proc-name=\"" + _sProcedure + "\" " ;
			}
			else
			{
				_error("action 'exec' without procedure name ! ");
			}
		}
		if ( _sElements != null ) 
		{
			sXml += " elements=\"" + _sElements + "\" " ;
		}
		if ( _sFlagGet != null ) 
		{
			sXml += " get=\"" + _sFlagGet + "\" " ;
		}
		if ( _sFlagSet != null ) 
		{
			sXml += " set=\"" + _sFlagSet + "\" " ;
		}
		if ( _sFlagClear != null ) 
		{
			sXml += " clear=\"" + _sFlagClear + "\" " ;
		}		
		sXml += " >" + _sEOL ;

		//--- Parameters list 
		if ( _oParameters != null )
		{
			var sParams = "" ;
			var iParamsCount = 0 ;
			for ( p	in _oParameters )
			{
				sParams += "<param " + p + "=\"" + _oParameters[p] + "\" />" + _sEOL;
				iParamsCount++ ;
			}
			if ( iParamsCount > 0 ) 
			{
				sXml += "<params>" + _sEOL + sParams + "</params>" + _sEOL ;
			}
		}
		
		//--- Views list 
		if ( _oViews != null )
		{
			var sViews = "" ;
			var iViewsCount = 0 ;
			for ( v	in _oViews )
			{
				sViews += "<view name=\"" + v + "\" renderer=\"" + _oViews[v] + "\" />" + _sEOL;
				iViewsCount++ ;
			}
			if ( iViewsCount > 0 ) 
			{
				sXml += "<views>" + _sEOL + sViews + "</views>" + _sEOL;
			}
		}		
		
		//--- Data Tags ( 1 to N )
		if ( _sDataTags != null )
		{
			sXml += "<data>" + _sEOL ;
			sXml += _sDataTags ;
			sXml += "</data>" + _sEOL ;						
		}
		
		//--- End of requet
		sXml += "</request>" + _sEOL ;
		
		_trace( "getXmlRequest() : \n" + sXml ); 
		return sXml ;
	}	
	//------------------------------------------------------------------------------	
}
//------------------------------------------------------------------------------
//                                  F  I  N
//------------------------------------------------------------------------------
