//------------------------------------------------------------------------------
// fwkScreenServices.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 12/12/2005 )
// Author  : Laurent GUERIN 
// Changes : none (creation)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// CLASS DEFINITION
//------------------------------------------------------------------------------
function ScreenService( sServiceName, sRootUrl, fctProcessResponse ) 
{
	//--- Call the super constructor
	Ancestor.call(this, "ScreenService", false);
	var _this = this ;
	var _trace = this.trace ;
	var _error = this.error ;
	//------------------------------------------------------------------------------
	if ( arguments.length != 3 )
	{
		_error ( "Constructor : 3 arguments expected !" );
		return ;
	}
	if ( sServiceName == null )
	{
		_error ( "Constructor : 'ServiceName' argument is null !" );
	}
	if ( sRootUrl == null )
	{
		_error ( "Constructor : 'RootUrl' argument is null !" );
	}
	if ( fctProcessResponse == null )
	{
		_error ( "Constructor : 'callback function' argument is null !" );
	}

	_trace( "ScreenService( " + sServiceName + ", " + sRootUrl + ", function )") ;
	
	//==============================================================================
	// PRIVATE ATTRIBUTES
	//==============================================================================		
	var _sServiceName = sServiceName ;
	var _fctProcessResponse = fctProcessResponse ;
	//var _sRootURL = sRootUrl ;
	if ( sRootUrl.charAt( sRootUrl.length ) != '/' )
	{
		sRootUrl = sRootUrl + "/" ;
	}
	var _sServiceURL = sRootUrl + sServiceName + ".svc" ;
	var _oRequester = new ScreenServiceXHR( _sServiceURL ); // The XHR wrapper
	
	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================
	function _buildQueryString()
	{
		var s = "" ;
		//_trace("_buildQueryString() : " + arguments.length + " argument(s)");
		if ( arguments.length > 0 )
		{
			for ( var i = 0 ; i < arguments.length ; i++ )
			{
				if ( i == 0 )
				{
					s = s + "?" ;
				}
				else
				{
					s = s + "&" ;
				}
				s = s + "p"+(i+1)+"="+ escape(arguments[i]).replace(/\+/g, "%2B"); // escape added in v 1.0.2 (regexp is to force "+" conversion)
			}
		}
		//_trace("buildQueryString() : " + s );
		return s ;
	}
	//------------------------------------------------------------------------------
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================
	//------------------------------------------------------------------------------
	this.getURL = function () 
	{
		return _sServiceURL ;
	}		
	//------------------------------------------------------------------------------
	this.getName = function () 
	{
		return _sServiceName ;
	}		
	//------------------------------------------------------------------------------
	this.call = function () 
	{
		_trace("call() : " + arguments.length + " argument(s)");
		try {
			//--- Call _buildQueryString() with the arguments received 
			var sFullURL = _sServiceURL + _buildQueryString.apply(this, arguments); 
			//_oRequester.postSync(sFullURL, null, _fctProcessResponse ); 			
			_oRequester.postAsync(sFullURL, null, _fctProcessResponse );
			//_oRequester.getSync(sFullURL, null, _fctProcessResponse );
			//_oRequester.getAsync(sFullURL, null, _fctProcessResponse );
		}
		catch (ex) {
			_error ( "Exception : " + ex.message );
		}	
	}		
	//------------------------------------------------------------------------------
	this.callWithParams = function (params) 
	{
		_trace("callWithParams() : " + arguments.length + " argument(s)");
		if ( arguments.length < 1 ) 
		{
			alert ( "ERROR : callWithParams() : no arguments !" );
			return;
		}
		//--- Build the Query String 
		var s = "" ;
		for ( p in params )
		{
			if ( s == "" )
			{
				s = "?" ;
			}
			else
			{
				s = s + "&" ;
			}
			s = s + p + "=" + params[p] ; 
		}
		_trace("QueryString = '" + s + "'");

		try {
			//---  
			var sFullURL = _sServiceURL + s ; 
			//_oRequester.postSync(sFullURL, null, _fctProcessResponse ); 			
			_oRequester.postAsync(sFullURL, null, _fctProcessResponse );
			//_oRequester.getSync(sFullURL, null, _fctProcessResponse );
			//_oRequester.getAsync(sFullURL, null, _fctProcessResponse );
		}
		catch (ex) {
			_error ( "Exception : " + ex.message );
		}	
	}		
	//------------------------------------------------------------------------------
	this.doGet = function () 
	{
		_trace("doGet() : ");
		try {
			//--- Call  
			var sFullURL = _sServiceURL + "?action=get" ; 
			//_oRequester.postSync(sFullURL, null, _fctProcessResponse ); 			
			_oRequester.postAsync(sFullURL, null, _fctProcessResponse );
			//_oRequester.getSync(sFullURL, null, _fctProcessResponse );
			//_oRequester.getAsync(sFullURL, null, _fctProcessResponse );
		}
		catch (ex) {
			_error ( "Exception : " + ex.message );
		}	
	}		
	//------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------
//                                  F  I  N
//------------------------------------------------------------------------------
