//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
/**
 * ScreenView class 
 * @param  String  the container Id ( cannot be null )
 * @param  Object  the container object ( cannot be null )
 * @param  String  the screen zone name
 * @param  String  the XML tag name for the view
 * @param  String  the renderer name used to generate the view ( cannot be null )
 */	
function ScreenView ( argContainerId, argContainerObject, argScreenZone, argXmlTag, argRenderer )
{
	if ( argContainerId == null )
	{
		throw new Error("ScreenView constructor : Field ID is null" );
	}
	if ( argContainerObject == null )
	{
		throw new Error("ScreenView constructor : Unknown field with ID = " + argFieldId );
	}
	if ( argXmlTag == null )
	{
		throw new Error("ScreenView constructor : View name (XML tag) is null" );
	}
	if ( argRenderer == null )
	{
		throw new Error("ScreenView constructor : Renderer is null" );
	}

	//==============================================================================
	// PRIVATE ATTRIBUTES
	//==============================================================================
	var _sContainerId = argContainerId ;
	var _oContainer   = argContainerObject ;
	var _sScreenZone  = argScreenZone ;
	var _sXmlTag      = argXmlTag ;
	var _sRenderer    = argRenderer ;
	
	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================
	function _trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE ( ScreenView object ) : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	function _error ( sMsg ) 
	{
		alert ( "ERROR in 'ScreenView' object : \n\n" + sMsg ) ;
	}
	
	//==============================================================================
	// PUBLIC METHODS
	//==============================================================================
	this.getName = function()
	{
		return _sXmlTag ;
	}
	//------------------------------------------------------------------------------
	this.getRenderer = function()
	{
		return _sRenderer ;
	}
	//------------------------------------------------------------------------------
	this.getScreenZone = function()
	{
		return _sScreenZone ;
	}
	//------------------------------------------------------------------------------
	this.getContainerId = function()
	{
		return _sContainerId ;
	}
	//------------------------------------------------------------------------------
	this.getContainer = function()
	{
		return _oContainer ;
	}
	
	//------------------------------------------------------------------------------	
	/**
     * Set the given content in the view container 
     * @param  the content to set in the innerHTML attribute of the container
     */	
	this.setContent = function(argContent)
	{
		if ( _oContainer != null )
		{
			if ( argContent != null )
			{
				_oContainer.innerHTML = argContent ;
				//--- Eval View JS Scripts / NEW v 0.9.8 ( 13/12/07 )
				var scripts = _oContainer.getElementsByTagName("script");
				for(var i=0; i<scripts.length; i++) 
				{
					eval(scripts[i].innerHTML);
				}
			}
			else
			{
				_oContainer.innerHTML = "" ;			
			}
		}
		else
		{
			_error("View " + _sContainerId + " : container is null " );
		}
	}
	//------------------------------------------------------------------------------	
	/**
     * Clear the content of the view container if the view belongs to the
     * given ScreenZone. If the ScreenZone is null, the view is cleared.
     * @param  the screen zone (can be null) 
     */	
	this.clear = function(argZone)
	{
		if ( argZone != null )
		{
			if ( argZone != _sScreenZone ) return ;
		}
		this.setContent("");
	}
	//------------------------------------------------------------------------------
}
