//------------------------------------------------------------------------------
// fwkScreenActions.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 12/12/2005 )
// Author  : Laurent GUERIN 
// Changes : none (creation)
//------------------------------------------------------------------------------
// Standard screen actions behavior
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// CLASS DEFINITION
//------------------------------------------------------------------------------
function ScreenActions( oScreen )
{
	if ( oScreen == null )
	{
		alert ( "ERROR in ScreenActions constructor : no screen parameter !" );
	}
	
	//==============================================================================
	// PRIVATE ATTRIBUTES
	//==============================================================================
	
	var _bTrace  = false ;
	
	var _oScreen  = oScreen ; 
	
	var _oRequester = new TelosysRequester(); // The AJAX requester
	
	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================
	function _trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE ( ScreenActions object ) : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	function _error ( sMsg ) 
	{
		alert ( "ERROR in 'ScreenActions' object : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	this.getRequest = function ( sAction )
	//function _getRequest ( sAction ) 
	{
		//--- Create a new request 
		var request = new ScreenRequest();
		//--- Init the request 
		request.setScreenName( _oScreen.getScreenName() );
		request.setScreenId  ( _oScreen.getScreenId()   );
		if ( sAction )
		{
			request.setAction(sAction);
		}
		return request ;
	}
	
	//------------------------------------------------------------------------------
	function _sendRequest(req)
	{
		if ( _oRequester == null )
		{
			throw new Error( "ScreenActions._sendRequest() : The AJAX requester is null ! ");
		}
		if ( _oScreen == null )
		{
			throw new Error( "ScreenActions._sendRequest() : The SCREEN object is null ! ");
		}
		
		_oRequester.post(req.getXmlRequest(), _oScreen.getRequestURL() );	
		
		var response = _oRequester.getResponse();
		
		//response.show();
		//--- Is the return code OK ? 
		if ( response.getReturnCode() != "0" )
		{
			//--- Show the error
			_error("RESPONSE ERROR : " 
					+ "\nReturn Code = " + response.getReturnCode() 
					+ "\nReturn Message = " + response.getReturnMessage() );
		}
		return response ;
		//alert( "XML RESPONSE : \n" + sResponse );
	}
	//------------------------------------------------------------------------------
	//function _sendRequestAsync(req, callBack)
	this.sendRequestAsync = function (req, callBack)
	{
		if ( _oRequester == null )
		{
			throw new Error( "ScreenActions._sendRequest() : The AJAX requester is null ! ");
		}
		if ( _oScreen == null )
		{
			throw new Error( "ScreenActions._sendRequest() : The SCREEN object is null ! ");
		}
		
		_oRequester.postAsync(req.getXmlRequest(), callBack, _oScreen.getRequestURL() );	

/*		
		var response = _oRequester.getResponse();
		
		//response.show();
		return response ;
		//alert( "XML RESPONSE : \n" + sResponse );
*/
	}
	
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================
	this.getScreen = function () 
	{
		return _oScreen ;
	}
	//------------------------------------------------------------------------------
	//   A C T I O N S
	//------------------------------------------------------------------------------
	this.doOpen = function () 
	{
		_trace("Action : Open");
		var response = null ;
		try {
			//--- Build the request 
			//var request = _getRequest("open") ;
			var request = this.getRequest("open") ;
			//--- Send the request 
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doClose = function () 
	{
		_trace("Action : Close");
		var response = null ;
		try {
			//--- Build the request 
			//var request = _getRequest("close") ;
			var request = this.getRequest("close") ;
			//--- Send the request 
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	//------------------------------------------------------------------------------
	this.doGet = function ( bUpdateScreen ) 
	{
		_trace("Action : Get");
		var response = null ;
		try {	
			//--- Build the request 
			//var request = _getRequest("get") ;		
			var request = this.getRequest("get") ;		
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
				/*** OLD
				if ( response.dataVoid() )
				{
					//alert ( "No Data ( void ) !" );
					this.doGet_DataVoid(response);
				}
				else
				{
					//--- Populate the screen fields with XML data recieved
					_oScreen.XmlToScreen(response);
				}	
				***/		
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doSet = function () 
	{
		_trace("Action : Set");
		var response = null ;
		try {	
			//--- Build the request 
			//var request = _getRequest("set") ;
			var request = this.getRequest("set") ;
			
			var sXmlData = _oScreen.ScreenToXml();
			request.setData(sXmlData);
			
			//--- Send the request 
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}			
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doSetAsync = function (callBack) 
	{
		_trace("Action : SetAsync");
		var response = null ;
		try {	
			//--- Build the request 
			//var request = _getRequest("set") ;
			var request = this.getRequest("set") ;
			
			var sXmlData = _oScreen.ScreenToXml();
			request.setData(sXmlData);
			
			//--- Send the request 
			//_sendRequestAsync(request,callBack);
			this.sendRequestAsync(request,callBack);
		}
		catch (ex) {
			alert ( "ERROR: " + ex.message );
		}			
	}	
	//------------------------------------------------------------------------------
	this.doClear = function ( bUpdateScreen ) 
	{
		_trace("Action : Clear");
		var response = null ;
		try {
			//--- Build the request 
			//var request = _getRequest("clear") ;		
			var request = this.getRequest("clear") ;		
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( bUpdateScreen == null || bUpdateScreen == true )
				{
					//--- Clear the screen
					_oScreen.Clear();
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	//------------------------------------------------------------------------------
	this.doLoad = function ( params, bUpdateScreen ) 
	{
		_trace("Action : Load");
		var response = null ;
		try {
			//--- Build the request 
			var request = this.getRequest("load") ;

			//--- Call external method to set the request parameters
			//this.prepareToLoad(request); 
			
			//--- Set request parameters
			if ( params != null )
			{
				for ( p in params )
				{
					request.setParameter( p, params[p] );
				}			
			}
			
			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataNotFound() != true ) // Data Found
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}	
		return response ;
	}
	
	//------------------------------------------------------------------------------
	this.doLoadAsync = function (params, callBack) 
	{
		_trace("Action : LoadAsync");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("load") ;
			//--- Set request parameters
			for ( p in params )
			{
				request.setParameter( p, params[p] );
			}
			//--- Send the request
			this.sendRequestAsync(request,callBack);
		}
		catch (ex) {
			alert ( "ERROR: " + ex.message );
		}			
	}	
	//------------------------------------------------------------------------------
	this.populateFields = function (response) 
	{
		if ( response.isOK() )
		{
			if ( response.dataNotFound() != true ) // Data Found
			{
				//--- Populate the screen fields with XML data recieved
				_oScreen.XmlToScreen(response);
			}
		}
		else
		{
			alert("ERROR : cannot populate fields ( response status is not OK ) !");
		}
	}
	//------------------------------------------------------------------------------
	this.clearFields = function () 
	{
		//--- Clear the screen
		_oScreen.Clear();	
	}
	//------------------------------------------------------------------------------
	
	//------------------------------------------------------------------------------
	// param Action : "save", "insert", "update"
	//------------------------------------------------------------------------------
	this.sendScreenData = function (sAction)
	{
		_trace("sendScreenData : Action = '" + sAction + "'" );
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest(sAction) ;
			
			//--- Build the XML tree from screen fields 
			var sXmlData = _oScreen.ScreenToXml();
			request.setData(sXmlData);
			
			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doSave = function () 
	{
		_trace("Action : Save");
		return this.sendScreenData("save");
	}
	//------------------------------------------------------------------------------
	this.doInsert = function () 
	{
		_trace("Action : Insert");
		return this.sendScreenData("insert");
	}
	//------------------------------------------------------------------------------
	this.doUpdate = function () 
	{
		_trace("Action : Update");
		return this.sendScreenData("update");
	}
	//------------------------------------------------------------------------------
	this.doDelete = function ( params, bUpdateScreen ) 
	{
		_trace("Action : Delete");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("delete") ;
			
			//--- Call external method to set the request parameters
			//this.prepareToDelete(request);

			//--- Set request parameters
			if ( params != null )
			{
				for ( p in params )
				{
					request.setParameter( p, params[p] );
				}			
			}
			
			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( bUpdateScreen == null || bUpdateScreen == true )
				{
					//--- Clear the screen
					_oScreen.Clear();
				}
			}			
		}
		catch (ex) {
			_error ( ex.message );
		}	
		return response ;
	}
	
	//------------------------------------------------------------------------------
	// ACTIONS FOR LIST MANAGEMENT
	//------------------------------------------------------------------------------
	this.doListSelect = function ( sListName, sItemName, iOldIndex, iIndex, bUpdateScreen ) 
	{
		_trace("Action : SelectListElement");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_select") ;		

			request.setParameter( "list", sListName );
			request.setParameter( "item", sItemName );
			request.setParameter( "old-index", iOldIndex );
			request.setParameter( "new-index", iIndex );
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListInsert = function ( sListName, sItemName, iOldIndex, iIndex, bUpdateScreen ) 
	{
		_trace("Action : ListInsert");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_insert") ;		

			request.setParameter( "list", sListName );
			request.setParameter( "item", sItemName );
			request.setParameter( "new-index", iIndex );
			request.setParameter( "old-index", iOldIndex );
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListAdd = function ( sListName, sItemName, iOldIndex, bUpdateScreen ) 
	{
		_trace("Action : ListAdd");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_add") ;		

			request.setParameter( "list", sListName );
			request.setParameter( "item", sItemName );
			//request.setParameter( "new-index", iIndex );
			request.setParameter( "old-index", iOldIndex );
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListRemove = function ( sListName, sItemName, iOldIndex, iIndex, bUpdateScreen ) 
	{
		_trace("Action : ListRemove");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_remove") ;		

			request.setParameter( "list", sListName );
			request.setParameter( "item", sItemName );
			request.setParameter( "new-index", iIndex );
			request.setParameter( "old-index", iOldIndex );
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListClear = function ( sListName, sItemName, iOldIndex ) 
	{
		_trace("Action : ListClear");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_clear") ;		

			request.setParameter( "list", sListName );
			//request.setParameter( "item", sItemName );
			//request.setParameter( "new-index", iIndex );
			request.setParameter( "old-index", iOldIndex );
			
			//request.setData( _oScreen.ScreenToXml(sItemName) ); 
			
			//--- Send the request 
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListSort = function ( sListName, sItemName, iOldIndex, sFieldName, sOrder ) 
	{
		_trace("Action : ListSort");
		if ( arguments.length < 4 )
		{
			_error ( "doListSort() : 4 or 5 arguments expected ( " + arguments.length + " arguments )" ) ;
			return ;
		}
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_sort") ;		

			request.setParameter( "list",  sListName );
			request.setParameter( "item", sItemName );
			request.setParameter( "old-index", iOldIndex );
			//--- Sort parameters 
			request.setParameter( "field", sFieldName );
			if ( sOrder != null ) // order parameter is optional
			{
				request.setParameter( "order", sOrder );
			}		
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				//--- Populate the screen fields with list view recieved
				_oScreen.XmlToScreen(response);
			}			
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	
	//------------------------------------------------------------------------------
	this.doExec = function (sProcName, htParameters) 
	{
		_trace("Action : Exec");
		var response = null ;
		if ( sProcName == null )
		{
			_error ( "doExec() : No procedure name (argument is null) " );
		}
		try {	
			//--- Build the request 
			//var request = _getRequest("exec") ;
			var request = this.getRequest("exec") ;
			//--- Set the procedure name 
			request.setProcedure(sProcName);
			//--- Set the parameters
			if ( htParameters != null )
			{
				for ( name in htParameters )
				{
					_trace ( "request.setParameter(" + name + ", " + htParameters[name] + ")" ) ;
					request.setParameter(name, htParameters[name]);
				}
			}
			
			//--- Call external method to set the request parameters
			/*** OLD
			this.beforeExec(request);
			***/

			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);

			/*** OLD
			if ( response.isOK() )
			{
				//--- Call external method ????
				this.afterExec(request,response);
			}
			***/
		}
		catch (ex) {
			_error ( ex.message );
		}	
		return response ;
	}
		
	//------------------------------------------------------------------------------
	// Default external methods
	//------------------------------------------------------------------------------
	/*** OLD
	this.prepareToLoad = function ( request ) 
	{
		throw new Error( "ScreenActions : Specific method 'prepareToLoad' not implemented ! ");
	}
	***/
	//------------------------------------------------------------------------------
	/*** OLD
	this.prepareToDelete = function ( request ) 
	{
		throw new Error( "ScreenActions : Specific method 'prepareToDelete' not implemented ! ");
	}
	***/
	//------------------------------------------------------------------------------
	/*** OLD
	this.doGet_DataVoid = function ( response ) 
	{
		throw new Error( "ScreenActions : Specific method 'doGet_DataVoid' not implemented ! ");
	}
	//------------------------------------------------------------------------------
	this.doLoad_DataNotFound = function ( response ) 
	{
		throw new Error( "ScreenActions : Specific method 'doLoad_DataNotFound' not implemented ! ");
	}
	***/
	//------------------------------------------------------------------------------
	/***
	this.beforeExec = function ( request ) 
	{
		throw new Error( "ScreenActions : Specific method 'beforeExec' not implemented ! ");
	}
	//------------------------------------------------------------------------------
	this.afterExec = function ( request, response ) 
	{
		throw new Error( "ScreenActions : Specific method 'afterExec' not implemented ! ");
	}
	***/
	//------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------
//                                  F  I  N
//------------------------------------------------------------------------------
