//------------------------------------------------------------------------------
// fwkScreenActions.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 12/12/2005 )
// Author  : Laurent GUERIN 
// Changes : none (creation)
//------------------------------------------------------------------------------
// Standard screen actions behavior
//------------------------------------------------------------------------------

/**
 * Client stub to invoke server actions 
 * @class ScreenActions
 * @param Object the Screen instance 
 * @author Laurent GUERIN
 */

//------------------------------------------------------------------------------
// CLASS DEFINITION
//------------------------------------------------------------------------------
function ScreenActions( oScreen )
{
	if ( oScreen == null )
	{
		alert ( "ERROR in ScreenActions constructor : no screen parameter !" );
	}
	
	//==============================================================================
	// PRIVATE ATTRIBUTES
	//==============================================================================
	
	var _bTrace  = false ;
	
	var _oScreen  = oScreen ; 
	
	var _oRequester = new TelosysRequester(); // The AJAX requester
	
	var _oParameters = new Map(); // The parameters to send in the action request
	
	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================
	function _trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE ( ScreenActions object ) : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	function _error ( sMsg ) 
	{
		alert ( "ERROR in 'ScreenActions' object : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	this.getRequest = function ( sAction )
	//function _getRequest ( sAction ) 
	{
		//--- Create a new request 
		var request = new ScreenRequest();
		//--- Init the request 
		request.setScreenName( _oScreen.getScreenName() );
		request.setScreenId  ( _oScreen.getScreenId()   );
		//--- Set the action name ( "get", "set", "load", .. ) 
		if ( sAction )
		{
			request.setAction(sAction);
		}
		//--- Set the action parameters in the request
		if ( _oParameters.size() > 0 )
		{
			var name = null ;
			var names = _oParameters.keysArray();
			for ( i in names )
			{
				name = names[i] ;
				value = _oParameters.get(name);
		//alert("request.setParameter("+name+","+value+")");
				request.setParameter(name, value );
			}
			//--- Clear the action parameters 
			_oParameters.clear();
		}
		return request ;
	}
	
	//------------------------------------------------------------------------------
	function _sendRequest(req)
	{
		if ( _oRequester == null )
		{
			throw new Error( "ScreenActions._sendRequest() : The AJAX requester is null ! ");
		}
		if ( _oScreen == null )
		{
			throw new Error( "ScreenActions._sendRequest() : The SCREEN object is null ! ");
		}
		
		_oRequester.post(req.getXmlRequest(), _oScreen.getRequestURL() );	
		
		var response = _oRequester.getResponse();
		
		//response.show();
		//--- Is the return code OK ? 
		if ( response.getReturnCode() != "0" )
		{
			var sMsg1 = "RESPONSE ERROR : " 
					+ "\nReturn Code = " + response.getReturnCode() 
					+ "\nReturn Message = " + response.getReturnMessage();
						
			var sMsg2 = "" ;
			var exceptions = response.getExceptions();
			if ( exceptions != null )
			{
				sMsg2 = "\n\nSERVER EXCEPTIONS : "
				var ex = null ;
				for ( i in exceptions )
				{	
					ex = exceptions[i];
					sMsg2 = sMsg2 + "\n. " + ex.getClass() + " : \n   " + ex.getMessage();
				}
			}
			
			//--- Show the error
			/**
			_error("RESPONSE ERROR : " 
					+ "\nReturn Code = " + response.getReturnCode() 
					+ "\nReturn Message = " + response.getReturnMessage() );
			**/
			_error( sMsg1 + sMsg2 );
		}
		return response ;
		//alert( "XML RESPONSE : \n" + sResponse );
	}
	
	
	//------------------------------------------------------------------------------
	// Set request elements
	//------------------------------------------------------------------------------
	function _setElements(request, elements)
	{
//	alert("_setElements");
		if ( elements != null ) 
		{
			if ( elements.length == 0 )
			{
				//--- No elements => get all the elements of the screen
				request.setElements(_oScreen.getElementsString());				
			}
			else
			{
				//--- Build the elements list
				var sElementsList = "" ;
				var iNbElements = 0 ;
				for ( i=0 ; i < elements.length ; i++ )
				{
					if ( typeof elements[i] == "string" )
					{
						if ( iNbElements > 0 ) 
						{
							sElementsList += ",";
						}
						sElementsList += elements[i];
						iNbElements++;
					}
				}				
				//--- Set the given elements list in the request
				request.setElements(sElementsList);
			}	
		}
	}	
	//------------------------------------------------------------------------------
	// Get the UpdateScreen flag value using the last argument if it exist and is a boolean
	//------------------------------------------------------------------------------
	function _getUpdateScreenFlag(args, bDefault)
	{
	//alert("_getUpdateScreenFlag");
		if ( args != null ) 
		{
			if ( args.length > 0 )
			{
				//--- Is there a boolean argument at the end ?
				var lastArg = args[args.length-1] ;
				if ( typeof lastArg == "boolean" )
				{
	//alert("_getUpdateScreenFlag : return lastArg = " + lastArg );
					return lastArg ;
				}
			}
		}		
	//alert("_getUpdateScreenFlag : return bDefault = " + bDefault );
		return bDefault ;
	}
	//------------------------------------------------------------------------------
	// Set a view in the request (with renderer if any)
	//------------------------------------------------------------------------------
	function _setView(request, psViewName, psRenderer )
	{
		var oScreenView = _searchView( psViewName ) ;
		if ( oScreenView != null )
		{
			if ( psRenderer != null )
			{
				request.setView( oScreenView.getName(), psRenderer );
			}
			else
			{
				request.setView( oScreenView.getName(), oScreenView.getRenderer() );
			}
		}
	}
	//------------------------------------------------------------------------------
	// Set the given views in the request
	//------------------------------------------------------------------------------
	function _setViews(request, views)
	{
		if ( views != null ) 
		{
			if ( views.length == 0 )
			{
				//--- No views => set all the views of the screen in the request
				_setAllViews(request);
			}
			else
			{
				//--- Set the given views in the request
				for ( i=0 ; i < views.length ; i++ )
				{
					if ( typeof views[i] != "string" )
					{
						_error( "_setViews() : argument[" + i + "] is not a String ! type = " + typeof views[i] );					
					}
					else
					{
						var oScreenView = _searchView( views[i] ) ;
						if ( oScreenView != null )
						{
							request.setView( oScreenView.getName(), oScreenView.getRenderer() );						
						}
					}
				}				
			}	
		}
	}
	//------------------------------------------------------------------------------
	// Set all the views of the screen in the request
	//------------------------------------------------------------------------------
	function _setAllViews(request)
	{
		//--- Set views names if any
		var views = _oScreen.getViews();
		if ( views != null )
		{
			var oScreenView = null ;			
			//--- For each view registered in this screen 
			for ( var i = 0 ; i < views.length ; i++ )
			{
				oScreenView = views[i] ;
				if ( oScreenView != null )
				{
					request.setView( oScreenView.getName(), oScreenView.getRenderer() );			
				}
				else
				{
					_error( "doGet() : ScreenViews ["+i+"] is null ! " );					
				}
			}
		}
	}	
	//------------------------------------------------------------------------------
	// Search a view (by name) in the views list of the screen
	//------------------------------------------------------------------------------
	function _searchView(psViewName)
	{
		var views = _oScreen.getViews();
		if ( views != null )
		{
			var oScreenView = null ;			
			//--- For each view registered in this screen 
			for ( var i = 0 ; i < views.length ; i++ )
			{
				oScreenView = views[i] ;
				if ( oScreenView != null )
				{
					if ( oScreenView.getName() == psViewName )
					{
						return oScreenView ;
					}
				}
				else
				{
					_error( "doGetView() : ScreenViews ["+i+"] is null ! " );					
				}
			}
			_error( "doGetView() : unknown view '" + psViewName + "' ! " );					
		}
		else
		{
			_error( "doGetView() : no views in this screen ! " );					
		}	
		return null;
	}
	//------------------------------------------------------------------------------
	//function _sendRequestAsync(req, callBack)
	this.sendRequestAsync = function (req, callBack)
	{
		if ( _oRequester == null )
		{
			throw new Error( "ScreenActions._sendRequest() : The AJAX requester is null ! ");
		}
		if ( _oScreen == null )
		{
			throw new Error( "ScreenActions._sendRequest() : The SCREEN object is null ! ");
		}
		
		_oRequester.postAsync(req.getXmlRequest(), callBack, _oScreen.getRequestURL() );	

/*		
		var response = _oRequester.getResponse();
		
		//response.show();
		return response ;
		//alert( "XML RESPONSE : \n" + sResponse );
*/
	}
	
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================
	/**
     * Retuns the Screen instance 
     * @return Object the Screen instance 
     */	
	this.getScreen = function () 
	{
		return _oScreen ;
	}
	//------------------------------------------------------------------------------
	//   PARAMETERS
	//------------------------------------------------------------------------------
	this.setParameter = function (sName, sValue) 
	{
		if ( sName == null )
		{
			_error ( "setParameter(name,value) : name is null" ) ;
		}
		if ( sValue == null )
		{
			_error ( "setParameter(name,value) : value is null" ) ;
		}
		_oParameters.put(sName, sValue) ;
	}
	//------------------------------------------------------------------------------
	this.clearParameters = function () 
	{
		_oParameters.clear() ;		
	}
	//------------------------------------------------------------------------------
	this.showParameters = function () 
	{
		_oParameters.show() ;		
	}
	//------------------------------------------------------------------------------
	//   A C T I O N S
	//------------------------------------------------------------------------------
	/**
     * Invokes the standard "OPEN" action  
     * @return Object the server response object  
     */	
	this.doOpen = function () 
	{
		_trace("Action : Open");
		var response = null ;
		try {
			//--- Build the request 
			//var request = _getRequest("open") ;
			var request = this.getRequest("open") ;
			//--- Send the request 
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	/**
     * Invokes the standard "CLOSE" action  
     * @return Object the server response object  
     */	
	this.doClose = function () 
	{
		_trace("Action : Close");
		var response = null ;
		try {
			//--- Build the request 
			//var request = _getRequest("close") ;
			var request = this.getRequest("close") ;
			//--- Send the request 
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	/**
     * Invokes the standard "GET" action with an elements list
     * @param  Variable list of arguments ( 0 .. N ) 
     *         if the last argument is a boolean it is considered as the update screen flag
     * @return Object  the server response object  
     */	
	this.doGetElements = function () 
	{
		_trace("Action : GetElements");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("get") ;
			_setElements(request, arguments);
			var bUpdateScreen =_getUpdateScreenFlag(arguments, true);
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}

	//------------------------------------------------------------------------------
	this.doGetViews = function () 
	{
		_trace("Action : GetViews");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("get") ;
			//--- Don't want elements => set "elements" to void list
			request.setElements("");
			//--- Set views and renderer
			_setViews(request, arguments);
	/***		
			if ( arguments.length == 0 )
			{
				//--- No arguments => set all the views of the screen in the request
				_setAllViews(request);
			}
			else
			{
				//--- Set the given views in the request
				for ( i=0 ; i < arguments.length ; i++ )
				{
					if ( typeof arguments[i] != "string" )
					{
						_error( "doGetViews() : argument[" + i + "] is not a String ! type = " + typeof arguments[i] );					
					}
					else
					{
						var oScreenView = _searchView( arguments[i] ) ;
						if ( oScreenView != null )
						{
							request.setView( oScreenView.getName(), oScreenView.getRenderer() );						
						}
					}
				}				
			}
	***/
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				_oScreen.XmlToScreen(response);
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	
	//------------------------------------------------------------------------------
	this.doGetView = function ( psViewName, psRenderer ) 
	{
		_trace("Action : GetView");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("get") ;
			//--- Don't want elements => set "elements" to void list
			request.setElements("");
			//--- Set the given view
			_setView(request, psViewName, psRenderer ) ;

			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				_oScreen.XmlToScreen(response);
				/**
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
				**/
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	
	//------------------------------------------------------------------------------
	/**
     * Invokes the standard "GET" action to get data from the server 
     * Get all the Screen Context elements and all the views
     * @param  boolean true = update the screen, false = no screen update
     * @return Object  the server response object  
     */	
	this.doGet = function ( bUpdateScreen ) 
	{
		//return this.doGetElements( null, bUpdateScreen );
		_trace("Action : Get");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("get") ;
			//--- Get all the elements used by the screen
			request.setElements(_oScreen.getElementsString());
			//--- Set all the views names of the screen if any
			_setAllViews(request);
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	/**
     * Invokes the standard "SET" action to update data on the server 
     * Set all the Screen Context elements 
     * @return Object  the server response object  
     */	
	this.doSet = function () 
	{
		_trace("Action : Set");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("set") ;
			
			var sXmlData = _oScreen.ScreenToXml();
			request.setData(sXmlData);
			
			//--- Send the request 
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}			
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doSetAsync = function (callBack) 
	{
		_trace("Action : SetAsync");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("set") ;
			
			var sXmlData = _oScreen.ScreenToXml();
			request.setData(sXmlData);
			
			//--- Send the request 
			//_sendRequestAsync(request,callBack);
			this.sendRequestAsync(request,callBack);
		}
		catch (ex) {
			alert ( "ERROR: " + ex.message );
		}			
	}	
	//------------------------------------------------------------------------------
	/**
     * Invokes the standard "CLEAR" action to clear the Screen Context on the server 
     * @param  boolean true = clear the screen fields, false = do not clear the screen fields
     * @return Object  the server response object  
     */	
	this.doClear = function ( bUpdateScreen ) 
	{
		_trace("Action : Clear");
		var response = null ;
		try {
			//--- Build the request 
			//var request = _getRequest("clear") ;		
			var request = this.getRequest("clear") ;		
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( bUpdateScreen == null || bUpdateScreen == true )
				{
					//--- Clear the screen
					_oScreen.Clear();
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	
	//------------------------------------------------------------------------------
	//  LOAD ACTIONS
	//------------------------------------------------------------------------------
	this.doLoadViews = function () 
	{
		_trace("Action : doLoadViews");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("load") ;
			//--- Don't want elements => set "elements" to void list
			request.setElements("");
			//--- Set views and renderer
			_setViews(request, arguments);
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataNotFound() != true ) // Data Found
				{
					_oScreen.XmlToScreen(response);
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doLoadView = function ( psViewName, psRenderer ) 
	{
		_trace("Action : doLoadView");
		var response = null ;
		try {
			//--- Build the request 
			var request = this.getRequest("load") ;
			//--- Don't want elements => set "elements" to void list
			request.setElements("");
			//--- Set the given view
			_setView(request, psViewName, psRenderer ) ;
			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);
			//--- Use the response ( View => allways update screen )
			if ( response.isOK() )
			{
				if ( response.dataNotFound() != true ) // Data Found
				{
					_oScreen.XmlToScreen(response);
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}	
		return response ;
	}
	//------------------------------------------------------------------------------
	//this.doLoadElements = function ( sElements, bUpdateScreen ) 
	this.doLoadElements = function () 
	{
		_trace("Action : Load");
		var response = null ;
		try {
			//--- Build the request 
			var request = this.getRequest("load") ;
			//--- Elements to get after the load
			_setElements(request, arguments);
			//--- Flag to update (or not) the screen fields
			var bUpdateScreen =_getUpdateScreenFlag(arguments, true);
			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataNotFound() != true ) // Data Found
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}	
		return response ;
	}
	//------------------------------------------------------------------------------
	//this.doLoad = function ( params, bUpdateScreen ) 
	this.doLoad = function ( bUpdateScreen ) 
	{
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("load") ;
			//--- Get all the elements used by the screen
			request.setElements(_oScreen.getElementsString());
			//--- Set all the views names of the screen if any
			_setAllViews(request);
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataNotFound() != true ) // Data Found
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	
	//------------------------------------------------------------------------------
	this.doLoadAsync = function (params, callBack) 
	{
		_trace("Action : LoadAsync");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("load") ;
			//--- Set request parameters
			for ( p in params )
			{
				request.setParameter( p, params[p] );
			}
			//--- Send the request
			this.sendRequestAsync(request,callBack);
		}
		catch (ex) {
			alert ( "ERROR: " + ex.message );
		}			
	}	
	
	//------------------------------------------------------------------------------
	this.populateFields = function (response) 
	{
		if ( response.isOK() )
		{
			if ( response.dataNotFound() != true ) // Data Found
			{
				//--- Populate the screen fields with XML data recieved
				_oScreen.XmlToScreen(response);
			}
		}
		else
		{
			alert("ERROR : cannot populate fields ( response status is not OK ) !");
		}
	}
	//------------------------------------------------------------------------------
	this.clearFields = function () 
	{
		//--- Clear the screen
		_oScreen.Clear();	
	}
	//------------------------------------------------------------------------------
	
	//------------------------------------------------------------------------------
	// param Action : "save", "insert", "update"
	//------------------------------------------------------------------------------
	this.sendScreenData = function (sAction)
	{
		_trace("sendScreenData : Action = '" + sAction + "'" );
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest(sAction) ;
			
			//--- Build the XML tree from screen fields 
			var sXmlData = _oScreen.ScreenToXml();
			request.setData(sXmlData);
			
			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doSave = function () 
	{
		_trace("Action : Save");
		return this.sendScreenData("save");
	}
	//------------------------------------------------------------------------------
	this.doInsert = function () 
	{
		_trace("Action : Insert");
		return this.sendScreenData("insert");
	}
	//------------------------------------------------------------------------------
	this.doUpdate = function () 
	{
		_trace("Action : Update");
		return this.sendScreenData("update");
	}
	//------------------------------------------------------------------------------
	this.doDelete = function ( params, bUpdateScreen ) 
	{
		_trace("Action : Delete");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("delete") ;
			
			//--- Call external method to set the request parameters
			//this.prepareToDelete(request);

			//--- Set request parameters
			if ( params != null )
			{
				for ( p in params )
				{
					request.setParameter( p, params[p] );
				}			
			}
			
			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( bUpdateScreen == null || bUpdateScreen == true )
				{
					//--- Clear the screen
					_oScreen.Clear();
				}
			}			
		}
		catch (ex) {
			_error ( ex.message );
		}	
		return response ;
	}
	
	//------------------------------------------------------------------------------
	// ACTIONS FOR LIST MANAGEMENT
	//------------------------------------------------------------------------------
	this.doListSelect = function ( sListName, sItemName, iOldIndex, iIndex, bUpdateScreen ) 
	{
		_trace("Action : SelectListElement");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_select") ;		

			request.setParameter( "list", sListName );
			request.setParameter( "item", sItemName );
			request.setParameter( "old-index", iOldIndex );
			request.setParameter( "new-index", iIndex );
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListInsert = function ( sListName, sItemName, iOldIndex, iIndex, bUpdateScreen ) 
	{
		_trace("Action : ListInsert");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_insert") ;		

			request.setParameter( "list", sListName );
			request.setParameter( "item", sItemName );
			request.setParameter( "new-index", iIndex );
			request.setParameter( "old-index", iOldIndex );
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListAdd = function ( sListName, sItemName, iOldIndex, bUpdateScreen ) 
	{
		_trace("Action : ListAdd");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_add") ;		

			request.setParameter( "list", sListName );
			request.setParameter( "item", sItemName );
			//request.setParameter( "new-index", iIndex );
			request.setParameter( "old-index", iOldIndex );
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListRemove = function ( sListName, sItemName, iOldIndex, iIndex, bUpdateScreen ) 
	{
		_trace("Action : ListRemove");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_remove") ;		

			request.setParameter( "list", sListName );
			request.setParameter( "item", sItemName );
			request.setParameter( "new-index", iIndex );
			request.setParameter( "old-index", iOldIndex );
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				if ( response.dataVoid() != true ) // Data is not void
				{
					if ( bUpdateScreen == null || bUpdateScreen == true )
					{
						//--- Populate the screen fields with XML data recieved
						_oScreen.XmlToScreen(response);
					}
				}
			}
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListClear = function ( sListName, sItemName, iOldIndex ) 
	{
		_trace("Action : ListClear");
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_clear") ;		

			request.setParameter( "list", sListName );
			//request.setParameter( "item", sItemName );
			//request.setParameter( "new-index", iIndex );
			request.setParameter( "old-index", iOldIndex );
			
			//request.setData( _oScreen.ScreenToXml(sItemName) ); 
			
			//--- Send the request 
			response = _sendRequest(request);
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	this.doListSort = function ( sListName, sItemName, iOldIndex, sFieldName, sOrder ) 
	{
		_trace("Action : ListSort");
		if ( arguments.length < 4 )
		{
			_error ( "doListSort() : 4 or 5 arguments expected ( " + arguments.length + " arguments )" ) ;
			return ;
		}
		var response = null ;
		try {	
			//--- Build the request 
			var request = this.getRequest("list_sort") ;		

			request.setParameter( "list",  sListName );
			request.setParameter( "item", sItemName );
			request.setParameter( "old-index", iOldIndex );
			//--- Sort parameters 
			request.setParameter( "field", sFieldName );
			if ( sOrder != null ) // order parameter is optional
			{
				request.setParameter( "order", sOrder );
			}		
			
			//request.setData( _oScreen.ScreenToXml() );
			request.setData( _oScreen.ScreenToXml(sItemName) ); // NEW
			
			//--- Send the request 
			response = _sendRequest(request);
			if ( response.isOK() )
			{
				//--- Populate the screen fields with list view recieved
				_oScreen.XmlToScreen(response);
			}			
		}
		catch (ex) {
			_error ( ex.message );
		}		
		return response ;
	}
	//------------------------------------------------------------------------------
	
	//------------------------------------------------------------------------------
	this.doExec = function (sProcName, htParameters) 
	{
		_trace("Action : Exec");
		var response = null ;
		if ( sProcName == null )
		{
			_error ( "doExec() : No procedure name (argument is null) " );
		}
		try {	
			//--- Build the request 
			//var request = _getRequest("exec") ;
			var request = this.getRequest("exec") ;
			//--- Set the procedure name 
			request.setProcedure(sProcName);
			//--- Set the parameters
			if ( htParameters != null )
			{
				for ( name in htParameters )
				{
					_trace ( "request.setParameter(" + name + ", " + htParameters[name] + ")" ) ;
					request.setParameter(name, htParameters[name]);
				}
			}
			
			//--- Call external method to set the request parameters
			/*** OLD
			this.beforeExec(request);
			***/

			//--- Send the request ( retrieve the response )
			response = _sendRequest(request);

			/*** OLD
			if ( response.isOK() )
			{
				//--- Call external method ????
				this.afterExec(request,response);
			}
			***/
		}
		catch (ex) {
			_error ( ex.message );
		}	
		return response ;
	}
		
	//------------------------------------------------------------------------------
	// Default external methods
	//------------------------------------------------------------------------------
	/*** OLD
	this.prepareToLoad = function ( request ) 
	{
		throw new Error( "ScreenActions : Specific method 'prepareToLoad' not implemented ! ");
	}
	***/
	//------------------------------------------------------------------------------
	/*** OLD
	this.prepareToDelete = function ( request ) 
	{
		throw new Error( "ScreenActions : Specific method 'prepareToDelete' not implemented ! ");
	}
	***/
	//------------------------------------------------------------------------------
	/*** OLD
	this.doGet_DataVoid = function ( response ) 
	{
		throw new Error( "ScreenActions : Specific method 'doGet_DataVoid' not implemented ! ");
	}
	//------------------------------------------------------------------------------
	this.doLoad_DataNotFound = function ( response ) 
	{
		throw new Error( "ScreenActions : Specific method 'doLoad_DataNotFound' not implemented ! ");
	}
	***/
	//------------------------------------------------------------------------------
	/***
	this.beforeExec = function ( request ) 
	{
		throw new Error( "ScreenActions : Specific method 'beforeExec' not implemented ! ");
	}
	//------------------------------------------------------------------------------
	this.afterExec = function ( request, response ) 
	{
		throw new Error( "ScreenActions : Specific method 'afterExec' not implemented ! ");
	}
	***/
	//------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------
//                                  F  I  N
//------------------------------------------------------------------------------
