//------------------------------------------------------------------------------
// fwkSimpleDate.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 12/12/2005 )
// Author  : Laurent Gurin 
// Changes : none (creation)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// CLASS DEFINITION
//------------------------------------------------------------------------------
function SimpleDate( sDate, sFormat )
{
	//==============================================================================
	// PRIVATE ATTRIBUTES
	//==============================================================================
	var _bTrace  = false ;
	
	//--- Date Value  ( 0 => Date Value is NULL )
	var _iDay     = 0 ; 
	var _iMonth   = 0 ; 
	var _iYear    = 0 ; 
	
	//--- Date Format
	var _sFormat      = null ;
	var _parsedFormat = null ;
	
	try {
		if ( sFormat == null )
		{
			sFormat = "YYYY-MM-DD" ; // ISO format
		}
		//--- Build the Date Value
		_buildDate( sDate, sFormat ) ;
	} 
	catch (ex) {
		alert ( "SimpleDate(" + sDate + "," + sFormat + ") : ERROR : " + ex.message );
	}
	
	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================
	function _trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	function _setNull() 
	{
		_iDay     = 0 ; 
		_iMonth   = 0 ; 
		_iYear    = 0 ; 
	}
	//------------------------------------------------------------------------------
	function _parseFormat( sFormat ) // throws Error
	{
		//--- Check format length 
		if ( sFormat.length != 6 && sFormat.length != 8 && sFormat.length != 10 )
		{
			throw new Error ( "Invalid date format " + sFormat + " : length != 8 and != 10 " ); 
		}
		var sFormatUC = sFormat.toUpperCase();
		
		//--- Separator identification 
		var sSep = null ;
		var iNbSep = 0 ;
		for ( var i = 0 ; i < sFormatUC.length ; i++ ) 
		{
			c = sFormatUC.charAt(i) ;
			if ( c != 'D' && c != 'M' && c != 'Y'  )  // separator
			{
				//alert ( "Separator = " + c ) ;
				if ( sSep != null && sSep != c )
				{
					// 2 different separators => error 
					throw new Error ( "Invalid date format " + sFormat + " : different separators " ); 
				}
				sSep = c ;
				iNbSep++ ;
			}
		}

		//--- Split format parts ( "DD", "MM" and "YY" or "YYYY" ) 
		var arrayFormat = null ;
		if ( iNbSep == 2 ) // 2 separators
		{
			//--- Split the format in 3 parts 
			arrayFormat = sFormatUC.split(sSep);
			if ( arrayFormat.length != 3 )
			{
				throw new Error ( "Invalid date format " + sFormat + " : " + arrayFormat.length + " parts !" ); 
			}
		}
		else if ( iNbSep == 0 ) // No separators
		{
			arrayFormat = new Array(3) ;
			if ( sFormatUC.length == 6 ) //  2c + 2c + 2c 
			{
				arrayFormat[0] = sFormatUC.substr(0,2);
				arrayFormat[1] = sFormatUC.substr(2,2);
				arrayFormat[2] = sFormatUC.substr(4,2);				
			}
			else if ( sFormatUC.length == 8 )
			{
				var iYYYY = sFormatUC.indexOf("YYYY") ;
				if ( iYYYY == 0 ) // Format starts with "YYYY" ( "YYYY...." )
				{
					arrayFormat[0] = sFormatUC.substr(0,4);
					arrayFormat[1] = sFormatUC.substr(4,2);
					arrayFormat[2] = sFormatUC.substr(6,2);									
				}
				else if ( iYYYY == 2 ) // "YYYY" in the middle ( "..YYYY.." )
				{
					arrayFormat[0] = sFormatUC.substr(0,2);
					arrayFormat[1] = sFormatUC.substr(2,4);
					arrayFormat[2] = sFormatUC.substr(6,2);									
				}
				else if ( iYYYY == 4 ) // "YYYY" at the end ( "....YYYY" )
				{
					arrayFormat[0] = sFormatUC.substr(0,2);
					arrayFormat[1] = sFormatUC.substr(2,2);
					arrayFormat[2] = sFormatUC.substr(4,4);									
				}
				else
				{
					throw new Error ( "Invalid date format " + sFormat + " : length = 8 and YYYY not found " ); 
				}
			}
			else
			{
				throw new Error ( "Invalid date format " + sFormat + " : length != 6 and != 8 " ); 
			}
		}
		else
		{
			throw new Error ( "Invalid date format " + sFormat + " : number of separators != 2 " ); 
		}
		
		//--- Check format parts ( "DD", "MM" and "YY" or "YYYY" ) 
		for ( var x=0 ; x < 3 ; x++ )
		{
			var sPart = arrayFormat[x];
			if ( sPart != "DD" && sPart != "MM" && sPart != "YY" && sPart != "YYYY" )
			{
				throw new Error ( "Invalid date format " + sFormat + " : invalid part " + sPart ); 
			}
		}
		
		var parsedFormat = arrayFormat ;
		parsedFormat[3] = sSep ;
		return parsedFormat ;
	}
	//------------------------------------------------------------------------------
	function _parseDate( sDate, parsedFormat ) 
	{
		var sSep = parsedFormat[3] ;
		var parsedDate = null ;
		if ( sSep != null )
		{
			//--- Split the date with the format separator
			parsedDate = sDate.split(sSep);
		}
		else
		{
			//--- Split the date with the format parts 
			parsedDate = new Array(3) ;
			var iStart = 0 ;
			for ( var i=0 ; i < 3 ; i++ )
			{
				var iSize = parsedFormat[i].length ;
				parsedDate[i] = sDate.substr(iStart, iSize ) ;
				iStart = iStart + iSize ;
			}
		}
		return parsedDate ;
	}
	//------------------------------------------------------------------------------
	function _buildDate( sDate, sFormat ) 
	{
		//alert("_buildDate(" + sDate + "," + sFormat + ")" );
		
		var parsedFormat = _parseFormat(sFormat);  // throws Error	

		//--- Keep the format in the instance (original and parsed)
		_sFormat      = sFormat ;
		_parsedFormat = parsedFormat ;

		//--- Build the Date Value
		if ( sDate == null )
		{
			//--- No date value
			_setNull(); 
		}
		else
		{
			_buildDateWithParsedFormat( sDate, parsedFormat );
		}

	}
	//------------------------------------------------------------------------------	
	function _buildDateWithParsedFormat( sDate, parsedFormat )
	{
		var parsedDate   = _parseDate  (sDate, parsedFormat);
		//alert("_buildDateWithParsedFormat : " + sDate + "\n parsedDate = " + parsedDate + "\n parsedFormat = " + parsedFormat );
		
		//--- Get Integer Values for date parts ( Day, Month, Year )
		var iDay   = 0 ;
		var iMonth = 0 ;
		var iYear  = 0 ;
		for ( var i=0 ; i < 3 ; i++ )
		{
			     if ( parsedFormat[i] == "DD" ) iDay   = parseInt( parsedDate[i],10) ;
			else if ( parsedFormat[i] == "MM" ) iMonth = parseInt( parsedDate[i],10) ;
			else if ( parsedFormat[i] == "YY" || parsedFormat[i] == "YYYY" ) 
			{
				iYear  = parseInt(parsedDate[i],10) ;
				if ( iYear < 100 ) iYear = iYear + 2000 ;
			}
		}
		
		//--- If the date parts are valid 
		if ( _isValid( iYear, iMonth, iDay ) )
		{
			//--- Set instance attributes : Day, Month, Year
			_iYear  = iYear ;
			_iMonth = iMonth ;
			_iDay   = iDay ;
		}
		else
		{
			throw new Error ( "Invalid date : " + sDate );
		}
	}
	//------------------------------------------------------------------------------
	function _isValid( iYear, iMonth, iDay ) // boolean
	{
		var aDaysByMonth = new Array(0,31,28,31,30,31,30,31,31,30,31,30,31);
		//--- If lap year, february is one day longer
		if ( ( iYear % 4 == 0 && iYear % 100 != 0 ) || iYear % 400 == 0 )  
		{
			aDaysByMonth[2] = 29  ; // 29 days for February
		}
		if ( ( iYear  < 0 ) || ( iYear  > 9999 ) )  return false ;
		if ( ( iMonth < 1 ) || ( iMonth > 12 ) )  return false ;
		if ( ( iDay   < 1 ) || ( iDay   > aDaysByMonth[iMonth] ) )  return false ;
		//--- OK   
		return true ;		
	}
	//------------------------------------------------------------------------------
	function _format( parsedFormat )  // throws Error
	{
		//alert( "_format( " + parsedFormat + ")" );
		var sFormatedDate = "" ;
		var sSeparator = parsedFormat[3] ;
		if ( sSeparator == null ) sSeparator = "" ;
		//--- For each part of the format ( Day, Month, Year ) ... 
		var sDatePart   = "" ;
		var sFormatPart = null ;
		for ( var i=0 ; i < 3 ; i++ )
		{
			//alert( "_format() i = " + i );
			if ( i > 0 ) 
			{
				sFormatedDate += sSeparator ;
			}
			sFormatPart = parsedFormat[i] ;
			     if ( sFormatPart.charAt(0) == 'Y' ) sDatePart = ""+_iYear ;
			else if ( sFormatPart.charAt(0) == 'M' ) sDatePart = ""+_iMonth ;
			else if ( sFormatPart.charAt(0) == 'D' ) sDatePart = ""+_iDay ; 
			else throw new Error ( "Invalid format : part = " + sFormatPart );
			if ( sDatePart.length < sFormatPart.length )
			{
				var iDelta = sFormatPart.length - sDatePart.length ;
				     if ( iDelta == 1 ) sDatePart = "0"+sDatePart ;
				else if ( iDelta == 2 ) sDatePart = "00"+sDatePart ;
				else if ( iDelta == 3 ) sDatePart = "000"+sDatePart ;
				else throw new Error ( "Invalid format : part = " + sFormatPart + " invalid length "); 
			}
			sFormatedDate += sDatePart ;
		}
		return sFormatedDate ;
	}
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================
	this.toString = function () 
	{
		return "day=" + _iDay + " month=" + _iMonth + " year=" + _iYear + " (format=" + _sFormat + ")" ;
	}
	//------------------------------------------------------------------------------
	this.show = function () 
	{
		alert("Date Value : day = " + _iDay + " month = " + _iMonth + " year = " + _iYear
			+ ( this.isNull() ? " ( NULL ) " : "" )
		    + "\nFormat : " + _sFormat 
		    + "\nParsed Format : " + _parsedFormat 
		    ) ;
	}
	//------------------------------------------------------------------------------
	// sFormat can be null (use the initial format )
	this.format = function ( sFormat ) // throws Error
	{
		//alert( "format() : Date = " + this.toString() );
		if ( this.isNull() )
		{
			return null ;
		}
		else
		{
			var sFormatedDate = null ;
			//try {
				var parsedFormat = _parsedFormat ;
				if ( sFormat != null )
				{
					parsedFormat = _parseFormat( sFormat ) ;
				}
				sFormatedDate = _format ( parsedFormat );
			/**
			}
			catch ( ex ) {
				alert ( "ERROR : \nSimpleDate - format() : \n" + ex.message );
			}
			**/
			return sFormatedDate ;
		}
	}
	//------------------------------------------------------------------------------
	this.isNull = function () 
	{
		return ( _iDay == 0 ? true : false ) ;
	}
	//------------------------------------------------------------------------------
	this.getDay = function () 
	{
		return _iDay ;
	}
	//------------------------------------------------------------------------------
	this.getMonth = function () 
	{
		return _iMonth ;
	}
	//------------------------------------------------------------------------------
	this.getYear = function () 
	{
		return _iYear ;
	}
	//------------------------------------------------------------------------------
	this.getFormat = function () 
	{
		return _sFormat ;
	}
	//------------------------------------------------------------------------------
	this.setFormat = function ( sFormat ) // throws Error
	{
		// try {
			_parsedFormat = _parseFormat( sFormat ); // throws Error
			_sFormat      = sFormat ;
		/**
		}
		catch (ex) {
			alert ( "ERROR : \nSimpleDate - setFormat() : \n" + ex.message );
		}
		**/
	}
	//------------------------------------------------------------------------------
	this.setDate = function ( sDate ) // throws Error
	{
		// try {
			if ( sDate == null )
			{	
				_setNull();
			}
			else
			{
				var sSeparator  = null ;
				var sYearFormat = null ;
				if ( sDate.length == 6 ) // Only digits "DD", "MM", "YY"
				{
					sSeparator  = null ; // no separator
					sYearFormat = "YY" ; // Year on 2 digits
				}
				else if ( sDate.length == 8 ) // 6 digits + 2 separators OR 8 digits 
				{
					if ( sDate.indexOf("/") >= 0 ) // At least a "/"
					{
						sSeparator  = "/" ;  // separator
						sYearFormat = "YY" ; // Year on 2 digits
					}
					else if ( sDate.indexOf("-") >= 0 ) // At least a "-"
					{
						sSeparator  = "-" ;  // separator
						sYearFormat = "YY" ; // Year on 2 digits
					}
					else
					{
						sSeparator  = null ;   // no separator 
						sYearFormat = "YYYY" ; // Year on 4 digits
					}
				}
				else if ( sDate.length == 10 ) // 8 digits + 2 separators 
				{
					     if ( sDate.indexOf("/") >= 0 ) sSeparator  = "/" ;
					else if ( sDate.indexOf("-") >= 0 ) sSeparator  = "-" ;
					else throw new Error("Invalid date : " + sDate );
					sYearFormat = "YYYY" ;
				}
				else
				{
					throw new Error( "Invalid date : " + sDate );
				}
				
				//--- Copy the current parsed format (other instance)
				var parsedFormat = _parsedFormat.concat() ; // no "copy" method => use "concat()"
				//--- Set the new "INPUT FORMAT" for date parsing : can change 'YEAR format' (but keep order)
				for ( var i=0 ; i < 3 ; i++ )
				{
					if ( parsedFormat[i].charAt(0) == 'Y' )
					{
						parsedFormat[i] = sYearFormat ;
					}
				}
				//--- Set the separator detected in the date
				parsedFormat[3] = sSeparator ;
	
				//--- Build the Date Value 	
				_buildDateWithParsedFormat( sDate, parsedFormat ) ;
				
			}
		/**
		}
		catch (ex) {
			alert ( "ERROR : \nSimpleDate - setDate() : \n" + ex.message );
		}
		**/
	}
	//------------------------------------------------------------------------------
	//------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------
//                                  F  I  N
//------------------------------------------------------------------------------
