//------------------------------------------------------------------------------
// TelosysRequester.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 22/03/2005 )
// Author  : Laurent Gurin 
// Changes : . Creation
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// OBJECT 
//------------------------------------------------------------------------------
function TelosysRequester( sDefaultUrlParam )
{
	//alert ( "TelosysRequester constructor ..." );
	
	//------------------------------------------------------------------------------
	// PRIVATE ATTRIBUTES
	//------------------------------------------------------------------------------
	var _bTrace          = false ;
	var _sDefaultUrl     = null ;
	var _oXMLHTTP        = null ; // THE "XMLHttpRequest" object
	var _sRequestContent = null ; // The last request content
	var _oScreenResponse = null ; // The last response object
	var _fCallback       = null ; // The current callback function for Async calls
	
	if ( sDefaultUrlParam != null )
	{
		_sDefaultUrl = sDefaultUrlParam ;
	}
	
	//--- Instance init 	
    if (window.ActiveXObject) 
	{
    	//--- Internet Explorer
    	_oXMLHTTP = new ActiveXObject("Microsoft.XMLHTTP");
    }
    else if (window.XMLHttpRequest)
    {
    	//--- Mozilla & Safari
    	_oXMLHTTP = new XMLHttpRequest();
    }

	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================

	//------------------------------------------------------------------------------
	// 
	//------------------------------------------------------------------------------
	function trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE : \n\n" + sMsg ) ;
	}
	
	//------------------------------------------------------------------------------
	// Build the response object 
	//------------------------------------------------------------------------------
	function _buildScreenResponse () 
	{
		trace( "Method : _buildScreenResponse() : \n" + _oXMLHTTP.responseText ); // Show the response 
		
        var sTelosysRedirect = _oXMLHTTP.getResponseHeader("Telosys-Redirect");  // v 0.9.9
		if ( sTelosysRedirect != null )
		{
			//--- Redirection required (the content received is not an XML response)
			trace("_processAjaxResponse() : Telosys Redirection : '" + sTelosysRedirect + "' " );
			fwkLoadPage( sTelosysRedirect );
		}
		else
		{		
			_oScreenResponse = new ScreenResponse(_oXMLHTTP);
			return _oScreenResponse ;
		}
	}
	//------------------------------------------------------------------------------
	// "POST" SYNCHRONE
	//------------------------------------------------------------------------------
	function _asyncCallback() // Called for each state change (1,2,3 & 4)
	{
		trace ("Method : _asyncCallback() / readyState = " + _oXMLHTTP.readyState );
		//--- If new state is "COMPLETE" ...
		if ( _oXMLHTTP.readyState == 4 )
		{
			//--- Build the ScreenResponse object
			var response = _buildScreenResponse();
			//--- Run the callback if it exists
			if ( _fCallback != null )
			{
				trace ("Run callback function" );
				_fCallback(response) ;
			}
		}
	}
	//------------------------------------------------------------------------------
	// Send HttpRequest to the server 
	//------------------------------------------------------------------------------
	function _sendRequest ( sMethod, sURL, bAsync, sContent ) // RET : void 
	{
		if ( sMethod == null )
		{
			alert ( "Error : HTTP Method is null ! " ); 
			return ;
		}
		if ( sURL == null )
		{
			alert ( "Error : URL is null ! " ); 
			return ;
		}
		if ( _oXMLHTTP == null )
    	{
			alert ( "Error : XMLHttpRequest object is null !" );
			return ;
    	}

		// _sRequestContent = sContent ;
		if ( sContent != null )
		{
			//_sRequestContent = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\r\n" + sContent ;
			_sRequestContent = sContent ;
		}
		else
		{
			_sRequestContent = null ;
		}
		
	    trace( "_sendRequest() : \n" + _sRequestContent ); // Show the request 

		//--- (1) Setup the HTTP Request Method and URL 
	    _oXMLHTTP.open (sMethod, sURL, bAsync);  // false = SYNCHRONE  (true = ASYNCHRONE)	    
	    //_oXMLHTTP.setRequestHeader("Content-Type", "text/xml;charset=ISO-8859-1");
	    _oXMLHTTP.setRequestHeader("Content-Type", "text/xml");

		//--- (2) Configure the callback method (if ASYNCHRONE) 
	    if ( bAsync )
	    {
			_oXMLHTTP.onreadystatechange = _asyncCallback ;
		}

		//--- (3) Send the request to the server 
	    _oXMLHTTP.send (_sRequestContent); // Send the content (via HTTP)

	}
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================

	//------------------------------------------------------------------------------
	// Get the Xml String of the request
	//------------------------------------------------------------------------------
	this.getRequest = function() 
	{
		return _sRequestContent ;
	}

	//------------------------------------------------------------------------------
	// Get the Screen Response object
	//------------------------------------------------------------------------------
	this.getResponse = function() 
	{
		return _oScreenResponse ;
	}

	//------------------------------------------------------------------------------
	// "POST" SYNCHRONE
	//------------------------------------------------------------------------------
	this.post = function ( sContent, sUrlParam )
	{
		var sUrl = _sDefaultUrl ;
		if ( sUrlParam != null )
		{
			sUrl = sUrlParam ;
		}
		_sendRequest("POST", sUrl, false, sContent);
		//trace ("post - after sendRequest : readyState = " + _oXMLHTTP.readyState );		
		_buildScreenResponse();
	}
	//------------------------------------------------------------------------------
	// "POST" ASYNCHRONE ( with callback function )
	//------------------------------------------------------------------------------
	this.postAsync = function ( sContent, callbackFunction, sUrlParam )
	{
		var sUrl = _sDefaultUrl ;
		if ( sUrlParam != null )
		{
			sUrl = sUrlParam ;
		}
		if ( callbackFunction == null )
		{
			alert ("ERROR on postAsync() : callback function is NULL !");
		}
		_fCallback = callbackFunction ;
		_sendRequest("POST", sUrl, true, sContent);
	}

	//------------------------------------------------------------------------------
	//------------------------------------------------------------------------------
	//------------------------------------------------------------------------------
	//------------------------------------------------------------------------------

	//------------------------------------------------------------------------------
	// Fonction d'affichage de message de trace
	//------------------------------------------------------------------------------
	this.MsgTrace = function ( sMsg ) 
	{
		if ( this.bTrace ) alert ( "TRACE : \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	// Fonction d'affichage du contenu de l'arbre XML des donnes changes
	//------------------------------------------------------------------------------
	this.ShowXmlData = function () // RET : Void
	{
		alert ( "XmlData : \n" + this.oXMLData.xml );
	}
	
	//------------------------------------------------------------------------------
	// Envoi d'une requete au serveur 
	// Paramtres :
	//  . URL  solliciter ct serveur
	//  . flux XML  envoyer au serveur ( ou "null" si aucunes donnes  envoyer )
	//  . flag : true  = recharger les donnes XML de l'cran avec le XML reu du serveur
	//           false = les conserver en l'tat 
	//------------------------------------------------------------------------------
	this.GenericXmlRequest = function ( sURL, oXml, bReload ) // RET : boolean 
	{
	    //--- Envoi des donnes XML au serveur
	    var oXMLHTTP = null ;
	    if (window.ActiveXObject) 
		{
	    	//--- I.E.
	    	oXMLHTTP = new ActiveXObject("Microsoft.XMLHTTP");
	    }
	    else if (window.XMLHttpRequest)
	    {
	    	//--- Mozilla
	    	oXMLHTTP = new XMLHttpRequest();
	    }
	    
	    oXMLHTTP.open ("POST", sURL, false);  // false = SYNCHRONE  (true = ASYNCHRONE)
	    this.MsgTrace ( "Requete serveur ...\nURL="+sURL);
	    oXMLHTTP.send (oXml); // Send HTTP ( avec un flux XML ou NULL )

	    //--- Interpretation de la rponse du serveur : Cas d'erreur :
	    if ( oXMLHTTP.responseXML     == null ) return false ;
	    if ( oXMLHTTP.responseXML.xml == null ) return false ;
	    if ( oXMLHTTP.responseXML.xml == ""   ) return false ;
	    
		//--- Cas normal ( pas d'erreur ) ==> rcupration des codes retour et message retour
		this.GetResponseStatus( oXMLHTTP.responseXML );
		
    	//--- Doit-on recharger les donnes XML locales ?
    	if ( bReload == true )
    	{
		   	this.oXMLData.load( oXMLHTTP.responseXML );  // "Reload"
    	}
    	
    	return true ; // OK : fin normale de traitement 
	}
	//------------------------------------------------------------------------------
	// Envoi d'une requete au serveur de type "GET CURRENT VALUES" 
	//------------------------------------------------------------------------------
	this.RequestGetCurrentValues = function ( sUrlBase, iPopupLevel ) // RET : boolean
	{
		if ( arguments.length < 2 ) { alert("fonction RequestGetCurrentValues : Paramtre(s) incorrect(s)");return false;} 
		var sUrl = sUrlBase + "?action=GETCURVAL" + this.UrlPopupLevel(iPopupLevel) ;
		return  this.GenericXmlRequest ( sUrl, null, true ) ;
	}
	//------------------------------------------------------------------------------
	// Envoi d'une requete au serveur de type "SELECT" 
	//------------------------------------------------------------------------------
	this.RequestSelect = function ( sUrlBase, iPopupLevel ) // RET : boolean
	{
		if ( arguments.length < 2 ) { alert("fonction RequestSelect : Paramtre(s) incorrect(s)");return false;} 
		var sUrl = sUrlBase + "?action=SELECT" + this.UrlPopupLevel(iPopupLevel) + this.ComposeUrlKeyParams() ;
		return  this.GenericXmlRequest ( sUrl, null, true ) ;
	}
	//------------------------------------------------------------------------------
	// Envoi d'une requete au serveur de type "UPDATE" 
	//------------------------------------------------------------------------------
	this.RequestUpdate = function ( sUrlBase, iPopupLevel ) // RET : boolean
	{
		if ( arguments.length < 2 ) { alert("fonction RequestUpdate : Paramtre(s) incorrect(s)");return false;} 
		var sUrl = sUrlBase + "?action=UPDATE" + this.UrlPopupLevel(iPopupLevel) ;
		return  this.GenericXmlRequest ( sUrl, this.oXMLData, true ) ;
	}
	//------------------------------------------------------------------------------
	// Envoi d'une requete au serveur de type "INSERT" 
	//------------------------------------------------------------------------------
	this.RequestInsert = function ( sUrlBase, iPopupLevel ) // RET : boolean
	{
		if ( arguments.length < 2 ) { alert("fonction RequestInsert : Paramtre(s) incorrect(s)");return false;} 
		var sUrl = sUrlBase + "?action=INSERT" + this.UrlPopupLevel(iPopupLevel) ;
		return  this.GenericXmlRequest ( sUrl, this.oXMLData, true ) ;
	}
	//------------------------------------------------------------------------------
	// Envoi d'une requete au serveur de type "DELETE" 
	//------------------------------------------------------------------------------
	this.RequestDelete = function ( sUrlBase, iPopupLevel ) // RET : boolean
	{
		if ( arguments.length < 2 ) { alert("fonction RequestDelete : Paramtre(s) incorrect(s)");return false;} 
		var sUrl = sUrlBase + "?action=DELETE" + this.UrlPopupLevel(iPopupLevel) + this.ComposeUrlKeyParams() ;
		return  this.GenericXmlRequest ( sUrl, null, false ) ;
	}
	//------------------------------------------------------------------------------
	// Envoi d'une requete au serveur de type "PROCEDURE" avec envoi de donnes au serveur (ds flux XML)
	//------------------------------------------------------------------------------
	this.RequestProcedure = function ( sUrlBase, sProcName, iPopupLevel ) // RET : boolean
	{
		if ( arguments.length < 3 ) { alert("fonction RequestProcedure : Paramtre(s) incorrect(s)");return false;} 
		var sUrl = sUrlBase + "?action=PROC&procname=" + sProcName + this.UrlPopupLevel(iPopupLevel) ;
		return  this.GenericXmlRequest ( sUrl, this.oXMLData, true ) ;
	}
	//------------------------------------------------------------------------------
	// Envoi d'une requete au serveur de type "PROCEDURE" sans envoi de donnes au serveur (sans flux XML)
	//------------------------------------------------------------------------------
	this.RequestProcedureWithoutData = function ( sUrlBase, sProcName, iPopupLevel ) // RET : boolean
	{
		if ( arguments.length < 3 ) { alert("fonction RequestProcedureWithoutData : Paramtre(s) incorrect(s)");return false;} 
		var sUrl = sUrlBase + "?action=PROC&procname=" + sProcName + this.UrlPopupLevel(iPopupLevel) ;
		return  this.GenericXmlRequest ( sUrl, null, true ) ;
	}
	
}
//------------------------------------------------------------------------------
//                                  F  I  N
//------------------------------------------------------------------------------
