package org.objectweb.telosys.admin;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.net.URLClassLoader;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 
 */

public class ConsoleClassLoader extends Console
{
    //-----------------------------------------------------------------------------
    static protected void printServletClassLoader( HttpServletRequest request, HttpServletResponse response, ConsoleServlet servlet ) throws IOException
    {
        PrintWriter out = response.getWriter();
        response.setContentType(CONTENT_TYPE_TEXT_HTML);
        out.println("<b><u> Current servlet class Loader : </u></b> <br>");
        out.println("<br>");
        ClassLoader cl = servlet.getClass().getClassLoader() ;
        if ( cl != null )
        {
            printClassLoader(out, cl, 1 );
        }
        printBack(out, request);
    }
    //-----------------------------------------------------------------------------
    static protected void printCurrentThreadClassLoader( HttpServletRequest request, HttpServletResponse response) throws IOException
    {
        PrintWriter out = response.getWriter();
        response.setContentType(CONTENT_TYPE_TEXT_HTML);
        out.println("<b><u> Current thread class Loader : </u></b> <br>");
        out.println("<br>");
        ClassLoader cl = Thread.currentThread().getContextClassLoader() ;
        if ( cl != null )
        {
            printClassLoader(out, cl, 1 );
        }
        printBack(out, request);
    }
    
    //-----------------------------------------------------------------------------
    static private void printClassLoader(PrintWriter out, ClassLoader cl, int n )
    {
        if ( cl != null )
        {
            out.println("<b><u> Class Loader " + n + " : </u></b>");
            out.println("<ul>");
            //System.out.println(" * Class Loader " + cl.toString() ); // NB : very verbose !
            URL[] urls = ((URLClassLoader)cl).getURLs();
            for(int i=0; i< urls.length; i++)
            {
                out.println("<li> " + urls[i].getFile() + "</li>");
            }
            out.println("</ul> ");
            if ( cl.getParent() != null )
            {
                printClassLoader(out, cl.getParent(), n+1);
            }
        }
    }

    
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    protected static void printLoadResourceForm( PrintWriter out, HttpServletRequest request )
    {
        out.println("<b><u> Class loader test </u></b>");
        out.println("<hr>");

        out.println("<b> Try to load a RESOURCE : </b>");
        out.println("<br>");
        out.println("<form method='POST' action='" + href(request, "do=" + Console.LOAD_RESOURCE ) + "' target='result' >");
        
        printLoadEnd( out );
        
    }
    //-----------------------------------------------------------------------------
    protected static void printLoadJavaClassForm( PrintWriter out, HttpServletRequest request )
    {
        out.println("<b><u> Class loader test </u></b>");
        out.println("<hr>");

        out.println("<b> Try to load a JAVA CLASS : </b>");
        out.println("<br>");
        out.println("<form method='POST' action='" + href(request, "do=" + Console.LOAD_CLASS ) + "' target='result' >");
        
        printLoadEnd( out );
    }
    //-----------------------------------------------------------------------------
    private static void printLoadEnd( PrintWriter out )
    {
        out.println("<table>");
        out.println("<tr><td>Name : </td><td><input name='name' style='width:400px;' /> </td></tr>");
        out.println("<tr><td>Loader type : </td><td> ");
        out.println("<input type='radio' name='loader' value='standard' checked='checked'>Standard</input> <br>");
        out.println("<input type='radio' name='loader' value='thread'>Thread</input> ");         
        out.println("</td></tr>");
        out.println("</table>");
        out.println("<li> Standard loader search resources only in classpath folders</li>");
        out.println("<li> CurrentThread loader search resources in classpath folders AND in .jar files (ie telosys.jar )</li>");
        
        out.println("<input type='submit' value='try to load' /> <br>");
        out.println("<br><br>");
        out.println("</form>");
        out.println("<hr>");
        out.println("<iframe name='result' width='600' height='300'  frameborder='1' ></iframe>");

        
    }
    
    //-----------------------------------------------------------------------------
    private static ClassLoader getClassLoader( PrintWriter out, int iClassLoaderType, ConsoleServlet servlet  )
    {
        ClassLoader cl = null ;
        switch ( iClassLoaderType )
        {
        case STANDARD_CLASS_LOADER_TYPE :
            cl = servlet.getClass().getClassLoader() ;
            break ;
        case THREAD_CLASS_LOADER_TYPE :
            cl = Thread.currentThread().getContextClassLoader() ;
            break ;
        default:
	        out.println("<span style='color:red' > <b> Invalid class loader type : " + iClassLoaderType + " </b> </span> <br>");	        
        }
        return cl ;
    }

    //-----------------------------------------------------------------------------
    private static int getLoaderType(HttpServletRequest request) 
    {
        String sLoader = request.getParameter("loader") ;
        int iLoaderType = -1;
        if ( sLoader.equals("standard"))
        {
            iLoaderType = Console.STANDARD_CLASS_LOADER_TYPE ;
        }
        if ( sLoader.equals("thread"))
        {
            iLoaderType = Console.THREAD_CLASS_LOADER_TYPE ;
        }
        return iLoaderType ;
    }
    
    //-----------------------------------------------------------------------------
    private static String getClassLoaderType( int iClassLoaderType )
    {
        ClassLoader cl = null ;
        switch ( iClassLoaderType )
        {
        case STANDARD_CLASS_LOADER_TYPE :
            return "Standard Class Loader";
        case THREAD_CLASS_LOADER_TYPE :
            return "Current Thread Class Loader";
        }
        return "Unknown Class Loader type";
    }
    //-----------------------------------------------------------------------------
    protected static void loadResource( PrintWriter out, HttpServletRequest request, ConsoleServlet servlet )
    {
        String sName = request.getParameter("name") ;
        String sLoader = request.getParameter("loader") ;
        if ( sName != null && sLoader != null)
        {
            loadResource(out, sName, getLoaderType(request), servlet );
        }
        else
        {
            out.println("<b>invalid parameters : </b> ... <br>");
            out.println(". name = '" + sName + "'<br>");
            out.println(". loader = '" + sLoader + "'<br>");
        }
    }
    //-----------------------------------------------------------------------------
    protected static void loadResource( PrintWriter out, String sResource, int iClassLoaderType, ConsoleServlet servlet  )
    {
        out.println("<b>Try to load resource  <u> '" + sResource + "' : </u>  with " + getClassLoaderType( iClassLoaderType ) + " </b> ... <br>");
        // NB  
        // . The "standard class loader" search resources ONLY IN FOLDERS ( not in ".jar files" ) 
        // . The "currentThread class loader" search resources in folders AND IN  ".jar files" 
        ClassLoader cl = getClassLoader( out, iClassLoaderType, servlet );
        if ( cl != null )
        {
            URL url = cl.getResource(sResource);
            if ( url != null )
            {
    	        out.println("<span style='color:green' > <b> OK : Loaded. </b> </span> <br>");	        
    	        out.println("Resource URL path : '" + url.getPath() + "'");	        
            }
            else
            {
    	        out.println("<span style='color:red' > <b> Cannot load ! </b> </span> <br>");	        
            }
            
        }
        else
        {
	        out.println("ERROR : Cannot get class loader ! <br>");	        
        }
    }
    //-----------------------------------------------------------------------------
    protected static void loadClass( PrintWriter out, HttpServletRequest request, ConsoleServlet servlet )
    {
        String sName = request.getParameter("name") ;
        String sLoader = request.getParameter("loader") ;
        if ( sName != null && sLoader != null)
        {
            loadClass(out, sName, getLoaderType(request), servlet );
        }
        else
        {
            out.println("<b>invalid parameters : </b> ... <br>");
            out.println(". name = '" + sName + "'<br>");
            out.println(". loader = '" + sLoader + "'<br>");
        }
    }
    //-----------------------------------------------------------------------------
    protected static void loadClass( PrintWriter out, String sClass, int iClassLoaderType, ConsoleServlet servlet )
    {
        out.println("<b>Try to load JAVA CLASS  <u> '" + sClass + "' : </u>  with " + getClassLoaderType( iClassLoaderType ) + " </b> ... <br>");
        ClassLoader classLoader = getClassLoader( out, iClassLoaderType, servlet );
        if ( classLoader != null )
        {
            Class javaClass;
            try
            {
                javaClass = classLoader.loadClass(sClass);
    	        out.println("<span style='color:green' > <b> OK : Loaded. </b> </span><br>");	 
    	        out.println("Loaded class name : '" + javaClass.getName() + "'");	        
            } catch (ClassNotFoundException e)
            {
    	        out.println("<span style='color:red' ><b> Cannot load ! ( ClassNotFoundException ) </b> </span><br>");	        
            }
        }
        else
        {
	        out.println("ERROR : Cannot get class loader ! <br>");	        
        }
    }
    //-----------------------------------------------------------------------------

}