package org.objectweb.telosys.admin;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysServlet;

/**
 * 
 */

public class ConsoleServlet extends TelosysServlet
{
    private static final String CONTENT_TYPE_TEXT_HTML  = "text/html";
    private static final String CONTENT_TYPE_TEXT_PLAIN = "text/plain ";
    
    
        
    //-----------------------------------------------------------------------------
    private String getRealPath( String sResource )
    {
        ServletContext context = this.getServletContext();        
        if (context != null)
        {
            return context.getRealPath(sResource);
        }
        else
        {
            return "Cannot get ServletContext !" ;
        }
    }
    //-----------------------------------------------------------------------------
    private String getResourcePath( String sResource )
    {
        ServletContext context = this.getServletContext();        
        if (context != null)
        {
            // GET RESOURCE
            URL resource = null ;
            try
            {
                resource = context.getResource( sResource );
                if ( resource != null )
                {
                    return resource.getPath();
                }
                else
                {
                    return "Resource '" + sResource + "' not found !" ;
                }
            } catch (MalformedURLException e)
            {
                return "Cannot get Resource from ServletContext ( MalformedURLException )" ;
            }
        }
        else
        {
            return "Cannot get ServletContext !" ;
        }
    }
    //-----------------------------------------------------------------------------

    //-----------------------------------------------------------------------------
    //--- HTTP GET
    //-----------------------------------------------------------------------------
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        trace("InitServlet.doGet()...");
        process(request, response);
    }

    //-----------------------------------------------------------------------------
    //--- HTTP POST
    //-----------------------------------------------------------------------------
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        trace("InitServlet.doPost()...");
        process(request, response);
    }

    //-----------------------------------------------------------------------------
    private void printResourceContent( InputStream is, HttpServletResponse response ) throws IOException
    {
        if ( is != null )
        {
			try
            {
                response.setContentType(CONTENT_TYPE_TEXT_PLAIN);
			    ServletOutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int len = 0;
                while ((len = is.read(buffer)) > 0)
                {
                    os.write(buffer, 0, len);
                }
                os.close();
                is.close();
            } catch (IOException e)
            {
                System.out.println("IOException !");
                e.printStackTrace();
            }                
        }
        else
        {
            // File not found ! 
            response.setContentType(CONTENT_TYPE_TEXT_HTML);
            PrintWriter out = response.getWriter();
            out.println("ERROR : InputStream is NULL !");
        }
    }
    //-----------------------------------------------------------------------------
    private void showResourcePath( PrintWriter out, HttpServletRequest request )
    {
        String sResource = request.getParameter("resource");
        if ( sResource != null )
        {
	        out.println("<b>Resource : <u>  '" + sResource + "' : </u></b> <br>");
	        out.println("<br>");	        
	        out.println("Resource path : <b>" + getResourcePath( sResource ) + "</b> <br>");	        
	        out.println("<br>");	        
	        out.println("'RealPath' : <b>" + getRealPath( sResource ) + "</b> <br>");	        
        }
        else
        {            
	        out.println("<b><u> Try to get the PATH of a resource  : </u></b>");
	        out.println("<br><br>");
	        out.println("<form method='POST' action='" + href(request, "do=resource_path" ) + "' >");
	        out.println("Resource : <br>");
	        out.println("<input name='resource' style='width:400px;' /> <br>");
	        out.println("<input type='submit' value='get path' /> <br>");
	        out.println("<br><br>");
	        out.println("</form>");
        }
        printBack(out, request);
    }
    //-----------------------------------------------------------------------------
    private void showResource( HttpServletRequest request, HttpServletResponse response, String sPath ) throws IOException
    {
        ServletContext servletContext = this.getServletContext();        
        if (servletContext != null)
        {
            InputStream is = servletContext.getResourceAsStream(sPath);
            if ( is != null )
            {
                printResourceContent( is, response );
            }
            else
            {                
	            Set set = servletContext.getResourcePaths(sPath);
	            if ( set != null )
	            {
	                
	                LinkedList list = new LinkedList(set);
	                Collections.sort(list);

	                response.setContentType(CONTENT_TYPE_TEXT_HTML);
	                PrintWriter out = response.getWriter();
	                out.println("<b><u> '" + sPath + "' resources : </u></b>");
	                //Iterator it = set.iterator();
	                Iterator it = list.iterator();
	                if ( it != null )
	                {
	                    //openTable( out );
	                    while ( it.hasNext() )
	                    {
	                        Object o = it.next();
	                        //printTR(out, o.toString() );
	                        String sResource = o.toString() ;
	                        String sQuery = "do=" + Console.RESOURCES +  "&path=" + sResource ;
	                        out.println("<li> " + link(request, sResource, sQuery) + "</li>");
	                    }            
	                    //closeTable( out );
	                }
	                printBack(out, request);
	            }
                else
                {
                    response.setContentType(CONTENT_TYPE_TEXT_HTML);
                    PrintWriter out = response.getWriter();
                    out.println("<b><u> ERROR : Cannot get resource '" + sPath+ "' </u></b>");
                    printBack(out, request);
                }
            }
        }
        else
        {
            response.setContentType(CONTENT_TYPE_TEXT_HTML);
            PrintWriter out = response.getWriter();
            out.println("<b><u> ERROR : Cannot get Servlet Context ! </u></b>");
            printBack(out, request);
        }
    }

    //-----------------------------------------------------------------------------
    private String href(HttpServletRequest request, String sQuery) 
    {
        return request.getRequestURL() + "?" + sQuery ;
    }
    //-----------------------------------------------------------------------------
    private String link(HttpServletRequest request, String sTxt, String sQuery) 
    {
        String sLink = request.getRequestURL() + "?" + sQuery ;
        return "<a href=\"" + sLink + "\" >" + sTxt + "</a>" ;
    }
    //-----------------------------------------------------------------------------
    private void printLink(PrintWriter out, HttpServletRequest request, String sTxt, String sAction) //throws ServletException, IOException
    {
        String sLink = request.getRequestURL() + "?do=" + sAction ;
        out.print("<tr>");
        out.print("<td>" + " * " + "</td>");
        out.print("<td>");
        out.print("<a href=\"" + sLink + "\" >" + sTxt + "</a>" );
        out.print("</td>");
        out.println("</tr>");
    }
    //-----------------------------------------------------------------------------
    private void printMenu(PrintWriter out, HttpServletRequest request) //throws ServletException, IOException
    {
        out.println("<html>");
        out.println("<h1>Telosys console : </h1>");
        out.println("<h3>Show ...</h3>");
        out.println("<table>");
        out.println("<tbody>");
        
        printLink(out, request, "Java info", Console.JAVA_INFO);
        printLink(out, request, "Class Path ( servlet class loader )",   Console.SERVLET_CLASS_LOADER);
        printLink(out, request, "Class Path ( current thread class loader )", Console.THREAD_CLASS_LOADER);
        printLink(out, request, "Servlet Context", Console.SERVLET_CONTEXT);
        printLink(out, request, "Resources", Console.RESOURCES);
        printLink(out, request, "Resource path", Console.RESOURCE_PATH);
        printLink(out, request, "Class Loader test : java class", Console.LOAD_CLASS_FORM);
        printLink(out, request, "Class Loader test : resource", Console.LOAD_RESOURCE_FORM);
        printLink(out, request, "Telosys configuration", Console.TELOSYS_CONF);
        printLink(out, request, "Http Sessions", Console.SESSIONS);

        out.println("</tbody>");
        out.println("</table>");

        out.println("</html>");
    }
    //-----------------------------------------------------------------------------
    private void printBack(PrintWriter out, HttpServletRequest request) //throws ServletException, IOException
    {
        out.println("<hr>" );        
        out.println("<a href=\"" + request.getRequestURL() + "\" > &lt;&lt;&lt; Console menu </a>" );
        out.println("<hr>" );        
    }
    //-----------------------------------------------------------------------------
    private void process(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        //PrintWriter out = response.getWriter();

        String sParam = request.getParameter("do") ;
        if ( sParam != null )
        {
            if ( sParam.equals( Console.JAVA_INFO ) )
            {
                ConsoleJavaInfo.print(request, response); 
            }
            else if ( sParam.equals( Console.SERVLET_CLASS_LOADER) )
            {
                ConsoleClassLoader.printServletClassLoader(request, response, this);
            }
            else if ( sParam.equals( Console.THREAD_CLASS_LOADER) )
            {
                ConsoleClassLoader.printCurrentThreadClassLoader(request, response);
            }
            else if ( sParam.equals( Console.SERVLET_CONTEXT) )
            {
                ConsoleWebAppConf.print(request, response, this);
            }
            else if ( sParam.equals( Console.RESOURCES) )
            {
                String sPath = request.getParameter("path");
                if ( sPath == null )
                {
                    sPath = "/" ;
                }
                showResource(request, response, sPath);
            }
            else if ( sParam.equals( Console.RESOURCE_PATH) )
            {
                PrintWriter out = response.getWriter();
                response.setContentType(CONTENT_TYPE_TEXT_HTML);
                showResourcePath(out, request);
            }
            
            else if ( sParam.equals( Console.LOAD_RESOURCE_FORM ) )
            {
                PrintWriter out = response.getWriter();
                response.setContentType(CONTENT_TYPE_TEXT_HTML);
                ConsoleClassLoader.printLoadResourceForm(out, request);
            }
            else if ( sParam.equals( Console.LOAD_RESOURCE ) )
            {
                PrintWriter out = response.getWriter();
                response.setContentType(CONTENT_TYPE_TEXT_HTML);
                ConsoleClassLoader.loadResource(out, request, this );
            }
            else if ( sParam.equals( Console.LOAD_CLASS_FORM ) )
            {
                PrintWriter out = response.getWriter();
                response.setContentType(CONTENT_TYPE_TEXT_HTML);
                ConsoleClassLoader.printLoadJavaClassForm(out, request);
            }
            else if ( sParam.equals( Console.LOAD_CLASS ) )
            {
                PrintWriter out = response.getWriter();
                response.setContentType(CONTENT_TYPE_TEXT_HTML);
                ConsoleClassLoader.loadClass(out, request, this );
                
            }
            else if ( sParam.equals( Console.TELOSYS_CONF ) )
            {
                PrintWriter out = response.getWriter();
                response.setContentType(CONTENT_TYPE_TEXT_HTML);
                ConsoleTelosysConf.print(out, request);
            }
            else if ( sParam.equals( Console.SESSIONS ) )
            {
                PrintWriter out = response.getWriter();
                response.setContentType(CONTENT_TYPE_TEXT_HTML);
                ConsoleSessions.print(out, request);
            }
        }
        else
        {
            PrintWriter out = response.getWriter();
            response.setContentType(CONTENT_TYPE_TEXT_HTML);
            printMenu(out, request);
        }
    }

}