package org.objectweb.telosys.auth;

import java.io.Serializable;

/**
 * Technical user object containing login and password
 * 
 * @author Laurent GUERIN
 *  
 */

public class LoginUser implements Serializable
{
    private String _sLogin;

    private String _sPassword;

    private String _sIP ; // User IP Address or last proxy IP

    private String _sHost ; // User host name or last proxy name

    private int    _iRequestCount = 0 ;
    
    //-------------------------------------------------------------
    /**
     * Constructor
     * @param sLogin : User login 
     * @param sPassword : User password
     * @param sIP : User IP Address ( or last proxy IP )
     * @param sHost : User host name ( or last proxy name )
     */
    /* package */ LoginUser(String sLogin, String sPassword, String sIP, String sHost )
    {
        _sLogin = sLogin;
        _sPassword = sPassword;
        _sIP = sIP ;
        _sHost = sHost ;
        _iRequestCount = 0 ;
    }

    //-------------------------------------------------------------
    /**
     * @return the login
     */
    public String getLogin()
    {
        return _sLogin;
    }

    //-------------------------------------------------------------
    /**
     * @return the password
     */
    /* package */ String getPassword()
    {
        return _sPassword;
    }

    //-------------------------------------------------------------
    /**
     * @return the IP address
     */
    public String getIP()
    {
        return _sIP;
    }

    //-------------------------------------------------------------
    /**
     * @return the Host Name
     */
    public String getHost()
    {
        return _sHost;
    }

    //-------------------------------------------------------------
    /**
     * @return the number of request 
     */
    public int getRequestCount()
    {
        return _iRequestCount;
    }

    //-------------------------------------------------------------
    /**
     * Increment the number of request
     * @return the incremented number of request 
     */
    /* package */ int incrementRequestCount()
    {
        return _iRequestCount++;
    }

    //-------------------------------------------------------------
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return "LoginUser : login = '" + _sLogin + "' ( host : " + _sHost + " - IP : " + _sIP + " ) ";
    }
    //-------------------------------------------------------------
}