package org.objectweb.telosys.common;

/**
 * Console can be used to print trace, info, warning and error messages
 * 
 * @author Laurent GUERIN
 *  
 */

import java.io.PrintStream;

public final class Console
{
    private static int STACK_MAX_CALLERS = 10 ;
    
    public static final PrintStream out = System.out;

    //-------------------------------------------------------------------------------------
    /**
     * Print a trace message
     * @param sClassName
     * @param sMsg
     */
    public static void trace(String sClassName, String sMsg)
    {
        println("Trace : [" + sClassName + "] " + sMsg);
    }

    /**
     * Print a trace message
     * @param sMsg
     */
    public static void trace(String sMsg)
    {
        println("Trace : " + sMsg);
    }

    //-------------------------------------------------------------------------------------
    /**
     * Print an error message
     * @param sClassName
     * @param sMsg
     */
    public static void error(String sClassName, String sMsg)
    {
        println("ERROR : [" + sClassName + "] " + sMsg);
    }

    /**
     * Print an error message
     * @param sMsg
     */
    public static void error(String sMsg)
    {
        println("ERROR : " + sMsg);
    }

    //-------------------------------------------------------------------------------------
    /**
     * Print a warning message
     * @param sClassName
     * @param sMsg
     */
    public static void warn(String sClassName, String sMsg)
    {
        println("WARN  : [" + sClassName + "] " +  sMsg);
    }

    /**
     * Print a warning message
     * @param sMsg
     */
    public static void warn(String sMsg)
    {
        println("WARN  : " + sMsg);
    }

    //-------------------------------------------------------------------------------------
    /**
     * Print an info message
     * @param sClassName
     * @param sMsg
     */
    public static void info(String sClassName, String sMsg)
    {
        println("info  : [" + sClassName + "] " + sMsg);
    }

    /**
     * Print an info message
     * @param sMsg
     */
    public static void info(String sMsg)
    {
        println("info  : " + sMsg);
    }

    //-------------------------------------------------------------------------------------
    public static void exeptionReport( Throwable ex )
    {
        exeptionReport( ex, false );
    }
    //-------------------------------------------------------------------------------------
    public static void exeptionReport( Throwable ex, boolean bCause )
    {
        String sCause = bCause ? "CAUSE " : "" ;
        println("EXCEPTION " + sCause + ": [" + ex.getClass().getName() + "] : \"" + ex.getMessage() + "\"" );
        StackTraceElement st[] = ex.getStackTrace();
        StackTraceElement loc = null;
        if ( st.length > 0 )
        {
            //--- Exception origin ( 0 ) 
	        loc = st[0];
            //println(" in '" + loc.getFileName() + "', line " + loc.getLineNumber() + ", method '" + loc.getMethodName() + "'" );
            //println(" in '" + loc.getClassName() + "', line " + loc.getLineNumber() + ", method '" + loc.getMethodName() + "', callers :" );
            //--- Callers stack ( 1 to N )
	        for ( int i = 0 ; i < st.length && i < STACK_MAX_CALLERS+1 ; i++ )
	        {
	            loc = st[i];
	            println(" . " + loc.getClassName() + "." + loc.getMethodName() + " : " + loc.getLineNumber() );
	        }
        }
        if ( ex.getCause() != null )
        {
            exeptionReport( ex.getCause(), true );
        }
    }
    

    //-------------------------------------------------------------------------------------
    /**
     * Print a String
     * @param s
     */
    synchronized static private void println(String s)
    {
        System.out.println(s);
        System.out.flush();
    }
}