package org.objectweb.telosys.common;

/**
 * Interface for Application Logger<br>
 * This interface aims to provide a way for the Telosys framework to call a specific application logger<br>
 * The application logger can be implemented with any kind of logger ( Log4J, JSR47, Monolog, ... )
 *  
 * @author Laurent GUERIN
 *
 */
public interface IApplicationLogger
{
    /**
     * Trace log message ( the lowest level ) 
     * @param sClassName
     * @param sMessage
     */
    public void trace(String sClassName, String sMessage) ;
    
    /**
     * Debug log message ( at the moment, never used by Telosys, only for future use)
     * @param sClassName
     * @param sMessage
     */
    public void debug(String sClassName, String sMessage) ;

    /**
     * Info log message
     * @param sClassName
     * @param sMessage
     */
    public void info(String sClassName, String sMessage) ;
    
    /**
     * Warning log message
     * @param sClassName
     * @param sMessage
     */
    public void warn(String sClassName, String sMessage) ;
    
    /**
     * Error log message
     * @param sClassName
     * @param sMessage
     */
    public void error(String sClassName, String sMessage) ;
    
    /**
     * Error log with exception
     * @param sClassName
     * @param exception
     */
    public void error(String sClassName, Throwable exception) ;
    
    
}
