package org.objectweb.telosys.common;

import org.objectweb.telosys.auth.IUserValidator;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;
import org.objectweb.telosys.screen.core.IScreenDataProvider;
import org.objectweb.telosys.screen.core.IScreenProvider;
import org.objectweb.telosys.service.IServiceProvider;

/**
 * Centralize the getters to obtain all the Factory providers needed by the framework <br>
 * This interface must be implemented by only one class of the application <br>
 * 
 * @author Laurent GUERIN
 * 
 */
public interface IExternalMainFactory
{
    /**
     * Returns the user validator service provided by the application
     * @return
     * @since 0.9.6
     */
    public IUserValidator getUserValidator();

    /**
     * Returns the user validator service provided by the application
     * @return
     * @since 0.9.9
     */
    public IApplicationLogger getApplicationLogger();

    /**
     * Returns the application XML mappers provider ( usally the 'XML mappers registry' ) 
     * @return
     * @since 0.9.6
     */
    public IXmlWrapperProvider getXmlWrapperProvider();

    /**
     * Returns the application Screen provider ( usally the 'screens registry' )
     * @return
     * @since 0.9.6
     */
    public IScreenProvider getScreenProvider();

    /**
     * Returns the application ScreenData provider ( usally the 'screens registry' )
     * @return
     * @since 0.9.6
     */
    public IScreenDataProvider getScreenDataProvider();

    /**
     * Returns the application service provider ( usally the 'services registry' )
     * @return
     * @since 0.9.6
     */
    public IServiceProvider getServiceProvider();

}