package org.objectweb.telosys.common;

import java.io.InputStream;
import java.util.Properties;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilderFactory;

import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;

/**
 * General Telosys environment informations and utilities
 * 
 * @author Laurent GUERIN
 *  
 */
public final class Telosys
{
	//--- Conf files property name in "telosys.properties"
    
	private static final String   DB_CONF_FILE       = "DbConfFile";  // v 1.0.0
	
	private static final String   TRACE_CONF_FILE    = "TraceConfFile"; // v 1.0.0
	
	private static final String   TAGS_CONF_FILE     = "TagsConfFile"; // v 1.0.0
	
	private static final String   SCREENS_CONF_FILE  = "ScreensConfFile"; // v 1.0.0
	
	//--- XML parsers
	
	private static final String   SAX_XERCES  = "org.apache.xerces.parsers.SAXParser";
	
	private static final String   SAX_CRIMSON = "org.apache.crimson.parser.XMLReaderImpl";

    private static final TelosysClassLogger $log = new TelosysClassLogger(Telosys.class);

    //-----------------------------------------------------------------------------
    // PROPERTIES & CONFIGURATION
    //-----------------------------------------------------------------------------
    //--- Telosys config file ( absolute path )
    //private static String                   $TelosysPropFile   = null;
    
    private static TelosysProperties        $TelosysProperties = null;
    
    //--- The properties loaded from "telosys.properties"
    private static Properties               $properties = null;

    //--- The properties loaded from "telosys.properties"
    private static int                      $iTelosysPropertiesOrigin = TelosysProperties.UNKNOWN ;

    //--- Instance of the "Framework Internal Factory"
    private static TelosysFactory           $telosysFactory    = null;

    //--- Trace configuration file 
    //private static String                   $sTraceConfFile    = null;

    //--- Authentication required to create ScreenSession 
    private static boolean                  $bAuthenticationRequired = true ;

    //--- XML SAX Parser Class Name ( for SAX 2 XMLReaderFactory.createXMLReader() )
    private static String                   $sSAXParserClassName     = null;
    
    private static LogWriter                $TelosysLogger     = null;

    //--- External application logger
    private static ApplicationLoggerFacade  $ApplicationLogger = null; // v 0.9.9


    //-----------------------------------------------------------------------------
    // WEB APP 
    //-----------------------------------------------------------------------------
    //--- Telosys Web App root directory
    private static String                   $TelosysWebAppRootDir = null;

    //--- Telosys Web App Context Name ( from web.xml : tag "display-name" )
    private static String                   $TelosysWebAppName = null;
    
    //--- Current servlet context
    private static ServletContext           $ServletContext = null;
    //-----------------------------------------------------------------------------
    
    //-----------------------------------------------------------------------------
    private static void createLogWriter(Properties prop)
    {
        if (prop != null)
        {
            $log.info("Create LogWriter with properties. ");
            $TelosysLogger = new LogWriter(prop);
        }
        else
        {
            $log.error("initParameters : properties = null !");
            $log.info("Create LogWriter without properties.  ");
            $TelosysLogger = new LogWriter();
        }
    }

    //-----------------------------------------------------------------------------
    private static boolean checkNotInitialized()
    {
        if (isInitialized())
        {
            //--- Already initialized
            $log.info("INIT : Telosys.init(properties) : WARNING - Already initialized => No change !");
            return false; // Do not init again
        }
        else
        {
            return true; // OK to init
        }
    }

    //-----------------------------------------------------------------------------
    private static void initParamAuthentication(Properties prop)
    {
        //--- Authentication required ?
        $bAuthenticationRequired = true ;                    
        String sProperty = prop.getProperty("AuthenticationRequired");
        //log.info("INIT : Property Authentication Required = " + sAuthenticationRequired );
        if ( sProperty != null )
        {
            String sPropertyTrimLC = sProperty.trim().toLowerCase();
            //log.info("INIT : toLowerCase = '" + sPropTrimLC + "'");
            if ( sPropertyTrimLC.equals("false") )
            {
                //log.info("INIT : Flag is false" );
                $bAuthenticationRequired = false ; 
            }
            else
            {
                //log.info("INIT : Flag is NOT false" );                    
            }
        }        
    }
    //-----------------------------------------------------------------------------
    private static void initParamXmlSaxParser(Properties prop)
    {
        //--- Authentication required ?
        $sSAXParserClassName = null ;                    
        String sProperty = prop.getProperty("SAXParserClassName");
        if ( sProperty != null )
        {
            $sSAXParserClassName = sProperty.trim();
        }        
        else
        {
            $sSAXParserClassName = SAX_CRIMSON ;
        }
    }
    //-----------------------------------------------------------------------------
    /**
     * Generic central Telosys 'init' method called by all public 'init' methods
     * @param extFactory
     * @param tp
     * @param servletContext
     */
    //private static void initTelosys(IExternalMainFactory extFactory, TelosysProperties tp, ServletContext servletContext)
    private static void initTelosys(IExternalMainFactory extFactory, TelosysProperties tp )
    {
        if ( tp != null )
        {
            $TelosysProperties = tp ;
            $iTelosysPropertiesOrigin = tp.getOrigin() ;
            Properties prop = tp.getProperties();
            if ( prop != null )
            {
                //--- Global init 
                initGlobal(extFactory, prop);
                
                //--- Trace manager init
                initTraceConfig();
                
                //--- Create the log writer
                createLogWriter($properties);
                $log.info("INIT : Telosys log writer created. ");
            }
            else
            {
                $log.error("initTelosys() : cannot get Properties from TelosysProperties !");
            }
        }
        else
        {
            $log.error("initTelosys() : TelosysProperties == null !");
        }
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Telosys global initialization from properties
     * 
     * @param extFactory
     * @param prop
     * @return true if Ok / false if Error
     */
    private static boolean initGlobal(IExternalMainFactory extFactory, Properties prop)
    {
        if (checkNotInitialized())
        {
            //--- Check arguments
            if (extFactory == null)
            {
                $log.error("initTelosys() : No external factory ( extFactory == NULL ) !");
                return false;
            }

            if (prop == null)
            {
                $log.error("initTelosys() : properties = null !");
                return false;
            }

            // v 1.0.0 moved here
            //--- Store the properties ( must be set set before any other initialization )
            $properties = prop;
            $log.info("INIT : Telosys properties set. ");

            //--- Build the Telosys factory with the "external factory"
            $telosysFactory = new TelosysFactory(extFactory);
            $log.info("INIT : Telosys factory created. ");

// v 1.0.0
//            //--- Store the properties
//            $properties = prop;
//            $log.info("INIT : Telosys properties set. ");

            //--- Authentication required ?
            initParamAuthentication(prop);
            $log.info("INIT : Flag Authentication Required = " + $bAuthenticationRequired );

            //--- XML SAX parser
            initParamXmlSaxParser(prop);
            $log.info("INIT : XML SAX parser class = '" + $sSAXParserClassName + "'" );
            
            $log.info("INIT : Other config files :" );
            $log.info(" . Traces    : '" + getTraceConfigFileName() + "'" );
            $log.info(" . Databases : '" + getDbConfigFileName() + "'" );
            $log.info(" . Screens   : '" + getScreensConfigFileName() + "'" );
            $log.info(" . Tags      : '" + getTagsConfigFileName() + "'" );
        }
        return true;
    }

    //-----------------------------------------------------------------------------
    /**
     * Init the logger system
     * @return
     */
    private static void initTraceConfig()
    {
        //--- 1) Init the Telosys trace manager
        $log.info("INIT : Init trace properties ( file = '" + Telosys.getTraceConfigFileName() + "' ) ..." );
        InputStream is = Telosys.getTraceConfigInputStream();
        if ( is != null )
        {
            TraceConfig.init(is); 
            Tool.closeInputStream(is);
            $log.info("INIT : Trace manager initialized.");                                
        }
        else
        {
            $log.error("INIT : Telosys.initTraceConfig() : Cannot get TRACE CONFIG InputStream ! ");
        }
        
        //--- 2) Set up the application logger if any  ( v 0.9.9 )
        $log.info("INIT : Init application logger ..." );
        if ( $telosysFactory != null )
        {
            IApplicationLogger appLogger = $telosysFactory.getApplicationLogger();
            if ( appLogger != null )
            {
                try
                {
                    $ApplicationLogger = new ApplicationLoggerFacade(appLogger);
                    $log.info("INIT : Application logger ready.");                                
                } catch (TelosysException e)
                {
                    $log.error("INIT : Telosys.initTraceConfig() : Cannot create ApplicationLoggerFacade ! ");
                }
            }
            else
            {
                $log.info("INIT : No application logger");                                
            }
        }
        else
        {
            $log.error("INIT : Telosys factory is null !");                                
        }
    }
    
    //-----------------------------------------------------------------------------
//    /**
//     * Telosys initialization from a properties file
//     * 
//     * @param extFactory
//     * @param sPropFile
//     * @return true if Ok / false if Error
//     */
//    private static boolean initTelosys(IExternalMainFactory extFactory, String sPropFile)
//    {
//        $log.info("INIT : Telosys.init(" + sPropFile + ") ... ");
//        Properties propLoaded = Tool.loadProperties(sPropFile);
//        if (propLoaded != null)
//        {
//            //--- Store the file name
//            $TelosysPropFile = sPropFile ;
//            if ( $TelosysPropFile != null )
//            {
//                //--- Store the directory
//                File file = new File($TelosysPropFile); 
//                $TelosysConfDir = file.getParent();
//            }
//            //--- Initialize Telosys with properties loaded
//            return initTelosys(extFactory, propLoaded);
//        }
//        else
//        {
//            $log.error("initTelosys() : cannot load properties from '" + sPropFile + "' ");
//            return false;
//        }
//    }

    //-----------------------------------------------------------------------------
    // PUBLIC METHODS
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    /**
     * Returns the Telosys framework version ( e.g. "1.2.3" )
     * @return
     */
    public static String getVersion()
    {
        return TelosysVersion.VERSION ;
    }
    //-----------------------------------------------------------------------------
    /* package */ static void startup(ServletContext servletContext)
    {
        $log.info("INIT : Telosys startup ... ");
        $ServletContext =  servletContext ;
        if ( $ServletContext != null )
        {
            $log.info("Telosys startup : ServletContext ready.");
        }
        else
        {
            $log.error("Telosys startup : ServletContext is NULL !");
        }
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns true if Telosys is initialized <br>
     * ( true if the properties are loaded and the factory exists )
     * @return
     */
    public static boolean isInitialized()
    {
        if ($properties != null && $telosysFactory != null)
        {
            return true;
        }
        return false;
    }

    //-----------------------------------------------------------------------------
    /**
     * Try to initialize the Telosys properties : <br>
     * try to find 'telosys.properties' using the classpath if the file is find <br>
     * the properties are loaded and Telosys is initialized.
     * 
     * @return the properties
     */
    public static Properties init(IExternalMainFactory extFactory)
    {
        if (checkNotInitialized())
        {
            $log.info("INIT : Telosys.init(extFactory) ... ");
            TelosysProperties tp = TelosysPropertiesLoader.load();
            //initTelosys(extFactory, tp, null );
            initTelosys(extFactory, tp); // v 1.0.0
        }
        return $properties;
    }

    //-----------------------------------------------------------------------------
    /**
     * Initialize the Telosys properties. <br>
     * Try to find a properties file :<br>
     * 1/ specific file name in 'web.xml' ( servlet <init-param>TelosysProperties <init-param>)<br>
     * 2/ 'telosys.properties' located in 'WEB-INF/conf' <br>
     * 3/ 'telosys.properties' located via CLASSPATH <br>
     * 
     * @param extFactory
     * @param servletContext : the ServletContext to use
     * @return
     */
    public static Properties init(IExternalMainFactory extFactory, ServletContext servletContext)
    {
        if (checkNotInitialized())
        {
            $log.info("INIT : Telosys.init(IExternalMainFactory, ServletContext) ... ");
            // $ServletContext =  servletContext ; // done in startup()
                
            TelosysProperties tp = TelosysPropertiesLoader.load($ServletContext);
            //initTelosys(extFactory, tp, $ServletContext );
            initTelosys(extFactory, tp ); // v 1.0.0

            //--- Set the default Telosys application root directory from the ServletContext
            if ($ServletContext != null)
            {
                $TelosysWebAppRootDir = $ServletContext.getRealPath("/"); // NB : Can be null
                $TelosysWebAppName = $ServletContext.getServletContextName();
            }    
        }
        return $properties;
    }
    //-----------------------------------------------------------------------------
    /**
     * Initialize the Telosys properties. <br>
     * Try to find a properties file :<br>
     * 1/ specific file name in 'web.xml' ( servlet <init-param>TelosysProperties <init-param>)<br>
     * 2/ 'telosys.properties' located in 'WEB-INF/conf' <br>
     * 3/ 'telosys.properties' located via CLASSPATH <br>
     * 
     * @param extFactory
     * @param servletConfig : the ServletConfig to use ( can be null )
     * @return
     */
    public static Properties init(IExternalMainFactory extFactory, ServletConfig servletConfig)
    {
        if (checkNotInitialized())
        {
            $log.info("INIT : Telosys.init(IExternalMainFactory, ServletConfig) ... ");
            ServletContext servletContext = null ;
            if ( servletConfig != null )
            {
                servletContext = servletConfig.getServletContext() ;
            }
            init(extFactory, servletContext);
        }
        return $properties;
    }

    //-----------------------------------------------------------------------------
    /**
     * Initialize the Telosys properties using a specific properties file
     * 
     * @param extFactory
     * @param sPropFile
     * @return
     */
    public static Properties init(IExternalMainFactory extFactory, String sPropFile)
    {
        if (checkNotInitialized())
        {
            $log.info("INIT : Telosys.init(extFactory, file) ... ");
            TelosysProperties tp = TelosysPropertiesLoader.load(sPropFile);
            //initTelosys(extFactory, tp, null );
            initTelosys(extFactory, tp ); // v 1.0.0
        }
        return $properties;
    }

    //-----------------------------------------------------------------------------

    //-----------------------------------------------------------------------------
    // Rcupration des "properties" gnrales de TELOSYS
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    //--- PUBLIC GETTERS/SETTERS
    //-----------------------------------------------------------------------------
//    /**
//     * Returns the full path of the properties file<br>
//     * e.g. : "C:\dir1\dir2\mywebapp\WEB-INF\conf\telosys.properties"
//     * @return
//     */
//    public static String getPropertiesFile()
//    {
//        return $TelosysPropFile ;
//    }
    //-----------------------------------------------------------------------------
//    /**
//     * Returns the Telosys configuration directory ( absolute filesystem directory )<br>
//     * e.g. "C:\dir1\dir2\mywebapp\WEB-INF\conf" <br>
//     * @return
//     */
//    public static String getConfigDirectory()
//    {
//        return $TelosysConfDir ;
//    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the absolute filesystem root directory of the current Web App <br>
     * e.g. "C:\dir1\dir2\mywebapp\" <br> 
     * Set from ServletContext.getRealPath("/"), so it can be null
     * @return the full WebApp path or null if the WAR file is not exploded in the server filesystem
     */
    public static String getWebAppRootDir()
    {
        return $TelosysWebAppRootDir ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the name of the current Web App  ( from tag 'display-name' in web.xml ) <br>
     * Set from ServletContext.getServletContextName()
     * @return the WebApp display name or null if not defined
     */
    public static String getWebAppName()
    {
        return $TelosysWebAppName ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the SAX parser class name <br>
     * @return
     */
    public static String getSAXParserClassName()
    {
        return $sSAXParserClassName ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Return the full path for the given relative file<br> 
     * ( ie : "myfile.txt" -> "C:/dir1/dir2/mywebapp/myfile.txt" ) <br>
     * This method concats the "Application Root Dir" with the file name<br>
     * if the "Application Root Dir" is not set, it returns the file as is.<br>
     * @param sFile
     * @return 
     */
    public static String getFullPath (String sFile )
    {
        if ( $TelosysWebAppRootDir != null )
        {
            String sRoot2 = $TelosysWebAppRootDir ;
            if ( $TelosysWebAppRootDir.endsWith("/") || $TelosysWebAppRootDir.endsWith("\\") )
            {
                //--- Cut the last char
                sRoot2 = $TelosysWebAppRootDir.substring(0, $TelosysWebAppRootDir.length()-1 ) ;
            }

            if ( sFile != null )
            {
                String sFile2 = sFile ;
                if ( sFile.startsWith("/") )
                {
                    //--- Cut the first char
                    sFile2 = sFile.substring(1); 
                }
                return sRoot2 + "/" + sFile2 ;                                
            }
            else
            {
                return sRoot2 + "/" ;
            }        
            
        }
        else
        {
            $log.error("Telosys.getFullPath(" + sFile + ") : WebApp RootDir is null !");
            return sFile ; // No change 
        }
    }
    //-----------------------------------------------------------------------------
//    /**
//     * Transforms the given file name : <br>
//     * replace '.' by the 'config directory' <br>
//     *   ( e.g. './myfile' -> 'C:/dir1/dir2/mywebapp/WEB-INF/conf/myfile' ) <br>
//     * replace '$ROOT/' by the 'web app root directory' 
//     *   ( e.g. '$ROOT/myfile' -> 'C:/dir1/dir2/mywebapp/myfile' ) <br> 
//     * @param sFile
//     * @return
//     */
//    public static String transformFileName(String sFile)
//    {
//        if ( sFile == null )
//        {
//            return null ;
//        }
//        
//        if ( sFile.length() == 0 )
//        {
//            return sFile ;
//        }
//        
//        //--- If the file name starts with "." => it's a relative path 
//        if ( sFile.charAt(0) == '.' )
//        {
//            // ie : "./foo" => "Dir/foo"
//            return $TelosysConfDir + sFile.substring(1) ;
//        }
//        
//        //--- If the file name starts with "$ROOT" => it's a "WEB APP ROOT" path 
//        if ( sFile.startsWith("$ROOT/") )
//        {
//            if ( $TelosysWebAppRootDir != null )
//            {
//                // ie : "$ROOT/foo" => "WebAppRootDir/foo"
//                return $TelosysWebAppRootDir + sFile.substring(6) ;            
//            }
//            else
//            {
//                Telosys.error("Telosys.transformFileName(" + sFile + ") : WebApp RootDir is null !");
//                return sFile.substring(6) ;            
//            }
//        }
//
//        //--- Not special path => return "as is"
//        return sFile ;
//    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the Telosys properties ( loaded from the 'telosys.properties' file )
     * @return
     */
    public static Properties getProperties()
    {
        return $properties;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the Telosys factory
     * @return
     */
    public static TelosysFactory getFactory()
    {
        return $telosysFactory;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the web app ServletContext object
     * @return
     * @since  1.0.0
     */
    public static ServletContext getServletContext()
    {
        return $ServletContext;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the XmlWrapperProvider provided by the Telosys factory
     * @return
     */
    public static IXmlWrapperProvider getXmlWrapperProvider()
    {
        if ( $telosysFactory != null )
        {
            return $telosysFactory.getXmlWrapperProvider();
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Returns true if user authentication is required
     * @return
     */
    public static boolean isAuthenticationRequired()
    {
        return $bAuthenticationRequired ;
    }
    //-----------------------------------------------------------------------------
    public static DocumentBuilderFactory getXmlDomParserFactory()
    {
//        // NB: An implementation of the DocumentBuilderFactory class is NOT guaranteed to be thread safe.
//        //     => a factory is created for each call 
//        String sParserClass = null ;
//        if ( $sXmlDomParser != null )
//        {
//            sParserClass = $sXmlDomParser ;
//        }
//        else
//        {
//            //--- Use CRIMSON by default
//            sParserClass = DOM_CRIMSON ;            
//        }
//        //--- Set the JAXP system propety 
//        System.setProperty( DOM_PARSER, sParserClass );
        //--- Create a new factory instance via JAXP
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        //--- Configuring the Factory ...
        //factory.setValidating(true);
        //factory.setNamespaceAware(true);        

        $log.info("Telosys.getXmlDomParserFactory() : JAXP factory class = " + factory.getClass().getName() );
        
        return factory ;
    }
    
    //-----------------------------------------------------------------------------
    //--- PUBLIC LOG METHODS
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    // ERROR
    //-----------------------------------------------------------------------------
    public static void error(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sClassName, sMsg);
        }
        else
        {
            Console.error(sClassName, sMsg);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.error( sClassName, sMsg);
        }
    }
    //-----------------------------------------------------------------------------
    public static void error(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sMsg);
        }
        else
        {
            Console.error(sMsg);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.error( "", sMsg);
        }
    }

    //-----------------------------------------------------------------------------
    // ERROR - Exception
    //-----------------------------------------------------------------------------
    private static String getMsg(Throwable ex)
    {
        StackTraceElement t[] = ex.getStackTrace();
        StackTraceElement loc = t[0];
        return "Exception " + ex.toString() + " in file '" + loc.getFileName() + "', method '" + loc.getMethodName() + "', line " + loc.getLineNumber();
    }
    //-----------------------------------------------------------------------------
    public static void error(String sClassName, Throwable ex)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sClassName, getMsg(ex));
            $TelosysLogger.exeptionReport(ex);
        }
        else
        {
            Console.error(sClassName, getMsg(ex));
            Console.exeptionReport(ex);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.error( sClassName, ex);
        }
    }
    //-----------------------------------------------------------------------------
    public static void error(Throwable ex)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(getMsg(ex));
            $TelosysLogger.exeptionReport(ex);
        }
        else
        {
            Console.error(getMsg(ex));
            Console.exeptionReport(ex);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.error( "", ex);
        }
    }

    //-----------------------------------------------------------------------------
    // WARN
    //-----------------------------------------------------------------------------
    public static void warn(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.warn(sClassName, sMsg);
        }
        else
        {
            Console.warn(sClassName, sMsg);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.warn( sClassName, sMsg);
        }
    }
    //-----------------------------------------------------------------------------
    public static void warn(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.warn(sMsg);
        }
        else
        {
            Console.warn(sMsg);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.warn( "", sMsg);
        }
    }

    //-----------------------------------------------------------------------------
    // INFO
    //-----------------------------------------------------------------------------
    public static void info(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.info(sClassName, sMsg);
        }
        else
        {
            Console.info(sClassName, sMsg);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.info( sClassName, sMsg);
        }
    }
    //-----------------------------------------------------------------------------
    public static void info(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.info(sMsg);
        }
        else
        {
            Console.info(sMsg);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.info( "", sMsg);
        }
    }

    //-----------------------------------------------------------------------------
    // TRACE
    //-----------------------------------------------------------------------------
    public static void trace(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.trace(sClassName, sMsg);
        }
        else
        {
            Console.trace(sClassName, sMsg);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.trace( sClassName, sMsg);
        }
    }
    //-----------------------------------------------------------------------------
    public static void trace(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.trace(sMsg);
        }
        else
        {
            Console.trace(sMsg);
        }
        if ( $ApplicationLogger != null ) // v 0.9.9
        {
            $ApplicationLogger.trace("", sMsg);
        }
    }
    
    //=============================================================================
    // Acces to other configuration files/resources
    //=============================================================================
    
    //-----------------------------------------------------------------------------
    // dbconfig.xml
    //-----------------------------------------------------------------------------
    /**
     * Returns the file name (or resource name) of the DbConfig XML file.
     * @return
     */
    public static String getDbConfigFileName()
    {
        // return getConfigFile("DbConfFile");
        return getConfigFile(DB_CONF_FILE); // v 1.0.0
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the InputStream of the DbConfig XML file or resource.
     * 
     * @return the InputStream ( or null if cannot get it )
     */
    public static InputStream getDbConfigInputStream()
    {
//        String s = getDbConfigFileName() ;
//        return ( s != null ? getInputStream(s) : null ) ;
        return getInputStream(DB_CONF_FILE) ; // v 1.0.0
    }
    
    //-----------------------------------------------------------------------------
    // screens.xml
    //-----------------------------------------------------------------------------
    /**
     * Returns the file name (or resource name) of the DbConfig XML file.
     * @return
     */
    public static String getScreensConfigFileName()
    {
        //return getConfigFile("ScreensConfFile");
        return getConfigFile(SCREENS_CONF_FILE); // v 1.0.0
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the InputStream of the DbConfig XML file or resource.
     * 
     * @return the InputStream ( or null if cannot get it )
     */
    public static InputStream getScreensConfigInputStream()
    {
//        String s = getScreensConfigFileName() ;
//        return ( s != null ? getInputStream(s) : null ) ;
        return getInputStream(SCREENS_CONF_FILE) ; // v 1.0.0
    }

    //-----------------------------------------------------------------------------
    // tags.properties
    //-----------------------------------------------------------------------------
    public static String getTagsConfigFileName()
    {
        //return getConfigFile("TagsConfFile");
        return getConfigFile(TAGS_CONF_FILE); // v 1.0.0
    }
    //-----------------------------------------------------------------------------
    public static InputStream getTagsConfigInputStream()
    {
//        String s = getTagsConfigFileName() ;
//        return ( s != null ? getInputStream(s) : null ) ;
        return getInputStream(TAGS_CONF_FILE) ; // v 1.0.0
    }
    
    //-----------------------------------------------------------------------------
    // traces.properties
    //-----------------------------------------------------------------------------
    public static String getTraceConfigFileName()
    {
        //return getConfigFile("TraceConfFile");
        return getConfigFile(TRACE_CONF_FILE); // v 1.0.0
    }
    //-----------------------------------------------------------------------------
    public static InputStream getTraceConfigInputStream()
    {
        //String s = getTraceConfigFileName() ;
        //return ( s != null ? getInputStream(TRACE_CONF_FILE) : null ) ;
        return getInputStream(TRACE_CONF_FILE); // v 1.0.0
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Returns the property value for the given property name <br>
     * ( properties loaded from the "telosys.properties" file )
     * @param sPropertyName
     * @return
     */
    public static String getProperty(String sPropertyName)
    {
        if ( $TelosysProperties != null )
        {
            return $TelosysProperties.getProperty(sPropertyName);
        }
        else
        {
            error("getProperty('"+sPropertyName+"') : TelosysProperties == null ");
            return null ;
        }
    }
    //-----------------------------------------------------------------------------
    private static String getConfigFile(String sPropertyName)
    {
        if ( $TelosysProperties != null )
        {
            return $TelosysProperties.getConfigFile(sPropertyName);
        }
        else
        {
            error("getConfigFile('"+sPropertyName+"') : TelosysProperties == null ");
            return null ;
        }
    }
    //-----------------------------------------------------------------------------
    private static int getOrigin(String sPropertyName) // v 1.0.0
    {
        //--- Is there a specific origin for this property ?
        String sOrigin = getProperty(sPropertyName + ".origin");
        if ( sOrigin != null )
        {
            String s = sOrigin.trim().toUpperCase();
            if ( s.equals("WEBAPP"))     return TelosysProperties.WEBAPP ;
            if ( s.equals("FILESYSTEM")) return TelosysProperties.FILESYSTEM ;
            if ( s.equals("CLASSPATH"))  return TelosysProperties.CLASSPATH ;            
        }
        // by default use the same origin as "telosys.properties"
        return $iTelosysPropertiesOrigin ;
    }
    //-----------------------------------------------------------------------------
//    /**
//     * Returns the resource file InputStream using the same origin as the main Telosys properties
//     * @param sResource resource or file name
//     * @return
//     */
//    private static InputStream getInputStream(String sResource)
//    {
//        InputStream is = TelosysPropertiesLoader.getInputStream(sResource, $iTelosysPropertiesOrigin, $ServletContext );
//        if ( is == null )
//        {
//            $log.error("getInputStream('" + sResource + "') : return NULL" );
//        }
//        return is;
//    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the InputStream for the resource file associated with the given property name
     * @param sPropertyName
     * @return
     */
    private static InputStream getInputStream(String sPropertyName) // v 1.0.0
    {
        $log.trace("getInputStream("+sPropertyName+")");
        String sResource = getConfigFile(sPropertyName); 
        if ( sResource != null )
        {
            int iOrigin = getOrigin(sPropertyName) ;
            $log.trace("getInputStream("+sPropertyName+") --> '" + sResource + "', origin = " + iOrigin);
            // Get the input stream for the given resource, using the given origin ( WEBAPP, FILESYSTEM, CLASSPATH )
            InputStream is = TelosysPropertiesLoader.getInputStream(sResource, iOrigin, $ServletContext );
            if ( is == null )
            {
                $log.error("getInputStream('" + sResource + "') : return NULL" );
            }
            return is;
        }
        else
        {
            return null ;
        }
    }
    //-----------------------------------------------------------------------------
}