package org.objectweb.telosys.common.data;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.util.XmlUtil;

/**
 * 
 * @author Laurent GUERIN
 *  
 */
public class DataSetWriter extends TelosysObject
{
    private final static String END_OF_LINE = "\n" ;
    
    //-----------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------
    public static void toXml(Writer writer, DataSet ds )
    {
        toXml(writer, ds, null, null);
    }
    //-----------------------------------------------------------------------------------
    public static void toXml(OutputStream outputStream, DataSet ds )
    {
        toXml(outputStream, ds, null, null);
    }
    
    //-----------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------
    public static void toXml(Writer writer, DataSet ds, String sDataSetTagName)
    {
        toXml(writer, ds, sDataSetTagName, null);
    }
    //-----------------------------------------------------------------------------------
    public static void toXml(OutputStream outputStream, DataSet ds, String sDataSetTagName)
    {
        toXml(outputStream, ds, sDataSetTagName, null);
    }
    
    //-----------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------
    public static void toXml(OutputStream outputStream, DataSet ds, String sDataSetTagName, String sDataRowTagName)
    {
        Writer writer = new OutputStreamWriter(outputStream);
        toXml(writer, ds, sDataSetTagName, sDataRowTagName);
    }
    //-----------------------------------------------------------------------------------
    public static void toXml(Writer writer, DataSet ds, String sDataSetTagName, String sDataRowTagName)
    {
        if ( writer == null )
        {
            throw new TelosysRuntimeException("DataSetWriter.toXml() : PrintWriter is null");
        }
        if (ds == null)
        {
            throw new TelosysRuntimeException("DataSetWriter.toXml() : DataSet is null");
        }
        if (sDataSetTagName == null)
        {
            sDataSetTagName = "dataset";
        }
        if (sDataRowTagName == null)
        {
            sDataRowTagName = "row";
        }
        
        try
        {
            writer.write("<" + sDataSetTagName + ">");
            writer.write(END_OF_LINE);
            
            int iRowCount = ds.getRowCount();
            DataRow dr = null;
            int iSize = 0;
            for ( int iRow = 1 ; iRow <= iRowCount ; iRow++ )
            {
                dr = (DataRow) ds.getDataRow(iRow);
                if (dr != null)
                {
                    iSize = dr.getSize();
                    Object obj = null;
                    writer.write("<" + sDataRowTagName );
                    //--- For each column in the DataRow
                    for ( int iCol = 1 ; iCol <= iSize ; iCol++ )
                    {
                        obj = dr.getObject(iCol);
                        if (obj != null)
                        {
                            writer.write(" c" + iCol + "=\"" + XmlUtil.xmlString(obj.toString()) + "\"");
                        }
                    }
                    writer.write(" />");
                    writer.write(END_OF_LINE);
                }
            }
            writer.write("</" + sDataSetTagName + ">" );
            writer.write(END_OF_LINE);
            
            writer.flush();
            
        } catch (IOException ex)
        {
            throw new TelosysRuntimeException("DataSetWriter.toXml() : IOException", ex);
        }
    }
    //-----------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------
    public static void toText(Writer writer, DataSet ds )
    {
        toText(writer, ds, null, null, null);
    }
    //-----------------------------------------------------------------------------------
    public static void toText(OutputStream outputStream, DataSet ds )
    {
        toText(outputStream, ds, null, null, null);
    }
    //-----------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------
    public static void toText(Writer writer, DataSet ds, String sSeparator )
    {
        toText(writer, ds, sSeparator, null, null);
    }
    //-----------------------------------------------------------------------------------
    public static void toText(OutputStream outputStream, DataSet ds, String sSeparator )
    {
        toText(outputStream, ds, sSeparator, null, null);
    }
    //-----------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------
    public static void toText(OutputStream outputStream, DataSet ds, String sSeparator, String sHeader, String sFooter)
    {
        Writer writer = new OutputStreamWriter(outputStream);
        toText(writer, ds, sSeparator, sHeader, sFooter);
    }
    //-----------------------------------------------------------------------------------
    public static void toText(Writer writer, DataSet ds, String sSeparator, String sHeader, String sFooter)
    {
        if ( writer == null )
        {
            throw new TelosysRuntimeException("DataSetWriter.toText() : PrintWriter is null");
        }
        if (ds == null)
        {
            throw new TelosysRuntimeException("DataSetWriter.toText() : DataSet is null");
        }
        if (sSeparator == null)
        {
            sSeparator = "|";
        }
        
        try
        {
            if ( sHeader != null )
            {
                writer.write( sHeader );
                writer.write(END_OF_LINE);
            }
            
            int iRowCount = ds.getRowCount();
            DataRow dr = null;
            int iSize = 0;
            for ( int iRow = 1 ; iRow <= iRowCount ; iRow++ )
            {
                dr = (DataRow) ds.getDataRow(iRow);
                if (dr != null)
                {
                    String s = "" ;
                    iSize = dr.getSize();
                    Object obj = null;
                    //--- For each column in the DataRow
                    for ( int iCol = 1 ; iCol <= iSize ; iCol++ )
                    {
                        if ( iCol > 1)
                        {
                            s = sSeparator ;
                        }
                        obj = dr.getObject(iCol);
                        if (obj != null)
                        {
                            writer.write( s + obj.toString() );
                        }
                    }
                    writer.write(END_OF_LINE);
                }
            }
            
            if ( sFooter != null )
            {
                writer.write( sFooter );
                writer.write(END_OF_LINE);
            }            
            writer.flush();
            
        } catch (IOException ex)
        {
            throw new TelosysRuntimeException("DataSetWriter.toXml() : IOException", ex);
        }
    }
    //-----------------------------------------------------------------------------------
}