package org.objectweb.telosys.common.data;


/**
 * @author Laurent GUERIN
 * 
 * Provides getters to build dynamicaly list items<br>
 * A list item must provide a VALUE and a TEXT to populate Lists, ComboBoxes, ... 
 * 
 */
public interface ListItemProvider
{
    /**
     * Returns the number of rows of the ListItem provider
     * @return : row count
     */
    public int getRowCount();

    /**
     * Returns the item value for the given row  
     * @param iRow ( from 1 to RowCount )
     * @return item value
     */
    public String getListItemValue(int iRow);

    /**
     * Returns the item text for the given row (text to be shown in the combo/list)
     * @param iRow ( from 1 to RowCount )
     * @return item text
     */
    public String getListItemText(int iRow);
    
    /**
     * Notify the ListItem provider that it will be used
     */
    public void beforeUse();
    
    /**
     * Notify the ListItem provider that it has been used
     */
    public void afterUse();

}