package org.objectweb.telosys.common.data;

import java.util.Collection;
import java.util.Iterator;

import org.objectweb.telosys.common.TelosysException;

/**
 * Memory DataSet <br>
 * Special DataSet designed to be loaded from memory, using a Collection of arrays of objects<br>
 * Use the "load(Object)" method to initialize the DataSet with a collection<br>
 * ( for example a LinkedList of Object[] )
 * 
 * @author Laurent GUERIN 
 *  
 */
public class MemDataSet extends UpdatableDataSet
{
    //-----------------------------------------------------------------------------
    //--- CONSTRUCTORS
    //-----------------------------------------------------------------------------
    public MemDataSet()
    {
        super();
    }

    //-----------------------------------------------------------------------------
    //--- PRIVATE/PROTECTED METHODS
    //-----------------------------------------------------------------------------


    //-----------------------------------------------------------------------------
    //--- PUBLIC METHODS
    //-----------------------------------------------------------------------------

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.data.DataSet#load()
     */
    public int load() throws TelosysException
    {
        //--- Initialize the DataSet ( prepare to load )
        initBeforeLoad();
        //--- Return the number of rows in the DataSet
        return getRowCount();
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.data.DataSet#load(java.lang.Object)
     */
    public int load(Object obj) throws TelosysException
    {
        return init(obj);
    }
    
    private int init(Object obj) throws TelosysException
    {
        if ( obj == null )
        {
            throw new TelosysException("load(obj) : parameter is null");
        }
        
        //--- Initialize the DataSet ( prepare to load )
        initBeforeLoad();

        if ( obj instanceof Collection ) // Collection of "Object[]"
        {
            Collection c = (Collection) obj;
            Iterator iter = c.iterator();
            Object oRow = null ;
            while ( iter.hasNext() )
            {
                oRow = iter.next();
                if ( oRow instanceof Object[] )
                {
                    Object[] values = (Object[]) oRow ;
                    DataRow dr = new DataRow( values.length, values);
                    super.addRow(dr);
                }
            }
        }
        else if ( obj instanceof Object[][] ) // Collection of "Object[]"
        {
            //Object[][] grid = (Object[][]) obj;
            // TODO
            throw new TelosysException("unusable parameter " + obj.getClass().getName() );
        }
        else
        {
            throw new TelosysException("unusable parameter " + obj.getClass().getName() );
        }
        
        //--- Return the number of rows in the DataSet
        return getRowCount();
    }

    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
}