package org.objectweb.telosys.common.data;

import java.io.Serializable;


/**
 * 
 * @author Laurent GUERIN
 * Generic TreeNode to build objects trees
 *  
 */
public class TreeNode implements Serializable
{
    private final static TreeNodeList VOID_TREE_NODE_LIST = new TreeNodeList() ;
    private Object       _object       = null ;
    private String       _sId          = null ;
    private int          _iType        = 0 ;
    private TreeNodeList _children = null ;
    
    //----------------------------------------------------------------------------------------
    // Contructor(s)
    //----------------------------------------------------------------------------------------    
    public TreeNode ( Object object )
    {
        super();
        _object = object ;
        _sId    = null ;
        _iType  = 0 ;
    }
    //----------------------------------------------------------------------------------------    
    public TreeNode ( Object object, int type )
    {
        super();
        _object = object ;
        _sId    = null ;
        _iType  = type ;
    }
    //----------------------------------------------------------------------------------------    
    public TreeNode ( Object object, String id )
    {
        super();
        _object = object ;
        _sId    = id ;
        _iType  = 0 ;
    }
    //----------------------------------------------------------------------------------------    
    public TreeNode ( Object object, String id, int type )
    {
        super();
        _object = object ;
        _sId    = id ;
        _iType  = type ;
    }

    //----------------------------------------------------------------------------------------
    // Getters & Setters
    //----------------------------------------------------------------------------------------    
    public void setObject( Object v)
    {
        _object = v;
    }
    public Object getObject()
    {
        return _object;
    }
    //----------------------------------------------------------------------------------------
    public void setType( int v ) 
    {
        _iType = v ;
    }
    public int getType()
    {
        return _iType;
    }
    
    //----------------------------------------------------------------------------------------
    public void setId( String v ) 
    {
        _sId = v ;
    }
    public String getId()
    {
        return _sId;
    }
    
    //----------------------------------------------------------------------------------------
    // Child nodes management
    //----------------------------------------------------------------------------------------    
    public TreeNodeList getChildNodes() // like W3C DOM
    {
        if ( _children != null )
        {
            return _children;
        }
        else
        {
            return VOID_TREE_NODE_LIST;
        }
    }
    //----------------------------------------------------------------------------------------
    public boolean hasChildNodes() // like W3C DOM
    {
        if ( _children != null )
        {
            return ( _children.size() > 0 ) ;
        }
        return false ;
    }
    //----------------------------------------------------------------------------------------    
    /**
     * Appends a child node
     * @param node
     * @return
     */
    public boolean appendChild( TreeNode node ) // like W3C DOM
    {
        if ( _children == null )
        {
            _children = new TreeNodeList();
        }
        return _children.add(node);
    }
    
    //----------------------------------------------------------------------------------------    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() 
    {
        int n = ( _children != null ? _children.size() : 0 ) ;
        String s = "null" ;
        if ( _object != null )
        {
            s = _object.getClass().getName() ;
        }
        return "TreeNode [id:"+_sId+"|type:"+_iType+"|object:"+s+"|"+ n + " child node(s)]";
    }
    
}
