package org.objectweb.telosys.common.val;

import java.math.BigDecimal;

/**
 * This class wraps a BigDecimal value in an mutable value object. 
 * The current value can be changed with the 'setValue' method.
 * 
 * @author Laurent Guerin
 */
public class BigDecimalValue implements ISingleValue
{
    private final static BigDecimal ZERO = new BigDecimal(0.0);
    
    private BigDecimal _value = ZERO ;
        
    //-----------------------------------------------------------------------------------
    
    /**
     * Default constructor, the initial value is '0.0'.
     */
    public BigDecimalValue () 
    {
        _value = ZERO ;
    }
    
    /**
     * Constructor with initial value, if the parameter is null, the value is set to '0.0'
     * @param v
     */
    public BigDecimalValue ( BigDecimal v ) 
    {
        _value = ( v != null ? v : ZERO ) ;
    }
    
    //-----------------------------------------------------------------------------------

    public void setValue(BigDecimal v)
    {
        _value = v;
    }
    
    /**
     * Returns the BigDecimal instance
     * ( the value returned is never null )
     * @return
     */
    public BigDecimal getValue()
    {
        return ( _value != null ? _value : ZERO ) ;
    }
    
    //-----------------------------------------------------------------------------------
    
    public String toString()
    {
        return ( _value != null ? _value.toString() : "0.0" ) ;
    }
    
}
