package org.objectweb.telosys.common.val;

import java.util.Date;

import org.objectweb.telosys.util.DateUtil;

/**
 * This class wraps a Date value in an mutable value object. 
 * The current value can be changed with the 'setValue' method.
 * 
 * @author Laurent Guerin
 */
public class DateValue implements ISingleValue
{
    private final static Date ZERO = new Date(0);
    
    //private Date _value = ZERO ;
    private Date _value = null ;
        
    //-----------------------------------------------------------------------------------
    
    /**
     * Default constructor, the initial value is set to 'null'.
     */
    public DateValue () 
    {
        _value = null ;
    }
    
    /**
     * Constructor with initial value, 
     * the parameter can be null
     * @param v
     */
    public DateValue ( Date v ) 
    {
        //_value = ( v != null ? v : ZERO ) ;
        _value = v ;
    }
    
    //-----------------------------------------------------------------------------------

    public void setValue(Date v)
    {
        _value = v;
    }
    
    /**
     * Returns the Date instance
     * ( the value returned can be null )
     * @return
     */
    public Date getValue()
    {
        //return ( _value != null ? _value : ZERO ) ;
        return _value ;
    }
    
    //-----------------------------------------------------------------------------------
    
    public String toString()
    {
        return ( _value != null ? DateUtil.dateISO(_value) : "" ) ;
    }
    
}
