package org.objectweb.telosys.common.val;

/**
 * This class wraps a String in a mutable value object. 
 * The current value can be changed with the 'setValue' method.
 * 
 * @author Laurent Guerin
 */
public class StringValue implements ISingleValue
{
    private final static String VOID = "" ;
    
    private String _value = null ;
        
    //-----------------------------------------------------------------------------------
    
    /**
     * Default constructor, the initial value is a void string.
     */
    public StringValue () 
    {
        _value = VOID ;
    }
    
    /**
     * Constructor with initial value
     * @param v
     */
    public StringValue ( String v ) 
    {
        _value = v;
    }
    
    //-----------------------------------------------------------------------------------

    public void setValue(String v)
    {
        _value = v;
    }
    
    /**
     * Returns the string value 
     * ( the value returned is never null )
     * @return
     */
    public String getValue()
    {
        return ( _value != null ? _value : VOID ) ;
    }
    
    //-----------------------------------------------------------------------------------
    
    public String toString()
    {
        return ( _value != null ? _value : VOID ) ;
    }
    
}
