package org.objectweb.telosys.common.vo;

import java.io.PrintWriter;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.rpl.xml.IXmlWrapper;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;

/**
 * 
 * @author Laurent GUERIN
 *  
 */
public class BeanWriter //extends TelosysObject
{
    private static final TelosysClassLogger log = new TelosysClassLogger(BeanWriter.class);
    
    //-----------------------------------------------------------------------------------
    /**
     * Write the given object as an XML element 
     * @param out
     * @param oBean the bean to write
     * @param sTagName the XML tag name
     */
    public static void toXml(PrintWriter out, Object oBean, String sTagName )
    {
        if (oBean == null)
        {
            out.println("<!-- " + sTagName + " : Object is NULL -->");
            return;
        }
        
	    IXmlWrapperProvider wrapperProvider = Telosys.getXmlWrapperProvider() ;
        if (wrapperProvider != null)
		{
            IXmlWrapper wrapper = null;
            try
            {
                wrapper = wrapperProvider.getWrapper(oBean);
            } catch (TelosysRuntimeException e)
            {
                wrapper = null;
            }
            if (wrapper != null)
            {
                //--- Print the XML tag
                wrapper.toXml(out, oBean, sTagName );
            }
            else
            {
                String msg = "Cannot get XmlWrapper for '" + oBean.getClass().getName() + "'" ;
    			out.println("<!-- ERROR : " + msg + " -->" );
                log.error("toXml : " + msg );
            }
		}
        else
        {
			out.println("<!-- ERROR : Cannot get XmlWrapperProvider -->" );
            log.error("toXml : Cannot get XmlWrapperProvider ");
        }
    }
}