package org.objectweb.telosys.common.vo;

import java.io.PrintWriter;
import java.util.List;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.rpl.xml.IXmlWrapper;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;

/**
 * 
 * @author Laurent GUERIN
 *  
 */
public class GenericVOListWriter //extends TelosysObject
{
    private static final TelosysClassLogger log = new TelosysClassLogger(GenericVOListWriter.class);

    //-----------------------------------------------------------------------------------
    public static void toXml(PrintWriter out, GenericVOList volist, String sListTagName, String sElementTagName)
    {
        if (volist == null)
        {
            out.println("<" + sListTagName + "/> <!-- List is NULL -->");
            return;
        }
        
        out.println("<" + sListTagName + ">");
        List list = volist.getList();
        if (list != null)
        {
            IXmlWrapperProvider wrapperProvider = Telosys.getXmlWrapperProvider();
            if ( wrapperProvider != null )
            {
                //--- Get the Wrapper the elements type  
                Class elementClass = volist.getElementType();
                IXmlWrapper elementWrapper = wrapperProvider.getWrapper( elementClass ) ;
                if (elementWrapper != null)
                {
                    Object element = null;
                    int iSize = list.size();
                    //--- For each VO in the list ...
                    for ( int i = 0 ; i < iSize ; i++ )
                    {
                        element = list.get(i);
                        if (element != null)
                        {
                            elementWrapper.toXml(out, element, sElementTagName);
                        }
                        else
                        {
                            out.println("<!-- Element [" + i + "] is NULL -->");
                        }
                    }
                }
                else
                {
                    String msg = "Cannot get XmlWrapper for '" + elementClass.getName() + "'" ;
                    log.error("toXml : " + msg );
        			out.println("<!-- ERROR : " + msg + " -->" );
                }
            }
            else
            {
                log.error("toXml : Cannot get XmlWrapperProvider ");
    			out.println("<!-- ERROR : Cannot get XmlWrapperProvider -->" );
            }
        }
        else
        {
            log.error("toXml : List in GenericVOList is NULL ");
            out.println("<!-- ERROR : List in GenericVOList is NULL -->");
        }
        out.println("</" + sListTagName + ">");
    }
}