package org.objectweb.telosys.common.vo;

import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * Special VOList implementation with loosely typed methods <br>
 * Useful for internal framework usage.
 * 
 * @author Laurent GUERIN
 * @since 1.0.0
 *
 */
public class LooselyTypedVOList extends GenericVOList
{
    //-----------------------------------------------------------------------------
    // CONSTRUCTOR
    //-----------------------------------------------------------------------------
    public LooselyTypedVOList( Class cl )
    {
        super(cl); // null class checked in super class
    }
    
    //-----------------------------------------------------------------------------
    // V.O. LIST METHODS 
    //-----------------------------------------------------------------------------
    public Object add()
    {
        //--- Create the list element using the list element type
        Object bean = null ;
        Class cl = super.getElementType();
        if ( cl == null )
        {
            throw new TelosysRuntimeException("Cannot get element type from super class." );
        }
        try
        {
            bean = cl.newInstance();
        } catch (InstantiationException e)
        {
            throw new TelosysRuntimeException("Cannot create bean " + cl.getName(), e);
        } catch (IllegalAccessException e)
        {
            throw new TelosysRuntimeException("Cannot create bean " + cl.getName(), e);
        }

        //--- Add it in the list 
        super.addElement(bean);
        return bean ;
    }
    //-----------------------------------------------------------------------------------------------  
    public void add(Object vo)
    {
        super.addElement(vo);
    }
    //-----------------------------------------------------------------------------------------------  
    public void insert(int index, Object vo)
    {
        super.insertElement(index,vo);
    }
    //-----------------------------------------------------------------------------------------------  
    public Object replace(int index, Object vo)
    {
        return super.replaceElement(index,vo);
    }
    //-----------------------------------------------------------------------------------------------  
    public Object remove(int index)
    {
        return super.removeElement(index);
    }
    //-----------------------------------------------------------------------------------------------  
    public boolean remove(Object vo)
    {
        return super.removeElement(vo);
    }
    //-----------------------------------------------------------------------------------------------  
    //-----------------------------------------------------------------------------------------------  
    public Object get(int index)
    {
        return super.getElement(index);
    }
    //-----------------------------------------------------------------------------------------------  
    public Object getFirst()
    {
        return super.getFirstElement();
    }
    //-----------------------------------------------------------------------------------------------  
    public Object getNext()
    {
        return super.getNextElement();
    }
    //-----------------------------------------------------------------------------------------------  
}

