package org.objectweb.telosys.dal.dao;

import org.objectweb.telosys.dal.dao.StandardDAO;

/**
 * DAO provider interface 
 * Service providing DAOs for bean instance, bean class or table name
 * 
 * @since 1.0.0
 * @author Laurent GUERIN
 *
 */
public interface DAOProvider
{
    /**
     * Returns the DAO for the given object instance and the default database 
     * @param beanInstance
     * @return the DAO or null if not found
     */
    StandardDAO getDAO(Object beanInstance);
    
    /**
     * Returns the DAO for the given object instance and the given database ID 
     * @param beanInstance
     * @param databaseId
     * @return the DAO or null if not found
     */
    StandardDAO getDAO(Object beanInstance, int databaseId);
    
    
    /**
     * Returns the DAO for the given class and the default database 
     * @param beanClass
     * @return the DAO or null if not found
     */
    StandardDAO getDAO(Class beanClass);
    
    /**
     * Returns the DAO for the given class and the given database ID 
     * @param beanClass
     * @param databaseId
     * @return the DAO or null if not found
     */
    StandardDAO getDAO(Class beanClass, int databaseId);
    
    
    /**
     * Returns the DAO for the given table name and the default database 
     * @param tableName
     * @return the DAO or null if not found
     */
    StandardDAO getDAO(String tableName);

    /**
     * Returns the DAO for the given table name and the given database ID 
     * @param tableName
     * @param databaseId
     * @return the DAO or null if not found
     */
    StandardDAO getDAO(String tableName, int databaseId) ; 
        
}
