package org.objectweb.telosys.dal.dao;

import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.dal.dao.StandardDAO;

/**
 * 
 * DAO provider implementation
 * 
 * @since 1.0.0
 * @author Laurent GUERIN
 *
 */
public class DAOProviderImpl implements DAOProvider 
{

    /**
     * <code>registries</code> : DAO registries to use
     */
    private DAORegistries _registries = null ;
    
    /**
     * Constructor with Dependency Injection  
     */
    public DAOProviderImpl( DAORegistries registries )
    {
        super();
        if ( registries == null )
        {
            throw new TelosysRuntimeException("DAOProviderImpl constructor : no DAO registries");
        }
        this._registries = registries ;
    }
    
    /* (non-Javadoc)
     * @see daoprovider.DAOProvider#getDAO(java.lang.Object)
     */
    public StandardDAO getDAO(Object beanInstance)
    {
        if (beanInstance == null)
        {
            throw new TelosysRuntimeException("getDAO(Object bean) : bean argument is null ");
        }
        return getDAO(beanInstance.getClass());
    }

    /* (non-Javadoc)
     * @see daoprovider.DAOProvider#getDAO(java.lang.Class)
     */
    public StandardDAO getDAO(Class beanClass)
    {        
        return getDAO( beanClass, ConnectionManager.getDefaultDatabase() );
    }

    /* (non-Javadoc)
     * @see daoprovider.DAOProvider#getDAO(java.lang.String)
     */
    public StandardDAO getDAO(String table)
    {
        return getDAO( table, ConnectionManager.getDefaultDatabase() );
    }
    
    //------------------------------------------------------------------------------
    // Multiple databases methods
    //------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see daoprovider.DAOProvider#getDAO(java.lang.Object, int)
     */
    public StandardDAO getDAO(Object beanInstance, int databaseId)
    {
        if (beanInstance == null)
        {
            throw new TelosysRuntimeException("getDAO(Object bean, int db) : bean argument is null ");
        }
        return getDAO(beanInstance.getClass(), databaseId);
    }
    
    /* (non-Javadoc)
     * @see daoprovider.DAOProvider#getDAO(java.lang.Class, int)
     */
    public StandardDAO getDAO(Class beanClass, int databaseId)
    {
        if (beanClass == null)
        {
            throw new TelosysRuntimeException("getDAO(Class beanClass, int db) : class argument is null ");
        }
        DAORegistry reg = _registries.getRegistry(databaseId);
        if (reg != null)
        {
            return reg.getByBeanClass(beanClass);
        }
        else
        {
            throw new TelosysRuntimeException("getDAO(" + beanClass.getName() + "," 
                    + databaseId + ") : no registry for database " + databaseId );
        }
    }
    
    /* (non-Javadoc)
     * @see daoprovider.DAOProvider#getDAO(java.lang.String, int)
     */
    public StandardDAO getDAO(String tableName, int databaseId)
    {
        if (tableName == null)
        {
            throw new TelosysRuntimeException("getDAO(String tableName, int db) : table argument is null ");
        }
        DAORegistry reg = _registries.getRegistry(databaseId);
        if (reg != null)
        {
            return reg.getByTableName(tableName);
        }
        else
        {
            throw new TelosysRuntimeException("getDAO(" + tableName + "," 
                    + databaseId + ") : no registry for database " + databaseId );
        }
    }
}
