package org.objectweb.telosys.dal.dao;

import java.sql.Connection;

import org.objectweb.telosys.common.TelosysException;

/**
 * @author Laurent GUERIN
 *  
 */
public interface DatabaseSession
{
    /**
     * Returns the database session unique id 
     * 
     * @return
     * @since v 1.0.2
     */
    public long getId();
    
    /**
     * Returns true if the session is a "dummy session" (without real connection)
     * 
     * @return
     */
    public boolean isDummy();

    /**
     * Returns the database id of the session <br>
     * 0 to N for a real session <br>
     * -1 for a dummy session
     * 
     * @return
     */
    public int getDatabaseId();

    /**
     * Returns the database connection of the session <br>
     * A dummy session returns null
     * 
     * @return
     */
    public Connection getConnection();

    /**
     * Do a commit on the session connection
     * 
     * @throws TelosysException
     */
    public void commit() throws TelosysException;

    /**
     * Do a rollback on the session connection
     * 
     * @throws TelosysException
     */
    public void rollback() throws TelosysException;

    /**
     * Close the session connection
     * 
     * @throws TelosysException
     */
    public void close() throws TelosysException;
}