package org.objectweb.telosys.dal.dao;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.dal.sql.ConnectionManager;

/**
 * Central point to get a DatabaseSession
 * 
 * @author Laurent Guerin
 * @since v 1.0.2
 * 
 */
public class DatabaseSessionManager
{
    public  static final int  DEFAULT_DATABASE = -1;

    public  static final int  NO_DATABASE      = -99;
    
    private static final TelosysClassLogger log = new TelosysClassLogger(DatabaseSessionManager.class);
    
    /**
     * Returns a DatabaseSession instance for the given database ID.<br>
     * ( this is the basic behavior, without thread-local variable storage )
     * 
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    public static DatabaseSession getDatabaseSession( int iDatabaseId ) throws TelosysException
    {
        DatabaseSession dbSession = null ;
        if (iDatabaseId == NO_DATABASE)
        {
            //--- Return a dummy database session
            log.trace("getDatabaseSession() : return a 'Dummy Session' ");
            // return DummyDatabaseSession.getInstance();
            dbSession = ConnectionManager.getDummySession(); // v 1.0.2
        }
        else
        {
            if (iDatabaseId == DEFAULT_DATABASE)
            {
                //--- Return a database session for the default database
                log.trace("getDatabaseSession() : return a 'Real Session' from default database ");
                dbSession = ConnectionManager.getSession();
            }
            else
            {
                //--- Return a database session for the specific database
                log.trace("getDatabaseSession() : return a 'Real Session' from database " + iDatabaseId);
                dbSession = ConnectionManager.getSession(iDatabaseId);
            }
        }
        
        return dbSession ;
    }

    /**
     * Returns a DatabaseSession instance for the given database ID, <br>
     * and store it as thread-local variable.
     * 
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    public static DatabaseSession getThreadLocalDatabaseSession( int iDatabaseId ) throws TelosysException
    {
        DatabaseSession dbSession = getDatabaseSession( iDatabaseId );

        //--- Store the DatabaseSession instance in the current thread ( as thread-local variable )
        CurrentDatabaseSession.set(dbSession); 
        // Rem : the thread-local variable will be removed by the "close()" method in "DatabaseSession"
        
        return dbSession ;
    }

}
