package org.objectweb.telosys.dal.dao;

import java.sql.Connection;
import java.util.List;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.dal.sql.Criteria;

/**
 * Adapter to use a specific DAO as a generic DAO<br>
 * Provides the same methods as a specific DAO, but with generic parameters.
 * 
 * @author Laurent GUERIN
 * @since  1.0.0
 *
 */
public class StandardDAOAdapter
{
    private StandardDAO dao = null ;
    
    /**
     * @param dao the specific DAO
     */
    public StandardDAOAdapter(StandardDAO dao)
    {
        super();
        if ( dao != null )
        {
            this.dao = dao;
        }
        else
        {
            throw new TelosysRuntimeException("StandardDAOAdapter constructor : DAO parameter is null");
        }
    }
    //--------------------------------------------------------------------------------
    public Class getDAOClass() 
    {
        return dao.getClass();
    }
    
    //--------------------------------------------------------------------------------
    public Class getEntityBeanClass() 
    {
        return dao.getEntityBeanClass();
    }
    
    //--------------------------------------------------------------------------------
    public int count(ListQuery query) throws TelosysException
    {
        return dao.doCount(query);
    }
    public int count(ListQuery query, Connection con) throws TelosysException
    {
        return dao.doCount(query, con);
    }
    public int count(ListQuery query, DatabaseSession session) throws TelosysException
    {
        return dao.doCount(query, session);
    }
    public int count(ListQuery query, int iDatabaseId) throws TelosysException
    {
        return dao.doCount(query, iDatabaseId);
    }
    
    //--------------------------------------------------------------------------------
    public int delete(Object bean) throws TelosysException
    {
        return dao.doDelete(bean);
    }
    public int delete(Object bean, Connection con) throws TelosysException
    {
        return dao.doDelete(bean, con);
    }
    public int delete(Object bean, DatabaseSession session) throws TelosysException
    {
        return dao.doDelete(bean, session);
    }
    public int delete(Object bean, int iDatabaseId) throws TelosysException
    {
        return dao.doDelete(bean, iDatabaseId);
    }
    
    //--------------------------------------------------------------------------------
    public int deleteList(ListQuery query) throws TelosysException
    {
        return dao.doDeleteList(query);
    }
    public int deleteList(ListQuery query, Connection con) throws TelosysException
    {
        return dao.doDeleteList(query, con);
    }
    public int deleteList(ListQuery query, DatabaseSession session) throws TelosysException
    {
        return dao.doDeleteList(query, session);
    }
    public int deleteList(ListQuery query, int iDatabaseId) throws TelosysException
    {
        return dao.doDeleteList(query, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int deleteListItems(GenericVOList listData) throws TelosysException
    {
        return dao.doDeleteListItems(listData);
    }
    public int deleteListItems(GenericVOList listData, Connection con) throws TelosysException
    {
        return dao.doDeleteListItems(listData, con);
    }
    public int deleteListItems(GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        return dao.doDeleteListItems(listData, session);
    }
    public int deleteListItems(GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        return dao.doDeleteListItems(listData, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int deleteListItems(List listData) throws TelosysException
    {
        return dao.doDeleteListItems(listData);
    }
    public int deleteListItems(List listData, Connection con) throws TelosysException
    {
        return dao.doDeleteListItems(listData, con);
    }
    public int deleteListItems(List listData, DatabaseSession session) throws TelosysException
    {
        return dao.doDeleteListItems(listData, session);
    }
    public int deleteListItems(List listData, int iDatabaseId) throws TelosysException
    {
        return dao.doDeleteListItems(listData, iDatabaseId);
    }
    
    //--------------------------------------------------------------------------------
    public boolean exists(Object bean) throws TelosysException
    {
        return dao.doExists(bean);
    }
    public boolean exists(Object bean, Connection con) throws TelosysException
    {
        return dao.doExists(bean, con);
    }
    public boolean exists(Object bean, DatabaseSession session) throws TelosysException
    {
        return dao.doExists(bean, session);
    }
    public boolean exists(Object bean, int iDatabaseId) throws TelosysException
    {
        return dao.doExists(bean, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int insert(Object bean) throws TelosysException
    {
        return dao.doInsert(bean);
    }
    public int insert(Object bean, Connection con) throws TelosysException
    {
        return dao.doInsert(bean, con);
    }
    public int insert(Object bean, DatabaseSession session) throws TelosysException
    {
        return dao.doInsert(bean, session);
    }
    public int insert(Object bean, int iDatabaseId) throws TelosysException
    {
        return dao.doInsert(bean, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public Long insertKeyGen(Object bean) throws TelosysException
    {
        return dao.doInsertKeyGen(bean);
    }
    public Long insertKeyGen(Object bean, Connection con) throws TelosysException
    {
        return dao.doInsertKeyGen(bean, con);
    }
    public Long insertKeyGen(Object bean, DatabaseSession session) throws TelosysException
    {
        return dao.doInsertKeyGen(bean, session);
    }
    public Long insertKeyGen(Object bean, int iDatabaseId) throws TelosysException
    {
        return dao.doInsertKeyGen(bean, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int insertListItems(GenericVOList listData) throws TelosysException
    {
        return dao.doInsertListItems(listData);
    }
    public int insertListItems(GenericVOList listData, Connection con) throws TelosysException
    {
        return dao.doInsertListItems(listData, con);
    }
    public int insertListItems(GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        return dao.doInsertListItems(listData, session);
    }
    public int insertListItems(GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        return dao.doInsertListItems(listData, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int insertListItems(List listData) throws TelosysException
    {
        return dao.doInsertListItems(listData);
    }
    public int insertListItems(List listData, Connection con) throws TelosysException
    {
        return dao.doInsertListItems(listData, con);
    }
    public int insertListItems(List listData, DatabaseSession session) throws TelosysException
    {
        return dao.doInsertListItems(listData, session);
    }
    public int insertListItems(List listData, int iDatabaseId) throws TelosysException
    {
        return dao.doInsertListItems(listData, iDatabaseId);
    }
    
    //--------------------------------------------------------------------------------
    public int load(Object bean) throws TelosysException
    {
        return dao.doLoad(bean);
    }
    public int load(Object bean, Connection con) throws TelosysException
    {
        return dao.doLoad(bean, con);
    }
    public int load(Object bean, DatabaseSession session) throws TelosysException
    {
        return dao.doLoad(bean, session);
    }
    public int load(Object bean, int iDatabaseId) throws TelosysException
    {
        return dao.doLoad(bean, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int load(Object bean, String sOptionalClause) throws TelosysException
    {
        return dao.doLoad(bean, sOptionalClause);
    }
    public int load(Object bean, String sOptionalClause, Connection con) throws TelosysException
    {
        return dao.doLoad(bean, sOptionalClause, con);
    }
    public int load(Object bean, String sOptionalClause, DatabaseSession session) throws TelosysException
    {
        return dao.doLoad(bean, sOptionalClause, session);
    }
    public int load(Object bean, String sOptionalClause, int iDatabaseId) throws TelosysException
    {
        return dao.doLoad(bean, sOptionalClause, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int loadList(ListQuery query, GenericVOList listData) throws TelosysException
    {
        return dao.doLoadList(query, listData);
    }
    public int loadList(ListQuery query, GenericVOList listData, Connection con) throws TelosysException
    {
        return dao.doLoadList(query, listData, con);
    }
    public int loadList(ListQuery query, GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        return dao.doLoadList(query, listData, session);
    }
    public int loadList(ListQuery query, GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        return dao.doLoadList(query, listData, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int loadList(ListQuery query, List listData) throws TelosysException
    {
        return dao.doLoadList(query, listData);
    }
    public int loadList(ListQuery query, List listData, Connection con) throws TelosysException
    {
        return dao.doLoadList(query, listData, con);
    }
    public int loadList(ListQuery query, List listData, DatabaseSession session) throws TelosysException
    {
        return dao.doLoadList(query, listData, session);
    }
    public int loadList(ListQuery query, List listData, int iDatabaseId) throws TelosysException
    {
        return dao.doLoadList(query, listData, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int save(Object bean) throws TelosysException
    {
        return dao.doSave(bean);
    }
    public int save(Object bean, Connection con) throws TelosysException
    {
        return dao.doSave(bean, con);
    }
    public int save(Object bean, DatabaseSession session) throws TelosysException
    {
        return dao.doSave(bean, session);
    }
    public int save(Object bean, int iDatabaseId) throws TelosysException
    {
        return dao.doSave(bean, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int saveList(ListQuery query, GenericVOList listData) throws TelosysException
    {
        return dao.doSaveList(query, listData);
    }
    public int saveList(ListQuery query, GenericVOList listData, Connection con) throws TelosysException
    {
        return dao.doSaveList(query, listData, con);
    }
    public int saveList(ListQuery query, GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        return dao.doSaveList(query, listData, session);
    }
    public int saveList(ListQuery query, GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        return dao.doSaveList(query, listData, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int saveList(ListQuery query, List listData) throws TelosysException
    {
        return dao.doSaveList(query, listData);
    }
    public int saveList(ListQuery query, List listData, Connection con) throws TelosysException
    {
        return dao.doSaveList(query, listData, con);
    }
    public int saveList(ListQuery query, List listData, DatabaseSession session) throws TelosysException
    {
        return dao.doSaveList(query, listData, session);
    }
    public int saveList(ListQuery query, List listData, int iDatabaseId) throws TelosysException
    {
        return dao.doSaveList(query, listData, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int update(Object bean) throws TelosysException
    {
        return dao.doUpdate(bean);
    }
    public int update(Object bean, Connection con) throws TelosysException
    {
        return dao.doUpdate(bean, con);
    }
    public int update(Object bean, DatabaseSession session) throws TelosysException
    {
        return dao.doUpdate(bean, session);
    }
    public int update(Object bean, int iDatabaseId) throws TelosysException
    {
        return dao.doUpdate(bean, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int updateListItems(GenericVOList listData) throws TelosysException
    {
        return dao.doUpdateListItems(listData);
    }
    public int updateListItems(GenericVOList listData, Connection con) throws TelosysException
    {
        return dao.doUpdateListItems(listData, con);
    }
    public int updateListItems(GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        return dao.doUpdateListItems(listData, session);
    }
    public int updateListItems(GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        return dao.doUpdateListItems(listData, iDatabaseId);
    }

    //--------------------------------------------------------------------------------
    public int updateListItems(List listData) throws TelosysException
    {
        return dao.doUpdateListItems(listData);
    }
    public int updateListItems(List listData, Connection con) throws TelosysException
    {
        return dao.doUpdateListItems(listData, con);
    }
    public int updateListItems(List listData, DatabaseSession session) throws TelosysException
    {
        return dao.doUpdateListItems(listData, session);
    }
    public int updateListItems(List listData, int iDatabaseId) throws TelosysException
    {
        return dao.doUpdateListItems(listData, iDatabaseId);
    }
    
    //--------------------------------------------------------------------------------
    public ListQuery createQuery(Criteria criterias) throws TelosysException
    {
        return dao.createQuery(criterias);
    }
    public ListQuery createQuery(Criteria criterias, String sQueryEndOfSelect) throws TelosysException
    {
        return dao.createQuery(criterias, sQueryEndOfSelect);
    }
    public ListQuery createQuery(int iParamCount, String sQueryCriteria) throws TelosysException
    {
        return dao.createQuery(iParamCount, sQueryCriteria);
    }
    public ListQuery createQuery(int iParamCount, String sQueryCriteria, String sQueryEndOfSelect) throws TelosysException
    {
        return dao.createQuery(iParamCount, sQueryCriteria, sQueryEndOfSelect);
    }
    public ListQuery createQuery(String sQueryCriteria) throws TelosysException
    {
        return dao.createQuery(sQueryCriteria);
    }
    public ListQuery createQuery(String sQueryCriteria, String sQueryEndOfSelect) throws TelosysException
    {
        return dao.createQuery(sQueryCriteria, sQueryEndOfSelect);
    }
    //--------------------------------------------------------------------------------
    public ListQuery createQueryAll() throws TelosysException
    {
        return dao.createQueryAll();
    }
    public ListQuery createQueryAll(String sQueryEndOfSelect) throws TelosysException
    {
        return dao.createQueryAll(sQueryEndOfSelect);
    }
    //--------------------------------------------------------------------------------
    public String describe()
    {
        return dao.describe();
    }
    //--------------------------------------------------------------------------------
    public SqlRequests getSqlRequests()
    {
        return dao.getSqlRequests();
    }
    //--------------------------------------------------------------------------------
    public String getTableName()
    {
        return dao.getTableName();
    }
    //--------------------------------------------------------------------------------
    public String toString()
    {
        return dao.toString();
    }
}
