package org.objectweb.telosys.dal.dao;

import org.objectweb.telosys.common.TelosysRuntimeException;


/**
 * @author Laurent GUERIN
 * 
 * Standard implementation of ListQuery
 *  
 */
public class StandardQuery implements ListQuery
{
    //--- SQL request to SELECT the elements of the list
    private String   _SqlSelect = "";

    //--- SQL request to delete the elements of the list
    private String   _SqlDelete = "";

    //--- SQL request to count the elements of the list
    private String   _SqlCount  = "";

    //--- Key columns ( query parameters )
    private Object[] _params    = null;

    //-----------------------------------------------------------------------------
    // CONSTRUCTOR(S)
    //-----------------------------------------------------------------------------
    /**
     * Constructor for a standard LIST QUERY object
     * @param iKeySize
     * @param sSqlSelectData
     * @param sSqlSelectCount
     * @param sSqlDelete
     */
    public StandardQuery(int iKeySize, String sSqlSelectData, String sSqlSelectCount, String sSqlDelete)
    {
        if ( iKeySize < 0 )
        {            
            _params = new Object[1];            
        }
        else
        {
            _params = new Object[iKeySize+1];            
        }
        //--- Init all the array (even the first ZERO index element)
        for (int i=0 ; i < _params.length ; i++ )
        {
            _params[i] = null ;
        }
        _SqlSelect = sSqlSelectData;
        _SqlCount = sSqlSelectCount;
        _SqlDelete = sSqlDelete;
    }

    //-----------------------------------------------------------------------------
    // INTERFACE IMPLEMENTATION
    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#getParamCount()
     */
    public int getParamCount()
    {
        if ( _params != null )
        {
            return _params.length - 1;            
        }
        else
        {
            return 0 ;
        }
    }
    
//    //-----------------------------------------------------------------------------
//    /* (non-Javadoc)
//     * @see org.objectweb.telosys.dal.dao.ListQuery#getParametersArray()
//     */
//    public Object[] getParametersArray()
//    {
//        Object[] paramsCopy = (Object[]) _params.clone();
//        return paramsCopy;
//    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#getParametersString()
     */
    public String getParametersString()
    {
        StringBuffer sb = new StringBuffer(128);
        sb.append("[");
        for ( int i = 1 ; i < _params.length ; i++ )
        {
            Object obj = _params[i];
            if (i > 1)
            {
                sb.append("|");
            }
            if ( obj == null )
            {                
                sb.append("(null)");                
            }
            else
            {
                sb.append(obj.toString());                
            }
        }
        sb.append("]");
        return sb.toString();
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#getSqlSelect()
     */
    public String getSqlSelect()
    {
        return _SqlSelect;
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#getSqlDelete()
     */
    public String getSqlDelete()
    {
        return _SqlDelete;
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#getSqlCount()
     */
    public String getSqlCount()
    {
        return _SqlCount;
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#setParameter(int, java.lang.Object)
     */
    public void setParameter(int iIndex, Object oValue)
    {
        if ( iIndex > 0 && iIndex < _params.length )
        {
            _params[iIndex] = oValue ;            
        }
        else
        {
            throw new TelosysRuntimeException("Query : setParameter : invalid index " + iIndex + "( 1 to " + (_params.length-1) + " expected )" );
        }
    }
    
    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#setParameter(int, short)
     */
    public void setParameter(int iIndex, short shValue)
    {
    	setParameter(iIndex, new Short(shValue) );
    }
    
    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#setParameter(int, int)
     */
    public void setParameter(int iIndex, int iValue)
    {
    	setParameter(iIndex, new Integer(iValue) );
    }
    
    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#setParameter(int, long)
     */
    public void setParameter(int iIndex, long lValue)
    {
    	setParameter(iIndex, new Long(lValue) );
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.ListQuery#getParameter(int)
     */
    public Object getParameter(int iIndex)
    {
        if ( iIndex > 0 && iIndex < _params.length )
        {
            return _params[iIndex];
        }
        else
        {
            throw new TelosysRuntimeException("Query : getParameter : invalid index " + iIndex + "( 1 to " + (_params.length-1) + " expected )" );
        }        
    }
}