package org.objectweb.telosys.dal.rest;

import java.io.PrintWriter;

import org.objectweb.telosys.common.vo.BeanWriter;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.common.vo.GenericVOListWriter;
import org.objectweb.telosys.util.XmlUtil;

/**
 * XML response renderer
 * 
 * @author Laurent GUERIN
 * @since 1.0.0
 */
public class DAORendererXML implements DAORenderer
{
    private PrintWriter pw = null ;
    
    /**
     * @param pw
     */
    public DAORendererXML(PrintWriter pw)
    {
        super();
        this.pw = pw;
    }
    
    //--------------------------------------------------------------------------------------------------
    public void printResponse( DAOResponse response )
    {
        prologXml(pw);
        beginningOfXml ( response, pw);
        genReturn ( response, pw);
        genData ( response, pw);
        endOfXml (pw);
        pw.flush();
    }

    //--------------------------------------------------------------------------------------------------
    public void printError( Throwable t )
	{
        prologXml(pw);
        beginningOfXml (pw);
        genReturn ( t, pw);
        endOfXml (pw);
        pw.flush();  
	}

    //--------------------------------------------------------------------------------------------------
	private void prologXml ( PrintWriter out)
	{
		out.println("<?xml  version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
	}
    //--------------------------------------------------------------------------------------------------
	private void beginningOfXml ( PrintWriter out)
	{
		out.print("<response>" );
	}
    //--------------------------------------------------------------------------------------------------
	private void beginningOfXml ( DAOResponse response, PrintWriter out)
	{
		out.print("<response"
			+ " dao=\""    + response.getDAOClassName() + "\""
			+ " action=\"" + response.getAction() + "\""
			+ " >");
	}
	
	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	private void genReturn ( DAOResponse response, PrintWriter out)
	{
	    //log.trace ( "genReturn()" );
		out.println("<return"
		        + " code=\"" + response.getRetCode() + "\"" 
		        + " >" );		            
		//--- Flag "FOUND" or "NOT-FOUND" (or nothing)
		if ( response.isNotFound() == true )
		{
			out.println(" <not-found/>" );
		}
		if ( response.isFound() == true )
		{
			out.println(" <found/>" );
		}
				
		/**
		//--- List of exceptions 
		int iCount = response.getNumberOfExceptions() ;
		if ( iCount > 0 )
		{
		    Exception ex = null ;
			out.println("<exceptions>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        ex = response.getException(i);
		        if ( ex != null )
		        {
					out.println(" <exception num=\"" + (i+1) +"\" class=\"" + ex.getClass().getName() + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) + "\" />" );		            
		        }
		    }
			out.println("</exceptions>" );
		}
		**/
		
		out.println("</return>" );
		
	}
	//--------------------------------------------------------------------------------------------------
	/**
	 * Return tag for EXCEPTION
	 * @param t
	 * @param out
	 */
	private void genReturn ( Throwable t, PrintWriter out)
	{
	    //log.trace ( "genReturn()" );
		out.println("<return"
		        + " code=\"-1\"" 
		        + " >" );		            
				
		out.println("<exceptions>" );
		Throwable ex = t ;
		int i = 0 ;
		while ( ex != null )
		{
		    i++;
		    genException ( i, ex, out);
		    ex = ex.getCause();
		}		
		out.println("</exceptions>" );
		
		out.println("</return>" );
		
	}
	//--------------------------------------------------------------------------------------------------
	private void genException ( int i, Throwable t, PrintWriter out)
	{
        if ( t != null )
        {
			out.println(" <exception"
			        + " num=\"" + i + "\""
			        + " class=\"" + t.getClass().getName() + "\""
			        + " message=\"" + XmlUtil.xmlString( t.getMessage() ) + "\""
			        + " />" );		            
        }
	}
	
	//--------------------------------------------------------------------------------------------------
	private void genData ( DAOResponse response, PrintWriter out) //throws TelosysException
	{
//	    log.trace ( "genData()" );
//		if ( checkPrintWriter(out) == false ) return ;
		
	    Object oResult = response.getResultObject() ;
	    if ( oResult != null )
	    {
			out.println("<data>" );
			if ( oResult instanceof GenericVOList )
			{
                GenericVOList volist = (GenericVOList) oResult ;
                GenericVOListWriter.toXml(out, volist, "list", "item" );
			}
			else
			{
			    BeanWriter.toXml(out, oResult, "entity");
			}
			out.println("</data>" );         
	    }
	    else
	    {
			out.println("<data void=\"true\" />" );
	    }
	}

	//--------------------------------------------------------------------------------------------------
	private void endOfXml ( PrintWriter out ) 
	{
	    //log.trace ( "endOfXml()" );
		//if ( checkPrintWriter(out) == false ) return ;
		out.println("</response>");
	}    
}
