package org.objectweb.telosys.dal.rest;

/**
 * @author Laurent GUERIN
 * @since 1.0.0
 */
/* package */ class DAORequest
{
    public static final int TYPE_HTML = 1 ; 
    public static final int TYPE_TEXT = 2 ; 
    public static final int TYPE_XML  = 3 ; 
    public static final int TYPE_JSON = 4 ; 

    protected static final int DEFAULT_TYPE = TYPE_XML ; 
    
    private static final String[] types = { "?", "html", "text", "xml", "json" } ;
    
    private int    _iDatabaseId = 0 ;
    
    private String _sRecordBean = null ;

    private String _sAction = null ;
        
    private int    _iType = TYPE_HTML ;

    /**
     * Returns the integer type from the given string
     * @param type : "html", "xml", "text", etc ... ( not case sensitive )
     * @return
     */
    public static int getIntType(String type)
    {
        int iType = DEFAULT_TYPE ;
        if ( type != null )
        {
            String s = type.trim().toUpperCase();
            
                 if ( s.equals("HTML") ) iType = TYPE_HTML ;
                 
            else if ( s.equals("TEXT") ) iType = TYPE_TEXT ;
            else if ( s.equals("TXT") )  iType = TYPE_TEXT ;
            
            else if ( s.equals("XML")  ) iType = TYPE_XML ;
            
            else if ( s.equals("JSON") ) iType = TYPE_JSON ;
        }
        return iType;
    }
    
    /**
     * @param databaseId
     * @param recordBean
     * @param action
     */
    public DAORequest(int databaseId, String recordBean, String action, String type )
    {
        super();
        _iDatabaseId = databaseId;
        _sRecordBean = recordBean;
        _sAction = action;
        _iType = getIntType(type);
    }
    
    public int getDatabaseId()
    {
        return _iDatabaseId;
    }
    
    public String getAction()
    {
        return _sAction;
    }
    
    public String getRecordBean()
    {
        return _sRecordBean;
    }
    
    public int getType()
    {
        return _iType;
    }
    
    public String getTypeAsString()
    {
        if ( _iType > 0 && _iType < types.length )
        {
            return types[_iType];
        }
        return "???" ;
    }
    
    public boolean isListAction()
    {
        if ( _sAction != null )
        {
            String s = _sAction.trim().toLowerCase();
            return s.endsWith("list") ;
        }
        return false ;
    }
    
    public String toString()
    {
        return "db=" + _iDatabaseId + " bean='" + _sRecordBean + "' action='" + _sAction + "' type=" + getTypeAsString() ;
    }
}
