package org.objectweb.telosys.dal.rest;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.sql.Types;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;

/* package */ class Util {

	/**
	 * Cherche une ressource
	 * 
	 * @param sResource
	 *            nom de la ressource
	 * @return url de la ressource
	 */
	public static String loadResource(String sResource) {
		// NB
		// . The "standard class loader" search resources ONLY IN FOLDERS (
		// not in ".jar files" )
		// . The "currentThread class loader" search resources in folders
		// AND IN ".jar files"
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl != null) {
			URL url = cl.getResource(sResource);
			if (url != null) {
				return url.getPath();
			} else {
				return "";
			}
		} else {
			return null;
		}
	}

	/**
	 * Sachant le nom de l'attribut et sa valeur, rempli un attribut du
	 * bean.
	 * 
	 * @param oBean
	 *            Bean sur lequel on agit
	 * @param sFieldName
	 *            Nom de l'attribut
	 * @param sFieldValue
	 *            Valeur de l'attribut
	 * @throws TelosysException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	public static void setFieldValue(Object oBean, String sFieldName, String sFieldValue) throws TelosysException,
			IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		if (oBean == null) {
			throw new TelosysException("setFieldValue : bean instance is null ");
		}
		if (sFieldName == null) {
			throw new TelosysException("setFieldValue : field name is null ");
		}

		String sSetterName = "set" + sFieldName.substring(0, 1).toUpperCase() + sFieldName.substring(1);

		Method[] voMethods = oBean.getClass().getMethods();
		for (int i = 0; i < voMethods.length; i++) {
			Method voMethod = voMethods[i];
			if (voMethod.getName().equals(sSetterName)) {
				Class[] voMethodParametersType = voMethod.getParameterTypes();
				
				if (voMethodParametersType.length == 1) {
					Object[] setterParameter = new Object[1];

					if (voMethodParametersType[0].isPrimitive()) {

						if (voMethodParametersType[0].getName().equals("int")) {
							setterParameter[0] = new Integer(sFieldValue);
							voMethod.invoke(oBean, setterParameter);
						}
						if (voMethodParametersType[0].getName().equals("long")) {
							setterParameter[0] = new Long(sFieldValue);
							voMethod.invoke(oBean, setterParameter);
						}
						if (voMethodParametersType[0].getName().equals("boolean")) {
							setterParameter[0] = new Boolean(sFieldValue);
							voMethod.invoke(oBean, setterParameter);
						}
						if (voMethodParametersType[0].getName().equals("byte")) {
							setterParameter[0] = new Byte(sFieldValue);
							voMethod.invoke(oBean, setterParameter);
						}
						if (voMethodParametersType[0].getName().equals("double")) {
							setterParameter[0] = new Double(sFieldValue);
							voMethod.invoke(oBean, setterParameter);
						}
						if (voMethodParametersType[0].getName().equals("float")) {
							setterParameter[0] = new Float(sFieldValue);
							voMethod.invoke(oBean, setterParameter);
						}
						if (voMethodParametersType[0].getName().equals("short")) {
							setterParameter[0] = new Short(sFieldValue);
							voMethod.invoke(oBean, setterParameter);
						}
						if (voMethodParametersType[0].getName().equals("date")) {
							DateFormat dateFormat = new SimpleDateFormat();
							try {
								setterParameter[0] = dateFormat.parse(sFieldValue);
							} catch (ParseException e) {
								e.printStackTrace();
							}
							voMethod.invoke(oBean, setterParameter);
						}
					} else {
						setterParameter[0] = sFieldValue;
						voMethod.invoke(oBean, setterParameter);
					}
				} else {
					throw new TelosysException("setFieldValue : " + sSetterName + " : wrong parameter's number");
				}
			}
		}

	}
	
	public static String truncateString(String sTruncate,int size) {
		//On tronque la string
		if(sTruncate.length() > size) {
			return sTruncate.substring(0, size)+"...";
		} else {
			return sTruncate;
		}
	}
	
	public static String toLowerCaseExceptQuote(String sString) {
		String sResult ="";
		String[] splitedString = sString.split("'");
		for(int i = 0; i < splitedString.length; i++) {
			if(i%2 == 0) {
				sResult += splitedString[i].toLowerCase();
			} else {
				sResult += "'"+splitedString[i]+"'";
			}
		}
		return sResult;
	}
	
	public static String getSQLTypeByInt(int sqlType){
		String res ="";
		
		if (Types.ARRAY == sqlType) res ="Array";
		if (Types.BIGINT == sqlType) res ="Bigint";
		if (Types.BINARY == sqlType) res ="Binary";
		if (Types.BIT == sqlType) res ="Bit";
		if (Types.BLOB == sqlType) res ="Blob";
		if (Types.BOOLEAN == sqlType) res ="Boolean";
		if (Types.CHAR == sqlType) res ="Char";
		if (Types.CLOB == sqlType) res ="Clob";
		if (Types.DATALINK == sqlType) res ="Datalink";
		if (Types.DATE == sqlType) res ="Date";
		if (Types.DECIMAL == sqlType) res ="Decimal";
		if (Types.DISTINCT == sqlType) res ="Distinct";
		if (Types.DOUBLE == sqlType) res ="Double";
		if (Types.FLOAT == sqlType) res ="Float";
		if (Types.INTEGER == sqlType) res ="Integer";
		if (Types.JAVA_OBJECT == sqlType) res ="JavaObject";
		if (Types.LONGVARBINARY == sqlType) res ="LongVarBinary";
		if (Types.LONGVARCHAR == sqlType) res ="LongVarChar";
		if (Types.NULL == sqlType) res ="Null";
		if (Types.NUMERIC == sqlType) res ="Numeric";
		if (Types.OTHER == sqlType) res ="Other";
		if (Types.REAL == sqlType) res ="Real";
		if (Types.REF == sqlType) res ="Ref";
		if (Types.SMALLINT == sqlType) res ="SmallInt";
		if (Types.STRUCT == sqlType) res ="Struct";
		if (Types.TIME == sqlType) res ="Time";
		if (Types.TIMESTAMP == sqlType) res ="TimeStamp";
		if (Types.TINYINT == sqlType) res ="TinyInt";
		if (Types.VARBINARY == sqlType) res ="VarBinary";
		if (Types.VARCHAR == sqlType) res ="VarChar";
					
		/**
		// New Types in Java 5 
		if (Types.LONGNVARCHAR == sqlType) res ="LongNVarchar";
		if (Types.NCHAR == sqlType) res ="NChar";
		if (Types.NCLOB == sqlType) res ="NClob";
		if (Types.NVARCHAR == sqlType) res ="NVarChar";
		if (Types.ROWID == sqlType) res ="RowId";
		if (Types.SQLXML == sqlType) res ="SqlXml";
		**/
		
		return res;
	}
	
	/** Method to retrieve sql type by reflection on Types.class and jdbc int type 
	 * @param iJdbcType
	 * @return String
	 * @throws IllegalAccessException 
	 * @throws IllegalArgumentException 
	 */
	public static String getSqlTypeByJdbcType(int iJdbcType) {
		String res="???";
		Class cTypes = Types.class;
		Field[] arrayFields = cTypes.getDeclaredFields();
		for (int i=0; i < arrayFields.length;i++ ) {
			try {
				if(arrayFields[i].getInt(arrayFields[i]) == iJdbcType) {
					res = arrayFields[i].getName();
				}
			} catch (IllegalArgumentException e) {
				throw new TelosysRuntimeException("Illegal Argument Exception",e);
			} catch (IllegalAccessException e) {
				throw new TelosysRuntimeException("Illegal Acces Exception",e);
			}	
		}
		return res;
	}

	public static String quoteRetourChariot(String txt) {
		return txt.replaceAll("[\n\r]+", "\"+\n\r\"");
	}
}
