package org.objectweb.telosys.dal.sql;

import org.objectweb.telosys.common.TelosysException;

/**
 * @author Laurent GUERIN
 *  
 */

public class Cast
{

    public static Object toObject(String s, int iType) throws TelosysException
    {
        if ( s == null )
        {
            return null ;
        }
        
        try 
        {
            switch (iType)
            {
            case ParamType.BOOLEAN :
                return Boolean.valueOf(s); // "true" (ignore case) / other = false

            case ParamType.BYTE :
                return Byte.valueOf(s);
            case ParamType.DOUBLE :
                return Double.valueOf(s);
            case ParamType.FLOAT :
                return Float.valueOf(s);
            case ParamType.INTEGER :
                return Integer.valueOf(s);
            case ParamType.LONG :
                return Long.valueOf(s);
            case ParamType.SHORT :
                return Short.valueOf(s);
            case ParamType.STRING :
                return s;

            case ParamType.DATE :
                return java.sql.Date.valueOf(s); // ""yyyy-mm-dd" ( throws IllegalArgumentException )

            case ParamType.TIMESTAMP :
                return java.sql.Timestamp.valueOf(s); // ""yyyy-mm-dd hh:mm:ss.ffffff" ( throws IllegalArgumentException )

            default :
                throw new TelosysException("Cast.toObject() : Unknowned type " + iType);
            }
        } catch (NumberFormatException e)
        {
            throw new TelosysException("Cast.toObject() : Invalid number '" + s + "' ", e);
        } catch (IllegalArgumentException e)
        {
            throw new TelosysException("Cast.toObject() : Invalid date (ISO) '" + s + "' ", e);
        }
    }
}