package org.objectweb.telosys.dal.sql;

import java.io.Serializable;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *  
 */
public class Criterion implements Serializable
{    
    private String  _sSqlExpression = null; // SQL where criterion ( ie : "agency = ?" )

    private int     _iParamType     = ParamType.UNKNOWN ;   // see ParamType class

    private String  _sParamValue    = null;

    private Object  _oParamValue    = null;

    private boolean _bUsed          = false;

    /**
     * Creates a new Criterion object
     * @param sSqlExpression the SQL criterion expression ( eg : "agency = ?", "code > ?", ... )
     * @param iType the criterion parameter type ( eg : STRING, INTEGER, ... ) ( see "ParamType" )
     */
    public Criterion(String sSqlExpression, int iType)
    {
        super();
        _sSqlExpression = sSqlExpression ;
        _iParamType = iType;
    }

    public String getSqlExpression()
    {
        return _sSqlExpression;
    }

    public int getParamType()
    {
        return _iParamType;
    }

    public String getParamValueAsString()
    {
        return _sParamValue ;
    }

    public Object getParamValueAsObject()
    {
        return _oParamValue ;
    }

    public void useWithValue(String sParamVal)
    {
        _oParamValue = null ;
        _sParamValue = null ;
        try
        {
            _oParamValue = Cast.toObject(sParamVal, _iParamType) ;
            _sParamValue = sParamVal;
        } catch (TelosysException e)
        {
            throw new TelosysRuntimeException("Criteria : useWithValue(" + sParamVal + ") invalid parameter, type = " + _iParamType );
        }
        _bUsed = true;
    }

    public void doNotUse()
    {
        _oParamValue = null ;
        _sParamValue = null;
        _bUsed = false;
    }

    public boolean isUsed()
    {
        return _bUsed ;
    }
}