package org.objectweb.telosys.dal.sql;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.util.StrUtil;

/**
 * Database definition loaded from the configuration file 'dbconfig.xml' 
 * 
 * @author Laurent GUERIN
 *  
 */
public class Database extends TelosysObject 
{
    private int        _id                 = 0;

    private String     _sName              = null ;

    private String     _sDriver            = null ;

    private String     _sUrlJdbc           = null ;

    private String     _sIsolationLevel    = null ; // 0 (TRANSACTION_NONE), 1, 2, 4, 8

    private String     _sPoolSize          = null ;

    private String     _sDAORegistryClass  = null ; // v 1.0.0

    private String     _sDAOClassPattern   = null ; // v 1.0.0

    private String     _sRecordClassPattern = null ; // v 1.0.0
    
    //-----------------------------------------------------------------------------
    // Constructor(s)
    //-----------------------------------------------------------------------------
    //public Database(int id, String sName, String sDriver, String sUrl, String sIsolationLevel, String sPoolSize )
    public Database(int id, String sName, String sDriver, String sUrl, String sIsolationLevel, String sPoolSize, 
            String sDAORegistry, String sDAOClassPattern, String sRecordClassPattern  )
    {
        trace("Constructor ...");
        _id = id;
        _sName = sName;
        _sDriver = sDriver;
        _sUrlJdbc = sUrl;
        _sIsolationLevel = sIsolationLevel;
        _sPoolSize = sPoolSize ;
        _sDAORegistryClass = sDAORegistry ;
        _sDAOClassPattern = sDAOClassPattern ;
        _sRecordClassPattern = sRecordClassPattern ;
    }

    //-----------------------------------------------------------------------------
    // Getters
    //-----------------------------------------------------------------------------
    /**
     * Returns the database ID ( from 0 to max id )
     * @return
     */
    public int getId()
    {
        return _id;
    }
    
    /**
     * Returns the database name
     * @return
     */
    public String getName()
    {
        return _sName;
    }

    /**
     * Returns the JDBC driver used for this database
     * @return
     */
    public String getDriver()
    {
        return _sDriver;
    }
    
    /**
     * Returns the JDBC URL used for this database
     * @return
     */
    public String getJdbcURL()
    {
        return _sUrlJdbc;
    }

    /**
     * Returns the original 'isolation level' parameter 
     * @return
     */
    public String getIsolationLevel()
    {
        return _sIsolationLevel;
    }
    
    /**
     * Returns the original 'pool size' parameter 
     * @return
     */
    public String getPoolSize()
    {
        return _sPoolSize;
    }

    /**
     * Returns the 'pool size' parameter value <br>
     * ( returns 0 if the original parameter string cannot be converted to int ) 
     * @return
     */
    public int getPoolSizeValue()
    {
        return StrUtil.getInt(_sPoolSize, 0);
    }
    
    /**
     * Returns the DAO registry class name or null if none 
     * @return
     * @since v 1.0.0
     */
    public String getDAORegistryClass() 
    {
        return _sDAORegistryClass ;
    }

    /**
     * Returns the DAO class name pattern or null if none 
     * @return
     * @since v 1.0.0
     */
    public String getDAOClassPattern()
    {
        return _sDAOClassPattern ;
    }
    
    /**
     * Returns the record class name pattern or null if none 
     * @return
     * @since v 1.0.0
     */
    public String getRecordClassPattern()
    {
        return _sRecordClassPattern ;
    }
    
    //-----------------------------------------------------------------------------
    public String toString()
    {
        return _id + " : '" + _sName + "' (" + _sDriver + ", '" + _sUrlJdbc + "', " + _sIsolationLevel + ", " + _sPoolSize + ")";
    }
}