package org.objectweb.telosys.rpl;

import java.util.HashMap;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysFactory;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.screen.core.ScreenData;

/**
 * @author Laurent GUERIN
 *  
 */
public class ElementClassResolver extends TelosysObject
{
    private static final TelosysClassLogger log                = new TelosysClassLogger(ElementClassResolver.class);

    private final static HashMap $hmScreenData = new HashMap(128);

    /**
     *  
     */
    private ElementClassResolver()
    {
    }

    //-----------------------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------------------
    private static ScreenData createScreenData(String sScreenName)
    {
        log.trace("createScreenData(" + sScreenName + ")...");
        ScreenData screenData = null;
        TelosysFactory factory = Telosys.getFactory();
        if (factory != null)
        {
            try
            {
                screenData = factory.createScreenData(sScreenName);
            } catch (TelosysException e)
            {
                throw new TelosysRuntimeException("createScreenData() : cannot create ScreenData ", e);
            }
        }
        else
        {
            throw new TelosysRuntimeException("createScreenData() : cannot get factory ");
        }
        return screenData;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Ensure that the ScreenData definition of the given ScreenName is known
     * @param sScreenName
     */
    public static void initScreenDataDefinition(String sScreenName)
    {
        log.trace("initScreenDataDefinition(" + sScreenName + ")...");
        
        //--- Lock the HashMap
        synchronized ($hmScreenData)
        {
            if ($hmScreenData.get(sScreenName) == null) // Not found ( unknonw screen type )
            {
                log.trace("initScreenDataDefinition(" + sScreenName + ") : not found => initialized... ");
                //--- Creation
                ScreenData screenData = createScreenData(sScreenName);
                //--- Storage
                $hmScreenData.put(sScreenName, screenData);
                log.trace("initScreenDataDefinition(" + sScreenName + ") : initialized. ");
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the class of the given element
     * 
     * @param sScreenName
     * @param sElementName
     */
    public static Class getElementClass(String sScreenName, String sElementName)
    {
        log.trace("getElementClass(" + sScreenName + "," + sElementName + ")...");
        
        ScreenData screenData = (ScreenData) $hmScreenData.get(sScreenName);
        if (screenData != null)
        {
            return screenData.getElementClass(sElementName);
        }
        else
        {
            return null;
        }
    }

    //-----------------------------------------------------------------------------------------------
}