package org.objectweb.telosys.rpl;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.env.ScreenContextManager;
import org.objectweb.telosys.util.StrUtil;

/**
 * @author Laurent GUERIN
 *
 */
public class ScreenElementProvider extends TelosysObject
{
    
    private static final TelosysClassLogger log = new TelosysClassLogger(ScreenElementProvider.class);

    /**
     * Returns the screen data element from the screen context associated with the screen id 
     * and screen name stored in the given screen request.
     * @param screenRequest
     * @param sElementName
     * @return the screen data element or null if not found
     * @throws TelosysException
     */
    public static Object getScreenDataElement( ScreenRequest screenRequest, String sElementName ) throws TelosysException
    {
		HttpServletRequest httpRequest = screenRequest.getHttpRequest();
		
		int iScreenId = StrUtil.getInt ( screenRequest.getScreenId(), -1 );
		String sScreenName = screenRequest.getScreenName();		

		log.trace("getScreenDataElement : screen ID = " + iScreenId + ", screen name = '" + sScreenName + "'");
		
		//--- Is there an existing screen context with the given Screen ID and Sccreen Name ?
		ScreenContext screenContext = ScreenContextManager.getScreenContextForName( httpRequest, iScreenId, sScreenName );
        if (screenContext != null) 
		{
		    //--- The screen context exists => get the screen data 
		    ScreenData screenData = screenContext.getCurrentData();
			if (screenData != null ) 
			{
			    //--- The screen data exists => get the screen data element 
				Object screenDataElement = screenData.getElement(sElementName);
				if ( screenDataElement != null )
				{
				    log.trace("getScreenDataElement : screen data element found : class = " + screenDataElement.getClass() );
					return screenDataElement ;
				}
				else
				{
				    log.trace("getScreenDataElement : screen data element not found.");
				}
			}
			else
			{
			    log.trace("getScreenDataElement : screen data not found.");
			}
		}
        else
        {
            log.trace("getScreenDataElement : screen context not found.");
        }
		
		return null ;
    }
}
