package org.objectweb.telosys.rpl;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.objectweb.telosys.common.RequestParameters;
import org.objectweb.telosys.common.TelosysException;

/**
 * Parameters embeded in a ScreenRequest (parameters sent by a 'screen action')<br>
 * or parameters received in events 'eventCreate' or 'eventReuse'  
 * 
 * @author Laurent GUERIN
 * 
 */
public class ScreenRequestParameters extends RequestParameters // TelosysObject
{
    private Hashtable      _htParameters = null;

    /**
     * Creates a void parameters collection
     */
    public ScreenRequestParameters()
    {
    }

    /**
     * Creates a parameters collection and initializes it with the given map
     * @param map 
     */
    public ScreenRequestParameters(Map map)
    {
        if ( map != null )
        {
            if ( map.size() > 0 )
            {
                _htParameters = new Hashtable();
                
    	        Set set = map.entrySet();
    	        Iterator iter = set.iterator() ;
    	        for ( int i = 0 ; iter.hasNext() ; i++ )
    	        {
    	            Map.Entry entry = (Map.Entry) iter.next();
    	            String   sKey  = (String)   entry.getKey() ;
    	            String[] array = (String[]) entry.getValue() ;
    	            String   sVal  = array.length > 0 ? array[0] : "" ;
    	            
    	            //this.setParameter( sKey, sVal );
    	            _htParameters.put(sKey, sVal);
    	        }
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    public void setParameter(String sParamName, String sParamValue)
    {
        if (_htParameters == null)
        {
            _htParameters = new Hashtable();
        }
        _htParameters.put(sParamName, sParamValue);
//        if (_htParameters != null)
//        {
//            _htParameters.put(sParamName, sParamValue);
//        }
//        else
//        {
//            error("Cannot create parameters Hashtable !");
//        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given parameter name, or NULL if not found
     * @param sParamName
     * @return
     */
    public String getParameter(String sParamName)
    {
        String sVal = null ;
        if (_htParameters != null)
        {
            sVal = (String) _htParameters.get(sParamName);
        }
        return sVal ;        
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given parameter name, or the default value if not found
     * @param sParamName
     * @param sDefaultValue : the default value used if the parameter is not found
     * @return 
     */
    public String getParameter(String sParamName, String sDefaultValue)
    {
        String sVal = getParameter(sParamName) ;
        return sVal != null ? sVal : sDefaultValue ;        
    }

    //-----------------------------------------------------------------------------------------------
    private int parseInt(String sVal, String sParamName) throws TelosysException
    {
        int iVal = 0 ;
        try
        {
            iVal = Integer.parseInt(sVal);
        } catch (NumberFormatException e)
        {
            throw new TelosysException("Cannot convert parameter '" + sParamName + "' to int (value=" + sVal + ")");
        }
        return iVal ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the int value of the given parameter name
     * @param sParamName
     * @return
     * @throws TelosysException : if parameter not found or not convertible to int
     * @deprecated use getParamAsInt 
     */
    public int getIntParameter(String sParamName) throws TelosysException
    {
        String sVal = getParameter(sParamName) ;
        if (sVal != null)
        {
            return parseInt(sVal, sParamName);
        }
        else
        {
            throw new TelosysException("ScreenRequest parameter '" + sParamName + "' not found !");            
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the int value of the given parameter name, or the default value if not found
     * @param sParamName
     * @param iDefaultValue
     * @return
     * @throws TelosysException : if the parameter value is not convertible to int
     * @deprecated use getParamAsInt 
     */
    public int getIntParameter(String sParamName, int iDefaultValue ) throws TelosysException
    {
        String sVal = getParameter(sParamName) ;
        if (sVal != null)
        {
            return parseInt(sVal, sParamName);
        }
        else
        {
            return iDefaultValue ;
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Return the number of parameters 
     * @return parameters count ( 0 to N )
     */
    public int count()
    {
        if ( _htParameters != null )
        {
            return _htParameters.size() ;
        }
        return 0 ;
    }
    
    
    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        //--- Parameters
        String sParameters = "(no parameter)";
        if ( _htParameters != null )
        {
            sParameters = _htParameters.toString();
        }
        return sParameters;
    }
    //-----------------------------------------------------------------------------------------------
}