package org.objectweb.telosys.rpl;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysFactory;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.common.View;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.IServiceProvider;
import org.objectweb.telosys.service.ScreenService;
import org.objectweb.telosys.service.ServiceRenderer;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.service.ServiceResponse;
import org.objectweb.telosys.service.ServiceResultHolder;
import org.objectweb.telosys.service.XmlServiceOutput;
import org.objectweb.telosys.util.web.WebUtil;

/**
 * @author Laurent GUERIN
 *  
 */
public class ServiceServlet extends TelosysServlet //extends HttpServlet
{
    public final static String SERVICE_RESPONSE = "TELOSYS_SERVICE_RESPONSE";

    public final static int    ACTION_INVALID   = -1;

    public final static int    ACTION_NONE      = 0;

    public final static int    ACTION_GET       = 1;

    private IServiceProvider   _serviceProvider = null;

    //---------------------------------------------------------------------------
    public void init() throws ServletException
    {
    }

    //---------------------------------------------------------------------------
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        process(request, response);
    }

    //---------------------------------------------------------------------------
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        process(request, response);
    }

    //---------------------------------------------------------------------------
    public void destroy()
    {
    }

    //---------------------------------------------------------------------------
    private ScreenService getService(String sServiceName) throws TelosysException
    {
        if (_serviceProvider == null)
        {
            //--- Get the Telosys factory
            TelosysFactory factory = Telosys.getFactory();
            if (factory == null)
            {
                throw new TelosysException("openScreenContext : Cannot get the TelosysFactory !");
            }
            //--- Get the service provider
            _serviceProvider = factory.getServiceProvider();
            if (_serviceProvider == null)
            {
                throw new TelosysException("openScreenContext : Cannot get the ServiceProvider from the TelosysFactory !");
            }
        }
        return _serviceProvider.getService(sServiceName);
    }

    //---------------------------------------------------------------------------
    private void process(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        String sRenderer = null;
        String sServiceName = null;
        String sSessionKey = null;
        ServiceRequest  serviceRequest  = null;
        ServiceResponse serviceResponse = null;
        
//        int iParamCount = 0 ;
//        Enumeration paramNames = httpRequest.getParameterNames();
//        while ( paramNames.hasMoreElements() )
//        {
//            paramNames.nextElement();
//            iParamCount++;
//        }
//        trace("Param count = " + iParamCount);
        

        // Prevent caching 
        WebUtil.noCache(httpResponse);
        
        try
        {
            //------------------------------------------------------------------------
            //--- Init the call environment
            //------------------------------------------------------------------------
            //--- Get the service name
            sServiceName = getNameFromURI(httpRequest);

            //--- Get the service action parameter
            int iAction = ACTION_NONE;
            String sAction = httpRequest.getParameter("action");
            if (sAction != null)
            {
                if (sAction.equalsIgnoreCase("get"))
                {
                    iAction = ACTION_GET;
                }
                else
                {
                    iAction = ACTION_INVALID;
                }
            }

            //--- Get the current "Screen Session" ( create a "dummy user" session if Authentication Not Requiered )
            ScreenSession screenSession = ScreenRequestProcessor.getScreenSession(httpRequest);

            //--- Build a Service Request instance
            serviceRequest = new ServiceRequest(sServiceName, httpRequest, screenSession );
 
            //--- Build a Service Response instance
            //serviceResponse = new ServiceResponse(sServiceName, httpRequest, screenSession );
            serviceResponse = new ServiceResponse(sServiceName, httpRequest, httpResponse, screenSession ); // v 0.9.9

            //--- Get a service instance from the provider
            ScreenService oScreenService = getService(sServiceName);

            //--- The view where to forward (if any)
            //String sTargetView = null ;
            View view = null;

            if (oScreenService != null)
            {
                //--- Get the renderer
                sRenderer = oScreenService.getRenderer();
                //--- Get the session key (for session storage if any)
                sSessionKey = oScreenService.getSessionKey();
                trace("Service '" + sServiceName + "' found ( renderer = '" + sRenderer + "', session key = '" + sSessionKey + "' )");
                if (iAction != ACTION_NONE)
                {
                    //------------------------------------------------------------------------
                    //--- Execute the special action
                    //------------------------------------------------------------------------
                    if (iAction == ACTION_GET)
                    {
                        doGetAction(screenSession, sSessionKey, serviceResponse);
                    }
                    else if (iAction == ACTION_INVALID)
                    {
                        String sMsg = "Invalid service action : " + sAction;
                        error(sMsg);
                        serviceResponse.addMessage(new ScreenMessage(sMsg, ScreenMessage.ERROR));
                    }
                }
                else
                {
                    //------------------------------------------------------------------------
                    //--- Execute the service
                    //------------------------------------------------------------------------
                    view = oScreenService.execute(screenSession, serviceRequest, serviceResponse);
                }
                //--- Store the result and the renderer in the Screen Session (only after "execute" not after "action" )
                if (iAction == ACTION_NONE && sSessionKey != null)
                {
                    trace("Store service result in session (key=" + sSessionKey + ")");
                    ServiceResultHolder srh = new ServiceResultHolder(oScreenService, serviceResponse);
                    //screenSession.put(sSessionKey, srh);
                    screenSession.setServiceResultHolder(sSessionKey, srh);
                }

                //--- Set the Service Response in the http request (to be used by renderer/view)
                httpRequest.setAttribute(SERVICE_RESPONSE, serviceResponse);
                
                // removed in v 1.0.1 
                //RequestEnv.setEnv(httpRequest, screenSession);                
            }
            else
            {
                sRenderer = null;
                sSessionKey = null;
                String sMsg = "ScreenService '" + sServiceName + "' not found.";
                error(sMsg);
                serviceResponse.addMessage(new ScreenMessage(sMsg, ScreenMessage.ERROR));
            }

            //------------------------------------------------------------------------
            //--- Print the response ( View or Ajax response )
            //------------------------------------------------------------------------
            if (view != null)
            {
                //--- The service delegated the entire response management to an external VIEW
                trace("Service '" + sServiceName + "' : forward to view = '" + view.getTarget() + "' ...");
                view.generate(httpRequest, httpResponse);
                //--- Navigation notification
                navigation(httpRequest, oScreenService, view );
            }
            else
            {
                //--- The service generate the XML AJAX response by itself (using a partial view renderer, if any)
                trace("Service '" + sServiceName + "' : generate XML response ( renderer = '" + sRenderer + "' ) ...");
                ServiceRenderer svcRenderer = null;
                // if (serviceResponse.getResult() != null) // Suppr in v 1.0.2
                // {
                    if (sRenderer != null)
                    {
                        svcRenderer = new ServiceRenderer(sRenderer, httpRequest, httpResponse);
                    }
                    else
                    {
                        svcRenderer = null;
                    }
                // }
                //--- Generate the response
                PrintWriter out = httpResponse.getWriter();
                //--- The response is in XML format
                httpResponse.setContentType(XML_CONTENT_TYPE);
                XmlServiceOutput.generate(serviceResponse, out, svcRenderer);
                out.close();
            }

        } catch (TelosysException e)
        {
            String sMsg = "Call service '" + sServiceName + "' : TelosysException : " + e.getMessage();
            error(sMsg, e);
            if (serviceResponse != null)
            {
                //serviceResponse.addError(e);                
                serviceResponse.addException(e);                
            }
            sendErrorResponse(httpResponse, serviceResponse, 1003, sMsg);
        } catch (Throwable t)
        {
            String sMsg = "Call service '" + sServiceName + "' : Exception/Throwable : " + t.getMessage();
            error(sMsg, t);
            if (serviceResponse != null)
            {
                //serviceResponse.addError(t);                
                serviceResponse.addException(t);                
            }
            sendErrorResponse(httpResponse, serviceResponse, 1004, sMsg);
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Hook method designed to be overridden in a subclasse
     * @param request
     * @param service
     * @param view
     */
    protected void navigation(HttpServletRequest request, ScreenService service, View view )
    {
        trace("navigation() : " + request.getRequestURI() +")");
    }
    //---------------------------------------------------------------------------
    private void sendErrorResponse(HttpServletResponse response, ServiceResponse theResponse, int iReturnCode, String sMsg) //throws ServletException, IOException
    {
        //--- Log the error
        error(sMsg);

        //--- Create a ScreenResponse object if it doesn't exist
        ServiceResponse serviceResponse = theResponse;
        if (serviceResponse == null)
        {
            //serviceResponse = new ServiceResponse("", null, null);
            serviceResponse = new ServiceResponse("", null, null, null); // v 0.9.9
        }

        //--- Set the return code and message
        serviceResponse.setReturnCode(iReturnCode);
        serviceResponse.setReturnMessage(sMsg);

        //--- Send the HTTP Response
        try
        {
            //trace("error - step 1");
            PrintWriter out = response.getWriter();
            if (out == null)
            {
                error("Servlet output stream is null ");
                return;
            }
            //trace("error - step 1");
            //--- Generate the XML response
            XmlServiceOutput.generate(serviceResponse, out, null);

            //trace("error - step 3");
            out.close();
            //trace("error - step 4");

        } catch (Exception e)
        {
            //--- Log the error
            error("Error : " + e.getMessage());
        }
        //trace("error - end ");
    }

    //---------------------------------------------------------------------------
    // Dtermination du nom de la classe  partir de l'URI de la requte HTTP
    // ( nom de classe uniquement sans le nom de package )
    //---------------------------------------------------------------------------
    private static String getNameFromURI(HttpServletRequest request)
    {
        String sURI = request.getRequestURI();
        int iFirst = sURI.lastIndexOf("/");
        if (iFirst < 0)
        {
            iFirst = 0;
        }
        else
        {
            iFirst++;
        }
        int iLast = sURI.lastIndexOf(".");
        if (iLast < 0)
        {
            return null;
        }
        return sURI.substring(iFirst, iLast);
    }

    //---------------------------------------------------------------------------
    private void doGetAction(ScreenSession screenSession, String sSessionKey, ServiceResponse serviceResponse)
    {
        trace("GET action : session key = '" + sSessionKey + "'");

        //--- Is there a result object stored with this KEY in the ScreenSession ?        
        ServiceResultHolder srh = screenSession.getServiceResultHolder(sSessionKey);
        if ( srh != null )
        {
            //--- Store the service result in the current service response (for the renderer)
            serviceResponse.setResult(srh.getResult());            
        }
        else
        {
            trace("Object NOT FOUND in session (key='" + sSessionKey + "').");            
        }
    }
}