package org.objectweb.telosys.rpl;

import java.util.Properties;

import org.objectweb.telosys.common.Telosys;

/**
 * @author Laurent GUERIN
 *  
 */
public final class TelosysRPL
{
    private final static String XML_WRAPPER_PACKAGE = "XmlWrapperPackage";

    private final static String XML_WRAPPER_SUFFIX  = "XmlWrapperSuffix";

    private final static String CLASS_NAME          = "TelosysRPL";

    //--- Default wrapper package
    private static String       $sXmlWrapperPackage = null;

    //--- Default suffix (for the Wrapper class name)
    private static String       $sXmlWrapperSuffix  = null;

    //-----------------------------------------------------------------------------
    private static void checkParserClass() // v 1.0.2
    {
        String sClassName = Telosys.getSAXParserClassName();
        Telosys.info(CLASS_NAME + " : XML SAX parser class name = '" + sClassName + "' ");
        try
        {
            Class.forName(sClassName);
            Telosys.info(CLASS_NAME + " : class '" + sClassName + "' successfully loaded. ");
            
        } catch (ClassNotFoundException e)
        {
            Telosys.error(CLASS_NAME +" : cannot load XML SAX parser class ( ClassNotFoundException ) !");
            e.printStackTrace();
        }
    }
    //-----------------------------------------------------------------------------
    private static boolean initWithProperties(Properties prop)
    {
        boolean bRet = true;
        String s = null;

        //--- XML wrapper package
        s = prop.getProperty(XML_WRAPPER_PACKAGE);
        if (s == null)
        {
            Telosys.info(CLASS_NAME + " : property '" + XML_WRAPPER_PACKAGE + "' not found ");
            bRet = false;
        }
        else
        {
            $sXmlWrapperPackage = s.trim();
            Telosys.info(CLASS_NAME + " : property '" + XML_WRAPPER_PACKAGE + "' = '" + $sXmlWrapperPackage + "'");
        }

        //--- XML wrapper suffix
        s = prop.getProperty(XML_WRAPPER_SUFFIX);
        if (s == null)
        {
            Telosys.info(CLASS_NAME + " : property '" + XML_WRAPPER_SUFFIX + "'  not found ");
            bRet = false;
        }
        else
        {
            $sXmlWrapperSuffix = s.trim();
            Telosys.info(CLASS_NAME + " : property '" + XML_WRAPPER_SUFFIX + "' = '" + $sXmlWrapperSuffix + "'");
        }

        return bRet;
    }

    //-----------------------------------------------------------------------------
    // PUBLIC METHODS
    //-----------------------------------------------------------------------------
    /**
     * R.P.L. initialization
     * 
     * @return
     */
    public static boolean init()
    {
        Telosys.info("TelosysRPL : INIT ...");
        
        checkParserClass(); // v 1.0.2
        
        //--- Get the Telosys properties ...
        Properties prop = Telosys.getProperties();
        if (prop != null)
        {
            return initWithProperties(prop);
        }
        else
        {
            Telosys.error(CLASS_NAME +".init() : cannot get Telosys properties !");
            return false;
        }
    }

    //-----------------------------------------------------------------------------
    // PUBLIC GETTERS
    //-----------------------------------------------------------------------------
    public static String getXmlWrapperPackage()
    {
        return $sXmlWrapperPackage;
    }

    public static String getXmlWrapperSuffix()
    {
        return $sXmlWrapperSuffix;
    }

}