package org.objectweb.telosys.rpl.xml;

import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenRequestView;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.core.ScreenDataElement;
import org.objectweb.telosys.screen.core.ScreenDataElements;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.util.XmlUtil;

/**
 * @author Laurent GUERIN
 *
 */
public class XmlOutput 
{
    private static final TelosysClassLogger log = new TelosysClassLogger(XmlOutput.class);
    
	//--------------------------------------------------------------------------------------------------
	// Mthodes de vrification des paramtres
	//--------------------------------------------------------------------------------------------------
	private static boolean checkPrintWriter(PrintWriter out)
	{
		if ( out == null )
		{
			log.error( "out parameter is null !" );
			return false;
		}
		else
		{
			return true ;
		}
	}
	//--------------------------------------------------------------------------------------------------
    

	//--------------------------------------------------------------------------------------------------
	private static void beginningOfXml ( ScreenResponse resp, PrintWriter out)
	{
	    log.trace ( "beginningOfXml()" );
		if ( checkPrintWriter(out) == false ) return ;
		
//		out.println("<?xml  version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
//		out.println("<?xml  version=\"1.0\" encoding=\"utf-8\" ?>");
		
		out.print("<response");
		out.print(" screen-name=\"" + resp.getScreenName() + "\"");
		out.print(" screen-id=\"" + resp.getScreenId() + "\"");
		out.print(" action=\"" + resp.getAction() + "\"" );
		out.println(" >");

	}

	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	private static void genReturn ( ScreenResponse resp, PrintWriter out)
	{
	    log.trace ( "genReturn()" );
		int iCount = 0 ;
		if ( checkPrintWriter(out) == false ) return ;

		out.println("<return code=\"" + resp.getReturnCode() + "\" message=\"" + XmlUtil.xmlString( resp.getReturnMessage() ) + "\" >" );		            

		//--- Flag "FOUND" or "NOT-FOUND" (or nothing)
		if ( resp.isFlagNotFound() == true )
		{
			out.println(" <not-found/>" );
		}
		if ( resp.isFlagFound() == true )
		{
			out.println(" <found/>" );
		}
		
		//--- List of exceptions 
		iCount = resp.getNumberOfExceptions() ;
		if ( iCount > 0 )
		{
		    //Exception ex = null ;
		    Throwable ex = null ;		     
			out.println("<exceptions>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        ex = resp.getException(i);
		        if ( ex != null )
		        {
					out.println(" <exception num=\"" + (i+1) +"\" class=\"" + ex.getClass().getName() + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) + "\" />" );		            
		        }
		    }
			out.println("</exceptions>" );
		}

		//--- List of errors 
		iCount = resp.getNumberOfErrors() ;
		if ( iCount > 0 )
		{
		    ScreenError error = null ;
			out.println("<errors>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        error = resp.getError(i);
		        if ( error != null )
		        {
					out.println(" <error num=\"" + (i+1) +"\" code=\"" + error.getCode() + "\" text=\"" + XmlUtil.xmlString( error.getText() ) + "\" />" );		            
		        }
		    }
			out.println("</errors>" );
		}

		//--- List of messages 
		iCount = resp.getNumberOfMessages() ;
		if ( iCount > 0 )
		{
		    ScreenMessage message = null ;
			out.println("<messages>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        message = resp.getMessage(i);
		        if ( message != null )
		        {
					out.println(" <message num=\"" + (i+1) 
					        + "\" type=\"" + message.getType()
					        + "\" code=\"" + message.getCode()
					        + "\" text=\"" + XmlUtil.xmlString( message.getText() ) + "\" />" );		            
		        }
		    }
			out.println("</messages>" );
		}

		out.println("</return>" );
	}
	
	//--------------------------------------------------------------------------------------------------
	// <values> ... </values>
	//--------------------------------------------------------------------------------------------------
	private static void genValues ( ScreenResponse resp, PrintWriter out)
	{
	    log.trace ( "genValues()" );
		if ( checkPrintWriter(out) == false ) return ;
		if ( resp.getNumberOfValues() > 0 )
		{
		    String sValueName = null ;
			Enumeration eNames = resp.getValueNames();
			if ( eNames != null )
			{
				out.println("<values>" );
			    while ( eNames.hasMoreElements() )
			    {
			        sValueName = (String) eNames.nextElement() ;
					out.println("<value " + sValueName + "=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
			    }
				out.println("</values>" );			    
			}
		}
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates a tag in the <<data>> section for each element of the given elements list
	 * @param elements
	 * @param out
	 * @param wrapperProvider
	 * @throws TelosysException
	 */
	//private static void genDataElements ( ScreenDataElements elements, PrintWriter out, IXmlWrapperProvider wrapperProvider) throws TelosysException
	private static void genDataElements ( ScreenDataElements elements, PrintWriter out) throws TelosysException
	{
	    log.trace ( "genDataElements()" );

	    IXmlWrapperProvider wrapperProvider = Telosys.getXmlWrapperProvider() ;
        if (wrapperProvider == null)
		{
            throw new TelosysRuntimeException("toXml : cannot get XmlWrapperProvider ");
		}

	    
		if ( checkPrintWriter(out) == false ) return ;
		//--- Get the elements list 
        if ( elements != null ) 
        {
            //--- Generate a tag in the "<data> section" for each element 
            if ( elements.size() > 0 )
            {
                ScreenDataElement elem = null ;
        		out.println("<data>" );
        		Iterator iter = elements.iterator() ;
        		while ( iter.hasNext() )
        		{
        		    elem = (ScreenDataElement) iter.next();
        		    if ( elem != null )
        		    {
//        		        Renderer renderer = elem.getRenderer();
//        		        if ( renderer == null )
//        		        {
//        		            //--- No renderer => generate DATA
//            		        elem.toXmlData(out, wrapperProvider);        		            
//        		        }
        		        elem.toXmlData(out, wrapperProvider);        		            
        		    }
        		}
        		out.println("</data>" );         
            }
        }
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void genDataVoid ( PrintWriter out ) throws TelosysException
	{
	    log.trace ( "genDataVoid()" );
		if ( checkPrintWriter(out) == false ) return ;
		out.println("<data void=\"true\" />" );
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void genViews ( ScreenRequestView[] views, PrintWriter out, HttpServletRequest httpRequest, HttpServletResponse httpResponse) //throws TelosysException
	{
	    log.trace ( "genViews()" );
		if ( checkPrintWriter(out) == false ) return ;

		if ( views != null )
		{
		    if ( views.length > 0 )
		    {		        
	    		out.println("<view>" );
	    		//--- For each VIEW : generate the view using the associated renderer
				for ( int i=0 ; i < views.length ; i++ )
				{
				    ScreenRequestView view = views[i];
			        if ( view != null )
			        {
			            String sName = view.getName() ;
			            String sRenderer = view.getRenderer() ;
			    		out.println("<"+ sName + " renderer=\"" + sRenderer + "\" >");    		
			    		out.println("<![CDATA[");    	
			    		//--- Render the view content 
			    		view.generate(httpRequest, httpResponse); 
	    	    		out.println("]]>");    		
			    		out.println("</"+ sName +">");    		
			        }			    
				}		    
	    		out.println("</view>" );         
		    }
		}
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void endOfXml ( PrintWriter out ) 
	{
	    log.trace ( "endOfXml()" );
		if ( checkPrintWriter(out) == false ) return ;
		out.println("</response>");
	}
	//--------------------------------------------------------------------------------------------------
    
    /**
     * Generates the XML response flow for the current ACTION REQUEST
     * @param screenRequest
     * @param screenResponse
     * @param out
     * @param httpRequest
     * @param httpResponse
     * @throws TelosysException
     */
    public final static void generate ( ScreenRequest screenRequest, ScreenResponse screenResponse, PrintWriter out, HttpServletRequest httpRequest, HttpServletResponse httpResponse ) throws TelosysException
    {
        log.trace ( "generate()" );
//        IXmlWrapperProvider wrapperProvider = null ;
//        try
//        {
//            //--- Get the wrapper provider (throws Exception if not found)
//            wrapperProvider = XmlWrapperManager.getXmlWrapperProvider();
//        } catch (TelosysException e)
//        {
//            throw new TelosysRuntimeException("toXml : cannot get XmlWrapperProvider ");
//        }
        
        beginningOfXml ( screenResponse, out);
        genReturn ( screenResponse, out);
        genValues ( screenResponse, out);
        
        //--- If the response status is OK : generate ELEMENTS and VIEWS (if any) 
        if ( screenResponse.getReturnCode() == ScreenResponse.ACTION_OK )
        {            
            if ( screenRequest.getFlagGetData() == true )
            {                
		        //--- Generate data elements 
	            ScreenDataElements elements = screenResponse.getDataElements();             
		        if ( elements != null )
		        {
//		            genDataElements ( elements, out, wrapperProvider);
		            genDataElements ( elements, out);
		        }
		        else
		        {
		            // No element 
		            genDataVoid (out);
		        }	        
		        //--- Generate views  ( NEW )
		        ScreenRequestView[] views = screenRequest.getViews() ; 
		        if ( views != null )
		        {
		            genViews(views, out, httpRequest, httpResponse);
		        }
            }
            else
            {
	            genDataVoid (out); // get="false" => Do not return elements               
            }
        }
        
        endOfXml (out);
        out.flush();
    }
    
	//--------------------------------------------------------------------------------------------------
    /**
     * Generates the XML response flow for an ACTION ERROR
     * @param response
     * @param out
     */
    public final static void generateError ( ScreenResponse response, PrintWriter out ) 
    {
        beginningOfXml ( response, out);
        genReturn ( response, out);        
        endOfXml (out);
        out.flush();        
    }
}
