package org.objectweb.telosys.screen.core;

import java.io.Serializable;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * ScreenData interface
 * 
 * @author Laurent GUERIN
 *  
 */
public interface ScreenData extends Serializable
{
    /**
     * Returns true if the given element name is defined in the current ScreenData.
     * @param sElementName : the element name
     * @return : true if defined, else false
     */
    public boolean isElementDefined(String sElementName);

    /**
     * Sets the new element instance for the given element name
     * @param sElementName : the element name
     * @param oElementInstance : the element value to set ( instance )
     * @throws TelosysRuntimeException if the element is not defined in the current screen data
     */
    public void setElement(String sElementName, Object oElementInstance);

    /**
     * Returns the element instance referenced by the given name
     * @param sElementName : the element name
     * @return : the element value ( instance )
     * @throws TelosysRuntimeException if the element is not defined in the current screen data
     */
    public Object getElement(String sElementName) ;
    
    /**
     * Returns the class of the element referenced by the given name
     * @param sElementName : the element name
     * @return : the class
     * @throws TelosysRuntimeException if the element is not defined in the current screen data
     */
    public Class getElementClass(String sElementName) ;

    /**
     * Returns the ScreenDataElement object for the given name
     * @param sElementName : the element name
     * @return : the ScreenDataElement instance ( element definition )
     * @throws TelosysRuntimeException if the element is not defined in the current screen data
     */
    public ScreenDataElement getScreenDataElement(String sElementName);

    /**
     * Returns the list of all the elements defined for the ScreenData  
     * @return
     */
    public ScreenDataElements getElements(); 

    /**
     * Clears all the elements instances
     */
    public void clearElements() ;

    /**
     * Clear all the elements instances and reset the "found flag". 
     * Reset the instance in the same state as after a "new instance".
     */
    public void clear() ;
    
    /**
     * Copy all the elements to an other ScreenData instance
     * @param destination : where to copy the elements
     */
    public void copyElementsTo( ScreenData destination) ;

    /**
     * Copy all the elements and the "found flag" to an other ScreenData instance.
     * Creates an image of the initial instance.
     * @param theClone : where to copy ( the clone/image instance ) 
     */
    public void copyTo( ScreenData theClone ) ;

//    /**
//     * Generates the XML flow for all the elements 
//     * @param out
//     */
//    public void toXml(PrintWriter out) ;

    /**
     * Declares the DATA as "found"
     * ( sets the 'found flag' to false )
     */
    public void setFound();

    /**
     * Declares the DATA as "not found"
     * ( sets the 'found flag' to true )
     */
    public void setNotFound();

    /**
     * Returns true is the screen data is considered as found
     * @return
     */
    public boolean isFound();
    
    /**
     * Exposes all the data elements in the request scope
     * @param httpRequest 
     */
    public void exposeData( HttpServletRequest httpRequest ) ;

}

