package org.objectweb.telosys.screen.core;

import java.io.PrintWriter;
import java.io.Serializable;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.common.vo.GenericVOListWriter;
import org.objectweb.telosys.rpl.xml.IXmlWrapper;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenDataElement extends TelosysObject implements Serializable
{
    private String            _sElementName     = null; // serializable (std in Java)

    private Class             _oElementClass    = null; // serializable (std in Java)

    private Object            _oElementInstance = null; // serializable (std in Java)

    //--- The renderer this element is associated with 
    //private Renderer          _oRenderer        = null;

    //--- List element this element is binded with 
    private ScreenDataElement _oListElement     = null; // serializable (current class)

    private void init(String sElementName, Class oElementClass) //throws TelosysException
    {
        if (sElementName != null)
        {
            _sElementName = sElementName;
        }
        else
        {
            throw new TelosysRuntimeException("Cannot create ScreenDataElement : name is null");
        }
        if (oElementClass != null)
        {
            _oElementClass = oElementClass;
        }
        else
        {
            throw new TelosysRuntimeException("Cannot create ScreenDataElement : class is null");
        }
        _oElementInstance = null;
    }

    /**
     * Creates a new element (without instance)
     * 
     * @param sElementName
     * @param oElementClass
     */
    public ScreenDataElement(String sElementName, Class oElementClass)
    {
        super();
        init(sElementName, oElementClass);
    }

//    /**
//     * Creates a new element with an associated renderer (without instance)
//     * 
//     * @param sElementName
//     * @param oElementClass
//     * @param sRendererPath
//     */
//    public ScreenDataElement(String sElementName, Class oElementClass, String sRendererPath)
//    {
//        super();
//        init(sElementName, oElementClass);
//        //--- Creates the renderer associated with this new element
//        _oRenderer = new Renderer(sRendererPath);
//    }

    //    /**
    //     * Creates an new element with initial instance
    //     *
    //     * @param sElementName
    //     * @param oElementClass
    //     * @param oElementInstance
    //     */
    //    public ScreenDataElement(String sElementName, Class oElementClass, Object oElementInstance)
    //    {
    //        super();
    //        init(sElementName, oElementClass);
    //        if (oElementInstance != null)
    //        {
    //            //--- Check class compatibility
    //            if (_oElementClass.isAssignableFrom(oElementInstance.getClass()))
    //            {
    //                _oElementInstance = oElementInstance;
    //            }
    //            else
    //            {
    //                throw new TelosysRuntimeException("Cannot create ScreenDataElement : instance and class are incompatible");
    //            }
    //        }
    //    }

    /**
     * Returns the current element class
     * 
     * @return
     */
    public Class getInstanceClass()
    {
        return _oElementClass;
    }

    /**
     * Returns the current element name
     * 
     * @return
     */
    public String getName()
    {
        return _sElementName;
    }

    /**
     * Returns the current element instance
     * 
     * @return
     */
    public Object getInstance()
    {
        return _oElementInstance;
    }

    /**
     * Set the new element instance ( general use, binding is not apply here )
     * 
     * @param oInstance
     */
    public void setInstance(Object oInstance) //throws TelosysRuntimeException
    {
        trace("setInstance(object) for element '" + getName() + "'" );
        if (oInstance != null)
        {
            if (_oElementClass.isAssignableFrom(oInstance.getClass()))
            {
                _oElementInstance = oInstance;
            }
            else
            {
                throw new TelosysRuntimeException("Cannot set element instance : instance and class are incompatible");
            }
        }
        else
        {
            _oElementInstance = null;
        }
    }

    /**
     * Clear the element instance 
     */
    public void clearInstance() 
    {
        trace("clearInstance() for element '" + getName() + "'" );
        _oElementInstance = null;
    }
    
    /**
     * Apply the binding of the element if any 
     */
    public void applyBinding() // NEW #CURRENT_LIST_ITEM#
    {
        trace("applyBinding() for element '" + getName() + "'" );
        //--- If this element is the current selected item of a list => update the list
        if (_oListElement != null) // Yes it is binded to another ScreenDataElement
        {
            trace("applyBinding() for element '" + getName() + "' : binded to " + _oListElement.getName() );
            Object oElementInstance = _oListElement.getInstance();
            if (oElementInstance != null) // The ScreenDataElement has an instance
            {
                if (oElementInstance instanceof GenericVOList)
                {
                    trace("applyBinding() for element '" + getName() + "' : list.replaceSelectedElement(instance) " );
                    GenericVOList oList = (GenericVOList) oElementInstance;
                    //--- Replaces the current selected element of the list if any
                    oList.replaceSelectedElement(_oElementInstance);
                }
            }
        }
    }

    /**
     * Create a new element instance for this element
     *  
     */
    public void newInstance()
    {
        Object oNewInstance = null;
        try
        {
            //_oElementInstance = _oElementClass.newInstance();
            oNewInstance = _oElementClass.newInstance();
        } catch (InstantiationException ex)
        {
            throw new TelosysRuntimeException("Cannot create element instance : InstantiationException ", ex);
        } catch (IllegalAccessException ex)
        {
            throw new TelosysRuntimeException("Cannot create element instance : IllegalAccessException ", ex);
        }
        setInstance(oNewInstance);
    }

    //    public int getXmlOutputType()
    //    {
    //        if ( _oRenderer != null )
    //        {
    //            return TelosysConst.XML_VIEW ;
    //        }
    //        return TelosysConst.XML_DATA ; // Default standard type
    //    }

    public void toXmlData(PrintWriter out, IXmlWrapperProvider wrapperProvider)
    {
        trace("toXmlData(out, oElementInstance, '" + _sElementName + "' )... ");
        if (out == null)
        {
            throw new TelosysRuntimeException("toXmlData : output stream is null (PrintWriter 'out' argument) ");
        }
        if (wrapperProvider == null)
        {
            throw new TelosysRuntimeException("toXmlData : wrapperProvider is null ( IXmlWrapperProvider argument) ");
        }
        if (_sElementName == null)
        {
            throw new TelosysRuntimeException("toXmlData : no element name (String '_sElementName' attribute) ");
        }
        if (_oElementInstance != null)
        {
            if ( _oElementInstance instanceof GenericVOList )
            {
                GenericVOList volist = (GenericVOList) _oElementInstance ;
                String sItemName = "item" ;
//                Class cl = volist.getElementType();
//                if ( cl != null )
//                {
//                    sItemName = cl.getName();
//                    sItemName.replace('.', '_');
//                }
                GenericVOListWriter.toXml(out, volist, _sElementName, sItemName );
            }
            else
            {
                toXmlEntity(out, wrapperProvider);
            }
        }
        else
        {
            out.println("<!-- Element '" + _sElementName + "' instance is null -->");
            return;
        }
        
    }

	private void toXmlEntity(PrintWriter out, IXmlWrapperProvider wrapperProvider)
	{
        //--- Get the XML Wrapper for the given object
        IXmlWrapper wrapper = null;
        try
        {
            wrapper = wrapperProvider.getWrapper(_oElementClass);
        } catch (TelosysRuntimeException e)
        {
            wrapper = null;
        }
        if (wrapper != null)
        {
            //--- Print the XML tag
            wrapper.toXml(out, _oElementInstance, _sElementName);
        }
        else
        {
            throw new TelosysRuntimeException("ScreenDataElement : toXmlData() : cannot get XmlWrapper for the bean class '" + _oElementClass.getName() + "' ( name = '" + _sElementName + "' ) ");
        }
    }

//    public Renderer getRenderer()
//    {
//        return _oRenderer;
//    }

    public void bindToList(ScreenDataElement oList) // NEW #CURRENT_LIST_ITEM#
    {
        _oListElement = oList;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        String sElementToString = "null";
        String sElementClass    = "null";
        if ( _oElementClass != null)
        {
            sElementClass    = _oElementClass.getName() ;
        }
        if ( _oElementInstance != null)
        {
            sElementToString = _oElementInstance.toString();
            sElementClass = _oElementInstance.getClass().getName();
        }
        
        return _sElementName + " (class:" + sElementClass + ") : " + sElementToString ;
    }
}