package org.objectweb.telosys.screen.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenDataElements extends TelosysObject
{
    private ArrayList _elements = null;

    /**
     * Default constructor
     */
    public ScreenDataElements()
    {
        super();
        _elements = new ArrayList();
    }

    /**
     * Constructor from elements collection
     * @param elements
     */
    public ScreenDataElements(Collection elements)
    {
        super();
        _elements = new ArrayList(elements);        
    }
    
    //-----------------------------------------------------------------------------------------------
    public void add(ScreenDataElement oElement)
    {
        _elements.add(oElement);
    }

    //-----------------------------------------------------------------------------------------------
    public void add( String sElementName, Object oBean)
    {
        if ( sElementName == null )
        {
            throw new TelosysRuntimeException("ScreenDataElements - add() : element name is null ");
        }
        if ( oBean == null )
        {
            throw new TelosysRuntimeException("ScreenDataElements - add() : bean object is null ");
        }
        //--- Element creation
        ScreenDataElement element = new ScreenDataElement( sElementName, oBean.getClass() ) ;
        element.setInstance(oBean);            

        //--- Add the element ...
        _elements.add(element);
    }

    //-----------------------------------------------------------------------------------------------
    public int size()
    {
        return _elements.size();
    }

    //-----------------------------------------------------------------------------------------------
    public ScreenDataElement get(int index)
    {
        return (ScreenDataElement) _elements.get(index);
    }

    //-----------------------------------------------------------------------------------------------
    public Iterator iterator()
    {
        return _elements.iterator();
    }
    
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        StringBuffer sb = new StringBuffer(256);
        sb.append("ScreenDataElements ( " + _elements.size() + " elements ) : ");
        Iterator iter = _elements.iterator();
        if ( iter != null )
        {
            while ( iter.hasNext() )
            {
                ScreenDataElement elt = (ScreenDataElement) iter.next();
                sb.append("\n . " + elt.toString() );                
            }            
        }
        return sb.toString();
    }
}