package org.objectweb.telosys.screen.core;

/**
 * @author Laurent GUERIN
 *
 * Application message to be return and display on the client side 
 *   
 */
public class ScreenMessage
{
    public final static int NO_TYPE     = 0;

    public final static int INFORMATION = 1;

    public final static int WARNING     = 2;

    public final static int ERROR       = 3;

    private int             _iCode      = 0;

    private String          _sText      = null ;

    private int             _iType      = 0;

    /**
     * Build a message without type
     * @param iCode : Message Code
     * @param sText : Message Text
     */
    public ScreenMessage(int iCode, String sText)
    {
        _iCode = iCode;
        _sText = sText;
        _iType = NO_TYPE;
    }

    /**
     * Build a message with all its attributes
     * @param iCode : Message Code
     * @param sText : Message Text
     * @param iType : Message Type
     */
    public ScreenMessage(int iCode, String sText, int iType)
    {
        _iCode = iCode;
        _sText = sText;
        _iType = iType;
    }

    /**
     * Build a message without text and type (only code)
     * @param iCode : Message Code
     */
    public ScreenMessage(int iCode)
    {
        _iCode = iCode;
        _sText = null;
        _iType = NO_TYPE;
    }

    /**
     * Build a message without text (only code and type)
     * @param iCode : Message Code
     * @param iType : Message Type
     */
    public ScreenMessage(int iCode, int iType)
    {
        _iCode = iCode;
        _sText = null;
        _iType = iType;
    }

    /**
     * Build a message with only text (no code and no type)
     * @param sText
     */
    public ScreenMessage(String sText)
    {
        _iCode = 0;
        _sText = sText;
        _iType = NO_TYPE;
    }

    /**
     * Build a message without code (only text and type)
     * @param sText
     * @param iType
     */
    public ScreenMessage(String sText, int iType)
    {
        _iCode = 0;
        _sText = sText;
        _iType = iType;
    }

    public int getCode()
    {
        return _iCode;
    }

    public String getText()
    {
        return _sText;
    }

    public int getType()
    {
        return _iType;
    }
}