package org.objectweb.telosys.screen.core;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * @author Laurent GUERIN
 *  
 */
public class StandardScreenRegistry extends TelosysObject implements IScreenProvider, IScreenDataProvider
{

    //=========================================================================================

    private final HashMap hmRegistry = new HashMap(40);

    //=========================================================================================

    protected final void register(String screenName, Class screenDataClass, Class screenManagerClass, Class screenActionsClass, Class screenTriggersClass, Class screenProceduresClass)
    {
        trace("register screen " + screenName);
        try
        {
            ScreenContextDefinition def = new ScreenContextDefinition(screenName, screenDataClass, screenManagerClass, screenActionsClass, screenTriggersClass, screenProceduresClass);
            hmRegistry.put(screenName, def);
        } catch (TelosysException e)
        {
            error("Cannot register screen " + screenName);
            error(e.getMessage());
        }
    }

    private final ScreenContextDefinition lookup(String sScreenName) throws TelosysException
    {
        ScreenContextDefinition screenDef = (ScreenContextDefinition) hmRegistry.get(sScreenName);
        if (screenDef != null)
        {
            return screenDef;
        }
        else
        {
            throw new TelosysException("Screen " + sScreenName + " not found in the registy ");
        }
    }

    public ScreenContext getScreen( ScreenSession screenSession, String sScreenName, int iScreenId ) throws TelosysException
    {
        trace(" getScreen(" + sScreenName + ")");

        ScreenContextDefinition screenDef = lookup(sScreenName);
        if (screenDef != null)
        {
            return screenDef.createScreenContext(screenSession, iScreenId);
        }
        return null;
    }

    public ScreenData getScreenData(String sScreenName) throws TelosysException
    {
        trace(" getScreenData(" + sScreenName + ")");

        ScreenContextDefinition screenDef = lookup(sScreenName);
        if (screenDef != null)
        {
            return screenDef.createScreenData();
        }
        return null;
    }

    public ScreenActions getScreenActions(String sScreenName) throws TelosysException
    {
        trace(" getScreenActions(" + sScreenName + ")");

        ScreenContextDefinition screenDef = lookup(sScreenName);
        if (screenDef != null)
        {
            return screenDef.createScreenActions();
        }
        return null;
    }
}