package org.objectweb.telosys.screen.env;

import java.util.Hashtable;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.TelosysObject;

/**
 * The mono instance ScreenApplication object, <br>
 * designed to store shared attributes at application level.
 * 
 * @author Laurent GUERIN
 *  
 */
public class ScreenApplication extends TelosysObject
{
    // private transient ServletContext _servletContext = null; // v 1.0.0

    //--- Open storage space (to store anything)
    private Hashtable      _htAttributes   = new Hashtable();

    //---------------------------------------------------------------------------
// v 1.0.0
//    /**
//     * Constructor 
//     * @param context
//     */
//    public ScreenApplication(ServletContext context)
//    {
//        _servletContext = context;
//    }

    public ScreenApplication() // v 1.0.0
	{
        super();
        info("ScreenApplication object created.");
	}

    /**
     * Return the ServletContext (WebApp context) linked with the ScreenApplication
     * @return
     * @deprecated use ScreenApplicationManager.getServletContext() instead
     */
    public ServletContext getServletContext()
    {
        // return _servletContext;
        return ScreenApplicationManager.getServletContext() ; // v 1.0.0
    }

    //---------------------------------------------------------------------------
    // Screen Application attributes
    //---------------------------------------------------------------------------
    /**
     * Sets an object as a 'screen application attribute', associated with a unique name <br>
     * If name is null : do nothing <br>
     * If value is null : remove the attribute associated with the given name
     * @param name : a string specifying the name of the object 
     * @param value : the value  
     * @return : the previous value of the specified key in this hashtable, or null 
	 have one
     */
    public Object setAttribute(String name, Object value)
    {
        //--- Avoid NullPointerException if key is null
        //if (name != null && value != null)
        if ( name != null ) // v 1.0.0
        {
            if ( value != null ) // v 1.0.0
            {
                return _htAttributes.put(name, value);
            }
            else
            {
                return removeAttribute(name); // v 1.0.0
            }
        }
        else
        {
            error("setAttribute(name,value) : name is null !");
            return null;
        }
    }

    /**
     * Gets an object from the 'screen application attributes' using the given name <br>
     * If name is null : return null
     * @param name : a string specifying the name of the object 
     * @return : value instance ( or null if key is null or not found ) 
     */
    public Object getAttribute(String name)
    {
        //--- Avoid NullPointerException if key is null
        if (name != null)
        {
            return _htAttributes.get(name);
        }
        else
        {
            error("getAttribute(name) : name is null !");
            return null;
        }
    }

    /**
     * Removes an object from the 'screen application attributes' <br>
     * If name is null : do nothing and return null
     * @param name : a string specifying the name of the object 
     * @return : the value removed, or null if not found
     */
    public Object removeAttribute(String name)
    {
        //--- Avoid NullPointerException if key is null
        if (name != null)
        {
            return _htAttributes.remove(name);
        }
        else
        {
            error("removeAttribute(name) : name is null !");
            return null;
        }
    }

    //---------------------------------------------------------------------------
    
    /**
     * Close the ScreenApplication (clean all the references held)
     */
    public void close()
    {
        _htAttributes = null;
        // _servletContext = null;  // v 1.0.0
    }
    //---------------------------------------------------------------------------
}