package org.objectweb.telosys.screen.env;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.Telosys;

/**
 * Class for "ScreenApplication" object management<br>
 * The "ScreenApplication" is a singleton always accessible, shared by all the users  
 * 
 * @author Laurent GUERIN
 *
 */
public final class ScreenApplicationManager
{
    private static ScreenApplication $screenApplication = new ScreenApplication() ; // v 1.0.0 
    
    // private static final String TELOSYS_APPLICATION = "TELOSYS_APPLICATION" ;

    /**
     * No constructor for this class
     */
    private ScreenApplicationManager()
    {
    }
    
    //---------------------------------------------------------------------------
    /**
     * Returns the current ScreenApplication object<br>
     * @return the ScreenApplication ( never null )
     * @since 1.0.0
     */
    public static ScreenApplication getScreenApplication() // v 1.0.0
    {
        return $screenApplication ;
    }
    
    /**
     * Returns the current ScreenApplication object<br>
     * @return
     * @since 1.0.0
     */
    public static ServletContext getServletContext() // v 1.0.0
    {
        return Telosys.getServletContext() ; 
    }
    
    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenApplication object <br>
     * Since v 1.0.0 the ScreenApplication is not stored in the ServletContext<br>
     * ( same as getScreenApplication() without parameter )
     * @param servletContext : not used since v 1.0.0
     * @return the ScreenApplication instance
     * @deprecated use getScreenApplication() instead
     */
    public static ScreenApplication getScreenApplication( ServletContext servletContext )
    {
        return $screenApplication ; // v 1.0.0
//        if ( servletContext != null )
//        {
//    		Object obj = servletContext.getAttribute( TELOSYS_APPLICATION ) ; 
//    		if ( obj != null )
//    		{
//    			if ( obj instanceof ScreenApplication ) 
//    			{
//    				return (ScreenApplication) obj ;
//    			}
//    			else 
//    			{
//    	            Telosys.error("getScreenApplication: ServletContext corrupted : attribute is not a ScreenApplication instance !");
//    			}
//    		}                
//        }
//        else
//        {
//            Telosys.error("getScreenApplication : ServletContext parameter is null");
//        }
//        return null ;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenApplication object <br>
     * Since v 1.0.0 the ScreenApplication is not stored in the ServletContext<br>
     * ( same as getScreenApplication() without parameter )
     * 
     * @param httpSession : not used since v 1.0.0
     * @return the ScreenApplication object 
     * @deprecated use getScreenApplication() instead
     */
    public static ScreenApplication getScreenApplication( HttpSession httpSession )
    {
        return $screenApplication ; // v 1.0.0
//        if ( httpSession != null )
//        {
//            ServletContext servletContext = httpSession.getServletContext() ;
//            if ( servletContext != null )
//            {            
//                return getScreenApplication( servletContext );
//            }
//            else
//            {
//                Telosys.error("getScreenApplication : Cannot get ServletContext from HTTP session");
//            }            
//        }
//        else
//        {
//            Telosys.error("getScreenApplication : HttpSession parameter is null");
//        }
//        return null ;
    }

//    //---------------------------------------------------------------------------
//    /**
//     * Open and return the Telosys "ScreenApplication" object <br> 
//     * ( create the object and put it in the standard ServletContext object )<br>
//     * No effect if the "Screen Application" is already open ( just return the existing object )
//     * @param servletContext
//     * @return the "ScreenApplication" instance 
//     */
//    public static ScreenApplication openScreenApplication( ServletContext servletContext )
//    {
//        if (servletContext != null)
//        {
////            ScreenApplication screenApplication = getScreenApplication( servletContext ) ;
////            
////            //--- If not yet created ...
////            if ( screenApplication == null )
////            {
////                //--- 1) Create a screen application object 
////                screenApplication = new ScreenApplication(servletContext);   
////                
////                //--- 2) Store the screen application object in the servlet context
////                servletContext.setAttribute( TELOSYS_APPLICATION, screenApplication ) ;                        
////            }
////    		return screenApplication ;
//
//// v 1.0.0
//    		//--- If not yet created ...
//            if ( $screenApplication == null )
//            {
//                //--- Create the screen application object and keep it 
//                $screenApplication = new ScreenApplication(servletContext); // v 1.0.0   
//            }
//    		return $screenApplication ;
//        }
//        else
//        {
//            Telosys.error("openScreenApplication : ServletContext parameter is null");
//            return null ;
//        }
//    }
    
//    //---------------------------------------------------------------------------
//    /**
//     * Open and return the Telosys "ScreenApplication" object <br> 
//     * ( create the object and put it in the standard ServletContext object )<br>
//     * No effect if the "Screen Application" is already open ( just return the existing object )
//     * @param httpSession
//     * @return the "ScreenApplication" instance
//     */
//    public static ScreenApplication openScreenApplication( HttpSession httpSession )
//    {
//        if (httpSession != null)
//        {
//            //--- Get the WebApp context
//            ServletContext servletContext = httpSession.getServletContext() ;
//            
//            if ( servletContext != null )
//            {
//                return openScreenApplication( servletContext );
//            }
//            else
//            {
//                Telosys.error("openScreenApplication : Cannot get ServletContext from HTTP session");
//            }
//        }
//        else
//        {
//            Telosys.error("openScreenApplication : HttpSession parameter is null");
//        }
//		return null ;
//    }
    
}
