package org.objectweb.telosys.service;

import java.io.PrintWriter;
import java.util.Enumeration;

import javax.servlet.ServletException;

import org.objectweb.telosys.common.JavaObjectWriter;
import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.data.DataSet;
import org.objectweb.telosys.common.data.DataSetWriter;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.common.vo.GenericVOListWriter;
import org.objectweb.telosys.rpl.xml.IXmlWrapper;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.util.XmlUtil;

/**
 * @author Laurent GUERIN
 *
 */
public class XmlServiceOutput 
{
    private static final TelosysClassLogger log = new TelosysClassLogger(XmlServiceOutput.class);
    
    private static final String RESULT_TAG_NAME = "service-result" ;
    
	//--------------------------------------------------------------------------------------------------
	// Mthodes de vrification des paramtres
	//--------------------------------------------------------------------------------------------------
	private static boolean checkPrintWriter(PrintWriter out)
	{
		if ( out == null )
		{
			log.error( "out parameter is null !" );
			return false;
		}
		else
		{
			return true ;
		}
	}
	//--------------------------------------------------------------------------------------------------
    

	//--------------------------------------------------------------------------------------------------
	private static void beginningOfXml ( ServiceResponse resp, PrintWriter out)
	{
	    log.trace ( "beginningOfXml()" );
		if ( checkPrintWriter(out) == false ) return ;
		
		out.println("<?xml  version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
		
		out.print("<response");
		out.print(" service-name=\"" + resp.getServiceName() + "\"");
		out.println(" >");

	}

	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	private static void genReturn ( ServiceResponse resp, PrintWriter out)
	{
	    log.trace ( "genReturn()" );
		int iCount = 0 ;
		if ( checkPrintWriter(out) == false ) return ;

		out.println("<return code=\"" + resp.getReturnCode() + "\" message=\"" + XmlUtil.xmlString( resp.getReturnMessage() ) + "\" >" );		            
		
		//--- List of exceptions 
		iCount = resp.getNumberOfExceptions() ;
		if ( iCount > 0 )
		{
		    Exception ex = null ;
			out.println("<exceptions>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        ex = resp.getException(i);
		        if ( ex != null )
		        {
					out.println(" <exception num=\"" + (i+1) +"\" class=\"" + ex.getClass().getName() + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) + "\" />" );		            
		        }
		    }
			out.println("</exceptions>" );
		}

		//--- List of errors 
		iCount = resp.getNumberOfErrors() ;
		if ( iCount > 0 )
		{
		    ScreenError error = null ;
			out.println("<errors>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        error = resp.getError(i);
		        if ( error != null )
		        {
					out.println(" <error num=\"" + (i+1) +"\" code=\"" + error.getCode() + "\" text=\"" + XmlUtil.xmlString( error.getText() ) + "\" />" );		            
		        }
		    }
			out.println("</errors>" );
		}

		//--- List of messages 
		iCount = resp.getNumberOfMessages() ;
		if ( iCount > 0 )
		{
		    ScreenMessage message = null ;
			out.println("<messages>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        message = resp.getMessage(i);
		        if ( message != null )
		        {
					out.println(" <message num=\"" + (i+1) 
					        + "\" type=\"" + message.getType()
					        + "\" code=\"" + message.getCode()
					        + "\" text=\"" + XmlUtil.xmlString( message.getText() ) + "\" />" );		            
		        }
		    }
			out.println("</messages>" );
		}

		out.println("</return>" );
	}
	
	//--------------------------------------------------------------------------------------------------
	// <values> ... </values>
	//--------------------------------------------------------------------------------------------------
	private static void genValues ( ServiceResponse resp, PrintWriter out)
	{
	    log.trace ( "genValues()" );
		if ( checkPrintWriter(out) == false ) return ;
		if ( resp.getNumberOfValues() > 0 )
		{
		    String sValueName = null ;
			Enumeration eNames = resp.getValueNames();
			if ( eNames != null )
			{
				out.println("<values>" );
			    while ( eNames.hasMoreElements() )
			    {
			        sValueName = (String) eNames.nextElement() ;
					//OLD: out.println(" <" + sValueName + " value=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
					out.println("<value " + sValueName + "=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
			    }
				out.println("</values>" );			    
			}
		}
	}

	private static void printError ( PrintWriter out, Throwable ex)
	{
	    if ( ex != null ) // v 1.0.2
	    {	        
	        out.println("<b>Exception</b> : " + ex.getClass().getName() + " ( " + ex.getMessage() + " ) <br>");
            if ( ex instanceof ServletException )
            {
                //--- ServletException : get "Root Cause"
                ServletException ex2 = (ServletException) ex ;
                Throwable cause = ex2.getRootCause();
                if ( cause != null )
                {
	                out.println("<b>Root Cause</b> : <br>");
	                printError ( out, cause );                
                }
                else
                {
	                out.println("<b>No root cause</b><br>");
                }
            }
            else
            {
                //--- Other Exception : get "Standard Cause"
                Throwable cause = ex.getCause();
                if ( cause != null )
                {
	                out.println("<b>Cause</b> : <br>");
	                printError ( out, cause );                
                }
                else
                {
	                out.println("<b>No cause</b><br>");
                }
            }
	    }
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void genView ( ServiceResponse response, PrintWriter out, ServiceRenderer renderer)// throws TelosysException
	{
		out.println("<view>");
		out.println("<service-view>");
		out.println("<![CDATA[");    		
        try
        {
            renderer.render(response);
        } catch (Throwable e)
        {
            out.println("<b>ERROR</b> in renderer <b>'" + renderer.getPath() + "'</b><br>");
            printError(out, e);
//            out.println("<b>Exception</b> : " + e.getClass().getName() + "<br>");
//            out.println(" . Message : " + e.getMessage() + "<br>" );
//            out.println(" . Cause : " + e.getCause() + "<br>" );
        }
        finally {
    		out.println("]]>");    		
    		out.println("</service-view>");
    		out.println("</view>");                            
        }
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void genData ( ServiceResponse serviceResponse, PrintWriter out) //throws TelosysException
	{
	    log.trace ( "genData()" );
		if ( checkPrintWriter(out) == false ) return ;
		
		out.println("<data>" );
	    Object oResult = serviceResponse.getResult() ;
	    if ( oResult != null )
	    {
	        // Try to generate a generic object ( VOList, DataSet )
	        boolean bGenerated = genGenericObject ( oResult, out) ;
	        if ( ! bGenerated )
	        {
		        // Not yet generated => Try to generate as a standard Java object 
	            bGenerated = genJavaObject ( oResult, out) ;
	        }
	        if ( ! bGenerated )
	        {
		        // Not yet generated => Try to generate as a specific object ( with specific mapper )
	            genSpecificObject ( oResult, out) ;
	        }
	    }
	    else
	    {
    		out.println("<!-- No result object -->" );
	    }
		out.println("</data>" );         
	}

	//--------------------------------------------------------------------------------------------------
	private static boolean genJavaObject ( Object oResult, PrintWriter out) 
	{
	    String sClassName = oResult.getClass().getName() ;
	    //if ( sClassName.startsWith("java.lang") || sClassName.startsWith("java.math")  )
		if ( sClassName.startsWith("java.") ) // java.lang, java.math, java.util 
	    {
	        JavaObjectWriter.toXml(out, oResult, RESULT_TAG_NAME);
	        return true ;
	    }
	    return false ;
	}
	
	//--------------------------------------------------------------------------------------------------
	private static boolean genGenericObject ( Object oResult, PrintWriter out) 
	{
	    if ( oResult instanceof GenericVOList )
	    {
	        GenericVOListWriter.toXml(out, (GenericVOList) oResult, RESULT_TAG_NAME, "item" );
	        return true ;
	    }
	    else if ( oResult instanceof DataSet )
	    {
	        DataSetWriter.toXml(out, (DataSet) oResult, RESULT_TAG_NAME );
	        return true ;
	    }
	    return false ;
	}
	
	//--------------------------------------------------------------------------------------------------
	private static boolean genSpecificObject ( Object oResult, PrintWriter out) //throws TelosysException
	{
	    boolean bRet = false ;
	    IXmlWrapperProvider wrapperProvider = Telosys.getXmlWrapperProvider() ;
        if (wrapperProvider == null)
		{
    		out.println("<!-- ERROR : cannot get XmlWrapperProvider -->" );
            //throw new TelosysRuntimeException("toXml : cannot get XmlWrapperProvider ");
		}
        else
        {
	        //--- Get the XML Wrapper for the given object
            IXmlWrapper wrapper = null;
            try
            {
                wrapper = wrapperProvider.getWrapper(oResult);
            } catch (TelosysRuntimeException e)
            {
                wrapper = null;
            }
	        if (wrapper != null)
	        {
	            //--- Print the XML tag
	            wrapper.toXml(out, oResult, RESULT_TAG_NAME);
	            bRet = true ;
	        }
	        else
	        {
	    		out.println("<!-- Cannot get mapper for class " + oResult.getClass().getName() + " -->" );
	        }
        }
        return bRet ;
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void endOfXml ( PrintWriter out ) 
	{
	    log.trace ( "endOfXml()" );
		if ( checkPrintWriter(out) == false ) return ;
		out.println("</response>");
	}
	//--------------------------------------------------------------------------------------------------
    
    public final static void generate ( ServiceResponse response, PrintWriter out, ServiceRenderer renderer ) throws TelosysException
    {
        log.trace ( "generate()" );
        beginningOfXml ( response, out);
        genReturn ( response, out);
        genValues ( response, out);
        
        if ( renderer != null )
        {
            genView(response, out, renderer );
        }
        else
        {
            genData(response, out );
        }
        
        endOfXml (out);
        out.flush();
    }
}
