package org.objectweb.telosys.service.search;

import java.util.Map;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.View;
import org.objectweb.telosys.dal.dataset.SQLDataSet;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.service.ServiceResponse;

/**
 * @author Laurent GUERIN
 *  
 */
public class SQLSearch extends SQLSearchAncestor
{
    public View execute(ScreenSession screenSession, ServiceRequest serviceRequest, ServiceResponse serviceResponse) throws TelosysException
    {
        //--- Call the "BEFORE" hook method
        before( screenSession, serviceRequest, serviceResponse);
        
        Map requestParameters = serviceRequest.getParameterMap();
        String strParams[] = null;

        if (_bDynamicParameters)
        {
            //--- Set the useful criterias
            setCriterias(requestParameters);
            //--- Change the SQLDataSet definition
            _dsDefinition.alterWhere(_criterias);
            strParams = _criterias.getDynamicParameters();
            trace("WHERE  = " + _dsDefinition.getSqlWhere());
            trace("TYPES  = " + _dsDefinition.getParametersTypes());
            trace("PARAMS length = " + strParams.length);
            trace("PARAMS = " + strParams.toString());
        }
        else
        {
            //--- Get the SQL parameters values (as String)
            strParams = getStringParams(requestParameters);
            if (strParams.length < _dsDefinition.getNbParams())
            {
                throw new TelosysException(_dsDefinition.getNbParams() + " parameters expected (" + strParams.length + " received)");
            }
        }

        //--- Set the "MAX" limit 
        
        // if (_iMaxi <= 0)
        if ( maxRowsIsNotSet() )
        {
            //error("Maxi <= 0 : set the default value !");
            //_iMaxi = getDefaultMaxRows() ; // by default
            setDefaultMaxRows();
        }
        int iMaxi = getMaxRows();
        
        //_dsDefinition.setLinesPerPage(iMaxi); // useless : loadPage() not used
        _dsDefinition.setMaxRows(iMaxi);

        //--- DataSet creation
        SQLDataSet ds = null;
        try
        {
            ds = new SQLDataSet(_dsDefinition, strParams);
        } catch (TelosysException e)
        {
            //serviceResponse.addError(e);
            serviceResponse.addException(e);
            serviceResponse.addMessage(new ScreenMessage("Cannot create SQLDataSet ", ScreenMessage.ERROR));
            throw e;
        }
        trace("DataSet created");

        try
        {
            //--- Load the dataset ( limited to  )
            trace(" DataSet.load() , maxi = " + iMaxi);
            int iRet = ds.load();
            trace(" DataSet.load() : return code = " + iRet);
            trace(" DataSet.getRowCount() : " + ds.getRowCount() );
            trace(" DataSet.endOfResultSet() : " + ds.endOfResultSet() );

            boolean bMoreRows = false ;
            if (ds.getRowCount() == 0)
            {
                //String sMsg = "No row";
                serviceResponse.addMessage(_messages.noRow());
            }
            else
            {
                if (ds.endOfResultSet())
                {
                    //--- The end of the ResultSet was reached : all the lines are loaded in the DataSet
//                    String sMsg = ds.getRowCount() + " rows ";
//                    serviceResponse.addMessage(new ScreenMessage(sMsg, ScreenMessage.INFORMATION));
                    serviceResponse.addMessage(_messages.numberOfRows(ds.getRowCount()));
                }
                else
                {
                    //--- The end of the ResultSet was not reached : there's more lines than DataSet capacity
//                    String sMsg = "Too much rows ( more than " + _iMaxi + " )";
//                    serviceResponse.addMessage(new ScreenMessage(sMsg, ScreenMessage.WARNING));
                    serviceResponse.addMessage(_messages.tooMuchRows(iMaxi) );
                    bMoreRows = true ;
                }
                //--- Store the service result in the response (for the renderer)
                serviceResponse.setResult(ds);
            }

            //--- Set response values 
            serviceResponse.setValue("rowcount", ds.getRowCount());
            serviceResponse.setValue("max", iMaxi);
            serviceResponse.setValue("morerows", ""+bMoreRows);
                     
        } catch (Exception ex)
        {
            if (ex.getCause() != null)
            {
                serviceResponse.addException(ex.getCause());
            }
            serviceResponse.addException(ex);
            throw new TelosysException("DataSet cannot execute SQL operation ", ex);
        }
        
        //--- Call the "AFTER" hook method
        after( screenSession, serviceRequest, serviceResponse);

        return null ;
    }
}