package org.objectweb.telosys.service.search;

import java.util.Map;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.View;
import org.objectweb.telosys.dal.dataset.SQLDataSet;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.service.ServiceResponse;

/**
 * @author Laurent GUERIN
 *  
 */
public class SQLSearchWithCount extends SQLSearchAncestor
{
    //public View execute(ScreenSession screenSession, Map requestParameters, ServiceResponse serviceResponse) throws TelosysException
    public View execute(ScreenSession screenSession, ServiceRequest serviceRequest, ServiceResponse serviceResponse) throws TelosysException
    {
        //--- Call the "BEFORE" hook method
        before( screenSession, serviceRequest, serviceResponse);
        
        Map requestParameters = serviceRequest.getParameterMap();        
        String strParams[] = null;

        if (_bDynamicParameters)
        {
            //--- Set the useful criterias
            setCriterias(requestParameters);
            //--- Change the SQLDataSet definition
            _dsDefinition.alterWhere(_criterias);
            strParams = _criterias.getDynamicParameters();
            trace("WHERE  = " + _dsDefinition.getSqlWhere());
            trace("TYPES  = " + _dsDefinition.getParametersTypes());
            trace("PARAMS length = " + strParams.length);
            trace("PARAMS = " + strParams.toString());
        }
        else
        {
            //--- Get the SQL parameters values (as String)
            strParams = getStringParams(requestParameters);
            if (strParams.length < _dsDefinition.getNbParams())
            {
                throw new TelosysException(_dsDefinition.getNbParams() + " parameters expected (" + strParams.length + " received)");
            }
        }

//        if ( maxRowsIsSet() )
//        //if (_iMaxi > 0)
//        {
//            _dsDefinition.setLinesPerPage( getMaxRows() );
//        }
        if ( maxRowsIsNotSet() )
        {
            //error("Maxi <= 0 : set the default value !");
            //_iMaxi = getDefaultMaxRows() ; // by default
            setDefaultMaxRows();
        }
        int iMaxi = getMaxRows();
        
        //_dsDefinition.setLinesPerPage(iMaxi); // useless : loadPage() not used
        _dsDefinition.setMaxRows(iMaxi);


        //--- DataSet creation
        SQLDataSet ds = null;
        try
        {
            ds = new SQLDataSet(_dsDefinition, strParams);
        } catch (TelosysException e)
        {
            //serviceResponse.addError(e);
            serviceResponse.addException(e);
            serviceResponse.addMessage(new ScreenMessage("Cannot create SQLDataSet ", ScreenMessage.ERROR));
            throw e;
        }
        trace("DataSet created");

        try
        {
            //int iMaxi = getMaxRows();
            
            //--- Count the number of rows
            trace(" DataSet.count");
            int iCount = ds.count();

            if (iCount == 0)
            {
                //--- No row => no load
                serviceResponse.addMessage(_messages.noRow());
            }
            else
            {
                //--- Load the rows ...
//                if ( maxRowsIsSet() )
//                //if (_iMaxi > 0)
//                {
//                    //--- Limited load ( only one page )
//                    trace(" Load only one page : DataSet.loadPage(1) , maxi = " + getMaxRows() );
//                    int iRet = ds.loadPage(1);
//                    trace(" DataSet.loadPage(1) : return code = " + iRet);
//                }
//                else
//                {
//                    //--- Not limited load
//                    trace(" Load all : DataSet.load() , maxi is not set ");
//                    int iRet = ds.load();
//                    trace(" DataSet.load() : return code = " + iRet);
//                }
                trace(" DataSet.load() , maxi = " + iMaxi);
                int iRet = ds.load();
                trace(" DataSet.load() : return code = " + iRet);
                trace(" DataSet.getRowCount() : " + ds.getRowCount() );
                trace(" DataSet.endOfResultSet() : " + ds.endOfResultSet() );

                
                if (ds.getRowCount() == 0) // It would never happened ( because _iCount > 0 )
                {
                    // Hum... something has changed in the database between the 2 requests !!!
                    serviceResponse.addMessage(_messages.noRow());                    
                }
                else
                {
                    //if (_iMaxi > 0 && iCount > _iMaxi)
                    if ( iCount > iMaxi ) 
                    {
                        serviceResponse.addMessage(_messages.tooMuchRows(iCount, iMaxi) );                        
                    }
                    else
                    {
                        serviceResponse.addMessage(_messages.numberOfRows(ds.getRowCount()));                        
                    }
                    //--- Store the service result in the response (for the renderer)
                    serviceResponse.setResult(ds);
                    
                }
            }
            
            //--- Set response values 
            serviceResponse.setValue("rowcount", ds.getRowCount());
            serviceResponse.setValue("max", iMaxi);
            serviceResponse.setValue("dbrowcount", ""+iCount);
            

        } catch (Exception ex)
        {
            if (ex.getCause() != null)
            {
                //serviceResponse.addError(ex.getCause());
                serviceResponse.addException(ex.getCause());
            }
            //serviceResponse.addError(ex);
            serviceResponse.addException(ex);
            throw new TelosysException("DataSet cannot execute SQL operation ", ex);
        }
        
        //--- Call the "AFTER" hook method
        after( screenSession, serviceRequest, serviceResponse);

        return null ;
    }

}