package org.objectweb.telosys.service.search;

import java.util.Map;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.View;
import org.objectweb.telosys.dal.dataset.SQLDataSet;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.service.ServiceResponse;
import org.objectweb.telosys.util.StrUtil;

/**
 * @author Laurent GUERIN
 *  
 */
public class SQLSearchWithPages extends SQLSearchAncestor
{
    private int _iLinesPerPage = 20; // = page size

    protected void setLinesPerPage(int v)
    {
        _iLinesPerPage = v ;
    }

    //public View execute(ScreenSession screenSession, Map requestParameters, ServiceResponse serviceResponse) throws TelosysException
    public View execute(ScreenSession screenSession, ServiceRequest serviceRequest, ServiceResponse serviceResponse) throws TelosysException
    {
        //--- Call the "BEFORE" hook method
        before( screenSession, serviceRequest, serviceResponse);
        
        Map requestParameters = serviceRequest.getParameterMap();        
        String strParams[] = null;

        if (_bDynamicParameters)
        {
            //--- Set the useful criterias
            setCriterias(requestParameters);
            //--- Change the SQLDataSet definition
            _dsDefinition.alterWhere(_criterias);
            strParams = _criterias.getDynamicParameters();
            trace("WHERE  = " + _dsDefinition.getSqlWhere());
            trace("TYPES  = " + _dsDefinition.getParametersTypes());
            trace("PARAMS length = " + strParams.length);
            trace("PARAMS = " + strParams.toString());
        }
        else
        {
            //--- Get the SQL parameters values (as String)
            strParams = getStringParams(requestParameters);
            if (strParams.length < _dsDefinition.getNbParams())
            {
                throw new TelosysException(_dsDefinition.getNbParams() + " parameters expected (" + strParams.length + " received)");
            }
        }

        //--- Set the page size ( number of lines per page )
        _dsDefinition.setLinesPerPage(_iLinesPerPage);

        //--- Get the requested page number
        int iPage = 0;
        //String sPage = (String) requestParameters.get("page");
        String sPage = getParamValue(requestParameters,"page");
        if (sPage != null)
        {
            trace("Page parameter = '" + sPage + "'");
            //--- Page parameter found => use it
            iPage = StrUtil.getInt(sPage, 0);
            if (iPage <= 0)
            {
                String sMsg = "Invalid page number : " + sPage;
                serviceResponse.addMessage(new ScreenMessage(sMsg, ScreenMessage.ERROR));
                return null;
            }            
            trace("Page number = " + iPage);
        }
        else
        {
            //--- No page parameter => page = 1
            trace("No page parameter => use default page number : 1");
            iPage = 1;
        }

        //--- DataSet creation
        SQLDataSet ds = null;
        try
        {
            ds = new SQLDataSet(_dsDefinition, strParams);
        } catch (TelosysException e)
        {
            //serviceResponse.addError(e);
            serviceResponse.addException(e);
            serviceResponse.addMessage(new ScreenMessage("Cannot create SQLDataSet ", ScreenMessage.ERROR));
            throw e;
        }
        trace("DataSet created");

        try
        {
            //--- Count the number of rows
            trace(" DataSet.count");
            int iCount = ds.count();
            int iPageSize = 0 ;

            if (iCount == 0)
            {
                //--- No row => no load
                serviceResponse.addMessage(_messages.noRow());
            }
            else
            {
                //--- Load the rows for the given page ...
//                trace(" DataSet.loadPage(" + iPage + ") , maxi = " + _iMaxi);
                int iRet = ds.loadPage(iPage);
                trace(" DataSet.loadPage(" + iPage + ") : return code = " + iRet);

                if (ds.getRowCount() == 0) // It would never happened ( because _iCount > 0 )
                {
                    // Hum... something has changed in the database between the 2 requests !!!
                    serviceResponse.addMessage(_messages.noRow());
                }
                else
                {
//                    if (_iMaxi > 0 && iCount > _iMaxi)
//                    {
//                        serviceResponse.addMessage(_messages.tooMuchRows(iCount, _iMaxi));
//                    }
//                    else
//                    {
//                        serviceResponse.addMessage(_messages.numberOfRows(iCount));
//                    }
                    //--- Store the service result in the response (for the renderer)
                    serviceResponse.setResult(ds);
                }
                iPageSize = ds.getRowCount(); 
            }
            //--- 
            int iPageCount = iCount / _iLinesPerPage ;
            if ( iCount % _iLinesPerPage > 0 )
            {
                iPageCount++;
            }
            serviceResponse.setValue("RowCount",""+iCount);
            serviceResponse.setValue("PageNum",""+iPage);
            serviceResponse.setValue("PageCount",""+iPageCount);
            serviceResponse.setValue("RowsInPage",""+iPageSize);
        } catch (Exception ex)
        {
            if (ex.getCause() != null)
            {
                //serviceResponse.addError(ex.getCause());
                serviceResponse.addException(ex.getCause());
            }
            //serviceResponse.addError(ex);
            serviceResponse.addException(ex);
            throw new TelosysException("DataSet cannot execute SQL operation ", ex);
        }

        //--- Call the "AFTER" hook method
        after( screenSession, serviceRequest, serviceResponse);

        return null;
    }
}