package org.objectweb.telosys.uil;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.ScreenData;

/**
 * The Telosys request environment. <br>
 * The 'request ecosystem' for an 'action request' or a 'service request' <br>
 * Holds the ScreenContext for the current http request <br>
 * 
 * @author Laurent GUERIN
 *  
 */
public class RequestEnv extends TelosysObject
{
    private static final String  TELOSYS_REQUEST_ENV = "TELOSYS_REQUEST_ENV";

    public  static final int EXPOSE_DATA = 1 ;

    public  static final int DO_NOT_EXPOSE_DATA = 0 ;

    private ScreenContext               _screenContext      = null;

    //-----------------------------------------------------------------------------------------
// removed in v 1.0.1
//    public static void setEnv(HttpServletRequest httpServletRequest, ScreenSession screenSession)
//    {
//        setEnv(httpServletRequest, new RequestEnv(httpServletRequest));
//    }

//    //-----------------------------------------------------------------------------------------
//    public static void setEnv(HttpServletRequest httpServletRequest, ScreenContext screenContext)
//    {
//        setEnv(httpServletRequest, new RequestEnv(httpServletRequest, screenContext));
//    }
//
//    //-----------------------------------------------------------------------------------------
//    private static void setEnv(HttpServletRequest httpServletRequest, RequestEnv requestEnv)
//    {
//        httpServletRequest.setAttribute(TELOSYS_REQUEST_ENV, requestEnv);
//    }

    //-----------------------------------------------------------------------------------------
    /**
     * Constructs a RequestEnv instance and store it in the request attributes<br>
     * Since v 1.0.1 expose the ScreenData elements (if any) as request attributes
     * @param httpServletRequest
     * @param screenContext
     * @param flag
     */
    public static void setEnv(HttpServletRequest httpServletRequest, ScreenContext screenContext, int flag)
    {
        RequestEnv requestEnv = new RequestEnv(httpServletRequest, screenContext);
        //--- Store the ScreenContext holder at "request level" (for TagLib usage)
        httpServletRequest.setAttribute(TELOSYS_REQUEST_ENV, requestEnv);
        
        if ( flag == EXPOSE_DATA )
        {
            //--- Expose the ScreenData elements if any
            if ( screenContext != null )
            {
                ScreenData screenData = screenContext.getCurrentData() ;
                if ( screenData != null )
                {
                    screenData.exposeData(httpServletRequest);
                }
            }
        }
    }
    
    //-----------------------------------------------------------------------------------------
    /**
     * Returns the RequestEnv instance stored in the request attributes
     * @param servletRequest
     * @return the RequestEnv available for the given request (or null if none)
     */
    public static RequestEnv getEnv(ServletRequest servletRequest)
    {
        Object obj = servletRequest.getAttribute(TELOSYS_REQUEST_ENV);
        if (obj != null)
        {
            if (obj instanceof RequestEnv)
            {
                return (RequestEnv) obj;
            }
        }
        return null;
    }

    //-----------------------------------------------------------------------------------------

//    private void initEnvironment(HttpServletRequest httpServletRequest, ScreenContext screenContext)
//    {
//        _screenContext = screenContext;
//
//        //--- Expose as a request attribute
//        httpServletRequest.setAttribute(TELOSYS_REQUEST_ENV, this);
//    }

    //-----------------------------------------------------------------------------------------
    // Private constructors ( no public constructor )
    //-----------------------------------------------------------------------------------------
    private RequestEnv()
    {
    }

//    private RequestEnv(HttpServletRequest httpServletRequest)
//    {
//        initEnvironment(httpServletRequest, null);
//    }

    private RequestEnv(HttpServletRequest httpServletRequest, ScreenContext screenContext)
    {
//        initEnvironment(httpServletRequest, screenContext);
        _screenContext = screenContext;
    }

    //----------------------------------------------------------------
    /**
     * Returns the ScreenContext associated with the current ScreenAction request
     * @return the ScreenContext ( or null if there is no ScreenContext available for this request )
     */
    public ScreenContext getScreenContext()
    {
        return _screenContext;
    }

    //----------------------------------------------------------------
    /**
     * Returns the ScreenData of the ScreenContext associated with the current ScreenAction request
     * @return the ScreenData ( or null if there is no ScreenData available for this request )
     */
    public ScreenData getScreenData()
    {
        ScreenData data = null ;
        if (_screenContext != null)
        {
            data = _screenContext.getCurrentData();
        }
        //trace("getScreenData() : " + ( data != null ? "NOT NULL" : "NULL" ) );
        return data;
    }

    //----------------------------------------------------------------

}