package org.objectweb.telosys.uil;

import java.io.Serializable;
import java.util.Hashtable;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 *  
 */
public class SessionPostback extends TelosysObject implements Serializable
{
    private Hashtable _htScreensPostback = null;

    /**
     *  
     */
    public SessionPostback()
    {
        super();
        _htScreensPostback = new Hashtable();
    }

    /**
     * If the string is null or contains only spaces : returns null,
     * else returns the string after "trim"
     * @param s
     * @return the significant string 
     */
    private final static String getSignificant(String s)
    {
        if ( s == null )
        {
            return null ;                
        }
        else
        {
            String s2 = s.trim();
            if ( s2.length() > 0 )
            {
                return s2;
            }
            else
            {
                return null ;                
            }
        }
    }
    
    /**
     * Compose a "Screen Key" with the given Screen Name & Screen Id
     * @param sArgScreenName
     * @param sArgScreenId
     * @return the Screen Key
     */
    public final static String composeScreenKey(String sArgScreenName, String sArgScreenId)
    {
        String sScreenName = getSignificant(sArgScreenName) ;
        String sScreenId   = getSignificant(sArgScreenId) ;
        //--- Compose the "screen key" for postback values storage in session
        if (sScreenName != null && sScreenId != null)
        {
            return sScreenName + "." + sScreenId;
        }
        else if (sScreenName != null && sScreenId == null)
        {
            return sScreenName;
        }
        else if (sScreenName == null && sScreenId != null)
        {
            return sScreenId;
        }
        else
        {
            return "(no_screen_key)";
        }
    }
    
    /**
     * Compose a "Screen Key" with the given Screen Name & Screen Id
     * @param sArgScreenName
     * @param iArgScreenId
     * @return the Screen Key
     */
    public final static String composeScreenKey(String sArgScreenName, int iArgScreenId)
    {
        String sScreenId = null;
        if ( iArgScreenId >= 0 )
        {
            sScreenId = ""+iArgScreenId ;
        }
        else
        {
            //--- iArgScreenId < 0 equiv to NULL
            sScreenId = null;
        }
        return composeScreenKey(sArgScreenName, sScreenId );
    }
    
    /**
     * Set the ScreenPostback values for the given Screen Key
     * @param sKey
     * @param oValues
     */
    private void setScreenPostback(String sKey, ScreenPostback oValues)
    {
        trace("setScreenValues(" + sKey + ", values )");
        _htScreensPostback.put(sKey, oValues);
    }

    /**
     * Get the ScreenPostback values for the given Screen Key
     * @param sKey
     * @return
     * @throws TelosysException
     */
    public ScreenPostback getScreenPostback(String sKey) throws TelosysException
    {
        trace("getScreenPostback(" + sKey + ")");
        Object obj = _htScreensPostback.get(sKey);
        if (obj != null)
        {
            if (obj instanceof ScreenPostback)
            {
                return (ScreenPostback) obj;
            }
            else
            {
                throw new TelosysException("Object for key '" + sKey + "' is not a ScreenPostback instance");
                //return null ;
            }
        }
        else
        {
            //--- Create a new instance, register it and return it
            ScreenPostback screenPostback = new ScreenPostback();
            setScreenPostback(sKey, screenPostback);
            return screenPostback;
        }
    }

    //    private void removeScreenPostback(String sKey)
    //    {
    //        trace("removeScreenValues(" + sKey + ")" );
    //        _htScreensPostback.remove(sKey);
    //    }

}