package org.objectweb.telosys.uil;

import java.io.File;
import java.io.InputStream;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.Tool;
import org.objectweb.telosys.uil.screenmap.ScreenDefConfig;
import org.objectweb.telosys.uil.screenmap.ScreenDefLoader;
import org.objectweb.telosys.uil.screenmap.ScreenDefinitions;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;
import org.objectweb.telosys.uil.taglib.widget.TagDefaultValues;
import org.objectweb.telosys.util.JarUtil;

/**
 * @author Laurent GUERIN
 * 
 */
public final class TelosysUIL
{
    public  final static String      TELOSYS_CSS_FILE        = "telosys.css";
    
    private final static String      CLASS_NAME              = "TelosysUIL";

    //--- Fichiers de configuration
    private static String            $sScreensConfigFileName = null;

    private static String            $sTagsConfigFileName    = null;

    private static ScreenDefinitions $screenDefinitions      = null;

    //-----------------------------------------------------------------------------
//    private static ScreenDefinitions initScreenMapDefinitions(ServletContext servletContext)
    private static ScreenDefinitions initScreenMapDefinitions(InputStream is, ServletContext servletContext)
    {
//        if ($sScreensConfigFileName != null)
//        {
//            Telosys.info(CLASS_NAME + " : Init Screen Definitions from file " + $sScreensConfigFileName );
//            try {
//	            ScreenDefLoader loader = new ScreenDefLoader();
//	            
//	            Telosys.info(CLASS_NAME + " : Load screens definitions ... "  );
//	            //ScreenDefinitions definitions = loader.load($sScreensConfigFileName);
//	            ScreenDefinitions definitions = loader.load(is);
//	            
//	            Telosys.info(CLASS_NAME + " : Load screens translations ... "  );
//	            loader.loadTranslations(servletContext);
//	            return definitions ;	            
//            }
//            catch ( Exception ex ) 
//            {
//                return null ;
//            }
//        }
//        else
//        {
//            Telosys.error(CLASS_NAME + ".initScreenMapDefinitions() : No configuration file name => Cannot initialize ScreenMapDefinitions !");
//            return null;
//        }

        try {
            ScreenDefLoader loader = new ScreenDefLoader();
            
            Telosys.info(CLASS_NAME + " : Load screens definitions ... "  );
            //ScreenDefinitions definitions = loader.load($sScreensConfigFileName);
            ScreenDefinitions definitions = loader.load(is);
            
            Telosys.info(CLASS_NAME + " : Load screens translations ... "  );
            loader.loadTranslations(servletContext);
            return definitions ;	            
        }
        catch ( Exception ex ) 
        {
            return null ;
        }
    
    }

//    //-----------------------------------------------------------------------------
//    // PUBLIC METHODS
//    //-----------------------------------------------------------------------------
//    public static boolean init( ServletContext servletContext )
//    {
//        Telosys.info(CLASS_NAME + " : INIT without properties ...");
//        //--- Get the Telosys properties ...
//        Properties prop = Telosys.getProperties();
//        if (prop == null)
//        {
//            Telosys.error(CLASS_NAME + ".init() : cannot get Telosys properties !");
//            return false;
//        }
//        else
//        {
//            return init(servletContext, prop);
//        }
//    }

    //-----------------------------------------------------------------------------
//    private static boolean init(ServletContext servletContext, Properties prop)
//    {
//        Telosys.info(CLASS_NAME + " : INIT with properties ...");
//        boolean bRet1 = false;
//        boolean bRet2 = false;
//        if (prop != null)
//        {
//            //-------------------------------------------------------------------
//            //--- SCREENS configuration file
//            //-------------------------------------------------------------------
//            $sScreensConfigFileName = prop.getProperty("ScreensConfFile");
//            Telosys.info(CLASS_NAME + " : Init screens definitions ( file = " + $sScreensConfigFileName + ") ..." );
//            if ($sScreensConfigFileName == null)
//            {
//                Telosys.error(CLASS_NAME + ".init(Properties) : Screens Config File not found ");
//            }
//            else
//            {
//                $sScreensConfigFileName = $sScreensConfigFileName.trim();
//                $sScreensConfigFileName = Telosys.transformFileName($sScreensConfigFileName);
//                if ( Tool.fileExists( $sScreensConfigFileName ) )
//                {
//	                //--- Init ScreenMap Definitions
//	                $screenDefinitions = initScreenMapDefinitions(servletContext);
//	                if ( $screenDefinitions != null )
//	                {
//	                    Telosys.info(CLASS_NAME + " : Init : screen definitions loaded.");                    
//	                    bRet1 = true;                    
//	                }
//	                else
//	                {
//	                    Telosys.error(CLASS_NAME + ".init(Properties) : Cannot init screen definitions ! ");                    
//	                    bRet1 = false;                    
//	                }
//                }
//                else
//                {
//                    Telosys.error(CLASS_NAME + ".init(Properties) : File " + $sScreensConfigFileName + " not found !");                                        
//                }
//            }
//
//            //-------------------------------------------------------------------
//            //--- TAGS configuration file
//            //-------------------------------------------------------------------
//            $sTagsConfigFileName = prop.getProperty("TagsConfFile");
//            Telosys.info(CLASS_NAME + " : Init tags configuration ( file = " + $sTagsConfigFileName + ") ..." );
//            if ($sTagsConfigFileName == null)
//            {
//                Telosys.error(CLASS_NAME + ".init(Properties) : Tags Config File not found ");
//            }
//            else
//            {
//                $sTagsConfigFileName = $sTagsConfigFileName.trim();
//                $sTagsConfigFileName = Telosys.transformFileName($sTagsConfigFileName);
//                if ( Tool.fileExists( $sTagsConfigFileName ) )
//                {
//	                //--- Init Tags Default Values
//	                bRet2 = TagDefaultValues.init($sTagsConfigFileName);
//                }
//                else
//                {
//                    Telosys.error(CLASS_NAME + ".init(Properties) : File " + $sTagsConfigFileName + " not found !");                                        
//                }
//            }
//
//            //-------------------------------------------------------------------
//        }
//        else
//        {
//            Telosys.error(CLASS_NAME + ".init(Properties) : properties = null !");
//        }
//        return bRet1 && bRet2;
//    }
    
    //-----------------------------------------------------------------------------
    public static boolean init( ServletContext servletContext )
    {
        boolean bRet1 = false;
        boolean bRet2 = false;
        Telosys.info("TelosysUIL.init( ServletContext ) : Screens definitions initialization ...");                    
        
        //-------------------------------------------------------------------
        //--- SCREENS.XML configuration file
        //-------------------------------------------------------------------
        Telosys.info("TelosysUIL.init() : Init screens definitions ( file = '" + Telosys.getScreensConfigFileName() + "' ) ..." );
        InputStream is = Telosys.getScreensConfigInputStream();
        if ( is != null )
        {            
            //--- Init ScreenMap Definitions
            $screenDefinitions = initScreenMapDefinitions(is, servletContext);
            if ( $screenDefinitions != null )
            {
                $sScreensConfigFileName = Telosys.getScreensConfigFileName() ;
                Telosys.info("TelosysUIL.init() : Screens definitions initialized. Count = " + $screenDefinitions.getScreenDefCount() );                    
                bRet1 = true;                    
            }
            else
            {
                Telosys.error("TelosysUIL.init() : Cannot init screen definitions ! ");                    
                bRet1 = false;                    
            }
            Tool.closeInputStream(is);
        }
        else
        {
            Telosys.error("TelosysUIL.init() : Cannot get SCREENS CONFIG InputStream ! ");
        }
        
        //-------------------------------------------------------------------
        //--- TAGS configuration file
        //-------------------------------------------------------------------
        Telosys.info("TelosysUIL.init() : Init tags properties ( file = '" + Telosys.getTagsConfigFileName() + "' ) ..." );
        is = Telosys.getTagsConfigInputStream();
        if ( is != null )
        {            
            //--- Init Tags Default Values
            bRet2 = TagDefaultValues.init(is);
            if ( bRet2 )
            {
                Telosys.info("TelosysUIL.init() : Tags default values initialized.");                    
            }
            else
            {
                Telosys.error("TelosysUIL.init() : Cannot init tags default values ! ");                    
            }
            Tool.closeInputStream(is);
        }
        else
        {
            Telosys.error("TelosysUIL.init() : Cannot get TAGS CONFIG InputStream ! ");
        }

        //-------------------------------------------------------------------
        //--- Extract JS and CSS files from the 'telosys.jar' file
        //-------------------------------------------------------------------
        if ( Telosys.getWebAppRootDir() != null )
        {
            Telosys.info("TelosysUIL.init() : exploded WAR ( WebApp root dir = ' " + Telosys.getWebAppRootDir() + " ) ..." );
            Telosys.info("-----");                    
            Telosys.info("Try to extract files from Telosys JAR ... ");
            extractFilesFromJar($screenDefinitions);
            Telosys.info("-----");                    
        }
        else
        {
            Telosys.info("TelosysUIL.init() : WAR not exploded ( no WebApp root dir ) => no file extraction from JAR" );
        }
        
        //-------------------------------------------------------------------
        return bRet1 && bRet2;
    }

    //-----------------------------------------------------------------------------
    public static String getScreensConfigFileName()
    {
        return $sScreensConfigFileName;
    }

    //-----------------------------------------------------------------------------
    public static String getTagsConfigFileName()
    {
        return $sTagsConfigFileName;
    }

    //-----------------------------------------------------------------------------
    public static ScreenDefinitions getScreenDefinitions()
    {
        return $screenDefinitions;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the Screens Definitions Configuration <br>
     * ( the "config" part of the screens.xml file )
     * @return
     */
    public static ScreenDefConfig getScreenDefConfig()
    {
        if ($screenDefinitions != null)
        {
            return $screenDefinitions.getConfig();
        }
        return null;
    }    

    //-----------------------------------------------------------------------------
    /**
     * Returns the default screen type ( "html", "xul", ... )
     * @return
     */
    public static String getDefaultScreenType()
    {
        // TODO : load from screens.xml
        return ScreenMapConst.HTML_SCREEN_TYPE;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the default screen id 
     * @return
     */
    public static int getDefaultScreenId()
    {
        // TODO : load from screens.xml
        return 0 ;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the "server side" screen map servlet path <br>
     * defined in the "config" tag of the screens XML file<br>
     * ( e.g. : "/screenmap" )
     * 
     * @return
     */
    public static String getScreenMapServletPath()
    {
        ScreenDefinitions screenDefinitions = TelosysUIL.getScreenDefinitions();
        if (screenDefinitions == null)
        {
            //log.error("getConfig() : Cannot get 'ScreenDefinitions' from 'TelosysUIL' ");
            return null;
        }
        
        ScreenDefConfig config = screenDefinitions.getConfig();
        if (config == null)
        {
            return null;
        }
        return config.getScreenMapUrl();
    }
    
    //-----------------------------------------------------------------------------
    private static void extractFilesFromJar( ScreenDefinitions screenDefinitions )
    {
        if ( screenDefinitions == null )
        {
            Telosys.error("No screen definitions !");
            return ;
        }
        ScreenDefConfig config = screenDefinitions.getConfig();
        if (config == null)
        {
            Telosys.error("Cannot get config from screen definitions !");
            return ;
        }
        if ( config.getTelosysJarFile() == null )
        {
            Telosys.error("Cannot get Telosys Jar file from Screen Config !");
            return ;
        }
        
        //--- Web App resources names ( WAR root relative path ) 
        String sTelosysJarFile          = config.getTelosysJarFile();
        String sFrameworkJavascriptFile = config.getFrameworkJavascriptFile();
        //String sFrameworkCSSFile        = config.getFrameworkCSSFile(); // v 0.9.9
        String sFrameworkCSSDir         = config.getFrameworkCSSDir();
        
        Telosys.info("Application configuration : ");
        Telosys.info(". Telosys JAR file : '" + sTelosysJarFile + "'");
        Telosys.info(". Javascript file  : '" + sFrameworkJavascriptFile + "'");
        //Telosys.info(". CSS file         : '" + sFrameworkCSSFile + "'"); // v 0.9.9
        Telosys.info(". CSS directory    : '" + sFrameworkCSSDir + "'");
            
        //--- File system files 
        String sTelosysJarFullPath        = getFullPathFile(sTelosysJarFile, true);
        String sTelosysJavascriptFullPath = getFullPathFile(sFrameworkJavascriptFile, false); 
        //String sFrameworkCSSFileFullPath  = getFullPathFile(sFrameworkCSSFile, false);  // v 0.9.9
        String sFrameworkCSSDirFullPath   = getFullPathFile(sFrameworkCSSDir, false); 
        
        Telosys.info("File system files : ");
        Telosys.info(". Telosys JAR file : '" + sTelosysJarFullPath + "'");
        Telosys.info(". Javascript file  : '" + sTelosysJavascriptFullPath + "'");
        //Telosys.info(". CSS file         : '" + sFrameworkCSSFileFullPath + "'"); // v 0.9.9
        Telosys.info(". CSS directory    : '" + sFrameworkCSSDirFullPath + "'");
            
        if ( sTelosysJarFullPath != null )
        {                
            Telosys.info("Extraction from JAR '" + sTelosysJarFullPath + "' :");
            //--- Extract the Javascript framework file 
            extractFile(sTelosysJarFullPath, "js/telosys.js", sTelosysJavascriptFullPath );            
            //--- Extract all the CSS files ( all the "css" directory in the JAR file )
            extractDir(sTelosysJarFullPath, "css", sFrameworkCSSDirFullPath ) ; // v 0.9.9
            //--- Extract the file "telosys.css" 
            // v 0.9.9 USELESS already in the "css" dir
            //extractFile(sTelosysJarFullPath, "css/telosys.css", sFrameworkCSSFileFullPath ); 
        }
        else
        {
            Telosys.error("Cannot get or find Telosys JAR file path => cannot extract files !");
        }
    }
    //---------------------------------------------------------------------------
    private static void extractFile(String sJarFile, String sJarEntry, String sDestFile) 
    {
        Telosys.info(". Extracting JAR file '" + sJarEntry + "' to '" + sDestFile + "' ...");
        if ( sDestFile != null )
        {
	        try
	        {
	            JarUtil.extract(sJarFile, sJarEntry, sDestFile);
	        } catch (TelosysException e)
	        {
	            Telosys.error("Cannot extract '" + sDestFile + "' from '" + sJarFile + "' (TelosysException) !");
	        }
        }
        else
        {
            Telosys.error("Destination file is null => Cannot extract !");            
        }
    }
    //---------------------------------------------------------------------------
    private static void extractDir(String sJarFile, String sJarDir, String sDestDir ) 
    {
        Telosys.info(". Extracting JAR dir '" + sJarDir + "' to '" + sDestDir + "' ...");
        if ( sDestDir != null )
        {
	        //--- Extract all the files from the given JAR file directory 
	        try
	        {
	            JarUtil.extractFiles(sJarFile, sJarDir, sDestDir, JarUtil.REMOVE_PARENT_DIR );
	        } catch (TelosysException e)
	        {
	            Telosys.error("Cannot extract '"+sJarDir+"' from '" + sJarFile + "' (TelosysException) !");
	        }
        }
        else
        {
            Telosys.error("Destination directory is null => Cannot extract !");            
        }
    }

    //-----------------------------------------------------------------------------
    
    //---------------------------------------------------------------------------
    private static String getFullPathFile(String sParamFile, boolean bMustExist)
    {
        if (sParamFile != null)
        {
            String sFile = sParamFile.trim();
            //--- Determine Full Path
            String sFullPath = null;
            //--- Full path = Application root dir + relative file name
            sFullPath = Telosys.getFullPath(sFile);

            //--- Check existence (if needed)
            if (bMustExist)
            {
                File file = new File(sFullPath);
                if (file.exists())
                {
                    return sFullPath;
                }
                else
                {
    	            Telosys.error("The file '" + sFullPath + "' doesn't exist !");
    	            return null;
                }
            }
            else
            {
                return sFullPath;
            }
        }
        return null;
    }
}