package org.objectweb.telosys.uil.i18n;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * This class is the standard implementation of the Translator interface.
 * 
 * @author Laurent GUERIN
 *
 */
public class StandardTranslator extends TelosysObject implements Translator
{
    private static final String SEPARATOR = "_";
    
    //--- The hashmap dictionary ( e.g. "en_TagId.txt" --> "translated text" )
    HashMap _dict = null ;
    
    /**
     * The default constructor
     */
    public StandardTranslator()
    {
        super();
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.uil.i18n.Translator#translate(java.lang.String, java.lang.String)
     */
    // sKey : the tag attribute key ( e.g. : "MyTagId.txt")
    // sLanguage : the language ( e.g. : "fr", "en", "it", ... )
    public String translate(String sKey, String sLanguage)
    {
        if ( _dict != null )
        {
            //--- Get from the dictionary
            String sKey2 = sLanguage + SEPARATOR + sKey ;
            Object o = _dict.get(sKey2);
            if ( o != null )
            {
                return (String)o;
            }
        }
        return null;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.uil.i18n.Translator#size()
     */
    public int size()
    {
        if ( _dict != null ) return _dict.size();
        return 0 ;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.uil.i18n.Translator#isEmpty()
     */
    public boolean isEmpty()
    {
        if ( _dict != null ) return _dict.isEmpty();
        return true ;
    }
    
    /**
     * Loads a language translation from a properties input stream
     * @param inputStream : the properties input stream
     * @param sLanguage : the language ISO code ( "fr", "en", .. )
     */
    public void load( InputStream inputStream, String sLanguage )
    {
        if ( inputStream == null )
        {
            throw new TelosysRuntimeException("StandardTranslator.load() : input stream is null ");
        }
        if ( sLanguage == null )
        {
            throw new TelosysRuntimeException("StandardTranslator.load() : language is null ");
        }
        
        try
        {
            Properties prop = new Properties();
            prop.load(inputStream);
            Enumeration keys = prop.keys();
            while ( keys.hasMoreElements() )
            {
                Object oKey = keys.nextElement();
                String sKey = (String)oKey;
                String sVal = prop.getProperty(sKey);
                //--- Store in the dictionary
                String sKey2 = sLanguage + SEPARATOR + sKey ;
                if ( _dict == null )
                {
                    // Creates the dictionary only if necessary
                    _dict = new HashMap() ;
                }
                _dict.put(sKey2, sVal);
            }
        } catch (IOException e)
        {
            throw new TelosysRuntimeException("StandardTranslator.load() : IOException ", e);
        }
    }

}
