package org.objectweb.telosys.uil.i18n;

import java.io.InputStream;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.TelosysClassLogger;

/**
 * Tool class for translators loading 
 * 
 * @author Laurent GUERIN
 * 
 */
public final class TranslatorLoader
{
    private static final TelosysClassLogger $log = new TelosysClassLogger(TranslatorLoader.class);

    //-----------------------------------------------------------------------------
    /**
     * Loads the 0..N translations files for the given JSP file
     * @param servletContext the standard ServletContext
     * @param sJspFile the JSP file that need a translator
     * @param languages the list of languages to load 
     * @return
     */
    public static Translator loadTranslations(ServletContext servletContext, String sJspFile, String[] languages )
    {
        $log.trace("loadTranslationsFile(ctx, file='" + sJspFile + "', languages)");
        if ( sJspFile != null )
        {            
	        if ( sJspFile.endsWith(".jsp") )
	        {
	            String s = sJspFile.substring(0,sJspFile.length()-4);
	            String sFile = null ;
	            if ( languages != null )
	            {
	                StandardTranslator translator = new StandardTranslator();
	                for ( int i=0 ; i < languages.length ; i++ )
	                {
	                    String sLang = languages[i];
	                    sFile = s + "_" + sLang + ".txt" ;
	                    loadTranslationsFile( servletContext, sFile, translator, sLang );
	                }
	                if ( translator.isEmpty() )
	                {
	                    $log.trace("loadTranslationsFile(ctx, file='" + sJspFile + "', languages) : return NULL");
	                    return null ;
	                }
	                else
	                {
	                    $log.trace("loadTranslationsFile(ctx, file='" + sJspFile + "', languages) : return Translator instance");
		                return translator ;
	                }
	            }
	        }
	        else
	        {
	            $log.error("ScreenBody File doesn't end with '.jsp'");
	        }
        }
        else
        {
            $log.error("ScreenBody File is null");
        }
        $log.trace("loadTranslationsFile(ctx, file='" + sJspFile + "', languages) : return NULL");
        return null;
    }    
    
    //-----------------------------------------------------------------------------
    private static void loadTranslationsFile(ServletContext servletContext, String sFile, StandardTranslator translator, String sLang )
    {
        $log.trace("loadTranslationsFile(ctx, file='" + sFile + "', language='" + sLang + "')");
        InputStream inputStream = servletContext.getResourceAsStream(sFile) ;
        if ( inputStream != null )
        {
            $log.trace("+++ loading " + sFile );
//            ScreenMapTranslator translator = new ScreenMapTranslator();
            translator.load(inputStream, sLang);
        }
        else
        {
            $log.trace("File " + sFile + " not found ( input stream is null ) ");
        }        
    }
}