package org.objectweb.telosys.uil.screenmap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.TelosysVersion;
import org.objectweb.telosys.uil.TelosysUIL;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenDefConfig extends TelosysObject
{
    // <config default_action
    private String _sDefaultAction        = null ;

    // <config telosys_jar_file
    private String _sTelosysJarFile       = null ;

    // <config telosys_framework_file
    private String _sTelosysFrameworkFile = null ;

    // <config framework_css_dir
    private String _sFrameworkCSSDir      = null ;

    // <config ajax_request_url
    private String _sAjaxRequestUrl       = null ;

    // <config screen_map_url
    private String _sScreenMapUrl         = null ;

    // <config base_language
    private String _sBaseLanguage         = null ;

    // <config all_languages
    private String _sAllLanguages         = null ;

    
    /**
     * 
     */
    public ScreenDefConfig()
    {
        super();
        
        //--- Set the default values 
        _sDefaultAction        = "none";
        //_sTelosysJarFile       = getFullPathFile("/WEB-INF/lib/telosys-"+ TelosysVersion.VERSION.trim() + ".jar", true);
        _sTelosysJarFile       = "/WEB-INF/lib/telosys-"+ TelosysVersion.VERSION.trim() + ".jar" ;
        _sTelosysFrameworkFile = "/telosys/telosys.js";
        _sFrameworkCSSDir      = "/telosys";
        _sAjaxRequestUrl       = "/action" ;
        _sScreenMapUrl         = "/screenmap" ;
    }
    //---------------------------------------------------------------------------
//    private String getFullPathFile(String sFile, boolean bMustExist)
//    {
//        if (sFile != null)
//        {
//            //--- Determine Full Path
//            String sFullPath = null;
//            //            if ( sFile.startsWith(".") ) // RELATIVE
//            //            {
//            //                //--- Cut the first char
//            //                sFile = sFile.substring(1);
//            //--- Full path = Application root dir + relative file name
//            sFullPath = Telosys.getFullPath(sFile);
//            //            }
//            //            else // ABSOLUTE
//            //            {
//            //                sFullPath = sFile;
//            //            }
//
//            //--- Check existence (if needed)
//            if (bMustExist)
//            {
//                File file = new File(sFullPath);
//                if (file.exists())
//                {
//                    return sFullPath;
//                }
//            }
//            else
//            {
//                return sFullPath;
//            }
//        }
//        return null;
//    }

    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // Context : Auto-Open Context mode
    //---------------------------------------------------------------------------
    /* package */void setDefaultAction(String s)
    {
        trace("setDefaultAction(" + s +")");
        if ( s != null )
        {
            String s2 = s.trim().toLowerCase() ;
            if ( s2.equals("none") ||  s2.equals("open") ||  s2.equals("use") )
            {
                _sDefaultAction = s2;
                return ;
            }
        }
        throw new TelosysRuntimeException("Config : invalid default action : '" + s + "'");
    }

    //---------------------------------------------------------------------------
    public String getDefaultAction()
    {
        return _sDefaultAction;
    }

    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // Telosys Framework Javascript File
    //---------------------------------------------------------------------------
    /**
     * Set the web context path for the telosys javascript framework file
     * 
     * @param s ( ie: "/js/telosys.js" )
     */
    /* package */void setFrameworkJavascriptFile(String s)
    {
        trace("setFrameworkJavascriptFile(" + s +")");
        if ( s != null )
        {
            if ( s.trim().length() > 0 )
            {
                _sTelosysFrameworkFile = s.trim() ;  
                return ;
            }
        }
        throw new TelosysRuntimeException("Config : invalid Framework Javascript file : '" + s + "'");        
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the web context path of the javascript framework file
     * @return ( ie: "/js/telosys.js" )
     */
    public String getFrameworkJavascriptFile()
    {
        return _sTelosysFrameworkFile;
    }
    //---------------------------------------------------------------------------
//    /**
//     * Returns the full filesystem path of the Telosys Framework Javascript file
//     * 
//     * @return the path ( ie : "/filesystem_dir/web_dir/context_dir/js/telosys.js" )
//     */
//    public String getFrameworkJavascriptFilePath()
//    {
//        return getFullPathFile(_sTelosysFrameworkFile, false);
//    }
    //---------------------------------------------------------------------------
    // Telosys Javascript framework URL
    //---------------------------------------------------------------------------
//    /* package */void setTelosysFrameworkUrl(String s)
//    {
//        _sTelosysFrameworkUrl = s;
//    }
//    //---------------------------------------------------------------------------
//    public String getTelosysFrameworkUrl()
//    {
//        return _sTelosysFrameworkUrl;
//    }
//    public String getFrameworkJavascriptFilePath()
//    {
//        return _sTelosysFrameworkFile ;
//    }

    //---------------------------------------------------------------------------


    //---------------------------------------------------------------------------
    // Telosys CSS files
    //---------------------------------------------------------------------------
    /**
     * Set the web context path for the framework CSS directory 
     * @param s the directory ( ie: "/css/telosys" )
     */
    /* package */ void setFrameworkCSSDir(String s)
    {
        trace("setFrameworkCSSDir(" + s +")");
        if (s != null) 
        {
            if ( s.trim().length() > 0 )
            {
                _sFrameworkCSSDir = s.trim();                
                return ;
            }
        }
        throw new TelosysRuntimeException("Config : invalid Framework CSS dir : '" + s + "'");        
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the web context path of the framework CSS directory 
     * @return the directory ( ie: "/css/telosys" )
     */
    public String getFrameworkCSSDir()
    {
        return _sFrameworkCSSDir ;
    }

    //---------------------------------------------------------------------------
//    /**
//     * Returns the full filesystem path of the framework CSS directory
//     * @return the directory ( ie : "/filesystem_dir/web_dir/context_dir/css" )
//     */
//    public String getFrameworkCSSDirPath()
//    {
//        return Telosys.getFullPath ( _sFrameworkCSSDir );
//    }

    //---------------------------------------------------------------------------
    /**
     * Returns the web context path of a given framework CSS file 
     * @param sFileName ( ie : "image.png" )
     * @return the directory ( ie: "/css/telosys/image.png" )
     */
    public String getFrameworkCSSFile(String sFileName)
    {
        String sDir = _sFrameworkCSSDir;
        if (!sDir.endsWith("/"))
        {
            sDir = sDir + "/";
        }
        return sDir + sFileName ;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the web context path of the framework CSS file 
     * @return the directory ( ie: "/css/telosys/telosys.css" )
     */
    public String getFrameworkCSSFile()
    {
        return getFrameworkCSSFile( TelosysUIL.TELOSYS_CSS_FILE );
    }

    //---------------------------------------------------------------------------
//    /**
//     * Returns the full filesystem path of the given CSS file name
//     * @param sFileName ( ie : "image.png" )
//     * @return the path ( ie : "/filesystem_dir/web_dir/context_dir/css/image.png" )
//     */
//    public String getFrameworkCSSFilePath(String sFileName)
//    {
////        String sDir = _sFrameworkCSSDir;
////        if (!sDir.endsWith("/"))
////        {
////            sDir = sDir + "/";
////        }
//        return getFullPathFile( getFrameworkCSSFile(sFileName), false);
//    }
//    
    //---------------------------------------------------------------------------
//    /**
//     * Returns the full filesystem path of the Telosys Framework CSS file
//     * 
//     * @return the path ( ie : "/filesystem_dir/web_dir/context_dir/css/telosys.css" )
//     */
//    public String getFrameworkCSSFilePath()
//    {
//        return getFrameworkCSSFilePath( TelosysUIL.TELOSYS_CSS_FILE ) ;
//    }

    //---------------------------------------------------------------------------
    // Telosys JAR File
    //---------------------------------------------------------------------------
    /**
     * Set the full file name of the telosys jar file <br>
     * The file is set only if it is NOT NULL and NOT VOID
     * 
     * @param s
     */
    /* package */void setTelosysJarFile(String s)
    {
        trace("setTelosysJarFile(" + s +")");
        if ( s != null )
        {
            if ( s.trim().length() > 0 )
            {
                //_sTelosysJarFile = getFullPathFile(s.trim(), true);
                _sTelosysJarFile = s.trim();
                return ;
            }
        }
        // Else : keep the default file 
        //throw new TelosysRuntimeException("Config : invalid Telosys JAR file: '" + s + "'");                
    }

    //---------------------------------------------------------------------------
    public String getTelosysJarFile()
    {
        return _sTelosysJarFile;
    }

    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // AJAX request URL
    //---------------------------------------------------------------------------
    /* package */void setAjaxRequestUrl(String s)
    {
        trace("setAjaxRequestUrl(" + s +")");
        if ( s != null )
        {
            if ( s.trim().length() > 0 )
            {
                _sAjaxRequestUrl = s.trim() ;
                return ;
            }
        }
        throw new TelosysRuntimeException("Config : invalid Ajax Request URL : '" + s + "'");        
    }

    //---------------------------------------------------------------------------
    public String getAjaxRequestUrl()
    {
        return _sAjaxRequestUrl;
    }

    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // Screen Map URL
    //---------------------------------------------------------------------------
    /* package */void setScreenMapUrl(String s)
    {
        trace("setScreenMapUrl(" + s +")");
        if ( s != null )
        {
            if ( s.trim().length() > 0 )
            {
                _sScreenMapUrl = s.trim() ;  
                return ;
            }
        }
        throw new TelosysRuntimeException("Config : invalid ScreenMap URL : '" + s + "'");                
    }

    //---------------------------------------------------------------------------
    public String getScreenMapUrl()
    {
        return _sScreenMapUrl;
    }
    //---------------------------------------------------------------------------
    
    //---------------------------------------------------------------------------
    // ScreenMap language
    //---------------------------------------------------------------------------
    /* package */void setBaseLanguage(String s)
    {
        trace("setBaseLanguage(" + s +")");
        if ( s != null )
        {
            if ( s.trim().length() > 0 )
            {
                _sBaseLanguage = s.trim() ;
                return ;
            }
        }
        throw new TelosysRuntimeException("Config : invalid base language : '" + s + "'");        
    }
    //---------------------------------------------------------------------------
    public String getBaseLanguage()
    {
        return _sBaseLanguage;
    }
    
    //---------------------------------------------------------------------------
    // All the languages used for i18n
    //---------------------------------------------------------------------------
    /* package */void setAllLanguages(String s)
    {
        trace("setAllLanguages(" + s +")");
        if ( s != null )
        {
            if ( s.trim().length() > 0 )
            {
                _sAllLanguages = s.trim() ;
                return ;
            }
        }
        throw new TelosysRuntimeException("Config : invalid languages list : '" + s + "'");        
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the languages string ( e.g. "fr,en,de" )
     * @return the string or null if not set
     */
    public String getAllLanguages()
    {
        return _sAllLanguages;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the languages array ( e.g. { "fr","en","de" } )
     * @return the array or null
     */
    public String[] getAllLanguagesArray()
    {
        if ( _sAllLanguages != null )
        {
            String[] array = _sAllLanguages.split(",");
            for ( int i = 0 ; i < array.length ; i++)
            {	
                String s = array[i];
                if ( s != null )
                {
                    array[i] = s.trim();
                }
            }
            return array ;
        }
        else
        {
            return null ;
        }
    }
    //---------------------------------------------------------------------------

}