package org.objectweb.telosys.uil.screenmap;

import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 *  
 */
/* package */class ScreenDefLocation extends TelosysObject
{

    private final static int UNKNOWN_TYPE     = -1;

    //private final static int SCREEN_TYPE      = 0;

    private final static int HTML_TYPE        = 1;

    private final static int XUL_TYPE         = 2;

    private int              _iScreenType     = UNKNOWN_TYPE; // HTML or XUL

    private String           _sScreenTemplate = null;

    private String           _sScreenBody     = null;

    private String           _sScreenPage     = null;

    private String           _sScreenUrl      = null;

    private String           _sScreenCss      = null;

    private String           _sScreenScript   = null;

    //---------------------------------------------------------------------------
    //    private String buildURL(String sUrl, String sRootDir)
    //    {
    //        if (sUrl != null)
    //        {
    //            if (sUrl.indexOf('/') < 0) // No "/" in the URL
    //            {
    //                return sRootDir + "/" + sUrl;
    //            }
    //            else
    //            {
    //                return sUrl;
    //            }
    //        }
    //        else
    //        {
    //            return null;
    //        }
    //    }

    //---------------------------------------------------------------------------
    /**
     * Constructor for standard screen location parameters
     */
    public ScreenDefLocation()
    {
        super();
        _iScreenType = UNKNOWN_TYPE ;
    }

    //---------------------------------------------------------------------------
    /**
     * Constructor for specific type location parameters ( HTML, XUL )
     * 
     * @param sType ( ie : "html", "xul" )
     */
    public ScreenDefLocation(String sType)
    {
        super();
        _iScreenType = UNKNOWN_TYPE;
        String sTrimType = sType.trim();
        if (sTrimType.equalsIgnoreCase("html"))
        {
            _iScreenType = HTML_TYPE;
        }
        if (sTrimType.equalsIgnoreCase("xul"))
        {
            _iScreenType = XUL_TYPE;
        }
    }

//    //---------------------------------------------------------------------------
//    // Screen Type
//    //---------------------------------------------------------------------------
//    public final int getType()
//    {
//        return _iScreenType;
//    }

    //---------------------------------------------------------------------------
    // 
    //---------------------------------------------------------------------------
    private String getValidAttribute(String s)
    {
        if ( s == null )
        {
            return null ;
        }
        String sTrim = s.trim() ;
        if ( sTrim.length() == 0 )
        {
            return null ;
        }
        return sTrim ;
    }
    //---------------------------------------------------------------------------
    // template
    //---------------------------------------------------------------------------
    public String getScreenTemplate()
    {
        return _sScreenTemplate;
    }

    public void setScreenTemplate(String s)
    {        
        _sScreenTemplate = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // body
    //---------------------------------------------------------------------------
    public String getScreenBody()
    {
        return _sScreenBody;
    }

    public void setScreenBody(String s)
    {
        _sScreenBody = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // page
    //---------------------------------------------------------------------------
    public String getScreenPage()
    {
        return _sScreenPage;
    }

    public void setScreenPage(String s)
    {
        _sScreenPage = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // url
    //---------------------------------------------------------------------------
    public String getScreenUrl()
    {
        return _sScreenUrl;
    }

    public void setScreenUrl(String s)
    {
        _sScreenUrl = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // css
    //---------------------------------------------------------------------------
    public String getScreenCss()
    {
        return _sScreenCss;
    }

    public void setScreenCss(String s)
    {
        _sScreenCss = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // script
    //---------------------------------------------------------------------------
    public String getScreenScript()
    {
        return _sScreenScript;
    }

    public void setScreenScript(String s)
    {
        _sScreenScript = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    /**
     * Return the target for the current definition
     * Target determination by priority : 
     *  1. template attribute ( if it is specified )
     *  2. page attribute ( if it is specified and there is no template )
     *  3. url attribute ( if no template and no page )
     * @return the target (or null)
     */
    public String getScreenTarget()
    {
        if (_sScreenTemplate != null)
        {
            return _sScreenTemplate;
        }
        else if (_sScreenPage != null)
        {
            return _sScreenPage;
        }
        else
        {
            return _sScreenUrl;
        }
    }

    //---------------------------------------------------------------------------
//    public boolean isTemplate()
//    {
//        if (_sScreenTemplate != null)
//        {
//            return true;
//        }
//        return false;
//    }

    //---------------------------------------------------------------------------
//    public boolean isStandardScreen()
//    {
//        if (_iScreenType == SCREEN_TYPE)
//        {
//            return true;
//        }
//        return false;
//    }

    //---------------------------------------------------------------------------
    public boolean isHtml()
    {
        if (_iScreenType == HTML_TYPE)
        {
            return true;
        }
        return false;
    }

    //---------------------------------------------------------------------------
    public boolean isXul()
    {
        if (_iScreenType == XUL_TYPE)
        {
            return true;
        }
        return false;
    }
    //---------------------------------------------------------------------------
}