package org.objectweb.telosys.uil.screenmap;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.uil.i18n.Translator;

/**
 * @author Laurent GUERIN
 * 
 */
/* package */class ScreenDefinition extends TelosysObject
{
    private String            _sScreenName     = null;

    private ScreenDefLocation _htmlLocation    = null;

    private ScreenDefLocation _xulLocation     = null;

    private ScreenDefLocation _screenLocation  = null;

    private HashMap           _hmElements      = new HashMap();
    
    private Translator        _translator    = null ;
    
    //---------------------------------------------------------------------------

    /**
     * @param screenName
     * @param screenLoc
     */
    public ScreenDefinition( String screenName, ScreenDefLocation screenLoc )
    {
        super();
        _sScreenName = screenName;
        _screenLocation = screenLoc;
    }

    //---------------------------------------------------------------------------
    public final String getScreenName()
    {
        return _sScreenName;
    }

    //---------------------------------------------------------------------------
    // Screen Location
    //---------------------------------------------------------------------------
    public void setScreenDefLocation(ScreenDefLocation screenLoc)
    {
        if (screenLoc.isHtml())
        {
            _htmlLocation = screenLoc;
        }
        else if (screenLoc.isXul())
        {
            _xulLocation = screenLoc;
        }
    }

    //---------------------------------------------------------------------------
    public final ScreenDefLocation getScreenDefLocation(String sType)
    {
        ScreenDefLocation ret = null;
        if (sType.equals(ScreenMapConst.HTML_SCREEN_TYPE))
        {
            ret = _htmlLocation;
        }
        else if (sType.equals(ScreenMapConst.XUL_SCREEN_TYPE))
        {
            ret = _xulLocation;
        }
        return ret;
    }

    //---------------------------------------------------------------------------
    // Screen Map Elements
    //---------------------------------------------------------------------------
    public void addElement(ScreenMapElement elem)
    {
        if (elem != null)
        {
            if (elem.getName() != null)
            {
                _hmElements.put(elem.getName(), elem);
            }
        }
    }

    //---------------------------------------------------------------------------
    public void setElements(HashMap hm)
    {
        _hmElements = hm;
    }

    //---------------------------------------------------------------------------
    public HashMap getElements()
    {
        return _hmElements;
    }
    
    //---------------------------------------------------------------------------
    public int getElemCount()
    {
        if ( _hmElements == null ) 
        {
            return 0 ;
        }
        else
        {
            return _hmElements.size();
        }
    }
    
    //---------------------------------------------------------------------------
    // Screen Map Translator
    //---------------------------------------------------------------------------
    public void setTranslator(Translator t)
    {
        _translator = t;
    }

    //---------------------------------------------------------------------------
    public Translator getTranslator()
    {
        return _translator;
    }

    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    public boolean isBodyTarget(String sType)
    {
        //--- If this screen definition contains a template and a body ...
        if ( ( getScreenTemplate(sType) != null ) && ( getScreenBody(sType) != null ) )
        {
            return true ;
        }
        return false ;
    }
    //---------------------------------------------------------------------------
    public boolean isPageTarget(String sType)
    {
        //--- If this screen definition contains a "page" attribute ...
        if ( getScreenPage(sType) != null  )
        {
            return true ;
        }
        return false ;
    }
    //---------------------------------------------------------------------------
    private String getScreenTemplate(String sType)
    {
        String sRet = _screenLocation.getScreenTemplate();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenTemplate() != null )
            {
                sRet = loc.getScreenTemplate();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    private String getScreenPage(String sType)
    {
        String sRet = _screenLocation.getScreenPage();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenPage() != null )
            {
                sRet = loc.getScreenTemplate();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    public String getScreenTarget(String sType)
    {
        String sRet = _screenLocation.getScreenTarget();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenTarget() != null )
            {
                sRet = loc.getScreenTarget();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "screen body" file part for the given type 
     * ( without the "root dir" part ) 
     * @param sType
     * @return
     */
    public String getScreenBody(String sType)
    {
        String sRet = _screenLocation.getScreenBody();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenBody() != null )
            {
                sRet = loc.getScreenBody();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    public String getScreenCss(String sType)
    {
        String sRet = _screenLocation.getScreenCss();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenCss() != null )
            {
                sRet = loc.getScreenCss();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    public String getScreenScript(String sType)
    {
        String sRet = _screenLocation.getScreenScript();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenScript() != null )
            {
                sRet = loc.getScreenCss();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
//    public String getScreenImagesRootDir(String sType)
//    {
//        String sRet = _screenLocation.getScreenImagesRootDir();
//        ScreenDefLocation loc = getScreenDefLocation(sType);
//        if ( loc != null )
//        {
//            if ( loc.getScreenImagesRootDir() != null )
//            {
//                sRet = loc.getScreenImagesRootDir();
//            }
//        }
//        return sRet ;
//    }
    //---------------------------------------------------------------------------
}
