package org.objectweb.telosys.uil.screenmap;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenDefinitions extends TelosysObject
{
    private final static int         UNKNOWN_TYPE = -1;

    //private final static int SCREEN_TYPE = 0;

    private final static int         HTML_TYPE    = 1;

    private final static int         XUL_TYPE     = 2;

    private final TelosysClassLogger _log         = new TelosysClassLogger(ScreenDefinitions.class);

    private ScreenDefConfig          _config      = new ScreenDefConfig();

    private ScreenDefRootDir         _htmlRootDir = new ScreenDefRootDir();

    private ScreenDefRootDir         _xulRootDir  = new ScreenDefRootDir();

    //--- Screens Definitions : "screen name" --> ScreenDefinition instance
    private HashMap                  _hmScreenDef = new HashMap(20);

    /**
     *  
     */
    public ScreenDefinitions()
    {
        super();
    }

    //---------------------------------------------------------------------------
//    private int getType(String sScreenType)
//    {
//        if (isHtmlType(sScreenType))
//        {
//            return HTML_TYPE;
//        }
//        else if (isXulType(sScreenType))
//        {
//            return XUL_TYPE;
//        }
//        else
//        {
//            return UNKNOWN_TYPE;
//        }
//    }

    //---------------------------------------------------------------------------
    public ScreenDefConfig getConfig()
    {
            return _config;
    }

    //---------------------------------------------------------------------------
    private boolean isHtmlType(String sScreenType)
    {
        //return sScreenType.equalsIgnoreCase("html");
        return sScreenType.equalsIgnoreCase( ScreenMapConst.HTML_SCREEN_TYPE );
    }

    //---------------------------------------------------------------------------
    private boolean isXulType(String sScreenType)
    {
        //return sScreenType.equalsIgnoreCase("xul");
        return sScreenType.equalsIgnoreCase( ScreenMapConst.XUL_SCREEN_TYPE );
    }

    //---------------------------------------------------------------------------
    public ScreenDefRootDir getHtmlRootDir()
    {
            return _htmlRootDir;
    }
    /* package */ void setHtmlRootDir(ScreenDefRootDir x)
    {
        _htmlRootDir = x;
    }
    
    //---------------------------------------------------------------------------
    public ScreenDefRootDir getXulRootDir()
    {
            return _xulRootDir;
    }
    /* package */ void setXulRootDir(ScreenDefRootDir x)
    {
        _xulRootDir = x;
    }

    //---------------------------------------------------------------------------
    public ScreenDefRootDir getRootDir(String sScreenType)
    {
        if (isHtmlType(sScreenType))
        {
            return _htmlRootDir;
        }
        else if (isXulType(sScreenType))
        {
            return _xulRootDir;
        }
        else
        {
            return null ;            
        }
    }

    //---------------------------------------------------------------------------
    //public ScreenMap getScreenMap(String sScreenName, String sScreenType, int iScreenId)
    public ScreenMap getScreenMap(String sScreenName, String sScreenType, int iScreenContextId, String sScreenContextName ) // v 0.9.9
    {
        _log.trace("getScreenMap(" + sScreenName + "," + sScreenType + "," + iScreenContextId + "," + sScreenContextName + ")");

        //--- Search a screen definition for this screen name
        ScreenDefinition screenDef = (ScreenDefinition) _hmScreenDef.get(sScreenName);
        if (screenDef == null)
        {
            _log.error("No screen definition for the screen : '" + sScreenName + "' ");
            return null;
        }

        //--- Create and return the ScreenMap instance
        ScreenDefRootDir rootDir = getRootDir(sScreenType);
        if (rootDir == null)
        {
            _log.error("No root dir for screen type : '" + sScreenType + "' ");
            return null;
        }
        
        //return new ScreenMap(rootDir, screenDef, sScreenType, iScreenContextId);
        return new ScreenMap(rootDir, screenDef, sScreenType, iScreenContextId, sScreenContextName ); // v 0.9.9
    }

    //---------------------------------------------------------------------------
    /* package */ void addScreenDefinition(ScreenDefinition screenDef)
    {
        _log.trace("addScreenDefinition(" + screenDef.getScreenName() + ")");
        _hmScreenDef.put(screenDef.getScreenName(), screenDef);
    }

    //---------------------------------------------------------------------------
    public ScreenDefinition getScreenDefinition(String sScreenName)
    {
        _log.trace("getScreenDefinition(" + sScreenName + ")");

        //--- Search a screen definition for this screen name
        return (ScreenDefinition) _hmScreenDef.get(sScreenName);
    }

    //---------------------------------------------------------------------------
    public Iterator getScreenNames()
    {
        Set keys = _hmScreenDef.keySet();
        return ( keys != null ? keys.iterator() : null );
    }
    
    //---------------------------------------------------------------------------
    public int getScreenDefCount()
    {
        return _hmScreenDef.size();
    }
    //---------------------------------------------------------------------------
}