package org.objectweb.telosys.uil.taglib;

import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.data.DataRow;

/**
 * Static methods to retrieve informations from the "current bean"
 * 
 * @author Laurent GUERIN
 *  
 */
public class CurrentBean
{
    private static final TelosysClassLogger log = new TelosysClassLogger(CurrentBean.class);
    
    public static final int NULL = 0 ;
    public static final int STANDARD_BEAN = 1 ;
    public static final int DATA_ROW = 2 ;
    
    //----------------------------------------------------------------------------------
    private static int getColumn(String sCol, DataRow dr)
    {
        int iCol = 0;
        try
        {
            iCol = Integer.parseInt(sCol);
            if (iCol <= 0)
            {
                return -1;
            }
            if (iCol > dr.getSize())
            {
                return -2;
            }
        } catch (NumberFormatException e)
        {
            return -3;
        }
        return iCol;
    }

    //----------------------------------------------------------------------------------
    /**
     * Set the 'current bean'
     * @param pageContext
     * @param oBean
     */
    public static void setBean ( PageContext pageContext, Object oBean )
    {
        Page.setCurrentBean(pageContext, oBean) ;
    }
    
    //----------------------------------------------------------------------------------
    /**
     * Returns the 'current bean' ( or null if there's no current bean )
     * @param pageContext
     * @return
     */
    public static Object getBean ( PageContext pageContext )
    {
        return Page.getCurrentBean(pageContext) ;
    }
    
    //----------------------------------------------------------------------------------
    /**
     * Clear the 'current bean' 
     * @param pageContext
     * @return
     */
    public static void clearBean ( PageContext pageContext )
    {
        Page.clearCurrentBean(pageContext) ;
    }
    
    //----------------------------------------------------------------------------------
    /**
     * Returns TRUE if there's a 'current bean' ( if the 'current bean' is not null )
     * @param pageContext
     * @return
     */
    public static boolean isNotNull ( PageContext pageContext )
    {
        return ( Page.getCurrentBean(pageContext) != null ) ;
    }

    //----------------------------------------------------------------------------------
    /**
     * Returns the 'current bean' type  
     * @param pageContext
     * @return : DATA_ROW, STANDARD_BEAN, NULL
     */
    public static int getType ( PageContext pageContext )
    {
        Object oBean = Page.getCurrentBean(pageContext) ;
        if ( oBean == null )
        {
            return NULL ;
        }
        else if (oBean instanceof DataRow)
        {
            return DATA_ROW ;
        }
        return STANDARD_BEAN ;
    }
    
    //----------------------------------------------------------------------------------
    /**
     * Return the value of a column from the "current bean" <br>
     * ( usefull for a DataRow current bean ) 
     * @param pageContext
     * @param sCol
     * @return
     */
    public static Object getColumn ( PageContext pageContext, String sCol )
    {
        Object oBean = Page.getCurrentBean(pageContext);
        if (oBean instanceof DataRow)
        {
            DataRow dr = (DataRow) oBean;
            int iCol = getColumn(sCol, dr);
            if (iCol > 0 && iCol <= dr.getSize())
            {
                return dr.getObject(iCol) ;
            }
            else
            {
                return "(col='" + sCol + "':invalid)" ;
            }
        }
        else
        {
            return "(no DataRow)" ;
        }
    }

    //----------------------------------------------------------------------------------
//    /**
//     * Return the value of an attribute from the "current bean" <br>
//     * ( usefull for a "standard bean" current bean ) 
//     * @param pageContext
//     * @param sAttribute
//     * @return
//     */
//    public static Object getAttribute ( PageContext pageContext, String sAttribute )
//    {
//        //--- Try to find the "current bean" object 
//        Object oBean = Page.getCurrentBean(pageContext);
//	    if ( oBean != null  &&  sAttribute != null )
//	    {
//	        try
//	        {
//	            return BeanUtil.getFieldValue(oBean, sAttribute);
//	        } catch (TelosysException e)
//	        {
//	            // Unknown field
//	            return null ;
//	        }                    
//	    }
//	    return null ;        
//    }
}