package org.objectweb.telosys.uil.taglib;

import java.util.LinkedList;

import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysClassLogger;

/**
 * Static methods to managed a given JSP "page context"
 * 
 * @author Laurent GUERIN
 *  
 */
public class JSLines
{
    public  final static String TABBOX = "TABBOX";

    private static final TelosysClassLogger log = new TelosysClassLogger(JSLines.class);

    //==============================================================================================
    // Generic Javascript lines ( for <script> tag generation at the end )
    //==============================================================================================
    /**
     * Build the full key from the given ID
     * 
     * @param sId
     * @return
     */
    private static String getKey(String sId)
    {
        if (sId != null)
        {
            return "TELOSYS_JS_LINES_" + sId.trim();
        }
        else
        {
            return "TELOSYS_JS_LINES_GLOBAL";
        }
    }

    /**
     * Returns the Javascript lines for the given Id in the given page context
     * 
     * @param pageContext :
     *            the current page context to use
     * @param sId :
     *            the application level ID
     * @param bCreate :
     *            if true the list is created if not found
     * @return lines list (or null if not found and bCreate is set to false)
     */
    public static LinkedList getJSLines(PageContext pageContext, String sId, boolean bCreate)
    {
        String sKey = getKey(sId);

        LinkedList jsLines = null;
        //--- Try to found the list in the given page context
        Object oJavascriptLines = pageContext.getAttribute(sKey);
        if (oJavascriptLines != null)
        {
            if (oJavascriptLines instanceof LinkedList)
            {
                jsLines = (LinkedList) oJavascriptLines;
            }
            else
            {
                log.error("Page contex attribute '" + sKey + "' is not an instance of LinkedList ! ( => Create a new LinkedList instance ) ");
                jsLines = null; // Consider as not found
            }
        }
        //--- Create the list if not found
        if (jsLines == null && bCreate)
        {
            jsLines = new LinkedList();
            pageContext.setAttribute(sKey, jsLines);
        }
        //--- Return the list
        return jsLines;
    }

    /**
     * Returns the Javascript lines for the given Id in the given page context
     * 
     * @param pageContext :
     *            the current page context
     * @param sId :
     *            the application level ID
     * @return lines list (or null if not found)
     */
    public static LinkedList getJSLines(PageContext pageContext, String sId)
    {
        return getJSLines(pageContext, sId, false);
    }

    /**
     * Add a line in the list identified by the given ID 
     * @param pageContext
     * @param sId
     * @param sLine
     * @return
     */
    public static boolean addLine(PageContext pageContext, String sId, String sLine )
    {
        LinkedList jsLines = getJSLines(pageContext, sId, true);
        if (jsLines != null)
        {
            jsLines.add(sLine);
            return true ;
        }
        else
        {
            log.error("addLine(..) : Cannot get lines");
            return false ;
        }
    }

    /**
     * Clear the Javascript lines list (clear the list and remove it from the page context)
     * 
     * @param pageContext :
     *            the current page context
     * @param sId :
     *            the application level ID
     */
    public static void clearJSLines(PageContext pageContext, String sId)
    {
        String sKey = getKey(sId);

        LinkedList jsLines = getJSLines(pageContext, sId, false);
        if (jsLines != null)
        {
            jsLines.clear();
            pageContext.removeAttribute(sKey);
        }
    }

}