package org.objectweb.telosys.uil.taglib;

import java.io.IOException;
import java.util.HashMap;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

/**
 * @author Laurent GUERIN
 *  
 */
public class JSVariables
{
    private static final String VARIABLES    = "TELOSYS_GLOBAL_JS_VARIABLES";

    public  static final String TELOSYS_VAR  = "TELOSYS_VAR";

    public  static final String ACTIONS_VAR  = "ACTIONS_VAR";

    private final HashMap       _hmVariables = new HashMap();

    /**
     * Return a variable storage space for the given page context (create it if necessary)
     * 
     * @param pageContext
     * @return
     */
    public final static JSVariables getVariables(PageContext pageContext)
    {
        //--- Try to find an existing variables storage space in the page context
        ServletRequest request = pageContext.getRequest();
        if ( request != null )
        {            
	        JSVariables variables = (JSVariables) request.getAttribute(VARIABLES);
	        if (variables == null) // If not found => not yet created
	        {
	            //--- Variables storage space creation
	            variables = new JSVariables();
	            //--- Keep in the page context for the next time
	            request.setAttribute(VARIABLES, variables);
	        }
	        return variables;
        }
        else
        {
            return null ;
        }
    }

    /**
     * Set a variable value
     * 
     * @param sVarName
     * @param sVarValue
     */
    public final void set(String sVarName, String sVarValue)
    {
        if (_hmVariables != null)
        {
            _hmVariables.put(sVarName, sVarValue);
        }
    }

    /**
     * Get a variable value
     * 
     * @param sVarName
     * @return
     */
    public final String get(String sVarName)
    {
        if (_hmVariables != null)
        {
            Object oValue = _hmVariables.get(sVarName);
            if (oValue != null)
            {
                if (oValue instanceof String)
                {
                    return (String) oValue;
                }
            }
        }
        return null;
    }
    
    /**
     * Unset a variable
     * 
     * @param sVarName
     */
    public final void unset(String sVarName)
    {
        if (_hmVariables != null)
        {
            _hmVariables.remove(sVarName);
        }
    }

    /**
     * Print all the declared variables on the given output
     * 
     * @param out
     * @throws IOException
     */
    public final void print(JspWriter out) throws IOException
    {
        if (_hmVariables != null)
        {
            java.util.Set keys = _hmVariables.keySet();
            if (keys != null)
            {
                Object[] names = keys.toArray();
                String sName = null;
                String sValue = null;
                for ( int i = 0 ; i < names.length ; i++ )
                {
                    sName = (String) names[i];
                    sValue = (String) _hmVariables.get(sName);
                    out.println(" . " + sName + " = " + sValue);
                }
            }
        }
        else
        {
            out.println(" (no variables) ");
        }
    }

}