package org.objectweb.telosys.uil.taglib;

import java.util.Iterator;

import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.util.StrUtil;

/**
 * @author Laurent GUERIN
 *  
 */
public class LoopManager
{
    private final static int TYPE_NULL     = 0;

    private final static int TYPE_LOOP     = 1;

    private final static int TYPE_ITERATOR = 2;

    //----------------------------------------------------------------
    // Private variables
    //----------------------------------------------------------------

    //---
    private int              _iType        = TYPE_NULL;

    private PageContext      _pageContext  = null;

    //--- For loop ( FROM, TO, STEP )
    private int              _iFrom        = 0;

    private int              _iTo          = 0;

    private int              _iStep        = 0;

    //--- For COLLECTION ITERATOR
    private Iterator         _iter         = null;

    private int              _iIndexBase   = 0;        // First index value   ( v 0.9.9 )
    
    //--- Common variables
    private int              _iIndex       = 0;        // Current index

    private int              _iCount       = 0;        // Current count

    private String           _sItemName    = null ;    // Specific variable name for current item  ( v 1.0.1 )
    
    //----------------------------------------------------------------
    private void initLoop(PageContext pageContext, int from, int to, int step)
    {
        _pageContext = pageContext ;

        _iType = TYPE_LOOP;

        //--- Loop configuration
        _iFrom = from;
        _iTo = to;
        _iStep = step;

        _iIndex = _iFrom;
        _iCount = 0;
    }

    //----------------------------------------------------------------
    /**
     * Constructor
     * @param pageContext 
     * @param from : the "first" value of the loop
     * @param to   : the "last" value of the loop
     * @param step : the "step" used for each increment
     */
    public LoopManager(PageContext pageContext, int from, int to, int step)
    {
        super();
        initLoop(pageContext, from, to, step);
    }

    //----------------------------------------------------------------
    /**
     * Constructor
     * @param pageContext
     * @param sLoopDesc : the loop description string, syntaxe "from:to" ( ie "1:20" = from 1 to 20 )
     */
    public LoopManager(PageContext pageContext, String sLoopDesc)
    {
        super();

        //--- Parse the loop description
        int iFrom = 0;
        int iTo = 0;
        if (sLoopDesc != null)
        {
            String s = sLoopDesc.trim();
            if (s.length() > 0)
            {
                String[] tokens = s.split(":");
                if (tokens.length >= 2)
                {
                    iFrom = StrUtil.getInt(tokens[0], 0);
                    iTo = StrUtil.getInt(tokens[1], 0);
                }
            }
        }

        initLoop(pageContext, iFrom, iTo, 1); // Step 1
    }

    //----------------------------------------------------------------
    /**
     * Constructor
     * @param pageContext
     * @param iter : the Iterator to use for the loop
     * @param iIndexBase : the 'index base' ( first value of the index )
     */
    public LoopManager(PageContext pageContext, Iterator iter, int iIndexBase)
    {
        super();
        _pageContext = pageContext ;
        
        _iType = TYPE_ITERATOR;

        _iter = iter;

        _iIndex = 0;
        _iCount = 0;
        _iIndexBase = iIndexBase ;
    }

    //----------------------------------------------------------------
    /**
     * Set a specific variable name to the current item ( to store it in PageContext attribute )
     * @param v
     * @since v 1.0.1
     */
    public void setItemName( String v )
    {
        _sItemName = v ;
    }
    
    //----------------------------------------------------------------
    public int getCount()
    {
        return _iCount;
    }

    //----------------------------------------------------------------
    public int getFrom()
    {
        return _iFrom;
    }

    //----------------------------------------------------------------
    public int getIndex()
    {
        return _iIndex;
    }

    //----------------------------------------------------------------
    public int getStep()
    {
        return _iStep;
    }

    //----------------------------------------------------------------
    public int getTo()
    {
        return _iTo;
    }

    //----------------------------------------------------------------
    /**
     * @return true if the loop must continue
     */
    private boolean continueLoop()
    {
        if (_iFrom < _iTo) //--- ie : from 10 to 18
        {
            if (_iIndex <= _iTo && _iStep > 0)
            {
                //_iCount++; // 1 to N
                return true;
            }
            else
            {
                return false;
            }
        }
        if (_iFrom > _iTo) //--- ie : from 24 to 6
        {
            if (_iIndex >= _iTo && _iStep < 0)
            {
                //_iCount++; // 1 to N
                return true;
            }
            else
            {
                return false;
            }
        }
        if (_iFrom == _iTo) //--- ie : from 6 to 6
        {
            if (_iCount == 0) // Never done ...
            {
                //_iCount++; // 1 to N
                //_iIndex = _iFrom;
                return true; // First (and unique) iteration
            }
            else
            {
                return false;
            }
        }
        return false;
    }

    //----------------------------------------------------------------
    // UPDATE OBJECTS IN THE PAGE SCOPE
    //----------------------------------------------------------------
    private void updatePageObjects(Object oCurrentBean)
    {
        if ( _pageContext != null )
        {
            Page.setIterationCount(_pageContext, _iCount);
            Page.setIterationIndex(_pageContext, _iIndex);
            if (oCurrentBean != null)
            {
                // CurrentBean.setBean(_pageContext, oCurrentBean); // v 1.0.1
                // new in v 1.0.1 ( for nested loops )
                if ( _sItemName != null )
                {
                    // Specific variable name for the current item => stored it as a PageContext variable
                    Page.setVar(_pageContext, _sItemName, oCurrentBean);
                }
                else
                {
                    // No specific variable name for the current item => stored as "current bean"
                    CurrentBean.setBean(_pageContext, oCurrentBean); 
                }
            }            
        }
    }

    //----------------------------------------------------------------
    // FIRST
    //----------------------------------------------------------------
    private boolean firstLoop()
    {
        _iCount = 0;
        _iIndex = _iFrom; // First index value
        boolean bContinue = continueLoop();
        if (bContinue)
        {
            _iCount = 1; // First iteration
        }
        updatePageObjects(null);
        return bContinue;
    }

    //----------------------------------------------------------------
    private boolean firstIter()
    {
        boolean bRet = false;
        _iCount = 0;
        _iIndex = 0; // First index value
        Object oCurrentBean = null;
        if (_iter != null)
        {
            if (_iter.hasNext())
            {
                oCurrentBean = _iter.next();
                _iCount = 1; // First iteration
                //_iIndex = 1; // First iteration ( v 0.9.9 )
                _iIndex = _iIndexBase ; // First index value  ( v 0.9.9 )
                bRet = true;
            }
        }
        updatePageObjects(oCurrentBean);
        return bRet;
    }

    //----------------------------------------------------------------
    public boolean first()
    {
        if (_iType == TYPE_LOOP)
        {
            return firstLoop();
        }
        else if (_iType == TYPE_ITERATOR)
        {
            return firstIter();
        }
        else
        {
            return false;
        }
    }

    //----------------------------------------------------------------
    // NEXT
    //----------------------------------------------------------------
    private boolean nextLoop()
    {
        int iMemoIndex = _iIndex ;
        _iIndex = _iIndex + _iStep; // Next index value (next step)
        boolean bContinue = continueLoop();
        if (bContinue)
        {
            _iCount++; // One more iteration
        }
        else
        {
            _iIndex = iMemoIndex ; // Restore original index value ( for "last index" use after the loop )
        }
        updatePageObjects(null);
        return bContinue;
    }

    //----------------------------------------------------------------
    private boolean nextIter()
    {
        boolean bRet = false;
        Object oCurrentBean = null;
        if (_iter != null)
        {
            if (_iter.hasNext())
            {
                oCurrentBean = _iter.next();
                _iCount++;
                _iIndex++;
                bRet = true;
            }
        }
        updatePageObjects(oCurrentBean);
        return bRet;
    }

    //----------------------------------------------------------------
    public boolean next()
    {
        if (_iType == TYPE_LOOP)
        {
            return nextLoop();
        }
        else if (_iType == TYPE_ITERATOR)
        {
            return nextIter();
        }
        else
        {
            return false;
        }
    }

    //----------------------------------------------------------------

    //----------------------------------------------------------------
    public String toString()
    {
        String sType = "???";
        if (_iType == TYPE_LOOP)
        {
            sType = "LOOP (from " + _iFrom + " to " + _iTo + ", step " + _iStep + ")";
        }
        else if (_iType == TYPE_ITERATOR)
        {
            sType = "ITERATOR";
        }

        return "LoopManager : type = " + sType + ", index = " + _iIndex + ", count = " + _iCount;
    }
}