package org.objectweb.telosys.uil.taglib;

import java.util.LinkedList;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.uil.RequestEnv;
import org.objectweb.telosys.uil.taglib.widget.data.MenuItems;
import org.objectweb.telosys.uil.taglib.widget.data.TreeViewNode;

/**
 * Static methods to managed a given JSP "page context"
 * 
 * @author Laurent GUERIN
 *  
 */
public class Page
{
    private final static String             DEFAULT_DATA_ELEMENT    = "TELOSYS_DEFAULT_DATA_ELEMENT";

    private final static String             JS_LINES                = "TELOSYS_JS_LINES";

    private final static String             TABBOX_JS_LINES         = "TELOSYS_TABBOX_JS_LINES";

    private final static String             RADIO_GROUP_ITEMS       = "TELOSYS_RADIO_GROUP_ITEMS";

    private final static String             LAST_WIDGET_ID          = "TELOSYS_LAST_WIDGET_ID";

    private final static String             CURRENT_TAB_INDEX       = "TELOSYS_CURRENT_TAB_INDEX";

    private final static String             TAB_ID_SELECTED         = "TELOSYS_TAB_ID_SELECTED";

    private final static String             LOOP_MANAGER            = "TELOSYS_LOOP_MANAGER";

    private final static String             ITERATOR_COUNT          = "TELOSYS_ITERATOR_COUNT";

    private final static String             ITERATOR_INDEX          = "TELOSYS_ITERATOR_INDEX";

    private final static String             CURRENT_BEAN            = "TELOSYS_CURRENT_BEAN";

    private final static String             LIST_COLUMNS            = "TELOSYS_LIST_COLUMNS";
    
    private final static String             LIST_HEADER             = "TELOSYS_LIST_HEADER";

//    public final static String              LIST_BODY               = "TELOSYS_LIST_BODY";

    private final static String             LIST_BODY_LEVEL         = "TELOSYS_LIST_BODY_LEVEL";

    private final static String             LIST_DIV                = "TELOSYS_LIST_DIV";

    private final static String             PREFIX_FOR_GENERATED_ID = "fwk_id";

//    public final static String              REQUEST_ENV             = "TELOSYS_REQUEST_ENV";

    //--- Tree view 
//    public final static String              TREE_NODE_LEVEL         = "TELOSYS_TREE_NODE_LEVEL";
//
//    public final static String              TREE_INNER_LEVEL        = "TELOSYS_TREE_INNER_LEVEL";
    
    private final static String             TREE_CURRENT_NODE       = "TELOSYS_TREE_CURRENT_NODE";
    
    private final static String             TREE_ROOT_NODE          = "TELOSYS_TREE_ROOT_NODE";
    
    private final static String             TREE_DND_DIV_FLAG       = "TELOSYS_TREE_DND_DIV_FLAG";

    private final static String             MENU_ITEMS              = "TELOSYS_MENU_ITEMS";

    private final static TelosysClassLogger log                     = new TelosysClassLogger(Page.class);

    /**
     * Generate a tag ID (unique for the given page context)
     * 
     * @param pageContext
     * @return
     */
    public static String generateId(PageContext pageContext)
    {
        ServletRequest request = pageContext.getRequest(); // v 0.9.9
        int iLastId = 0;
        //--- Try to find the last widget id
        //Object oAttribute = pageContext.getAttribute(LAST_WIDGET_ID); // v 0.9.9
        Object oAttribute = request.getAttribute(LAST_WIDGET_ID); // v 0.9.9
        if (oAttribute != null)
        {
            if (oAttribute instanceof Integer)
            {
                iLastId = ((Integer) oAttribute).intValue();
            }
            else
            {
                log.error("generateId(..) '" + LAST_WIDGET_ID + "' is not an instance of Integer !");
                iLastId = 0;
            }
        }
        else
        {
            //--- Never used before => init
            iLastId = 0;
        }
        //--- Next ID
        iLastId++;
        //--- Store it in the page context ..
        //pageContext.setAttribute(LAST_WIDGET_ID, new Integer(iLastId)); // v 0.9.9
        request.setAttribute(LAST_WIDGET_ID, new Integer(iLastId)); // v 0.9.9
        //--- Return the generated string ID
        return PREFIX_FOR_GENERATED_ID + iLastId;
    }

    /**
     * Data binding :<br>
     * Set (or clear) the default data element name for the given page context
     * 
     * @param pageContext
     * @param sElementName
     */
    public static void setDefaultDataElement(PageContext pageContext, String sElementName)
    {
        // NB : Cannot set a pageContext attribute to NULL ( throws an exception )
        if (sElementName != null)
        {
            pageContext.setAttribute(DEFAULT_DATA_ELEMENT, sElementName);
        }
        else
        {
            pageContext.removeAttribute(DEFAULT_DATA_ELEMENT);
        }
    }

    /**
     * Data binding :<br>
     * Returns the default data element name from the given page context
     * 
     * @param pageContext
     *            the current page context to use
     * @return the data element name (or null if not found)
     */
    public static String getDefaultDataElement(PageContext pageContext)
    {
        Object oElement = pageContext.getAttribute(DEFAULT_DATA_ELEMENT);
        if (oElement != null)
        {
            if (oElement instanceof String)
            {
                String sElement = (String) oElement;
                return sElement.trim();
            }
        }
        return null;
    }

    //==============================================================================================
    // Register fields Javascript lines ( for <script> tag generation at the end )
    //==============================================================================================
    /**
     * Javascript generation :<br>
     * Returns the dynamic Javascript lines of the given page context
     * 
     * @param pageContext :
     *            the current page context to use
     * @param bCreate :
     *            if true the list is created if not found
     * @return lines list (or null if not found and bCreate is set to false)
     */
    public static LinkedList getJavascriptLines(PageContext pageContext, boolean bCreate)
    {
        ServletRequest request = pageContext.getRequest(); // v 0.9.9

        LinkedList jsLines = null;
        //--- Try to found the list in the given page context
        //Object oJavascriptLines = pageContext.getAttribute(JS_LINES); // v 0.9.9
        Object oJavascriptLines = request.getAttribute(JS_LINES); // v 0.9.9
        if (oJavascriptLines != null)
        {
            if (oJavascriptLines instanceof LinkedList)
            {
                jsLines = (LinkedList) oJavascriptLines;
            }
            else
            {
                log.error("Page contex attribute '" + JS_LINES + "' is not an instance of LinkedList ! ( => Create a new LinkedList instance ) ");
                jsLines = null; // Consider as not found
            }
        }
        //--- Create the list if not found
        if (jsLines == null && bCreate)
        {
            jsLines = new LinkedList();
            //pageContext.setAttribute(JS_LINES, jsLines); // v 0.9.9
            request.setAttribute(JS_LINES, jsLines); // v 0.9.9
        }
        //--- Return the list
        return jsLines;
    }

    /**
     * Javascript generation :<br>
     * Returns the dynamic Javascript lines of the given page context
     * 
     * @param pageContext
     * @return lines list (or null if not found)
     */
    public static LinkedList getJavascriptLines(PageContext pageContext)
    {
        return getJavascriptLines(pageContext, false);
    }

    /**
     * Javascript generation :<br>
     * Clear the Javascript lines list (clear the list and remove it from the page context)
     * 
     * @param pageContext
     */
    public static void clearJavascriptLines(PageContext pageContext)
    {
        LinkedList jsLines = getJavascriptLines(pageContext, false);
        if (jsLines != null)
        {
            jsLines.clear();
            //pageContext.removeAttribute(JS_LINES); // v 0.9.9
            pageContext.getRequest().removeAttribute(JS_LINES); // v 0.9.9
        }
    }

    //==============================================================================================
    // Radio items IDs management
    //==============================================================================================
    private static String getRadioItemsListName(String sRadioGroupId)
    {
        if (sRadioGroupId != null)
        {
            return RADIO_GROUP_ITEMS + "_" + sRadioGroupId.trim();
        }
        else
        {
            return RADIO_GROUP_ITEMS;
        }
    }

    private static LinkedList getRadioItems(PageContext pageContext, String sRadioGroupId, boolean bCreate)
    {
        LinkedList list = null;
        String sListName = getRadioItemsListName(sRadioGroupId);
        //--- Try to found the list in the given page context
        Object oJavascriptLines = pageContext.getAttribute(sListName);
        if (oJavascriptLines != null)
        {
            if (oJavascriptLines instanceof LinkedList)
            {
                list = (LinkedList) oJavascriptLines;
            }
            else
            {
                log.error("Page contex attribute '" + sListName + "' is not an instance of LinkedList ! ( => Create a new LinkedList instance ) ");
                list = null; // Consider as not found
            }
        }
        //--- Create the list if not found
        if (list == null && bCreate)
        {
            list = new LinkedList();
            pageContext.setAttribute(sListName, list);
        }
        //--- Return the list
        return list;
    }

    public static void registerRadioItem(PageContext pageContext, String sRadioGroupId, String sRadioItemId)
    {
        //--- Get the list from the page context (create it if not found)
        LinkedList list = getRadioItems(pageContext, sRadioGroupId, true);
        if (list != null)
        {
            list.add(sRadioItemId);
        }
        else
        {
            log.error("registerRadioItem(..) : Cannot get the list from page context ");
        }

    }

    public static LinkedList getRadioItems(PageContext pageContext, String sRadioGroupId)
    {
        return getRadioItems(pageContext, sRadioGroupId, false);
    }

    public static void clearRadioItems(PageContext pageContext, String sRadioGroupId)
    {
        //--- Get the list from the page context (create it if not found)
        LinkedList list = getRadioItems(pageContext, sRadioGroupId, false);
        if (list != null)
        {
            list.clear();
            String sListName = getRadioItemsListName(sRadioGroupId);
            pageContext.removeAttribute(sListName);
        }
    }

    //==============================================================================================
    // Tab Index management
    //==============================================================================================
    private static Integer getTabIndex(PageContext pageContext)
    {
        //--- Try to found the list in the given page context
        Object oTabIndex = pageContext.getAttribute(CURRENT_TAB_INDEX);
        Integer intTabIndex = null;
        if (oTabIndex != null)
        {
            if (oTabIndex instanceof Integer)
            {
                intTabIndex = ((Integer) oTabIndex);
            }
            else
            {
                log.error("Page contex attribute '" + CURRENT_TAB_INDEX + "' is not an instance of Integer ! ( => Create a new instance ) ");
                intTabIndex = null; // Consider as not found
            }
        }
        //--- Create the list if not found
        if (intTabIndex == null)
        {
            intTabIndex = new Integer(-1); // Not SET
            pageContext.setAttribute(CURRENT_TAB_INDEX, intTabIndex);
        }
        return intTabIndex;
    }

    //    public static void initTabIndex(PageContext pageContext, int iValue)
    //    {
    //        pageContext.setAttribute(CURRENT_TAB_INDEX, new Integer(iValue));
    //    }
    public static void initTabIndex(PageContext pageContext)
    {
        pageContext.setAttribute(CURRENT_TAB_INDEX, new Integer(-1));
    }

    /**
     * Set the next tab index and return it
     * 
     * @param pageContext
     * @return : the new tab index ( from 0 to N )
     */
    public static int getNextTabIndex(PageContext pageContext)
    {
        Integer intTabIndex = getTabIndex(pageContext);
        if (intTabIndex != null)
        {
            int i = intTabIndex.intValue();
            if (i < 0)
            {
                i = 0; // FIRST
            }
            else
            {
                i++; // NEXT
            }
            pageContext.setAttribute(CURRENT_TAB_INDEX, new Integer(i));
            return i;
        }
        return -1;
    }

    /**
     * Clear the current tab index ( reset for future use )
     * 
     * @param pageContext
     * @return
     */
    public static void clearTabIndex(PageContext pageContext)
    {
        pageContext.removeAttribute(CURRENT_TAB_INDEX);
    }

    //==============================================================================================
    // Selected Tab ID management
    //==============================================================================================
    /**
     * Set the ID of the selected tab in the current tabbox
     * 
     * @param pageContext
     * @param sTabId
     */
    public static void setSelectedTabId(PageContext pageContext, String sTabId)
    {        
        // NB : Cannot set a pageContext attribute to NULL ( throws an exception )
        if ( sTabId != null )
        {
            pageContext.setAttribute(TAB_ID_SELECTED, sTabId);
        }
        else
        {
            pageContext.removeAttribute(TAB_ID_SELECTED);
        }
    }

    /**
     * Get the ID of the selected tab in the current tabbox
     * 
     * @param pageContext
     * @return
     */
    public static String getSelectedTabId(PageContext pageContext)
    {
        Object obj = pageContext.getAttribute(TAB_ID_SELECTED);
        Integer intTabIndex = null;
        if (obj != null)
        {
            if (obj instanceof String)
            {
                return (String) obj;
            }
            else
            {
                log.error("Page contex attribute '" + TAB_ID_SELECTED + "' is not an instance of String !");
            }
        }
        return null;
    }

    /**
     * Clear the ID of the selected tab in the current tabbox
     * 
     * @param pageContext
     */
    public static void clearSelectedTabId(PageContext pageContext)
    {
        pageContext.removeAttribute(TAB_ID_SELECTED);
    }

    //==============================================================================================
    // Generic Javascript lines ( for <script> tag generation at the end )
    //==============================================================================================
//    /**
//     * Javascript generation :<br>
//     * Returns the dynamic Javascript lines of the given page context
//     * 
//     * @param pageContext :
//     *            the current page context to use
//     * @param bCreate :
//     *            if true the list is created if not found
//     * @return lines list (or null if not found and bCreate is set to false)
//     */
//    public static LinkedList getJSLines(PageContext pageContext, String sId, boolean bCreate)
//    {
//        LinkedList jsLines = null;
//        //--- Try to found the list in the given page context
//        Object oJavascriptLines = pageContext.getAttribute(JS_LINES);
//        if (oJavascriptLines != null)
//        {
//            if (oJavascriptLines instanceof LinkedList)
//            {
//                jsLines = (LinkedList) oJavascriptLines;
//            }
//            else
//            {
//                log.error("Page contex attribute '" + JS_LINES + "' is not an instance of LinkedList ! ( => Create a new LinkedList instance ) ");
//                jsLines = null; // Consider as not found
//            }
//        }
//        //--- Create the list if not found
//        if (jsLines == null && bCreate)
//        {
//            jsLines = new LinkedList();
//            pageContext.setAttribute(JS_LINES, jsLines);
//        }
//        //--- Return the list
//        return jsLines;
//    }

//    /**
//     * Javascript generation :<br>
//     * Returns the dynamic Javascript lines of the given page context
//     * 
//     * @param pageContext
//     * @return lines list (or null if not found)
//     */
//    public static LinkedList getJSLines(PageContext pageContext, String sId)
//    {
//        return getJSLines(pageContext, sId, false);
//    }

//    /**
//     * Javascript generation :<br>
//     * Clear the Javascript lines list (clear the list and remove it from the page context)
//     * 
//     * @param pageContext
//     */
//    public static void clearJSLines(PageContext pageContext, String sId)
//    {
//        LinkedList jsLines = getJSLines(pageContext, sId, false);
//        if (jsLines != null)
//        {
//            jsLines.clear();
//            pageContext.removeAttribute(JS_LINES);
//        }
//    }

    //==============================================================================================
    // Iteration count management
    //==============================================================================================
    /**
     * Set the iterator count
     * 
     * @param pageContext
     * @param iCount
     */
    public static void setIterationCount(PageContext pageContext, int iCount)
    {
        pageContext.setAttribute(ITERATOR_COUNT, new Integer(iCount));
    }

    /**
     * Get the iterator count
     * 
     * @param pageContext
     * @return
     */
    public static int getIterationCount(PageContext pageContext)
    {
        Object obj = pageContext.getAttribute(ITERATOR_COUNT);
        if (obj != null)
        {
            if (obj instanceof Integer)
            {
                return ((Integer) obj).intValue();
            }
            else
            {
                log.error("Page contex attribute '" + ITERATOR_COUNT + "' is not an instance of Integer !");
            }
        }
        return 0;
    }

    //==============================================================================================
    // Iteration index management
    //==============================================================================================
    /**
     * Set the iterator index
     * 
     * @param pageContext
     * @param iIndex
     */
    public static void setIterationIndex(PageContext pageContext, int iIndex)
    {
        pageContext.setAttribute(ITERATOR_INDEX, new Integer(iIndex));
    }

    /**
     * Get the iterator index
     * 
     * @param pageContext
     * @return
     */
    public static int getIterationIndex(PageContext pageContext)
    {
        Object obj = pageContext.getAttribute(ITERATOR_INDEX);
        if (obj != null)
        {
            if (obj instanceof Integer)
            {
                return ((Integer) obj).intValue();
            }
            else
            {
                log.error("Page contex attribute '" + ITERATOR_COUNT + "' is not an instance of Integer !");
            }
        }
        return 0;
    }
    
    //==============================================================================================
    // Loop management
    //==============================================================================================
    /**
     * Build the internal LoopManager name 
     * @param name
     * @return
     * @since v 1.0.1
     */
    private static String getLoopManagerName(String name)
    {
        if ( name != null )
        {
            return LOOP_MANAGER + "_" + name.trim();
        }
        return LOOP_MANAGER ;
    }
//  removed in v 1.0.1 : loop manager accessible only by name
//    /**
//     * Set the given LoopManager in the page context as the current anonymous LoopManager
//     * @param pageContext
//     * @param loop
//     */
//    public static void setLoopManager(PageContext pageContext, LoopManager loop )
//    {
//        // NB : Cannot set a pageContext attribute to NULL ( throws an exception )
//        if ( loop != null )
//        {
//            pageContext.setAttribute(LOOP_MANAGER, loop );
//        }
//        else
//        {
//            pageContext.removeAttribute(LOOP_MANAGER);
//        }
//    }
    /**
     * Set the given LoopManager in the page context with a specific name
     * @param pageContext
     * @param name
     * @param loop (if null remove)
     * @since v 1.0.1
     */
    public static void setLoopManager(PageContext pageContext, String name, LoopManager loop )
    {
        String internalName = getLoopManagerName(name);
        // NB : Cannot set a pageContext attribute to NULL ( throws an exception )
        if ( loop != null )
        {
            pageContext.setAttribute(internalName, loop );
        }
        else
        {
            pageContext.removeAttribute(internalName);
        }
    }
// removed in v 1.0.1 : loop manager accessible only by name
//    /**
//     * Returns the default anonymous LoopManager if any
//     * @param pageContext
//     * @return
//     */
//    public static LoopManager getLoopManager(PageContext pageContext )
//    {
//        Object obj = pageContext.getAttribute(LOOP_MANAGER);
//        if ( obj != null )
//        {
//            return (LoopManager) obj ;
//        }
//        return null ;        
//    }
    /**
     * Returns the named LoopManager if any
     * @param pageContext
     * @param name
     * @return
     * @since v 1.0.1
     */
    public static LoopManager getLoopManager(PageContext pageContext, String name )
    {
        String internalName = getLoopManagerName(name);
        Object obj = pageContext.getAttribute(internalName);
        if ( obj != null )
        {
            return (LoopManager) obj ;
        }
        return null ;        
    }

    //==============================================================================================
    // Current bean management
    //==============================================================================================
    /**
     * Set the current bean
     * 
     * @param pageContext
     * @param oBean :
     *            current bean to set
     */
    public static void setCurrentBean(PageContext pageContext, Object oBean)
    {
        // NB : Cannot set a pageContext attribute to NULL ( throws an exception )
        if ( oBean != null ) 
        {
            pageContext.setAttribute(CURRENT_BEAN, oBean);               
        }
        else
        {
            pageContext.removeAttribute(CURRENT_BEAN);
        }
    }

    /**
     * Get the current bean
     * 
     * @param pageContext
     * @return
     */
    public static Object getCurrentBean(PageContext pageContext)
    {
        return pageContext.getAttribute(CURRENT_BEAN);
    }
    
    /**
     * Clear the current bean ( reset for future use )
     * 
     * @param pageContext
     * @return
     */
    public static void clearCurrentBean(PageContext pageContext)
    {
        pageContext.removeAttribute(CURRENT_BEAN);
    }
    
    //==============================================================================================
    // List columns 
    //==============================================================================================
    public static ListColumns getListColumns(PageContext pageContext)
    {
        ListColumns cols = null ;
        Object obj = pageContext.getAttribute(LIST_COLUMNS);
        if ( obj == null )
        {
            cols = new ListColumns();
            pageContext.setAttribute(LIST_COLUMNS, cols);
        }
        else
        {
            cols = (ListColumns) obj ;
        }
        return cols ;
    }
    
    public static void setListHeader(PageContext pageContext, boolean bFlag)
    {
        pageContext.setAttribute(LIST_HEADER, Boolean.valueOf(bFlag) );
    }
    public static boolean inListHeader(PageContext pageContext)
    {
        Object obj = pageContext.getAttribute(LIST_HEADER);
        if ( obj != null )
        {
            Boolean b = (Boolean) obj ;
            return b.booleanValue() ;
        }
        return false ;
    }

    //----------------------------------------------------------------------------------
//    public static void setListBody(PageContext pageContext, boolean bFlag)
//    {
//        pageContext.setAttribute(LIST_BODY, Boolean.valueOf(bFlag) );
//    }

    private static int getListBodyLevel(PageContext pageContext) // v 1.0.1
    {
        Object obj = pageContext.getAttribute(LIST_BODY_LEVEL);
        if ( obj != null )
        {
            Integer i = (Integer) obj ;
            return i.intValue();
        }
        return 0 ;
    }
    private static void setListBodyLevel(PageContext pageContext, int iLevel) // v 1.0.1
    {
        if ( iLevel < 0 )
        {
            iLevel = 0 ;
        }
        pageContext.setAttribute(LIST_BODY_LEVEL, new Integer(iLevel) );
    }
    
    public static void openListBodyTag(PageContext pageContext) // v 1.0.1
    {
        int iLevel = getListBodyLevel(pageContext);
        iLevel++ ;
        setListBodyLevel(pageContext, iLevel);
    }
    
    public static void closeListBodyTag(PageContext pageContext) // v 1.0.1
    {
        int iLevel = getListBodyLevel(pageContext);
        iLevel-- ;
        setListBodyLevel(pageContext, iLevel);
    }
    
    public static boolean inNestedListBody(PageContext pageContext) // v 1.0.1
    {
        int iLevel = getListBodyLevel(pageContext);
        return ( iLevel > 1 );
    }
    
//    public static boolean inListBody(PageContext pageContext) 
//    {
//        Object obj = pageContext.getAttribute(LIST_BODY);
//        if ( obj != null )
//        {
//            Boolean b = (Boolean) obj ;
//            return b.booleanValue() ;
//        }
//        return false ;
//    }
    
    //==============================================================================================
    // List DIVs ( div width and ID ) 
    //==============================================================================================
    public static void setListDiv(PageContext pageContext, ListDiv listDiv)
    {
        // NB : Cannot set a pageContext attribute to NULL ( throws an exception )
        if ( listDiv != null )
        {
            pageContext.setAttribute(LIST_DIV, listDiv );            
        }
        else
        {
            pageContext.removeAttribute(LIST_DIV);
        }
    }
    
    public static ListDiv getListDiv(PageContext pageContext)
    {
        Object obj = pageContext.getAttribute(LIST_DIV);
        if ( obj != null )
        {
            return (ListDiv) obj ;
        }
        return null ;
    }
    
    //==============================================================================================
    // Request Environment 
    //==============================================================================================
    public static RequestEnv getRequestEnv(PageContext pageContext)
    {
        ServletRequest servletRequest = pageContext.getRequest();
        if ( servletRequest != null )
        {
            return RequestEnv.getEnv(servletRequest);
//            Object obj = servletRequest.getAttribute( REQUEST_ENV );
//            if ( obj != null )
//            {
//                if ( obj instanceof RequestEnv )
//                {
//                    return (RequestEnv) obj ;
//                }
//            }
        }
        return null ;
    }

    //==============================================================================================
    // Tree nodes 
    //==============================================================================================
    public static TreeViewNode getCurrentNode(PageContext pageContext)
    {
        if ( pageContext != null )
        {
	        Object obj = pageContext.getAttribute(TREE_CURRENT_NODE);
	        if ( obj != null )
	        {
	            if ( obj instanceof TreeViewNode )
	            {
	                return (TreeViewNode) obj ;
	            }
	        }
        }
        return null ;
    }
    //----------------------------------------------------------------
    public static void setCurrentNode( PageContext pageContext, TreeViewNode node )
    {
        if ( pageContext != null )
        {
            if ( node != null )
            {
    	        pageContext.setAttribute(TREE_CURRENT_NODE, node);
            }
            else
            {
                pageContext.removeAttribute(TREE_CURRENT_NODE);
            }
        }
    }
    //----------------------------------------------------------------
    public static TreeViewNode getRootNode(PageContext pageContext)
    {
        if ( pageContext != null )
        {
	        Object obj = pageContext.getAttribute(TREE_ROOT_NODE);
	        if ( obj != null )
	        {
	            if ( obj instanceof TreeViewNode )
	            {
	                return (TreeViewNode) obj ;
	            }
	        }
        }
        return null ;
    }
    //----------------------------------------------------------------
    public static void setRootNode( PageContext pageContext, TreeViewNode node )
    {
        if ( pageContext != null )
        {
            if ( node != null )
            {
    	        pageContext.setAttribute(TREE_ROOT_NODE, node);
            }
            else
            {
                pageContext.removeAttribute(TREE_ROOT_NODE);
            }
        }
    }
    //----------------------------------------------------------------
    public static boolean getTreeDnDDivFlag(PageContext pageContext)
    {
        if ( pageContext != null )
        {
	        Object obj = pageContext.getAttribute(TREE_DND_DIV_FLAG);
	        if ( obj != null )
	        {
	            return true ; // Found => Flag is set 
	        }
        }
        return false ;
    }
    //----------------------------------------------------------------
    public static void setTreeDnDDivFlag( PageContext pageContext )
    {
        if ( pageContext != null )
        {
	        pageContext.setAttribute(TREE_DND_DIV_FLAG, "done");
        }
    }
    
    //==============================================================================================
    // Menu items 
    //==============================================================================================
    public static MenuItems getCurrentMenuItems(PageContext pageContext)
    {
        if ( pageContext != null )
        {
	        Object obj = pageContext.getAttribute(MENU_ITEMS);
	        if ( obj != null )
	        {
	            if ( obj instanceof MenuItems )
	            {
	                return (MenuItems) obj ;
	            }
	        }
        }
        return null ;
    }
    //----------------------------------------------------------------
    public static void setCurrentMenuItems( PageContext pageContext, MenuItems items )
    {
        if ( pageContext != null )
        {
            if ( items != null )
            {
    	        pageContext.setAttribute(MENU_ITEMS, items);
            }
            else
            {
                pageContext.removeAttribute(MENU_ITEMS);
            }
        }
    }
    
    //==============================================================================================
    // Page variables management
    //==============================================================================================
    /**
     * Set a variable value in the PageContext ( stored as PageContext attribute )
     * @param pageContext
     * @param sVarName
     * @param oVarValue the variable value (if null remove the variable)
     */
    public static void setVar(PageContext pageContext, String sVarName, Object oVarValue)
    {
        if ( pageContext != null && sVarName != null )
        {
            // NB : Cannot set a pageContext attribute to NULL ( throws an exception )
            if ( oVarValue != null ) 
            {
                pageContext.setAttribute(sVarName, oVarValue);               
            }
            else
            {
                pageContext.removeAttribute(sVarName);
            }
        }
    }

    /**
     * Set a variable value from the PageContext ( retrieved from PageContext attribute )
     * @param pageContext
     * @param sVarName
     * @return
     */
    public static Object getVar(PageContext pageContext, String sVarName)
    {
        if ( pageContext != null && sVarName != null )
        {
            return pageContext.getAttribute(sVarName);
        }
        return null ;
    }    
    //----------------------------------------------------------------
}