package org.objectweb.telosys.uil.taglib;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.auth.IAppUser;
import org.objectweb.telosys.auth.LoginUser;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.data.DataRow;
import org.objectweb.telosys.common.data.DataSet;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.uil.screenmap.ScreenMap;
import org.objectweb.telosys.util.BeanUtil;
import org.objectweb.telosys.util.ObjUtil;
import org.objectweb.telosys.util.Parts;
import org.objectweb.telosys.util.StrUtil;
import org.objectweb.telosys.util.VarSetter;
import org.objectweb.telosys.util.web.ResourceURI;
import org.objectweb.telosys.util.web.ScreenURI;


/**
 * @author GUERIN
 *  
 */
public class TagVarSetter extends VarSetter
{
    PageContext _pageContext = null ;    
    
    //--------------------------------------------------------------------------------
    // CONSTRUCTOR
    //--------------------------------------------------------------------------------
    /**
     * @param pageContext
     */
    public TagVarSetter(PageContext pageContext)
    {
        super();
        if ( pageContext != null )
        {
            _pageContext = pageContext;            
        }
        else
        {
            throw new TelosysRuntimeException("TagVarSetter constructor : pageContext is null!");
        }
    }
    
    //--------------------------------------------------------------------------------
    private HttpSession getHttpSession()
    {
        if ( _pageContext != null )
        {
            return _pageContext.getSession();
        }
        return null ;
    }
    
    //--------------------------------------------------------------------------------
    /**
     * Returns the LoginUser if the user is logged, else null
     * @return
     */
    private LoginUser getLoginUser()
    {
        if ( _pageContext != null )
        {
            ScreenSession screenSession = ScreenSessionManager.findScreenSession(_pageContext);
            return screenSession != null ? screenSession.getLoginUser() : null ;
        }
        return null ;
    }
    
    //--------------------------------------------------------------------------------
    /**
     * Returns the IAppUser if the user is logged, else null
     * @return
     */
    private IAppUser getAppUser()
    {
        if ( _pageContext != null )
        {
            ScreenSession screenSession = ScreenSessionManager.findScreenSession(_pageContext);
            return screenSession != null ? screenSession.getAppUser() : null ;
        }
        return null ;
    }
    
    //--------------------------------------------------------------------------------
    /**
     * Return the integer value of the given string, or -1 if the parameter is not a valid positive integer
     * @param s
     * @return
     */
    private int getColumnNumber(String s)
    {
        if ( s != null )
        {
            // can be convert to int ?
            int i = StrUtil.getInt(s, -1); 
            if ( i >= 0 )
            {
                return i ;
            }
        }
        return -1 ;
    }
    
    //--------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.util.VarSetter#getVarValue(java.lang.String)
     * 
     * Search and returns the value for the given string 
     * The string is the content of the symbolic var { } 
     * e.g. "@{xxx}" s = "xxx" ( syntax : "value,default,format" )
     * Some value use cases :
     * Bean itself    : "bean_name"
     * Bean attribute : "bean.attribute", ".attribute",
     * DataRow column : "dr.3", "xxx.1", ".2" , "2",  ...
     * Prededined var : "#index", "#count", "#session-id", ...
     */
    protected String getVarValue(String s) 
    {
        trace("getVarValue(" + s + ")...");
        if ( s != null )
        {
            //---------------------------------------------------------------
            // Split the string in 1..3 parts
            //---------------------------------------------------------------
            String[] parts = Parts.getParts(s, ',', 10 );
            if ( parts.length == 0 ) return "{void!}";
            
            //--- 1) SYMBOLIC NAME
            String sSymbolicName = parts[0] ;
            if ( sSymbolicName.length() == 0 ) return "{void!}";
            
            //--- 2) DEFAULT VALUE
            String sDefaultValue = "" ;
            if ( parts.length >= 2 ) 
            {
                // There's a default value
                sDefaultValue = StrUtil.removeQuotes( parts[1], '\'' ) ;
            }
            
            //--- 3) FORMAT
            String sFormat = null ;
            if ( parts.length >= 3 ) 
            {
                // There's a format
                sFormat = StrUtil.removeQuotes( parts[2], '\'' ) ;
            }
            
            //---------------------------------------------------------------
            // Return the corresponding value ( using default and format if any )
            //---------------------------------------------------------------
            if ( sSymbolicName.charAt(0) == '#' ) // Starts with "#"
            {
                trace("getVarValue(" + s + ")... PREDEFINED VARIABLE");
                //--- Get PREDEFINED VARIABLE value    
//                String sValue = getPredefinedVariable( s2, sFormat ); // v 0.9.9
//                return ( sValue != null ? sValue : sDefaultValue ); // v 0.9.9
                return getPredefinedVariable( sSymbolicName, sDefaultValue, sFormat ); // v 0.9.9
            }
            else
            {                
	            char c = sSymbolicName.charAt(0);
	            if ( c >= '0' && c <= '9' ) // Numeric => Column  ( eg : "3", "2" ) 
	            {
	                trace("getVarValue(" + s + ")... COLUMN ( DATASET/DATAROW ) ");
	                //--- Get column value of the current DataRow    
//	                Object oValue = CurrentBean.getColumn(_pageContext, sSymbolicName);
//	                if ( oValue != null )
//	                {
//	                    return format(oValue, sFormat );
//	                }
//	                else
//	                {
//	                    return sDefaultValue ;
//	                }
	                
	                //--- No object => Current object 
	                String sValue = getBeanFieldValue(null, sSymbolicName, sFormat ); // v 0.9.9 ( 27/06/2008 )
	                return ( sValue != null ? sValue : sDefaultValue ); // v 0.9.9	( 27/06/2008 )                
	            }
	            else
	            {
	                trace("getVarValue(" + s + ")... BEAN ATTRIBUTE or COLUMN");
	                //--- Split the symbolic name ( "bean.attribute", ".attribute", "bean" )
	                String sBeanName = BeanUtil.getBeanName(sSymbolicName); // can be null ( no bean => current bean )
	                String sFieldName = BeanUtil.getAttributeName(sSymbolicName); // can be null ( no field => bean.toString() ) 

	                //--- Get bean, bean attribute, datarow column
	                String sValue = getBeanFieldValue(sBeanName, sFieldName, sFormat ); // v 0.9.9
	                return ( sValue != null ? sValue : sDefaultValue ); // v 0.9.9	                
	            }            
            }
        }
        return "{null!}";
    }
    
    //--------------------------------------------------------------------------------
    private String format( int i, String sFormat )
    {
        if ( sFormat != null )
        {
            return ObjUtil.format(new Integer(i), sFormat);
        }
        else
        {
            return ""+i;
        }
    }
    //--------------------------------------------------------------------------------
    private String format( Object oValue, String sFormat )
    {
        if ( sFormat != null )
        {
            return ObjUtil.format(oValue, sFormat);
        }
        else
        {
            return oValue.toString() ;
        }
    }
    
    //--------------------------------------------------------------------------------
    private String getPredefinedVariable( String sVarName, String sDefaultValue, String sFormat  )
    {
        trace("getBeanFieldValue('" + sVarName + "', '" + sDefaultValue + "', '" + sFormat + "' )...");
        String sValue = getPredefinedVariable( sVarName, sFormat ); 
        return ( sValue != null ? sValue : sDefaultValue );
    }
    //--------------------------------------------------------------------------------
    /**
     * Returns the value of the given predefined variable
     * @param sVarName
     * @return the value ( or null or "unknown" )
     */
    private String getPredefinedVariable( String sVarName, String sFormat )
    {
        trace("getBeanFieldValue('" + sVarName + "', '" + sFormat + "' )...");
        String s = sVarName.toLowerCase();
        if ( s.equals("#count") )
        {
            //--- Predefined variable "ITERATION COUNT"
            int iCount = Page.getIterationCount(_pageContext);
            return format(iCount, sFormat);
        }
        else if ( s.equals("#count-parity") )
        {
            //--- Predefined variable "ITERATION COUNT PARITY"
            int i = Page.getIterationCount(_pageContext);
            return i % 2 == 0 ? "even" : "odd" ;                
        }
        else if ( s.equals("#index") )
        {
            //--- Predefined variable "ITERATION INDEX"
            int iIndex = Page.getIterationIndex(_pageContext);
            return format(iIndex, sFormat);
        }
        else if ( s.equals("#index-parity") )
        {
            //--- Predefined variable "ITERATION INDEX PARITY"
            int iIndex = Page.getIterationIndex(_pageContext);
            return iIndex % 2 == 0 ? "even" : "odd" ;                
        }
        else if ( s.equals("#session-id") )
        {
            HttpSession sess = getHttpSession();
            return ( sess != null ? sess.getId() : null );
        }
        else if ( s.equals("#user-login") )
        {
            LoginUser user = getLoginUser();
            return ( user != null ? user.getLogin() : null );
        }
        else if ( s.equals("#user-language") )
        {
            IAppUser user = getAppUser();
            return ( user != null ? user.getLanguage() : null );
        }
        else if ( s.equals("#user-role") )
        {
            IAppUser user = getAppUser();
            return ( user != null ? user.getRole() : null );
        }
        else if ( s.equals("#screenmap-uri") )
        {
            HttpServletRequest req = ( HttpServletRequest ) _pageContext.getRequest() ;
            ScreenURI uri = new ScreenURI("");
            return uri.getContextURI( req );
        }
        else if ( s.equals("#screenmap-url") )
        {
            HttpServletRequest req = ( HttpServletRequest ) _pageContext.getRequest() ;
            ScreenURI uri = new ScreenURI("");
            return uri.getFullURL( req );
        }
        else if ( s.equals("#webapp-uri") )
        {
            HttpServletRequest req = ( HttpServletRequest ) _pageContext.getRequest() ;
            ResourceURI uri = new ResourceURI("");
            return uri.getContextURI( req );
        }
        else if ( s.equals("#webapp-url") )
        {
            HttpServletRequest req = ( HttpServletRequest ) _pageContext.getRequest() ;
            ResourceURI uri = new ResourceURI("");
            return uri.getFullURL( req ); 
        }
        else if ( s.equals("#context-id") )
        {
            //--- Returns the ID of the Context associated with the current ScreenMap
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return ""+screenMap.getScreenContextId(); // Never null ( always a default value )
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#context-name") )
        {
            //--- Returns the NAME of the Context associated with the current ScreenMap
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getScreenContextName(); // Null if none
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-name") )
        {
            //--- Returns the NAME of the current ScreenMap 
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getName(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-type") )
        {
            //--- Returns the TYPE of the current ScreenMap ( "html", "xul" )
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getType(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-path") )
        {
            //--- Returns the PATH of the current ScreenMap 
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getTargetPath(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-body") )
        {
            //--- Returns the Screen Body of the current ScreenMap 
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getScreenBody(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-body-path") )
        {
            //--- Returns the Screen Body PATH of the current ScreenMap 
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getScreenBodyPath(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-css") )
        {
            //--- Returns the CSS file of the current ScreenMap 
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getCss(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-css-path") )
        {
            //--- Returns the CSS PATH of the current ScreenMap 
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getCssPath(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-js") )
        {
            //--- Returns the JavaScript file of the current ScreenMap 
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getScript(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#screen-js-path") )
        {
            //--- Returns the JavaScript PATH of the current ScreenMap 
            ScreenMap screenMap = TagCommons.getScreenMap( _pageContext );
            if ( screenMap != null )
            {
                return screenMap.getScriptPath(); 
            }
            return null ; // No ScreenMap => null 
        }
        else if ( s.equals("#paramcount") ) 
        {
            HttpServletRequest req = ( HttpServletRequest ) _pageContext.getRequest() ;
            Map m = req.getParameterMap();
            if ( m != null ) return format(m.size(), sFormat);
            else return null ;
        }
        else 
        {
            if ( s.startsWith("#param-") ) // "#param-ParamName"
            {
                HttpServletRequest req = ( HttpServletRequest ) _pageContext.getRequest() ;
                String p = s.substring(7);
                return req.getParameter(p);
            }
        }
        return "('"+sVarName + "':unknown)";
    }
    
    //--------------------------------------------------------------------------------
    /**
     * Returns the value of a bean, or bean field name, or DataRow column
     *
     * @param sBeanName : the name of the bean ( if null search a "current bean" or "current DataRow" ) 
     * @param sFieldName : attribute name or column number
     * @param sFormat : the format to use
     * @return the attribute value ( can be null if the field value is null ) or error message 
     */
    private String getBeanFieldValue(String sBeanName, String sFieldName, String sFormat )
    {
        trace("getBeanFieldValue('" + sBeanName + "', '" + sFieldName + "', '" + sFormat + "' )...");
        //--- 1) try to find the bean 
        Object oBean = null;
        if (sBeanName != null)
        {
            //--- Try to find the object ( no SCOPE consideration )
            oBean = Finder.findObject(_pageContext, sBeanName );
        }
        else
        {
            //--- No bean name => Try to find the "current bean" object
            oBean = CurrentBean.getBean(_pageContext);
        }
        if ( oBean != null ) // Bean found
        {
            return getBeanFieldValue(oBean, sFieldName, sFormat);
        }
        else
        {
            if (sBeanName != null)
            {
	            // Unknown bean
	            //return "('" + sBeanName + "':unknown)"  ;
                return null ; // Not an error
            }
            else
            {
                // => ERROR
	            return "(no current bean)"  ;
            }
        }
    }
        
    //--------------------------------------------------------------------------------
    /**
     * Returns the attribute's value or column's value of the given object
     * @param oBean : Standard Java bean or DataRow
     * @param sFieldName : bean attribute name or DataRow column number (1 to N)
     * @param sFormat
     * @return
     */
    private String getBeanFieldValue(Object oBean, String sFieldName, String sFormat )
    {
        trace("getBeanFieldValue( Object, '" + sFieldName + "', '" + sFormat + "' )...");
        if ( oBean != null )
        {
            //--- Try to find the attribute of the bean 
            if ( sFieldName != null )
            {
                Object oField = null ;
                int iColumnNumber = getColumnNumber(sFieldName) ;
                if ( iColumnNumber >= 0 ) // is it a column number ?
                {
                    //--- DataRow column number
                    oField = getColumnValue(oBean, iColumnNumber);
                }
                else
                {
                    //--- Classical attribute bean
                    oField = getAttributeValue(oBean, sFieldName);
                }
                if ( oField != null )
                {
                    //return oField.toString();
                    return format(oField, sFormat);
                }
                else
                {
                    return null ; // field found, but null ( not an error )
                }
            }
            else
            {
                //--- No attribute name => return the object itself as value
                //return oBean.toString() ;
                return format(oBean, sFormat);
            }
        }
        return null ;
    }
    
    //--------------------------------------------------------------------------------
    /**
     * Returns the column's value of the given DataRow object  
     * @param oBean
     * @param iCol  ( 1 to N expected )
     * @return
     */
    private Object getColumnValue ( Object oBean, int iCol )
    {
        trace("getColumnValue( Object," + iCol + " )...");
        Object oRet = null ;
        if (oBean instanceof DataRow)
        {
            DataRow dr = (DataRow) oBean;
            if (iCol > 0 && iCol <= dr.getSize()) // 1..N 
            {
                oRet = dr.getObject(iCol) ;
            }
            else
            {
                // => ERROR
                oRet = "('" + iCol + "':invalid col)" ;
            }
        }
        else
        {
            // => ERROR
            oRet = "(not a DataRow)" ;
        }
        
        String sRet = "null";
        if ( oRet != null )
        {
            sRet = "'" + oRet.toString() + "' (class " + oRet.getClass().getName() + ")" ;
        }
        trace("getColumnValue( Object," + iCol + " ) return " + sRet );
        
        return oRet ;
    }
    
    //--------------------------------------------------------------------------------
//    private boolean isList ( Object o )
//    {
//        if ( o != null )
//        {
//            if ( o instanceof List )
//            {
//                return true ;
//            }
//        }
//        return false ;
//    }
    //--------------------------------------------------------------------------------
    private static final String SIZE = "size" ;
    private Object getListAttribute ( Object o, String attr ) // v 0.9.9
    {
        if ( o == null || attr == null )
        {
            return null ;
        }
        if ( o instanceof Collection )
        {
            Collection list = (Collection) o ;
            if ( attr.equals(SIZE) )
            {
                return new Integer( list.size() );
            }
        }
        if ( o instanceof GenericVOList )
        {
            GenericVOList list = (GenericVOList) o ;
            if ( attr.equals(SIZE) )
            {
                return new Integer( list.size() );
            }
        }
        if ( o instanceof DataSet )
        {
            DataSet ds = (DataSet) o ;
            if ( attr.equals(SIZE) )
            {
                return new Integer( ds.getRowCount() );
            }
        }
        return null ;
    }
    
    //--------------------------------------------------------------------------------
    /**
     * Returns the attribute's value of a standard Java bean 
     * @param oBean
     * @param sFieldName
     * @return
     */
    private Object getAttributeValue ( Object oBean, String sFieldName )
    {
        trace("getAttributeValue( Object, '" + sFieldName + "' )...");
        
        Object oValue = null ;
        
        //--- Try to find a "List" attribute ( "size" )
        if ( ( oValue = getListAttribute ( oBean, sFieldName ) ) != null ) // v 0.9.9
        {
            return oValue ;
        }
        else
        {            
	        //--- Try to find a standard Bean attribute ( obj.getXxxxx ) 
            try
	        {
	            //--- Is there a "getFieldName()" or "isFieldName()" accessor ?
	            oValue = BeanUtil.getFieldValue(oBean, sFieldName);
	        } catch (TelosysException e)
	        {
	            // Unknown field => ERROR
	            oValue = "('" + sFieldName + "':unknown)"  ;
	        }
        }
        return oValue ;
    }
    
}