package org.objectweb.telosys.uil.taglib.test;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * This tag handler is design only for test <br>
 * It outputs information about the different 'TagSupport' methods calls
 * 
 * @author Laurent GUERIN
 *  
 */
public class LifeCycle extends TagSupport
{
    private String attr1 = null ;
    
    private String attr2 = null ;

    //----------------------------------------------------------------    
    private void printConsole(String s)
    {
        System.out.println("((( LifeCycle TAG ))) : " + s );
    }
    //----------------------------------------------------------------    
    private void printPage(String s)
    {
        if ( pageContext != null )
        {            
	        JspWriter out = pageContext.getOut();
	        try
	        {
	            out.println("<!-- ((( LifeCycle TAG ))) : " + s + "-->");
	        } catch (IOException e)
	        {
	            trace("IOException in doStartTag() ");
	            e.printStackTrace();
	        }
        }
    }
    //----------------------------------------------------------------    
    private void trace(String s)
    {
        printConsole( s );
        if ( pageContext != null )
        {
            printPage(s);
        }
    }
    
    //----------------------------------------------------------------
    // Constructor
    //----------------------------------------------------------------

    public LifeCycle()
    {
        trace("===> new LifeCycle() ( constructor )");
    }

    //----------------------------------------------------------------
    // Methods inherited from TagSupport class
    //----------------------------------------------------------------

    public void setPageContext(PageContext p)
    {
        super.setPageContext(p);
        trace("----- (new tag usage) ...");
        trace("setPageContext(PageContext p)");
    }

    //----------------------------------------------------------------
    public void setParent(Tag t)
    {
        super.setParent(t);
        trace("setParent(Tag t)");
    }

    public Tag getParent()
    {
        trace("getParent()");
        return super.getParent();
    }

    //----------------------------------------------------------------
    // "id" attribute ( defined in TagSupport class )
    //----------------------------------------------------------------
    public void setId(String s)
    {
        super.setId(s);
        trace("setId('"+s+"')");
    }

    public String getId()
    {
        return super.getId();
    }

    //----------------------------------------------------------------
    // "attr1" attribute
    //----------------------------------------------------------------
    public void setAttr1(String s)
    {
        trace("setAttr1('" + s + "')" );
        attr1 = s;
    }
    public String getAttr1()
    {
        return attr1;
    }

    //----------------------------------------------------------------
    // "attr2" attribute
    //----------------------------------------------------------------
    public void setAttr2(String s)
    {
        trace("setAttr2('" + s + "')" );
        attr2 = s;
    }
    public String getAttr2()
    {
        return attr2;
    }

    //----------------------------------------------------------------
    public void setValue(String s, Object o)
    {
        trace("setValue('" + s + "', object)" );
        super.setValue(s, o);
    }
    public Object getValue(String s)
    {
        trace("getValue('" + s + "')" );
        return super.getValue(s);
    }
    public void removeValue(String s)
    {
        trace("removeValue('" + s + "')" );
        super.removeValue(s);
    }

    //----------------------------------------------------------------
    public int doStartTag() throws JspException
    {
        trace("doStartTag() ( return SKIP_BODY )" );
//        JspWriter out = pageContext.getOut();
//        try
//        {
//            out.println("<!-- LifeCycle test tag : id = '" + getId() + "'" 
//                    + " | attr1 = '" + getAttr1() + "'" 
//                    + " | attr2 = '" + getAttr2() + "'" 
//                    + "-->");
//        } catch (IOException e)
//        {
//            trace("IOException in doStartTag() ");
//            e.printStackTrace();
//        }
        String s = "doStartTag() attributes : "
            + " id = '" + getId() + "'" 
            + " | attr1 = '" + getAttr1() + "'" 
            + " | attr2 = '" + getAttr2() + "'" ;
        printPage(s);
        
        return SKIP_BODY;
    }

    //----------------------------------------------------------------
    public int doAfterBody() throws JspException
    {
        trace("doAfterBody()" );
        return super.doAfterBody();
    }
    
    //----------------------------------------------------------------
    public int doEndTag() throws JspException
    {
        trace("doEndTag()" );
        String s = "doEndTag() attributes : "
            + " id = '" + getId() + "'" 
            + " | attr1 = '" + getAttr1() + "'" 
            + " | attr2 = '" + getAttr2() + "'" ;
        printPage(s);
        return super.doEndTag();
    }
    
    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#release()
     * Called before JSP reuse (not called in the first use of the JSP) 
     */
    public void release()
    {
        trace("release()");
    }

}

