package org.objectweb.telosys.uil.taglib.util;

import org.objectweb.telosys.uil.taglib.CommonTagSupport;
import org.objectweb.telosys.uil.taglib.LoopManager;
import org.objectweb.telosys.uil.taglib.Page;

/**
 * @author Laurent GUERIN
 *  
 */
public class Loop extends CommonTagSupport 
{
    //----------------------------------------------------------------
    // Private variables
    //----------------------------------------------------------------
    private int    _iFrom  = 0;

    private int    _iTo    = 0;

    private int    _iStep  = 0;

    //----------------------------------------------------------------
    // Tag attributes
    //----------------------------------------------------------------
    private String _sFrom  = null;

    private String _sTo    = null;

    private String _sStep  = "1";

    private String _sLoopName  = "loop"; // v 1.0.1

    public void setFrom(String v)
    {
        this._sFrom = v;
    }

    public void setTo(String v)
    {
        this._sTo = v;
    }

    public void setStep(String v)
    {
        this._sStep = v;
    }

    public void setLoopname(String v) // v 1.0.1
    {
        this._sLoopName = v;
    }

    //----------------------------------------------------------------
    private boolean init()
    {
        String s = "";
        try
        {
            s = "from";
            _iFrom = Integer.parseInt(_sFrom);
            
            s = "to";
            _iTo = Integer.parseInt(_sTo);
            
            _iStep = 1 ;
            s = "step";
            if ( _sStep != null )
            {
                if ( _sStep.trim().length() > 0 )
                {
                    _iStep = Integer.parseInt(_sStep);                    
                }
            }
        } catch (NumberFormatException ex)
        {
            tagLibError("<!-- LOOP ERROR : Cannot parse attribute '" + s + "' -->");
            return false;
        }
        return true;
    }

    //----------------------------------------------------------------
    public int doStartTag()
    {
        //Page.setLoopManager(pageContext, null); // if NULL => remove
        // v 1.0.1 : do not remove the default LoopManager
        
        if (init())
        {
            LoopManager loopManager = new LoopManager ( pageContext, _iFrom, _iTo, _iStep );   
            //--- Set the loop manager in the current page context
            //Page.setLoopManager(pageContext, loopManager);
            Page.setLoopManager(pageContext, _sLoopName, loopManager); // v 1.0.1 use a LoopManager name
            if ( loopManager.first() )
            {
                return EVAL_BODY_INCLUDE;
            }
        }
        return SKIP_BODY;
    }

    //----------------------------------------------------------------
    public int doAfterBody()
    {
        //--- Retrieve the LOOP MANAGER
        //LoopManager loopManager = Page.getLoopManager(pageContext) ;   
        LoopManager loopManager = Page.getLoopManager(pageContext, _sLoopName ) ;   // v 1.0.1 use a LoopManager name
        //--- If there's a NEXT ITEM in the loop
        if ( loopManager != null )
        {
            if ( loopManager.next() )
            {
                //--- Another iteration ...
                return EVAL_BODY_AGAIN;
            }
        }

        //--- It's the end ( no other iteration ) 
        return SKIP_BODY;                               
    }
    //----------------------------------------------------------------
}