package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.widget.html.CheckBoxHTML;
import org.objectweb.telosys.uil.taglib.widget.xul.CheckBoxXUL;

/**
 * @author Laurent GUERIN
 *  
 */
public class CheckBox extends GenericPostbackTag // GenericTag 
{
    private static final String  TAG_NAME   = "checkbox";

    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget $htmlGen   = new CheckBoxHTML();

    private final static IWidget $xulGen    = new CheckBoxXUL();

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
    private String               _sChecked  = "false";

    private String               _sValueOff = null ;

    private String               _sTxtPos   = "right";           // "r", "right", "l", "left"

    /**
     * Constructor
     */
    public CheckBox()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
        setIdGeneration(true); // Automatic ID generation (if ID not set by user)
    }

    //----------------------------------------------------------------
//  moved in super class ( v 0.9.8 )
//    public String getValue() // Overrides the standard method to use POSTBACK
//    {
//        trace("getValue()...");
//        String sValue = getValueWithPostback();
//        trace("getValue() : return " + sValue );
//        return sValue;
//    }

    //----------------------------------------------------------------
    public void setValueoff(String v)
    {
        _sValueOff = v;
    }

    public String getValueoff()
    {
        //return _sValueOff; // v 0.9.9 
        return replaceSymbolicVar(_sValueOff); // v 0.9.9               
    }

    //----------------------------------------------------------------
    public void setTxtpos(String v)
    {
        _sTxtPos = v;
    }

    public String getTxtpos()
    {
        return _sTxtPos;
    }
    
    //----------------------------------------------------------------
    public void setChecked(String v)
    {
        _sChecked = v;
    }

// v 0.9.9
//    public void setChecked(boolean bFlag)
//    {
//        _sChecked = (bFlag == true ? "true" : "false" ) ;
//    }

    public String getChecked()
    {
        return _sChecked;
    }

//    public boolean isChecked() 
    public boolean getCheckedAsBoolean() // v 0.9.9  to avoid "setChecked(true/false)"
    {
        trace("getCheckedAsBoolean() ... _sChecked = '" + _sChecked + "'");
        //--- If postback is enabled AND there's is already a ScreenPostback in request or session scope
        if ( isPostbackEnabled() && ( getScreenPostback() != null ) ) 
        {
            trace("getCheckedAsBoolean() : postback is enabled and there's a ScreenPostback");
            if ( getPostbackValue() != null )
            {
                trace("getCheckedAsBoolean() : postback value = '" + getPostbackValue() + "' => return TRUE " );
                //--- There is a postback value for this checkbox => transmitted => checked
                return true ;
            }
            else
            {
                trace("getCheckedAsBoolean() : postback value is NULL => return FALSE " );
                return false ;
            }
        }
        else
        {
            //--- The postback management is disabled or not yet initialized => Just use the "checked" attribute 
            if (_sChecked != null)
            {
                // if (_sChecked.trim().equalsIgnoreCase("true")) // v 0.9.9
                String s = replaceSymbolicVar( _sChecked ); // v 0.9.9
                if ( s != null )
                {
                    return s.trim().equalsIgnoreCase("true") ; // v 0.9.9
                }
            }
            return false ;
        }        
    }


    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        startTag();
        return (SKIP_BODY);
    }
    
    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag()
    {
        endTag();
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }

}