package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.widget.html.FieldComboHTML;
import org.objectweb.telosys.util.StrUtil;

/**
 * @author Aurelien COUERON 
 * 
 */
public class FieldCombo extends Field implements ItemsContainer
{
    private static final String  TAG_NAME       = "fieldcombo";

    //----------------------------------------------------------------
    // Internal field type
    //----------------------------------------------------------------
    public static final int      WIDTH_DEFAULT    = 120;
    public static final int      HEIGHT_DEFAULT   =  19;
    public static final String   PICTO_DEFAULT    = "fieldcombo.bmp";
    public static final String   LISTSIZE_DEFAULT = "5";
    
    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget $htmlGen       = new FieldComboHTML();

    private final static IWidget $xulGen        = null;

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
    private String               _sListcl        = null;

    private String               _sListsize      = null;

    //----------------------------------------------------------------

    /**
     * Constructor
     */
    public FieldCombo()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
        setIdGeneration(true); // Automatic ID generation (if ID not set by user)        
    }
    
    //  -- OVERRIDE WIDTH AND HEIGHT --------------------------------
    public int getWidthAsInt()
    {        
        if (notVoid(super.getWidth()))
        {
        	return StrUtil.getInt ( super.getWidth() , WIDTH_DEFAULT ) ;        	
        }
        return WIDTH_DEFAULT;
    }
    public int getHeightAsInt()
    {
        if (notVoid(super.getHeight()))
        {
        	return StrUtil.getInt ( super.getHeight() , WIDTH_DEFAULT ) ;
        }
        return HEIGHT_DEFAULT;
    }

    //----------------------------------------------------------------
    public void setListcl(String v)
    {
        _sListcl = v;
    }

    public String getListcl()
    {
        return _sListcl;
    }
    
//  ----------------------------------------------------------------
    public void setListsize(String v)
    {
    	_sListsize = v;
    }

    public String getListsize()
    {
        if (notVoid(_sListsize))
        {
        	return _sListsize;            
        }
        return LISTSIZE_DEFAULT;
    }

    //----------------------------------------------------------------
    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        startTag();
        return (EVAL_BODY_INCLUDE);
    }

    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag()
    {
        endTag();
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }

    //----------------------------------------------------------------
    public String getFullPictoImageSrc()
    {
        String sPicto = getPicto();
        //--- If a specific PICTO as been set by the user 
    	if (notVoid(sPicto)) 
    	{
    	    //--- Get the absolute URL for the project IMAGE DIR
            return getFullImageSrc(sPicto);
    	}
    	else
    	{
    	    //--- Get the absolute URL for the default TELOSYS IMAGE DIR
    	    return getFullTelosysImageSrc(PICTO_DEFAULT);
    	}        
    }

}