package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.widget.GenericTag;

/**
 * Generic ancestor class for all tags generators (HTML, XUL, ...)
 * 
 * @author Laurent GUERIN
 *  
 */
public abstract class GeneratorAncestor
{
    protected final static int     POS_NONE = 1;

    protected final static int     POS_XY   = 2;

    protected final static boolean USE_VAR  = true;

    //protected final static boolean NOT_USE_VAR = false ;

    /**
     * Returns true if the string is not null and not void ("") and not blank ( " ", "  ", ... )
     * 
     * @param sTxt
     * @return
     */
    protected boolean notVoid(String sTxt)
    {
        if (sTxt != null)
        {
            if (sTxt.trim().length() > 0)
            {
                return true; // not void
            }
        }
        return false;
    }

    /**
     * Returns true if the string is null or void
     * 
     * @param sTxt
     * @return
     */
    protected boolean voidOrNull(String sTxt)
    {
        return !notVoid(sTxt);
    }

    /**
     * Returns the html "id" attribute ( ie : 'id="xx"' ) using the tag id
     * 
     * @param tag
     * @return 'id="xx"' ( or '' if ID is null )
     */
    protected String getIdAttr(GenericTag tag)
    {
        return getIdAttr(tag.getId());
        /**
         * String sId = tag.getId(); if (sId != null) { return "id=\"" + sId + "\""; } else { return ""; }
         */
    }

    /**
     * Returns the html "id" attribute ( ie : 'id="xx"' )
     * 
     * @param sId
     * @return 'id="xx"' ( or '' if ID is null )
     */
    protected String getIdAttr(String sId)
    {
        if (sId != null)
        {
            return "id=\"" + sId + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "id" attribute concat with index( ie : 'id="xx_nn"' )
     * 
     * @param tag
     * @param iIndex
     *            to concat
     * @return
     */
    protected String getIdAttr(GenericTag tag, int iIndex)
    {
        String sId = tag.getId();
        if (sId != null)
        {
            return "id=\"" + sId + "_" + iIndex + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "class" attribute ( ie : 'class="xx"' )
     * 
     * @param tag
     * @return
     */
    protected String getClassAttr(GenericTag tag)
    {
        String sClass = tag.getCl();
        if (sClass != null)
        {
            return "class=\"" + sClass + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "class" attribute ( ie : 'class="xx"' ),<br>
     * using the default parameter if the tag attribute is not set
     * 
     * @param tag
     * @param sDefaultClass :
     *            the class to use if the tag class is null
     * @return
     */
    protected String getClassAttr(GenericTag tag, String sDefaultClass)
    {
        String sClass = tag.getCl();
        if (sClass != null)
        {
            return "class=\"" + sClass + "\"";
        }
        else
        {
            if (sDefaultClass != null)
            {
                return "class=\"" + sDefaultClass + "\"";
            }
        }
        return "";
    }

    /**
     * Returns the html "tabindex" attribute or void ( ie : 'tabindex="xx"' )
     * 
     * @param tag
     * @return
     */
    protected String getTabindexAttr(GenericTag tag)
    {
        String s = tag.getTabindex();
        if (s != null)
        {
            return "tabindex=\"" + s + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "name" attribute or void ( ie : 'name="xx"' )
     * @param tag
     * @return
     */
    protected String getNameAttr(GenericTag tag)
    {
        String s = tag.getName();
        if (s != null)
        {
            return "name=\"" + s + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "disabled" attribute or void ( ie : 'disabled="disabled"' )
     * @param tag
     * @return
     */
    protected String getDisabledAttr(GenericTag tag)
    {
        if (tag.getDisabledAsBoolean())
        {
            return "disabled=\"disabled\"" ;
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "readonly" attribute or void ( ie : 'readonly="readonly"' )
     * @param tag
     * @return
     */
    protected String getReadonlyAttr(GenericTag tag)
    {
        if (tag.getReadonlyAsBoolean())
        {
            return "readonly=\"readonly\"" ;
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "accesskey" attribute or void ( ie : 'accesskey="x"' )
     * @param tag
     * @return
     */
    protected String getAccesskeyAttr(GenericTag tag)
    {
        String s = tag.getAccesskey();
        if ( s != null )
        {
            return "accesskey=\"" + s.trim() + "\"";
        }
        else
        {
            return "";
        }
    }

}