package org.objectweb.telosys.uil.taglib.widget;

import java.io.IOException;
import java.util.Iterator;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.common.data.ApplicationDataHolder;
import org.objectweb.telosys.common.data.ListItemProvider;
import org.objectweb.telosys.uil.taglib.Finder;
import org.objectweb.telosys.uil.taglib.LoopManager;
import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.TagCommons;
import org.objectweb.telosys.uil.taglib.widget.html.GenericItemsListHTML;
import org.objectweb.telosys.uil.taglib.widget.xul.GenericItemsListXUL;

/**
 * @author Laurent GUERIN
 *  
 */
public class GenericItemsListTag extends GenericTag 
{
    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static GenericItemsListHTML  $htmlGen  = new GenericItemsListHTML();

    private final static GenericItemsListXUL   $xulGen   = new GenericItemsListXUL();

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
    private String     _sProvider = null ;
    
    private String     _sList     = null ;
    
    private String     _sScope    = null ; 
    
    private String     _sValue    = null ;

    private String     _sLoopName = "items"; // v 1.0.1

    private String     _sItemName = null ; // v 1.0.1

    //  "txt"   is inherited
    
    /**
     * Constructor
     */
    public GenericItemsListTag(String sTagName)
    {
        super(sTagName, null, null);
        _sLoopName = sTagName ; // "comboitems" or "listitems"
    }

    //----------------------------------------------------------------
    public void setProvider(String v)
    {
        _sProvider = v;
    }
    public String getProvider()
    {
        return _sProvider ;
    }
    
    //----------------------------------------------------------------
    public void setList(String v)
    {
        _sList = v;
    }
    public String getList()
    {
        return _sList ;
    }
    
    //----------------------------------------------------------------
    public void setScope(String v)
    {
        _sScope = v;
    }
    public String getScope()
    {
        return _sScope ;
    }

    //----------------------------------------------------------------
    public void setValue(String v)
    {
        _sValue = v;
    }
	public String getValue() 
	{
		return _sValue;
		
	}
    //----------------------------------------------------------------
    public void setItemname(String v) // v 1.0.1
    {
        _sItemName = v;
    }
    //----------------------------------------------------------------
    private boolean isHTML()
    {
        String sScreenType = TagCommons.getScreenType(pageContext);
        if ( sScreenType != null )
        {
            return TagCommons.isHTML(sScreenType) ;            
        }
        return true ; // HTML by default 
    }
    
    //--------------------------------------------------------------------------------------------
    // 
    //--------------------------------------------------------------------------------------------
    public int doStartTag()
    {
        // No output generation at this level, but startTag() must be called for tag init
        startTag(NO_OUTPUT);

        boolean bHtmlType = isHTML();

        JspWriter out = pageContext.getOut();

        //--- Get the Application Data
        if ( notVoid ( _sProvider ) )
        {
            printItemsFromProvider(out, _sProvider, bHtmlType );
        }
        else if ( notVoid ( _sList ) )
        {
            printItemsFromDataList(out, _sList, _sScope, bHtmlType ) ;
        }
        else
        {
            printItem(out, bHtmlType, 0, "", "ERROR : no 'data' or 'provider' ", null );
        }
        
        return (SKIP_BODY);
    }
    
    //--------------------------------------------------------------------------------------------
    /**
     * Print items from 'provider' attribute
     * @param out
     * @param sProviderName
     * @param bHtmlType
     */
    private void printItemsFromProvider(JspWriter out, String sProviderName, boolean bHtmlType ) 
    {
        ListItemProvider listItemProvider = ApplicationDataHolder.getListItemProvider( sProviderName );
        if (listItemProvider != null)
        {
            listItemProvider.beforeUse();
            int iRowCount = listItemProvider.getRowCount();
            String sVal = "";
            String sTxt = "???";
            //--- Generate each item as a "combo item"
            for ( int iRow = 1 ; iRow <= iRowCount ; iRow++ )
            {
                sVal = listItemProvider.getListItemValue(iRow);
                sTxt = listItemProvider.getListItemText(iRow);
                //--- Print the list item 
                printItem(out, bHtmlType, iRow, sVal, sTxt, null );
            }
            listItemProvider.afterUse();
        }
        else
        {
            printItem(out, bHtmlType, 0, "", "ERROR : '" + sProviderName + "' not found", null );
        }
    }
    
    //--------------------------------------------------------------------------------------------
    private void printItemsFromDataList(JspWriter out, String sDataList, String sScope, boolean bHtmlType ) 
    {
        //--- Is there a current loop manager ? ( if the combo/list box is nested in a loop )
        //LoopManager memoLoopManager = Page.getLoopManager(pageContext); // v 1.0.1 useless since loop name
        
        //--- Search the list iterator in the given scope ( search everywhere if scope is null )
        Iterator iter = Finder.findIterator(pageContext, sDataList, sScope);
        //--- Init a loop manager ( with the ITERATOR )
        if ( iter != null )
        {
            int iIndexBase = 0 ;
            //--- New loop manager for the combo/list box items
            LoopManager loopManager = new LoopManager ( pageContext, iter, iIndexBase );  
            //--- If specific item variable name
            if ( _sItemName != null ) // v 1.0.1
            {
                loopManager.setItemName( _sItemName ); // v 1.0.1
            }
            //--- Set the loop manager in the current page context
            //Page.setLoopManager(pageContext, loopManager);
            Page.setLoopManager(pageContext, _sLoopName, loopManager); // v 1.0.1 use a LoopManager name
            if ( loopManager.first() )
            {
//                TagVarSetter varSetter = new TagVarSetter(pageContext);
                int iRow = 0 ;
                String sVal = "" ;
                String sTxt = "" ;
                String sTitle = "" ;
                do 
                {
                    //--- Print the list item 
                    iRow++ ;
//                    sVal   = varSetter.replaceVar(_sValue);
//                    sTxt   = varSetter.replaceVar(getTxt());   // NO Symbolic var replacement done by getTxt() because no "super.startTag"
//                    sTitle = varSetter.replaceVar(getTitle()); // NO Symbolic var replacement done by getTitle() because no "super.startTag"
                    sVal   = replaceSymbolicVar(_sValue);
                    sTxt   = getTxt();   // Symbolic var replacement done by getTxt() 
                    sTitle = getTitle(); // Symbolic var replacement done by getTitle() 
                    printItem(out, bHtmlType, iRow, sVal, sTxt, sTitle );
                } while ( loopManager.next() );

            }
            //--- Restore the original loop manager if any 
            // Page.setLoopManager(pageContext, memoLoopManager); // v 1.0.1 useless since loop name
        }
    }
    
    //--------------------------------------------------------------------------------------------
    /**
     * Prints a given item of the list provider
     * @param out
     * @param bHtmlType
     * @param iRow
     * @param sVal
     * @param sTxt
     * @param sTitle
     */
    private void printItem(JspWriter out, boolean bHtmlType, int iRow, String sVal, String sTxt, String sTitle) 
    {
        try
        {
            if ( sVal == null ) sVal = "" ;
            if ( sTxt == null ) sTxt = "???" ;
            
            boolean bSelected = false ;
            if (sameValueAsParent( sVal )) 
            {
                //--- Item value is the same as its parent value => selected
                bSelected = true; 
            }

            //--- Item generation 
            if (bHtmlType)
            {
                $htmlGen.printItem(out, this, iRow, sVal, sTxt, sTitle, bSelected);
            }
            else
            {
                $xulGen.printItem(out, this, iRow, sVal, sTxt, bSelected);
            }
        } catch (IOException ioex)
        {
            ioex.printStackTrace();
        }
    }    
}