package org.objectweb.telosys.uil.taglib.widget;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.Tag;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TraceConfig;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenApplicationManager;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.uil.screenmap.ScreenDefConfig;
import org.objectweb.telosys.uil.screenmap.ScreenMap;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;
import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.TagCommons;
import org.objectweb.telosys.uil.taglib.TagConst;
import org.objectweb.telosys.uil.taglib.TagVarSetter;
import org.objectweb.telosys.util.StrUtil;
import org.objectweb.telosys.util.VarSetter;

/**
 * @author Laurent GUERIN
 *  
 */
public class GenericTag extends BodyTagSupport
{
    protected static final int  NO_OUTPUT = 0 ;
    protected static final int  OUTPUT    = 1 ;
    
    //----------------------------------------------------------------
    // Trace management ( just like TelosysObject )
    //----------------------------------------------------------------
    protected int               _iLastTraceConfigState = -1;

    protected boolean           _bTrace                = false;

    //----------------------------------------------------------------
    // Widget generator implementations : HTML, XUL
    //----------------------------------------------------------------
    private IWidget             _htmlGenerator         = null;

    private IWidget             _xulGenerator          = null;

    //----------------------------------------------------------------
    // Widget tag name
    //----------------------------------------------------------------
    private String              _sTagName              = null;

    //----------------------------------------------------------------
    // Style values (CSS)
    //----------------------------------------------------------------
    private HashMap             _hmStyle               = new HashMap();

    //----------------------------------------------------------------
    // Standard tags attributes
    //----------------------------------------------------------------

    private String              _sId                   = null;

    private String              _sClass                = null;

    private String              _sStyle                = null;

    private String              _sTitle                = null;

    //----------------------------------------------------------------

    private String              _sX                    = null;

    private String              _sY                    = null;

    private String              _sWidth                = null;

    private String              _sHeight               = null;

    private String              _sTabIndex             = null;

    private String              _sZone                 = null;

    private String              _sDisabled             = null;

    private String              _sReadonly             = null;

    private String              _sName                 = null;

    private String              _sData                 = null;

    private String              _sDataElement          = null;

    private String              _sDataAttribute        = null;
    
    private String              _sTxt                  = null ;

    private String              _sAccessKey            = null ; // v 0.9.9 
    
    //----------------------------------------------------------------
    // Events
    //----------------------------------------------------------------
    private HashMap             _hmEvents              = new HashMap();

    //----------------------------------------------------------------
    // Other instance variables
    //----------------------------------------------------------------

    //--- Automatic ID generation
    // NB : the same instance can be used by different tags in the JSP ( cf. tag pool )
    // so the _sGeneratedId variable must be reset at each "doStartTag" -> "startTag" 

    private String              _sGeneratedId          = null;

    private boolean             _bIdGeneration         = false; // No automatic ID generation by default

    private VarSetter           _varSetter             = null;

    /**
     *  
     */
    public GenericTag(String sTagName, IWidget htmlGen, IWidget xulGen)
    {
        super();
        _sTagName = sTagName;
        _htmlGenerator = htmlGen;
        _xulGenerator = xulGen;
        
        // NB: Do not use "pageContext" before "doStartTag()" not yet set in the constructor !
        // Tag usage in the generated servlet :
        // _jspx_the_tag_0.setPageContext(pageContext); // SET HERE 
        // _jspx_the_tag_0.setParent(...);
        // _jspx_the_tag_0.setAnAttribute(...);
        // int _jspx_eval_the_tag_0 = _jspx_the_tag_0.doStartTag();

        //--- Create a "Variables Setter" for the current page context
        //_varSetter = new TagVarSetter( pageContext ); // v 0.9.8

    }
    
    /* Overrides the "TagSupport.setPageContext()" method <br>
     * Just for tag lifecycle trace
     * @see javax.servlet.jsp.tagext.Tag#setPageContext(javax.servlet.jsp.PageContext)
     */
    public void setPageContext(PageContext p)
    {
        super.setPageContext(p);
        trace("setPageContext(PageContext p) -----> new tag usage ");
    }

    /**
     * Returns the tag generator according to the screen type ( HTML, XUL )
     * @return the generator or null if 
     */
    private IWidget getGenerator()
    {
        IWidget gen = null ;
        String  sType = TagCommons.getScreenType(pageContext) ;
        if ( sType != null )
        {
            if (TagCommons.isHTML(sType))
            {
                gen = _htmlGenerator ;
            }
            else if (TagCommons.isXUL(sType))
            {
                gen = _xulGenerator ;
            }            
        }
        else
        {
            //--- No type defined for the current page context => HTML by default
            gen = _htmlGenerator ;
            sType = "HTML (default)" ;            
        }
        if ( gen == null)
        {
            error("No generator for screen type '" + sType + "'");
        }
        return gen ;
    }
    
    //===============================================================================
    // VARIABLES SUBTITUTION
    //===============================================================================
    protected String replaceSymbolicVar(String s)
    {
        String s2 = s ; // By default Return "as is"
        //--- If there's a VarSetter => use it to replace the var names by var values
        if ( _varSetter != null && s != null )
        {
            s2 = _varSetter.replaceVar(s);                    
        }
        trace("replaceSymbolicVar('" + s + "') --> '" + s2 +"'" );
        return s2 ; // Return "as is" 
    }

    //===============================================================================
    // Generic methods called from "doStartTag" and "doEndTag"
    //===============================================================================
    /**
     * Called by the standard "doStartTag" method in the subclass
     */
    public void startTag()
    {
        startTag(OUTPUT);
    }
    
    /**
     * @param iOutputFlag 'OUTPUT' for output generation, 'NO_OUTPUT' for only tag init without output generation
     */
    public void startTag(int iOutputFlag)
    {
        trace("startTag()...");

        //--- Force generated ID as "not yet generated"
        _sGeneratedId = null;
        
        //--- Init the data binding : "element name" and "attribute name"
        initDataBinding();
        
        _hmStyle.clear(); // Clear the programatic style definitions

        //--- Create a "Variables Setter" for the current page context
        _varSetter = new TagVarSetter( pageContext ); // v 0.9.8

        //--- Process (call the generators) ...
        if ( iOutputFlag != NO_OUTPUT )
        {
//            JspWriter out = pageContext.getOut();
            try
            {            
                IWidget generator = getGenerator();
                if ( generator != null )
                {
//                    generator.startTag(out,this);                
                    generator.startTag(pageContext.getOut(),this);                
                }            
            } catch (IOException ioex)
            {
                ioex.printStackTrace();
                error("IOException ( startTag ) : " + ioex.getMessage());
            }
        }
    }

    /**
     * Called by the standard "doEndTag" method in the subclass <br>
     * NB: not always called ( see doEndTag of each tag )
     */
    public void endTag()
    {
        trace("endTag()...");

//        JspWriter out = pageContext.getOut();
        try
        {            
            IWidget generator = getGenerator();
            if ( generator != null )
            {
//                generator.endTag(out,this);                
                generator.endTag(pageContext.getOut(),this);                
            }            
        } catch (IOException ioex)
        {
            ioex.printStackTrace();
            error("IOException ( endTag ) : " + ioex.getMessage());
        }
    }

    /**
     * Returns the name of the tag (ie: "field" for <x:field ..> )
     * 
     * @return : the tag name
     */
    public String getTagName()
    {
        return _sTagName;
    }

    /**
     * Returns the current page context
     * 
     * @return
     */
    public PageContext getPageContext()
    {
        return pageContext;
    }

    /**
     * Return the servlet class name of the current JSP
     * 
     * @return : servlet class name
     */
    public String getServletClassName()
    {
        Object oPage = pageContext.getPage();
        if (oPage != null)
        {
            return oPage.getClass().getName();
        }
        else
        {
            return null;
        }
    }

    //===============================================================================
    // "id"
    //===============================================================================
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.TagSupport#setId(java.lang.String)
     */
    public void setId(String v)
    {
        _sId = v;
        if (v != null)
        {
            _sId = v.trim();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.TagSupport#getId()
     */
    public String getId()
    {
        trace("getId()...");

        if (_sId != null) // Id set by user
        {
            // return _sId;
            //--- Apply the symbolic variables replacement  
            return replaceSymbolicVar(_sId);  // v 0.9.9
            
        }
        else
        // null : no id set by user
        {
            if (_bIdGeneration)
            {
                return getGeneratedId();
            }
            else
            {
                return null;
            }
        }
    }

    protected boolean getIdGeneration()
    {
        return _bIdGeneration;
    }

    protected void setIdGeneration(boolean bFlag)
    {
        _bIdGeneration = bFlag;
    }

    /**
     * Returns the generated ID for the current tag
     * 
     * @return
     */
    private String getGeneratedId()
    {
        trace("getGeneratedId()...");
        if (_sGeneratedId == null) // ID not yet generated
        {
            //--- Generate an ID
            _sGeneratedId = Page.generateId(pageContext);
            trace("generateId() --> '" + _sGeneratedId + "' ");
        }
        return _sGeneratedId;
    }

    /**
     * Returns true if the tag has a valid ID (set manualy or generated)
     * 
     * @return
     */
    public boolean hasId()
    {
        if (_sId != null) // ID set manualy
        {
            if (_sId.length() > 0)
            {
                return true;
            }
        }
        if (_sGeneratedId != null) // Generated ID (always valid)
        {
            return true;
        }
        return false;
    }

    //===============================================================================
    // "cl" ( "class" )
    //===============================================================================
    public void setCl(String v)
    {
        _sClass = v;
    }

    public String getCl()
    {
        // return _sClass;
        //--- Update v 0.9.9 ( 20 March 2008 ) : symbolic var + default value
        if (_sClass != null)
        {
            String s = replaceSymbolicVar( _sClass );
            trace("getCl()... return '" + s + "'");
            return s;
        }
        else
        {
            //--- Is there a default value for this attribute ?
            String sDefault = getDefaultAttributeValue(_sTagName, "cl");
            trace("getCl()... getDefaultAttributeValue : " + sDefault );
            return sDefault ;
        }
    }

    //===============================================================================
    // "style"
    //===============================================================================
    /**
     * @param v
     */
    public void setStyle(String v)
    {
        _sStyle = v;
    }

    /**
     * Returns the style set by the tag attribute "style"
     * 
     * @return a string ( never null, void if no style )
     */
    public String getStyle()
    {
        //--- Style set by the user
//        String sUserStyle = "";
//        if (_sStyle != null)
//        {
//            sUserStyle = _sStyle;
//        }
        //--- Update v 0.9.9 ( 20 March 2008 ) : symbolic var + default value
        String sUserStyle = getUserStyle() ; // NB : can be null
        if ( sUserStyle == null ) sUserStyle = "" ;

        //--- Styles set programmaticaly
        String sProgStyle = "";
        if (_hmStyle.size() > 0)
        {
            String sKey = null;
            String sVal = null;
            Iterator it = _hmStyle.keySet().iterator();
            while (it.hasNext())
            {
                sKey = (String) it.next();
                sVal = (String) _hmStyle.get(sKey);
                sProgStyle = sProgStyle + sKey + ":" + sVal + ";";
            }
        }
        // The user style is set AFTER prog style to override same values if any
        return sProgStyle + sUserStyle; 
    }

    /**
     * Returns the style set by the user ( after symbolic var replacement )
     * or the default attribute value if any 
     * @return the user style or null
     */
    private String getUserStyle()  //--- Update v 0.9.9 ( 20 March 2008 ) : symbolic var + default value
    {
	    if ( _sStyle != null )
	    {
	        String s = replaceSymbolicVar( _sStyle );
	        trace("getUserStyle()... return '" + s + "'");
	        return s; // Can be null
	    }
	    else
	    {
	        //--- Is there a default value for this attribute ?
	        String sDefault = getDefaultAttributeValue(_sTagName, "style");
	        trace("getUserStyle()... getDefaultAttributeValue : " + sDefault );
	        return sDefault ; // Can be null
	    }
    }
    
    //===============================================================================
    // "title"
    //===============================================================================
    public void setTitle(String v)
    {
        _sTitle = v;
    }

    public String getTitle()
    {
        //return _sTitle;
        //--- Apply the symbolic variables replacement  
        return replaceSymbolicVar(_sTitle);  // v 0.9.9
    }

    //===============================================================================
    // "x" coordinate
    //===============================================================================
    public void setX(String v)
    {
        _sX = v != null ? v.trim() : v ;
    }

//    public String getX()
//    {
//        return _sX;
//    }

    /**
     * Returns the X value
     * @return X value, never null ( returns "0" if the X value is not defined )
     */
    public String getXValue()
    {
        if (notVoid(_sX)) // Not Null and Not Void
        {
            if ( _sX.equals("x") )
            {
                //--- X = current value
                return Variables.getX(pageContext); // Never null
            }
            else if ( _sX.equals("x++") )
            { 
                //--- X = current value ( but set to x + GAP for future use ) 
                return Variables.getXPlusPlus(pageContext); // Never null
            }
            else if ( _sX.equals("++x") )
            { 
                //--- Y = current value + GAP 
                return Variables.getPlusPlusX(pageContext); // Never null
            }
            else if ( StrUtil.isFirstCharAlpha(_sX) ) 
            {
                //--- Variable name => return variable value : can be null
                //return Variables.getVarValue(pageContext, __sX); // v 1.0.2
                String s = Variables.getVarValue(pageContext, _sX); // v 1.0.2
                return s != null ? s : "0" ; // v 1.0.2
            }
            else
            {
                //--- X attribute AS IS (supposed to be a numeric value)
                return _sX;
            }
        }
        else
        {
            //--- X variable
            return Variables.getX(pageContext); // Never null
        }
    }

    //===============================================================================
    // "y" coordinate
    //===============================================================================
    public void setY(String v)
    {
        _sY = v != null ? v.trim() : v ;
    }

//    public String getY()
//    {
//        return _sY;
//    }

    public String getYValue()
    {
        if (notVoid(_sY)) // Not Null and Not Void
        {
            if ( _sY.equals("y") )
            {
                //--- Y = current value
                return Variables.getY(pageContext); // Never null
            }
            else if ( _sY.equals("y++") )
            { 
                //--- Y = current value ( but set to y + GAP for future use ) 
                return Variables.getYPlusPlus(pageContext); // Never null
            }
            else if ( _sY.equals("++y") )
            { 
                //--- Y = current value + GAP 
                return Variables.getPlusPlusY(pageContext); // Never null
            }
            else if ( StrUtil.isFirstCharAlpha(_sY) ) 
            {
                //--- Variable name => return variable value : can be null
                //return Variables.getVarValue(pageContext, _sY); // v 1.0.2
                String s = Variables.getVarValue(pageContext, _sY); // v 1.0.2
                return s != null ? s : "0" ; // v 1.0.2
            }
            else
            {
                //--- Y attribute AS IS (supposed to be a numeric value)
                return _sY;
            }
        }
        else 
        {
            return Variables.getY(pageContext); // Never null
        }
    }

    //===============================================================================
    // "width"
    //===============================================================================
    public void setWidth(String v)
    {
        _sWidth = v;
    }

    public void setWidth(int v) // for use with int Java variables ( "rtexprvalue" )
    {
        _sWidth = "" + v;
    }

    public String getWidth()
    {
        return _sWidth;
    }

    public int getWidthInt()
    {
        if ( _sWidth != null )
        {
            return StrUtil.getInt(_sWidth.trim(), TagConst.WIDTH_NULL);            
        }
        return TagConst.WIDTH_NULL ;
    }

    //===============================================================================
    // "height"
    //===============================================================================
    public void setHeight(String v)
    {
        _sHeight = v;
    }

    public void setHeight(int v) // for use with int Java variables ( "rtexprvalue" )
    {
        _sHeight = "" + v;
    }

    public String getHeight()
    {
        return _sHeight;
    }
    
    public int getHeightInt()
    {
        if ( _sHeight != null )
        {
            return StrUtil.getInt(_sHeight.trim(), TagConst.HEIGHT_NULL);            
        }
        return TagConst.HEIGHT_NULL ;
    }

    //===============================================================================
    // "tabindex"
    //===============================================================================
    public void setTabindex(String v)
    {
        _sTabIndex = v;
    }

    public void setTabindex(int v) // for use with int Java variables ( "rtexprvalue" )
    {
        _sTabIndex = "" + v;
    }

    public String getTabindex()
    {
        return _sTabIndex;
    }

    //===============================================================================
    // "name"
    //===============================================================================
    public void setName(String v)
    {
        _sName = v;
    }

    public String getName()
    {
        //return _sName;
        //--- Apply the symbolic variables replacement  
        return replaceSymbolicVar(_sName);  // v 0.9.9        
    }

    //===============================================================================
    // "accesskey"
    //===============================================================================
    public void setAccesskey(String v) // v 0.9.9
    {
        _sAccessKey = v;
    }

    public String getAccesskey() // v 0.9.9
    {
        //--- Apply the symbolic variables replacement  
        return replaceSymbolicVar(_sAccessKey);        
    }

    //===============================================================================
    // "zone"
    //===============================================================================
    public void setZone(String v)
    {
        _sZone = v;
        if (v != null)
        {
            _sZone = v.trim();
        }
    }

    public String getZone()
    {
        //return _sZone;
        //--- Update v 0.9.9 ( 20 March 2008 ) : symbolic var + default value
        if (_sZone != null)
        {
            String s = replaceSymbolicVar( _sZone );
            trace("getZone()... return '" + s + "'");
            return s;
        }
        else
        {
            //--- Is there a default value for this attribute ?
            String sDefault = getDefaultAttributeValue(_sTagName, "zone");
            trace("getZone()... getDefaultAttributeValue : " + sDefault );
            return sDefault ;
        }
    }

    public boolean hasZone()
    {
//        if (_sZone != null)
//        {
//            if (_sZone.length() > 0)
//            {
//                return true;
//            }
//        }
        //--- Update v 0.9.9 ( 20 March 2008 ) : symbolic var + default value
        String s = getZone();
        if (s != null)
        {
            if (s.length() > 0)
            {
                return true;
            }
        }
        return false;
    }
    
    //===============================================================================
    // "txt"   NEW in v. 0.9.8  ( from subclasses )
    //===============================================================================
    /**
     * Set the "txt" attribute value
     * @param v
     */
    public void setTxt(String v)
    {
        _sTxt = v;
    }

//    /**
//     * Returns the text for the given language code  
//     * @param sLanguage : the language code ( e.g. "fr", "en", ... )
//     * @return
//     */
//    public String getTxt(String sLanguage)
//    {
//        String sTranslatedTxt = null ;
//        if ( _sId != null )
//        {
//            //String sKey = _sId + ".txt" ; 
//            sTranslatedTxt = translate( _sId + ".txt", sLanguage) ;
//        }
//        return ( sTranslatedTxt != null ) ? sTranslatedTxt : _sTxt ;
//    }
    
    /**
     * Returns the "txt" attribute value 
     * @return
     */
    public String getTxt()
    {

//        String sText = _sTxt ;
//        //--- Get the base language used for sreens design 
//        //   ( cf screens.xml : <config base_language=".." /> )
//        String sScreenBaseLanguage = getScreenBaseLanguage();
//        if ( sScreenBaseLanguage != null )
//        {
//            //--- The current language ( for the current session ) 
//            String sSessionLanguage = getSessionLanguage();
//            if ( sSessionLanguage != null )
//            {
//                //--- Compare the 2 languages 
//                if ( sSessionLanguage.equals(sScreenBaseLanguage) != true )
//                {
//                    //--- The screen language is not the same as session language 
//                    //--- => translate the text ( get the text for the session language )
//                    sText = getTxt(sSessionLanguage);
//                }
//            }
//        }
//        // else : no language management (keep the txt as is )
        
        String sText = TagCommons.getTranslatedTxt( _sTxt, _sId, pageContext );
        
        //--- Apply the symbolic variables replacement 
        return replaceSymbolicVar(sText);        
    }

    //===============================================================================
    // "disabled"
    //===============================================================================
//    public void setDisabled(boolean v) // Used by the JSP Compiler if "isDisabled" exists
//    {
//        _sDisabled = "" + v;
//    }

    public void setDisabled(String v)
    {
        _sDisabled = v;
    }

    public String getDisabled()
    {
        return _sDisabled;
    }

    public boolean getDisabledAsBoolean() // NEW in v 0.9.8 to avoid "setDisabled(true/false)"
    {
        if (_sDisabled != null)
        {
            trace("getDisabledAsBoolean()... replaceSymbolicVar(" + _sDisabled +")");
            String s = replaceSymbolicVar(_sDisabled);
            if (s != null)
            {
	            if (s.trim().equalsIgnoreCase("true"))
	            {
	                trace("getDisabledAsBoolean()... return true");
	                return true;
	            }
            }
        }
        else
        {
            //--- Is there a default value for this attribute ?
            String sDefault = getDefaultAttributeValue(_sTagName, "disabled");
            trace("getDisabledAsBoolean()... getDefaultAttributeValue : " + sDefault );
            if (sDefault != null)
            {
                if (sDefault.trim().equalsIgnoreCase("true"))
                {
	                trace("getDisabledAsBoolean()... return true");
                    return true;
                }
            }
        }
        trace("getDisabledAsBoolean()... return false");
        return false;
    }

    //===============================================================================
    // "readonly" ( no "isReadonly" to avoid "setDisabled(boolean v)" ( JSP Compiler )
    // NEW in v 0.9.8
    //===============================================================================
    public void setReadonly(String v)
    {
        _sReadonly = v;
    }

    public String getReadonly()
    {
        return _sReadonly;
    }

    public boolean getReadonlyAsBoolean()  
    {
        if (_sReadonly != null)
        {
            String s = replaceSymbolicVar(_sReadonly);
            if (s != null)
            {
	            if (s.trim().equalsIgnoreCase("true"))
	            {
	                trace("getReadonlyAsBoolean()... return true");
	                return true;
	            }
            }
        }
        else
        {
            //--- Is there a default value for this attribute ?
            String sDefault = getDefaultAttributeValue(_sTagName, "readonly");
            trace("getReadonlyAsBoolean()... getDefaultAttributeValue : " + sDefault );
            if (sDefault != null)
            {
                if (sDefault.trim().equalsIgnoreCase("true"))
                {
	                trace("getReadonlyAsBoolean()... return true");
                    return true;
                }
            }
        }
        trace("getReadonlyAsBoolean()... return false");
        return false;
    }

    //===============================================================================
    // "data binding"
    //===============================================================================
    public void setData(String s)
    {
        //--- 1) Keep the original data attribute
        _sData = s ; 
        
//-----
// DO NOT SET THE DATA BINDING HERE beacuse sometimes "setId()" is called after "setData()" ( depending on JSP compiler setXXX() order ) 
// Init in startTag() --> call initDataBinding()
//-----
//        //--- 2) Replace "#" by "id value"
//        if (s != null)
//        {
//            if (s.trim().equals("#"))
//            {
//                //--- Use the field id as the "data" attribute
//                s = this.getId();
//            }
//        }
//
//        //--- 3) Split data in 2 parts ( element & attribute )
//        if (s != null)
//        {
//            s = s.trim();
//            //--- Is in "dot notation"
//            int iPointPosition = s.lastIndexOf('.');
//            if (iPointPosition >= 0)
//            {
//                //--- Syntaxe : "element.attribute" => split the String
//                _sDataElement = s.substring(0, iPointPosition);
//                _sDataAttribute = s.substring(iPointPosition + 1);
//            }
//            else
//            {
//                //--- Use the given name as the DataAttribute
//                _sDataElement = Page.getDefaultDataElement(pageContext);
//                _sDataAttribute = s;
//            }
//        }
//
//        //--- Trim ...
//        if (_sDataElement != null)
//        {
//            _sDataElement = _sDataElement.trim();
//        }
//        if (_sDataAttribute != null)
//        {
//            _sDataAttribute = _sDataAttribute.trim();
//        }
    }

    public String getData()
    {
        return _sData;
    }

    /**
     * Init the 2 parts of the data binding ( element + attribute )
     * "#" : screenbody default data element + tag id
     * "value" : screenbody default data element + "value"
     * "element.value" : "element" + "value"
     */
    private void initDataBinding()
    {
        String s = this.getData() ; 
        
        //--- Replace "#" by "id value" ( if necessary )
        if (s != null)
        {
            if (s.trim().equals("#"))
            {
                //--- Use the field id as the "data" attribute
                s = this.getId();
            }
        }

        //--- Split data in 2 parts ( element & attribute )
        if (s != null)
        {
            s = s.trim();
            //--- Is in "dot notation"
            int iPointPosition = s.lastIndexOf('.');
            if (iPointPosition >= 0)
            {
                //--- Syntaxe : "element.attribute" => split the String
                _sDataElement = s.substring(0, iPointPosition);
                _sDataAttribute = s.substring(iPointPosition + 1);
            }
            else
            {
                //--- Use the given name as the DataAttribute
                _sDataElement = Page.getDefaultDataElement(pageContext);
                _sDataAttribute = s;
            }
        }

        //--- Trim ...
        if (_sDataElement != null)
        {
            _sDataElement = _sDataElement.trim();
        }
        if (_sDataAttribute != null)
        {
            _sDataAttribute = _sDataAttribute.trim();
        }
    }
    
    public String getDataElement()
    {
        return _sDataElement;
    }

    public String getDataAttribute()
    {
        return _sDataAttribute;
    }

    public boolean hasDataBinding()
    {
        String sBindElement = getDataElement();
        String sBindAttribute = getDataAttribute();

        if (sBindElement != null && sBindAttribute != null)
        {
            if (sBindElement.length() > 0 && sBindAttribute.length() > 0)
            {
                return true;
            }
        }
        return false;
    }

    //===============================================================================
    // EVENTS
    //===============================================================================
    public void setOnclick(String v)
    {
        _hmEvents.put(Event.ON_CLICK, v);
    }

    public void setOndblclick(String v)
    {
        _hmEvents.put(Event.ON_DBL_CLICK, v);
    }

    public void setOnmouseover(String v)
    {
        _hmEvents.put(Event.ON_MOUSE_OVER, v);
    }

    public void setOnmouseout(String v)
    {
        _hmEvents.put(Event.ON_MOUSE_OUT, v);
    }

    public void setOnmousemove(String v)
    {
        _hmEvents.put(Event.ON_MOUSE_MOVE, v);
    }

    public void setOnmousedown(String v)
    {
        _hmEvents.put(Event.ON_MOUSE_DOWN, v);
    }

    public void setOnmouseup(String v)
    {
        _hmEvents.put(Event.ON_MOUSE_UP, v);
    }

    //----------------------------------------------------------------
    public void setOnchange(String v)
    {
        _hmEvents.put(Event.ON_CHANGE, v);
    }

    public void setOnselect(String v)
    {
        _hmEvents.put(Event.ON_SELECT, v);
    }

    public void setOnfocus(String v)
    {
        _hmEvents.put(Event.ON_FOCUS, v);
    }

    public void setOnblur(String v)
    {
        _hmEvents.put(Event.ON_BLUR, v);
    }

    //----------------------------------------------------------------
    public void setOnkeypress(String v)
    {
        _hmEvents.put(Event.ON_KEY_PRESS, v);
    }

    public void setOnkeydown(String v)
    {
        _hmEvents.put(Event.ON_KEY_DOWN, v);
    }

    public void setOnkeyup(String v)
    {
        _hmEvents.put(Event.ON_KEY_UP, v);
    }

    //----------------------------------------------------------------

    //===============================================================================
    //
    //===============================================================================
//    public int getOuputType(PageContext pageContext)
//    {
//        ScreenMap screenMap = TagCommons.getScreenMap(pageContext);
//        if (screenMap != null)
//        {
//            if (screenMap.isHtmlType())
//            {
//                return HTML_OUTPUT;
//            }
//            else if (screenMap.isXulType())
//            {
//                return XUL_OUTPUT;
//            }
//        }
//        return UNKNOWN_OUTPUT;
//    }

    public void defineStyle(String sName, String sValue)
    {
        _hmStyle.put(sName, sValue);
    }

    /**
     * Return the default attribute value for the given tag name and attribute name
     * @param tagName
     * @param sAttribute
     * @return the default value or null
     */
    private String getDefaultAttributeValue(String tagName, String sAttribute)
    {
        String sRetValue = null;
        //--- Try to find a default value in the current screen env.
        // A default value set with the "default" tag
        TagScreenEnv screenEnv = TagCommons.getScreenEnv(pageContext);
        if (screenEnv != null)
        {
            sRetValue = screenEnv.getDefaultAttributeValue(tagName, sAttribute);
            if (sRetValue == null)
            {
                // Try to get a default value for all tags
                sRetValue = screenEnv.getDefaultAttributeValue("*", sAttribute);
            }
        }

        if (sRetValue == null)
        {
            // Try to get a global default value (same for all screens)
            // Default values set with a properties file
            sRetValue = TagDefaultValues.getDefaultAttributeValue(tagName, sAttribute);
        }
        //        if (sRetValue == null)
        //        {
        //            //--- Try to find a variable with name "x" or "y"
        //            if (sAttribute.equals("x") || sAttribute.equals("y"))
        //            {
        //                Variables variables = Variables.getVariables(pageContext);
        //                if (variables != null)
        //                {
        //                    sRetValue = variables.get(sAttribute);
        //                }
        //                //sRetValue = getVariable(sAttribute);
        //            }
        //        }
        return sRetValue;
    }

    //    public String getAttributeValue(String sAttribute, String value)
    //    {
    //        return getAttributeValue(_sTagName, sAttribute, value);
    //    }

    protected String getHtmlTitle()
    {
        if (_sTitle != null)
        {
            return "title=\"" + _sTitle + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns a "label" tag with "for" attribute referencing the current tag Id. <br>
     * The label has no style, no class. Its Id is tag_id + "_label"
     * 
     * @param sTxt
     * @return
     */
    protected String getHtmlLabelForTag(String sTxt)
    {
        String sTagId = getId();
        return "<label for=\"" + sTagId + "\"" + " id=\"" + sTagId + "_label\"" + " >" + sTxt + "</label>";
    }

    /**
     * Return true if there at least one coordinate not null or not void
     * 
     * @return
     */
    public boolean isPositionAbsolute()
    {
        //--- If X is set => absolute
        if (_sX != null)
        {
            if (_sX.trim().length() > 0)
                return true;
        }
        //--- If Y is set => absolute
        if (_sY != null)
        {
            if (_sY.trim().length() > 0)
                return true;
        }
        // No X and no Y => not absolute
        return false;
    }

    public boolean isEventDefined(String sEventName)
    {
        return (_hmEvents.get(sEventName) != null ? true : false);
    }

    public String getEvent(String sEventName)
    {
        Object o = _hmEvents.get(sEventName) ;
        if ( o != null )
        {
            return o.toString() ;
        }
        return null ;
    }

    /**
     * Returns true if the string equals "true" ( after trim(), not case sensitive )
     * 
     * @param sTxt
     * @return
     */
    protected boolean isTrue(String sTxt)
    {
        if (sTxt != null)
        {
            if (sTxt.trim().equalsIgnoreCase("true"))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns true if the string is not null and not void
     * 
     * @param sTxt
     * @return
     */
    protected boolean notVoid(String sTxt)
    {
        if (sTxt != null)
        {
            if (sTxt.trim().length() > 0)
            {
                return true; // not void
            }
        }
        return false;
    }

    /**
     * Print all the defined events.
     * 
     * @param out : the output 
     * @throws IOException
     */
  public void printEvents(JspWriter out) throws IOException
  {
      printEvent(out, Event.ON_CLICK      );
      printEvent(out, Event.ON_DBL_CLICK  );
      printEvent(out, Event.ON_MOUSE_DOWN );
      printEvent(out, Event.ON_MOUSE_UP   );
      printEvent(out, Event.ON_MOUSE_OVER );
      printEvent(out, Event.ON_MOUSE_OUT  );
      printEvent(out, Event.ON_MOUSE_MOVE );

      printEvent(out, Event.ON_CHANGE    );
      printEvent(out, Event.ON_SELECT    );
      printEvent(out, Event.ON_FOCUS     );
      printEvent(out, Event.ON_BLUR      );

      printEvent(out, Event.ON_KEY_PRESS );
      printEvent(out, Event.ON_KEY_DOWN  );
      printEvent(out, Event.ON_KEY_UP    );

      printEvent(out, Event.ON_SUBMIT    );
      printEvent(out, Event.ON_RESET     );

      printEvent(out, Event.ON_LOAD      );
      printEvent(out, Event.ON_UNLOAD    );
  }
    
    /**
     * Prints the given event if it is defined.
     * @param out
     * @param sEventName
     * @throws IOException
     */
    public void printEvent(JspWriter out, String sEventName ) throws IOException
    {
        if (sEventName != null)
        {
	        String sEventScript = (String) _hmEvents.get(sEventName);
	        if (sEventScript != null)
	        {
	            printEvent(out, sEventName, sEventScript);
	        }
        }
    }
    
    /**
     * Prints a specific event script for a given event name
     * 
     * @param out
     * @param sEventName :
     *            ie onblur, onclick, ...
     * @param sEventScript :
     *            the specific script associated with the event
     * @throws IOException
     */
    public void printEvent(JspWriter out, String sEventName, String sEventScript ) throws IOException
    {
        if (sEventName != null && sEventScript != null)
        {
            if (sEventName.trim().length() > 0 && sEventScript.trim().length() > 0)
            {
                //--- Replace the var names by var values
                sEventScript = replaceSymbolicVar(sEventScript);
                
                //--- Print the event script
                out.print(" " + sEventName + "=\"" + sEventScript + "\" ");
            }
        }
    }
    
    /**
     * Returns the full image src, according with the image directory configuration <br>
     * ie : root context URL + image directory + the given src
     * 
     * @param sSrc
     * @return
     */
    public String getFullImageSrc(String sSrc)
    {
        if (sSrc != null)
        {
            return TagCommons.getImgDirURL(pageContext) + sSrc;
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the full standard http URL for the given context resource<br>
     * ie : for "myresource" returns "http://server:port/contextname/myresource" 
     * @param sResource
     * @return
     */
    public String getResourceURL(String sResource)
    {
        if (sResource != null)
        {
            //--- Get Root of "Context URL"  ( "http://server:port/contextname" ) 
            String sRootContextURL = TagCommons.getRootContextURL(pageContext);
            // add "/" at the end if necessary  
            if (!sRootContextURL.endsWith("/"))
            {
                sRootContextURL = sRootContextURL + "/";
            }
            // remove firts "/" if any 
            if (sResource.startsWith("/"))
            {
                sResource = sResource.substring(1, sResource.length());
            }
            return sRootContextURL + sResource ;
        }
        else
        {
            return "";
        }
    }
    //----------------------------------------------------------------
    /**
     * Returns the full URI for the given resource, or null if the given resource is null
     * e.g. '/context/resource' for 'resource' or '/resource'
     * @param httpRequest
     * @param sResource
     * @return URI or null
     */
    protected String getResourceURI( HttpServletRequest httpRequest, String sResource )
    {
		//String sURI = httpRequest.getRequestURI();
		String sContextPath = httpRequest.getContextPath() ; // "/context"
		if ( sContextPath != null && sResource != null )
		{
		    String s1 = sContextPath.trim();
		    String s2 = sResource.trim();
		    if ( s2.startsWith("/") )
		    {
		        return s1 + s2 ;
		    }
		    else
		    {
		        return s1 + "/" + s2 ;
		    }
		}
		return null ;
    }
    
    //----------------------------------------------------------------
    /**
     * Returns the full URI for the given resource, or null if the given resource is null
     * e.g. '/context/resource' for 'resource' or '/resource'
     * @param sResource
     * @return
     */
    public String getResourceURI(String sResource)
    {
        ServletRequest req = pageContext.getRequest();
        if ( req != null )
        {
            return getResourceURI( (HttpServletRequest)req, sResource );
        }
        return null ;
    }
    

    //----------------------------------------------------------------
    public String getFullTelosysImageSrc(String sImageFileName )
    {
        //--- Get Root of "Context URL"  ( "http://server:port/contextname" ) 
        String sRootContextURL = TagCommons.getRootContextURL(pageContext);
        // add "/" at the end if necessary  
        if (!sRootContextURL.endsWith("/"))
        {
            sRootContextURL = sRootContextURL + "/";
        }
        //--- Use the CSS directory as the IMAGES directory
        String sCSSDir = "";
        ScreenDefConfig config = TagCommons.getScreenDefConfig();
        if (config != null)
        {
            sCSSDir = config.getFrameworkCSSDir();
            if (sCSSDir != null)
            {
                // remove firts "/" if any 
                if (sCSSDir.startsWith("/"))
                {
                    sCSSDir = sCSSDir.substring(1, sCSSDir.length());
                }
                // add "/" at the end if necessary  
                if (!sCSSDir.endsWith("/"))
                {
                    sCSSDir = sCSSDir + "/";
                }
            }
            else
            {
                sCSSDir = "" ;
            }
        }

        //--- Set Image Dir ( ending with "/" )
        String sFullImgDir = sRootContextURL + sCSSDir ;
        if ( sImageFileName != null )
        {
            return sFullImgDir + sImageFileName ;            
        }
        else
        {
            return sFullImgDir ;                        
        }
    }

    /**
     * Returns the HttpServletRequest instance (the current request)
     * 
     * @return
     */
    protected HttpServletRequest getHttpServletRequest()
    {
        ServletRequest servletRequest = pageContext.getRequest();
        if (servletRequest == null)
        {
            // Error
            //error();
            return null;
        }
        if (servletRequest instanceof HttpServletRequest)
        {
            return (HttpServletRequest) servletRequest;
        }
        else
        {
            // Error
            return null;
        }
    }

    /**
     * Returns the HttpSession instance (or null if it doesn't exist)
     * 
     * @return
     */
    protected HttpSession getHttpSession()
    {
        HttpServletRequest httpServletRequest = this.getHttpServletRequest();
        if (httpServletRequest != null)
        {
            httpServletRequest.getSession(false);
        }
        return null;
    }

    /**
     * Returns the Telosys Screen Context ID ( 0 to N )
     * 
     * @return the context id (or -1 if not found )
     */
    public int getScreenContextId()
    {
        HttpServletRequest request = this.getHttpServletRequest();
        if (request != null)
        {
            ScreenMap screenMap = (ScreenMap) request.getAttribute(ScreenMapConst.SCREEN_MAP);
            if (screenMap != null)
            {
                return screenMap.getScreenContextId();
            }
        }
        return -1; // Not found
    }

    /**
     * Returns the Telosys Screen Name (or null if not found)
     * 
     * @return the Screen Name (or null)
     */
    public String getScreenName()
    {
        HttpServletRequest request = this.getHttpServletRequest();
        if (request != null)
        {
            ScreenMap screenMap = (ScreenMap) request.getAttribute(ScreenMapConst.SCREEN_MAP);
            if (screenMap != null)
            {
                return screenMap.getName();
            }
        }
        return null; // Not found
    }

    /**
     * Returns the screen marker name ( the screen name used to generate the 'screen marker') <br>
     * The screen marker name is the symbolic screen name if it is defined, <br>
     * else the servlet class name is used as screen identifier
     * 
     * @return
     */
    public String getScreenMarkerName()
    {
        String sScreenMarkerName = getScreenName();
        if (sScreenMarkerName != null)
        {
            //--- Use the current symbolic screen name
            return sScreenMarkerName;
        }
        else
        {
            //--- No symbolic screen name => use the servlet class name
            return getServletClassName();
        }
        //return null; // Not found
    }

    /**
     * Returns the Telosys ScreenSession instance (or null if it doesn't exist)
     * 
     * @return
     */
    protected ScreenSession getScreenSession()
    {
        HttpServletRequest request = this.getHttpServletRequest();
        if (request != null)
        {
            return ScreenSessionManager.findScreenSession(request);
        }
        else
        {
            // Error
            return null;
        }
    }

//    /**
//     * Returns the language of the current session ( or null )
//     * @return the language ( e.g. "fr", "en", ...) or null
//     */
//    protected String getSessionLanguage()
//    {
//        ScreenSession screenSession = getScreenSession();
//        if ( screenSession != null )
//        {
//            return screenSession.getLanguage();
//        }
//        return null ;
//    }
//    
//    /**
//     * Returns the screen configuration language 
//     * ( the language used to design the screen )  
//     * @return the language ( e.g. "fr", "en", ...) or null
//     */
//    protected String getScreenBaseLanguage()
//    {
//        String sLanguage = null ;
//        ScreenDefConfig config = TagCommons.getScreenDefConfig();
//        if ( config != null )
//        {
//            sLanguage = config.getBaseLanguage();
//        }
//        return sLanguage ;
//    }
    
    /**
     * Returns the Telosys ScreenApplication instance (never null)
     * 
     * @return
     */
    protected ScreenApplication getScreenApplication()
    {
//        HttpSession httpSession = getHttpSession();
//        return ScreenApplicationManager.getScreenApplication(httpSession);
        return ScreenApplicationManager.getScreenApplication(); // v 1.0.1
    }

    //---------------------------------------------------------------------------------
    // Current form name management (for usage in child tags)
    //---------------------------------------------------------------------------------

    private final static String CURRENT_FORM_NAME = "TELOSYS_CURRENT_FORM_NAME";

    protected void setCurrentFormName(String sFormName)
    {
        pageContext.setAttribute(CURRENT_FORM_NAME, sFormName);
    }

    protected String getCurrentFormName()
    {
        Object obj = pageContext.getAttribute(CURRENT_FORM_NAME);
        if (obj != null)
        {
            if (obj instanceof String)
            {
                return (String) obj;
            }
        }
        return null;
    }

    protected void removeCurrentFormName()
    {
        pageContext.removeAttribute(CURRENT_FORM_NAME);
    }

    //---------------------------------------------------------------------------------
    // Current form postback management (for usage in child tags)
    //---------------------------------------------------------------------------------
    private final static String CURRENT_FORM_POSTBACK = "TELOSYS_CURRENT_FORM_POSTBACK";

    protected void setCurrentFormPostback(String sPostback)
    {
        pageContext.setAttribute(CURRENT_FORM_POSTBACK, sPostback);
    }

    protected String getCurrentFormPostback()
    {
        Object obj = pageContext.getAttribute(CURRENT_FORM_POSTBACK);
        if (obj != null)
        {
            if (obj instanceof String)
            {
                return (String) obj;
            }
        }
        return null;
    }

    protected void removeCurrentFormPostback()
    {
        pageContext.removeAttribute(CURRENT_FORM_POSTBACK);
    }

    //-----------------------------------------------------------------------------
    /**
     * Print a standard trace message, using the active loggers (if the trace flag is "true")
     * 
     * @param sMsg
     */
    protected void trace(String sMsg)
    {
        String sClassName = this.getClass().getName();
        //--- If the configuration has changed since the last acces ...
        if (TraceConfig.getCurrentState() != _iLastTraceConfigState)
        {
            //--- ScreenTitle the flag from the configuration
            _bTrace = TraceConfig.flagIsOn(sClassName);
            //--- ScreenTitle the configuration state
            _iLastTraceConfigState = TraceConfig.getCurrentState();
        }
        if (_bTrace == true)
        {
            Telosys.trace(sClassName, "Tag '" + _sTagName + "' : " + sMsg);
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Print a standard tag error message, using the active loggers
     * 
     * @param sMsg
     */
    public void error(String sMsg)
    {
        Telosys.error(this.getClass().getName(), "Tag '" + _sTagName + "' : " + sMsg);
    }

    //-----------------------------------------------------------------------------

    public void addJavascriptLine(String sLine)
    {
        //--- Get the current Javascript lines (create it if not found)
        LinkedList lines = Page.getJavascriptLines(pageContext, true);
        if (lines != null)
        {
            trace("addJavascriptLine (" + sLine + ")");
            lines.add(sLine);
        }
        else
        {
            error("addJavascriptLine(..) : Cannot get lines from page context ");
        }
    }
    
    /**
     * Compares the Item value with the Parent value ( value of "combobox", "listbox", "radiogroup", "fieldcombo" )
     * ( used to know if the item is "selected" or not )
     * @param sTagValue the current tag value ( child tag value )
     * @return true if same values, false if different values or no parent
     */
    public boolean sameValueAsParent( String sTagValue )
    {
        //--- Get the current item tag value
        //String sTagValue = getValue();
        if (sTagValue == null)
        {
            return false;
        }

        //--- Get the parent tag value
//        String sParentValue = null;
//        Tag tagParent = this.getParent();
//        if (tagParent != null)
//        {
//            //--- There is only 3 parent tags with items :
//            if (tagParent instanceof ComboBox )
//            {
//                sParentValue = ((ComboBox) tagParent).getValue();
//            }
//            else if (tagParent instanceof ListBox)
//            {
//                sParentValue = ((ListBox) tagParent).getValue();
//            }
//            else if (tagParent instanceof RadioGroup)
//            {
//                sParentValue = ((RadioGroup) tagParent).getValue();
//            }
//            else if (tagParent instanceof FieldCombo)
//            {
//                return false; // Special case : Item never selected => never return true
//            }
//            else
//            {
//                error("invalid parent tag ( parent class = " + tagParent.getClass().getName() + " )" );
//                return false;
//            }
//        }
//        else
//        {
//            //error("no parent tag !" );
//            return false; // No parent tag => not the same value
//        }
        
        String sParentValue = getParentValue();
        
        //--- Compare the 2 values ...
        if (sParentValue != null && sTagValue != null)
        {
            return sParentValue.equals(sTagValue);
        }
        return false;
    }

    /**
     * Returns the 'items container' parent tag value ( value of "combobox", "listbox", "radiogroup", "fieldcombo" ) 
     * even if there is other tags between the parent and the current tag ( ie JSTL tags "if", "forEach", ... )
     * 
     * @return
     */
    private String getParentValue()
    {
        String sParentValue = null;
        ItemsContainer parentTag = getItemsContainerParentTag( this );
        if (parentTag != null)
        {
            sParentValue = parentTag.getValue();
        }
        else
        {
            //error("no 'items container' tag for '" + this.getClass().getName() + "'" );
        }
        return sParentValue ;
    }
    
    /**
     * Returns the 'items container' parent tag
     * @param currentTag
     * @return
     */
    private ItemsContainer getItemsContainerParentTag( Tag currentTag )
    {
        Tag tagParent = currentTag.getParent();
        if (tagParent != null)
        {
            //--- This is the expected container parent tag 
            if (tagParent instanceof ItemsContainer )
            {
                return (ItemsContainer) tagParent ;
            }
            else
            {
                //--- Try to find it at upper level ...
                return getItemsContainerParentTag( tagParent );
            }
        }
        return null ;
    }
}