package org.objectweb.telosys.uil.taglib.widget;

import java.util.Iterator;

import org.objectweb.telosys.uil.taglib.Finder;
import org.objectweb.telosys.uil.taglib.LoopManager;
import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.widget.html.ListBodyHTML;
import org.objectweb.telosys.util.StrUtil;


/**
 * @author Laurent GUERIN
 *  
 */
public class ListBody extends GenericTag
{
    protected static final String TAG_NAME = "listbody";

    private static final int NO_TYPE = 0 ;
    
    private static final int LOOP_TYPE = 1 ;
    
    private static final int DATA_TYPE = 2 ;
    
    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget  $htmlGen = new ListBodyHTML(); 

    private final static IWidget  $xulGen  = null;

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
    
    private String    _sScope     = null ;
    
    private String    _sLoop      = null ;
    
    private String    _sIndexbase = null;

    private String    _sLoopName  = "listbody"; // v 1.0.1
    
    private String    _sItemName  = null ; // v 1.0.1

    //----------------------------------------------------------------
    // Private variables
    //----------------------------------------------------------------
    
    /**
     * Constructor
     */
    public ListBody()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
    }

    //----------------------------------------------------------------
    public void setProvider(String v) // Deprecated (same as "data" attribute )
    {
        setData(v);
    }

    //----------------------------------------------------------------
    public void setScope(String v)
    {
        _sScope = v;
    }
    public String getScope()
    {
        return _sScope ;
    }

    //----------------------------------------------------------------
    public void setLoop(String v)
    {
        _sLoop = v;
    }

    public String getLoop()
    {
        return _sLoop ;
    }

    //----------------------------------------------------------------
    public void setIndexbase(String v)
    {
        this._sIndexbase = v;
    }

    //----------------------------------------------------------------
    private int getIterationType()  
    {
        int iType = NO_TYPE ;
        if ( _sLoop != null )
        {
            iType = LOOP_TYPE ;
        }
        //if ( _sProvider != null ) // override LOOP_TYPE
        if ( getData() != null ) // override LOOP_TYPE
        {
            iType = DATA_TYPE ;
        }        
        return iType;
    }

    //----------------------------------------------------------------
    public void setLoopname(String v) // v 1.0.1
    {
        this._sLoopName = v;
    }
    //----------------------------------------------------------------
    public void setItemname(String v) // v 1.0.1
    {
        _sItemName = v;
    }
    
    //----------------------------------------------------------------
    //----------------------------------------------------------------
    private LoopManager initLoopManager()
    {
        String sListDataToUse = getData() ;
        int iType = getIterationType();
        if ( iType != LOOP_TYPE && iType != DATA_TYPE)
        {
            //--- Type not defined no "loop" and no "provider"
            // By default, try to find a RENDERER-BEAN as provider
            sListDataToUse = "RENDERER-BEAN" ;
            iType = DATA_TYPE ;
        }
        
        LoopManager loopManager = null ;

        if ( iType == LOOP_TYPE ) // attribute loop="" 
        {
            //--- Init a loop manager 
            loopManager = new LoopManager ( pageContext, _sLoop );   
            //System.out.println(" initLoopManager :  " + loopManager );
        }
        else if ( iType == DATA_TYPE ) // attribute provider="" 
        {
            Iterator iter = Finder.findIterator(pageContext, sListDataToUse, null);
            //--- Init a loop manager ( with the ITERATOR )
            if ( iter != null )
            {
                int iIndexBase = 0 ;
                if ( _sIndexbase != null )
                {
                    iIndexBase = StrUtil.getInt(_sIndexbase, 0);
                }
                loopManager = new LoopManager ( pageContext, iter, iIndexBase );   
                //System.out.println(" initLoopManager :  " + loopManager );   
                //--- If specific item variable name
                if ( _sItemName != null ) // v 1.0.1
                {
                    loopManager.setItemName( _sItemName ); // v 1.0.1
                }
            }
        }
        
        return loopManager ;
    }
    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        startTag();
        LoopManager loopManager = initLoopManager();
        
        //--- Set the loop manager in the current page context ( if NULL => removed )
        //Page.setLoopManager(pageContext, loopManager);
        Page.setLoopManager(pageContext, _sLoopName, loopManager); // v 1.0.1 use a LoopManager name
        
        //--- If there's a FIRST ITEM in the loop
        if ( loopManager != null )
        {
            if ( loopManager.first() )
            {
                return EVAL_BODY_INCLUDE;
            }
        }
        return SKIP_BODY;        
    }
    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.IterationTag#doAfterBody()
     */
    public int doAfterBody()
    {
        //--- Retrieve the LOOP MANAGER
        //LoopManager loopManager = Page.getLoopManager(pageContext) ;   
        LoopManager loopManager = Page.getLoopManager(pageContext, _sLoopName ) ; // v 1.0.1 use a LoopManager name
        //--- If there's a NEXT ITEM in the loop
        if ( loopManager != null )
        {
            if ( loopManager.next() )
            {
                //--- Another iteration ...
                //System.out.println(" doAfterBody : EVAL_BODY_AGAIN ");
                return EVAL_BODY_AGAIN;
            }
        }

        //--- It's the end ( no other iteration ) 
//        System.out.println(" doAfterBody : SKIP_BODY ");
        return SKIP_BODY;                                        
    }

    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag()
    {
//        System.out.println(" doEndTag ");
        //--- It's the end ( no other iteration ) 
        endTag();
        return EVAL_PAGE; // EVAL_PAGE = continue , SKIP_PAGE = abort the processing                                      
    }
}
