package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.widget.data.MenuItems;
import org.objectweb.telosys.uil.taglib.widget.html.MenuBarHTML;

/**
 * @author Laurent GUERIN
 *  
 */
public class MenuBarTAG extends GenericTag
{
    private static final String  TAG_NAME = "menubar";

    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget $htmlGen = new MenuBarHTML();

    private final static IWidget $xulGen  = null;

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------

    private String              _sLayout            = null ;

    /**
     * Constructor
     */
    public MenuBarTAG()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
        setIdGeneration(true); // Automatic ID generation (if ID not set by user)        
    }

    //----------------------------------------------------------------
    public void setLayout(String v)
    {
        _sLayout = v;
    }

    public String getLayout()
    {
        return _sLayout;
    }

    public boolean isVerticalLayout()
    {
        if ( _sLayout != null )
        {
            return _sLayout.trim().toLowerCase().startsWith("v");
        }
        return false ;
    }


    //----------------------------------------------------------------
    public MenuItems getMenuItems()
    {
        return Page.getCurrentMenuItems(this.getPageContext());
    }
    
    //----------------------------------------------------------------
    // Open TAG
    //----------------------------------------------------------------
    public int doStartTag()
    {
        // No output generation at this level, but startTag() must be called for tag init
        startTag(NO_OUTPUT);
        
        //--- New "Menu Items List" for inner "menuitem" tags  
        MenuItems menuItems = new MenuItems();
        Page.setCurrentMenuItems(this.getPageContext(), menuItems);
        
        return (EVAL_BODY_INCLUDE);
    }

    //----------------------------------------------------------------
    // Close TAG
    //----------------------------------------------------------------
    public int doEndTag()
    {
        endTag(); // Generate ALL The MENUBAR
        
        //--- Remove the "Menu Items List"   
        Page.setCurrentMenuItems(this.getPageContext(), null);
        
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }
    
}
