package org.objectweb.telosys.uil.taglib.widget;

import javax.servlet.jsp.tagext.Tag;

import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.widget.html.RadioItemHTML;

/**
 * @author Laurent GUERIN
 *  
 */
public class RadioItem extends GenericItemTag // GenericTag
{
    private static final String  TAG_NAME   = "radioitem";

    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget htmlGen    = new RadioItemHTML();

    private final static IWidget xulGen     = null;

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------

    private String               _sTxtPos   = "right";            // "right" or "left"

    //----------------------------------------------------------------

    /**
     * Constructor
     */
    public RadioItem()
    {
        super(TAG_NAME, htmlGen, xulGen);
        setIdGeneration(true); // Automatic ID generation (if ID not set by user)
    }

    //----------------------------------------------------------------
    public void setTxtpos(String v)
    {
        _sTxtPos = v;
    }

    public String getTxtpos()
    {
        return _sTxtPos;
    }

    //----------------------------------------------------------------
    private RadioGroup getParentRadioGroup()
    {
        Tag tagParent = this.getParent();
        if (tagParent != null)
        {
            if (tagParent instanceof RadioGroup)
            {
                return (RadioGroup) tagParent;
            }
        }
        return null;
    }
    
    /**
     * Returns the radio group name, or the radio group id if there's no name
     * @return
     */
    public String getItemNameFromParent()
    {
        RadioGroup radioGroup = getParentRadioGroup();
        if ( radioGroup != null )
        {
            if (radioGroup.getName() != null)
            {
                return radioGroup.getName();
            }
            else if (radioGroup.getId() != null)
            {
                return radioGroup.getId();
            }            
        }
        return null;
    }
    
    /**
     * Return true if the parent tag (radio group) is disabled 
     * @return
     */
    public boolean isParentDisabled()
    {
        RadioGroup radioGroup = getParentRadioGroup();
        if ( radioGroup != null )
        {
            return radioGroup.getDisabledAsBoolean();            
        }
        return false;
    }

    //----------------------------------------------------------------
    public int doStartTag()
    {
        //--- Process ...
        startTag();
        return (SKIP_BODY);
    }

    //----------------------------------------------------------------
    public int doEndTag()
    {
        endTag();
        //--- Get the parent radio group Id
        String sRadioGroupId = null ;
        RadioGroup radioGroup = getParentRadioGroup();
        if ( radioGroup != null )
        {
            sRadioGroupId = radioGroup.getId() ;
        }
        //--- Register the current radio item (if it belongs to a radio group with an ID)
        if ( sRadioGroupId != null )
        {
            String sId =  this.getId() ;
            trace("doEndTag() : register RadioItem : " + sId );
            Page.registerRadioItem(pageContext, sRadioGroupId, sId ) ;            
        }
        
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }

    //----------------------------------------------------------------
}