package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.TagCommons;
import org.objectweb.telosys.uil.taglib.widget.html.RichTextEditorHTML;

/**
 * @author Laurent GUERIN
 *  
 */
public class RichTextEditor extends GenericTag
{
    private static final String TAG_NAME = "richtexteditor";

    public  static final int    WITDH_MIN      = 400 ;

    public  static final int    WITDH_DEFAULT  = 540 ;
    
    public  static final int    HEIGHT_MIN     = 100 ;

    public  static final int    HEIGHT_DEFAULT = 250 ;
    
    //----------------------------------------------------------------
    // Tool bar elements IDs
    //----------------------------------------------------------------
    public  static final int    BAR_FONT      = 0 ;
    public  static final int    BAR_SIZE      = 1 ;
    public  static final int    BAR_BOLD      = 2 ;
    public  static final int    BAR_ITALIC    = 3 ;
    public  static final int    BAR_UNDERLINE = 4 ;
    public  static final int    BAR_STRIKE    = 5 ; // --- Toolbar "Mini"
    
    public  static final int    BAR_JLEFT   = 6 ;
    public  static final int    BAR_JCENTER = 7 ;
    public  static final int    BAR_JRIGHT  = 8 ; // --- Toolbar "Basic"
    
    public  static final int    BAR_ULIST   = 9 ;
    public  static final int    BAR_OLIST   = 10 ;
    public  static final int    BAR_OUTDENT = 11 ; // <- Left 
    public  static final int    BAR_INDENT  = 12 ;  // -> Right
    public  static final int    BAR_FCOLOR  = 13 ;
    public  static final int    BAR_BCOLOR  = 14 ; // --- Toolbar "Standard"
    
    public  static final int    BAR_SUB = 15 ;
    public  static final int    BAR_SUP = 16 ;
    public  static final int    BAR_HR  = 17 ; // --- Toolbar "All"
    
    private static final String ELEM_NAMES[] = 
    { "Font","Size","B","I","U","S", "JL","JC","JR", "UL","OL","Left","Right","FC","BC", "Sub","Sup","HR"} ;
    
    private boolean[] _barElements = new boolean[20];
    
    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget $htmlGen = new RichTextEditorHTML();

    private final static IWidget $xulGen  = null;

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
//    private String              _sTxt    = "xxx";
//
//    private String              _sFor    = null ;
    
    private String              _sToolBar    = null ;

    /**
     * Constructor 
     */
    public RichTextEditor()
    {
        super(TAG_NAME, $htmlGen, $xulGen);    
        setIdGeneration(true); // Automatic ID generation (if ID not set by user)        
    }
    
	//----------------------------------------------------------------
	public void setToolbar(String v)
	{
	    _sToolBar = v;
	}
	public String getToolbar()
	{
	    return _sToolBar;
	}
    
	public boolean[] getToolbarElements()
	{
	    clearToolBar();
	    if ( _sToolBar != null ) // e.g. : "basic" or "basic,FC,BC" 
        {
	        if ( _sToolBar.trim().length() > 0 )
	        {
		        String[] tokens = _sToolBar.split( "," );
		        for ( int i = 0 ; i < tokens.length ; i++ )
		        {
		            String s = tokens[i];
		            if ( s != null )
		            {
			            initToolBar(s.trim()); 
		            }
		        }	        
	        }
	        else
	        {
			    //--- toolbar="" ( void) => Default toolbar  
		        initStandardToolBar();
	        }	            
        }
	    else
	    {
		    //--- attribute not used => Default toolbar  
	        initStandardToolBar();
	    }
	    return _barElements;	    
	}
    //----------------------------------------------------------------
	private void initToolBar(String sWhat) // "basic" or "FC" or "B" ....
	{
        if ( sWhat.equalsIgnoreCase("mini"))
        {
            setToolBarElements(BAR_FONT,  BAR_STRIKE ); 
        }
        else if ( sWhat.equalsIgnoreCase("basic"))
        {
            setToolBarElements(BAR_FONT,  BAR_JRIGHT ); 
        }
        else if ( sWhat.equalsIgnoreCase("all"))
        {
            setToolBarElements(BAR_FONT,  BAR_HR ); 
        }
        else if ( sWhat.equalsIgnoreCase("standard"))
        {
	        initStandardToolBar();
        }
        else 
        {
            int i = findElement(sWhat);
            if ( i >= 0 && i < _barElements.length ) // Tool Bar element found
            {
                _barElements[i] = true ; // Set the element in the toolbar
            }
        }
	}
    //----------------------------------------------------------------
	private void initStandardToolBar( )
	{
        setToolBarElements(BAR_FONT, BAR_BCOLOR ); 
	}
    //----------------------------------------------------------------
	private void setToolBarElements( int iFrom, int iTo )
	{
	    if ( iFrom < iTo && iFrom >= 0 && iTo < _barElements.length )
	    {
		    for ( int i = iFrom ; i <= iTo ; i++ )
		    {
		        _barElements[i] = true ;
		    }
	    }
	}
    //----------------------------------------------------------------
	private int findElement( String sWhat )
	{
        String s = null ;
	    for ( int i=0 ; i < ELEM_NAMES.length ; i++ )
	    {
	        s = ELEM_NAMES[i] ;
	        if ( s != null )
	        {
		        if ( s.equalsIgnoreCase(sWhat) )
		        {
		            return i;
		        }
	        }
	    }
	    return -1 ; // Not found
	}
    //----------------------------------------------------------------
	private void clearToolBar( )
	{
	    for ( int i = 0 ; i < _barElements.length ; i++ )
	    {
	        _barElements[i] = false ;
	    }
	}
    //----------------------------------------------------------------
//    public void setFor(String v)
//    {
//        _sFor = v;
//    }
//    public String getFor()
//    {
//        return _sFor ;
//    }

    //----------------------------------------------------------------
//    public String getFullImageSrc(String sResource)
//    {
//        if (sResource != null)
//        {
//            //return TagCommons.getImgDirURL(pageContext) + sSrc;
//            String sRootURL = TagCommons.getRootContextURL(pageContext);
//            if ( sRootURL != null )
//            {
//                return sRootURL + sResource ;
//            }
//        }
//        return "";
//    }
    
    //----------------------------------------------------------------
    public String getRootContextURL()
    {
        return TagCommons.getRootContextURL(pageContext);
    }
    
    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        startTag();
        return (SKIP_BODY);
    }

    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag()
    {
        endTag();        
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }
}
