package org.objectweb.telosys.uil.taglib.widget;

import javax.servlet.jsp.tagext.TagSupport;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.uil.taglib.TagCommons;

/**
 * @author T. Poingt, L. Guerin
 *  
 */
public class Set extends TagSupport
{
    private static final TelosysClassLogger log = new TelosysClassLogger(Set.class);
    
    //----------------------------------------------------------------
    // Constants
    //----------------------------------------------------------------
    private static final String TAG_NAME   = "set";

    //----------------------------------------------------------------
    // Tag attributes
    //----------------------------------------------------------------
    private String              _sVarName  = null;

    private String              _sVarValue = null;

    private String              _sXValue   = null;

    private String              _sYValue   = null;

    private String              _sScreenType = null;

    private String              _sXGap     = null;

    private String              _sYGap     = null;

    private String              _sScope    = null ;
    
    /**
     * Set "var" attribute
     * 
     * @param v
     */
    public void setVar(String v)
    {
        _sVarName = v;
    }

    /**
     * Set "value" attribute
     * 
     * @param v
     */
    public void setValue(String v)
    {
        _sVarValue = v;
    }

    /**
     * Set "scope" attribute
     * 
     * @param v
     */
    public void setScope(String v)
    {
        _sScope = v;
    }

    /**
     * Set "x" attribute
     * 
     * @param v
     */
    public void setX(String v)
    {
        _sXValue = v;
    }

    public void setXgap(String v)
    {
        _sXGap = v;
    }

    /**
     * Set "y" attribute
     * 
     * @param v
     */
    public void setY(String v)
    {
        _sYValue = v;
    }

    public void setYgap(String v)
    {
        _sYGap = v;
    }

    /**
     * Set "screen_type" attribute
     * 
     * @param v
     */
    public void setScreen_type(String v)
    {
        _sScreenType = v;
    }

    //----------------------------------------------------------------
    // Tag processing
    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        //--- SPECIFIC VARIABLE -----------------------
        if (_sVarName != null && _sVarValue != null)
        {
            if ( _sScope != null )
            {
                // Set the variable in "page", "request", "session" or "application" scope
                int iScope = TagCommons.getScope( _sScope );
    	        pageContext.setAttribute( _sVarName, _sVarValue, iScope );
            }
            else
            {
                // Set the variable in the Telosys Variables
                setVariable(_sVarName, _sVarValue);
            }
        }
        //--- X and XGAP ------------------------------
        if (_sXValue != null)
        {
            if ( _sXValue.trim().equals("++") ) // v 0.9.9
            {
                // increment and keep the new value 
                Variables.getPlusPlusX(pageContext);
            }
            else
            {
                setVariable(Variables.X, _sXValue);
            }
        }
        if (_sXGap != null)
        {
            setVariable(Variables.XGAP, _sXGap);
        }
        //--- Y and YGAP ------------------------------
        if (_sYValue != null)
        {
            if ( _sYValue.trim().equals("++") ) // v 0.9.9
            {
                // increment and keep the new value 
                Variables.getPlusPlusY(pageContext);
            }
            else
            {
                setVariable(Variables.Y, _sYValue);
            }
        }
        if (_sYGap != null)
        {
            setVariable(Variables.YGAP, _sYGap);
        }
        //---------------------------------------------
        if (_sScreenType != null)
        {
            setVariable(Variables.SCREEN_TYPE, _sScreenType);
        }
        return (SKIP_BODY);
    }

    private void setVariable(String sVarName, String sVarValue)
    {
        log.trace("setVariable(" + sVarName + "," + sVarValue + ")");
        Variables variables = Variables.getVariables(pageContext);
        if (variables != null)
        {
            variables.set(sVarName, sVarValue);
        }
        else
        {
            TagCommons.printError(pageContext, "SET : Cannot get the variables !");
        }
    }
}