package org.objectweb.telosys.uil.taglib.widget;

import javax.servlet.jsp.tagext.Tag;

import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.TagConst;
import org.objectweb.telosys.uil.taglib.widget.html.TabsHTML;
import org.objectweb.telosys.uil.taglib.widget.xul.TabsXUL;

/**
 * @author Laurent GUERIN
 *  
 */
public class Tabs extends GenericTag
{
    protected static final String TAG_NAME          = "tabs";

    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget  $htmlGen          = new TabsHTML();

    private final static IWidget  $xulGen           = new TabsXUL();

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
    private String                _sAlign           = null ;

    private String                _sSpacers         = null ;

    private String                _sSelectedClass   = null;

    private String                _sUnselectedClass = null;

    private String                _sSpacerClass     = null;

    private String                _sSelectedTab     = null; // v 0.9.9

    /**
     * Constructor
     */
    public Tabs()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
        reset();
    }
    //----------------------------------------------------------------
    private void applyDefaults()
    {
        if (_sSelectedClass == null)
        {
            _sSelectedClass = TagConst.TABS_DEFAULT_CL_SELECTED;
        }
        if (_sUnselectedClass == null)
        {
            _sUnselectedClass = TagConst.TABS_DEFAULT_CL_UNSELECTED;
        }
        if (_sSpacerClass == null)
        {
            _sSpacerClass = TagConst.TABS_DEFAULT_CL_SPACER;
        }
    }

    //----------------------------------------------------------------
    private void reset() // some same instance re-using
    {
        _sAlign = "left";
        _sSpacers = "all";
        _sSelectedClass = null;
        _sUnselectedClass = null;
        _sSpacerClass = null;
        _sSelectedTab = null ;
    }

    //----------------------------------------------------------------
    public void setAlign(String v)
    {
        _sAlign = v;
    }

    public String getAlign()
    {
        return _sAlign;
    }

    //----------------------------------------------------------------
    public void setSpacers(String v)
    {
        _sSpacers = v;
    }

    public String getSpacers()
    {
        return _sSpacers;
        //        TabBox parentTabBox = getParentTabBox();
        //        if (parentTabBox != null)
        //        {
        //            return parentTabBox.getSpacers();
        //        }
    }

    //----------------------------------------------------------------
    public void setCl_selected(String v)
    {
        _sSelectedClass = v;
    }

    //----------------------------------------------------------------
    public void setCl_unselected(String v)
    {
        _sUnselectedClass = v;
    }

    //----------------------------------------------------------------
    public void setCl_spacer(String v)
    {
        _sSpacerClass = v;
    }

    //----------------------------------------------------------------
    public TabBox getParentTabBox()
    {
        Tag parentTag = getParent();
        if (parentTag != null)
        {
            if (parentTag instanceof TabBox)
            {
                return (TabBox) parentTag;
            }
        }
        return null;
    }

    //----------------------------------------------------------------
    public String getTabBoxWidth()
    {
        TabBox parentTabBox = getParentTabBox();
        if (parentTabBox != null)
        {
            return parentTabBox.getWidth();
        }
        else
        {
            error("Cannot get parent TabBox");
        }
        return null;
    }

    //----------------------------------------------------------------
    //    public String getAlign()
    //    {
    //        TabBox parentTabBox = getParentTabBox();
    //        if (parentTabBox != null)
    //        {
    //            String sAlign = parentTabBox.getAlign();
    //            return (sAlign != null ? sAlign : "left");
    //        }
    //        else
    //        {
    //            error("Cannot get parent TabBox");
    //        }
    //        return "left";
    //    }

    //----------------------------------------------------------------
    public boolean hasExternalSpacer(String sAlignRequired)
    {
        String sSpacers = getSpacers();
        if (sSpacers != null)
        {
            sSpacers = sSpacers.trim().toLowerCase();
            if (sSpacers.startsWith("all") || sSpacers.startsWith("ext"))
            {
                String sAlign = getAlign();
                if (sAlign != null)
                {
                    sAlign = sAlign.trim().toLowerCase();
                    if (sAlign.startsWith(sAlignRequired))
                    {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    //----------------------------------------------------------------
    public boolean hasInternalSpacers()
    {
        String sSpacers = getSpacers();
        if (sSpacers != null)
        {
            sSpacers = sSpacers.trim().toLowerCase();
            if (sSpacers.startsWith("all") || sSpacers.startsWith("int"))
            {
                return true;
            }
        }
        return false;
    }


    //----------------------------------------------------------------
    public boolean hasLeftSpacer()
    {
        return hasExternalSpacer("left");
    }

    //----------------------------------------------------------------
    public boolean hasRightSpacer()
    {
        return hasExternalSpacer("right");
    }

    //----------------------------------------------------------------
    public String getUnselectedClass()
    {
        return _sUnselectedClass ;        
    }
    //----------------------------------------------------------------
    public String getSelectedClass()
    {
        return _sSelectedClass ;        
    }
    //----------------------------------------------------------------
    public String getSpacerClass()
    {
        return _sSpacerClass ;        
    }
    //----------------------------------------------------------------
    private String getSpacerClass(String sSide)
    {
        //String sClass = this.getCl();
        if ( _sSpacerClass != null )
        {
            return _sSpacerClass.trim() + "_" + sSide;
        }
        return null ;
    }

    //----------------------------------------------------------------
    public String getLeftSpacerClass()
    {
        return getSpacerClass("left");
    }

    //----------------------------------------------------------------
    public String getRightSpacerClass()
    {
        return getSpacerClass("right");
    }

    //----------------------------------------------------------------
    public void setSelectedtab(String v) // v 0.9.9
    {
        _sSelectedTab = v;
    }

    public String getSelectedtab() // v 0.9.9
    {
        return replaceSymbolicVar(_sSelectedTab);
    }

    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        applyDefaults();
        Page.initTabIndex(pageContext);

        startTag(); // Must be called before any "replaceSymbolicVar"
        
        //--- Is there a selected tab ? // v 0.9.9
        String s = getSelectedtab() ; // NB : call "replaceSymbolicVar"
        if ( s != null  ) 
        {
            //--- Init the selected tab ( can be overridden by each "tab" tag ) 
            Page.setSelectedTabId(pageContext, s);
        }

        return (EVAL_BODY_INCLUDE);
    }

    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag()
    {
        endTag();
        reset();
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }

}