package org.objectweb.telosys.uil.taglib.widget;

import java.io.InputStream;
import java.util.Properties;

import org.objectweb.telosys.common.Tool;

/**
 * @author T. Poingt / L. Guerin
 * 
 * This static class provides methods to managed the default values for
 * different tags. The default values are initialized from a standard properties
 * file using the syntax : tag.attribute = value
 */
public class TagDefaultValues
{

    private static Properties   _propDefaultValues           = null;

    /**
     * Init the default values with the given properties file name
     * @param sPropFileName
     * @return true if OK, else false
     */
    public static boolean init(String sPropFileName)
    {
        _propDefaultValues = Tool.loadProperties(sPropFileName);
        return ( _propDefaultValues != null ) ;
    }

    /**
     * Init the default values with the given properties InputStream
     * @param is
     * @return
     */
    public static boolean init( InputStream is )
    {
        _propDefaultValues = Tool.loadProperties(is);
        return ( _propDefaultValues != null ) ; 
    }

    /**
     * Return the default attribute value for a given tag name The default
     * values are
     * 
     * @param sTag
     * @param sAttrib
     * @return default value (or null if no default value)
     */
    public static String getDefaultAttributeValue(String sTag, String sAttrib)
    {
        if (_propDefaultValues != null)
        {
            return (String) _propDefaultValues.get(sTag + "." + sAttrib);
        }
        else
        {
            return null;
        }
    }

}