package org.objectweb.telosys.uil.taglib.widget;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.servlet.jsp.JspWriter;

/**
 * @author T. Poingt / L. Guerin
 * 
 * Contains and provides the current screen environment. 
 * There is one instance of this class for each "screen map", 
 * the instance is stored in the "JSP page context". It provides all the informations
 * needed for the tag processing : - attributes default values - id count 
 *  
 */
public class TagScreenEnv
{
    //--- CONST
//    private final static String PREFIX_FOR_GENERATED_ID   = "fwk_id";

    //--- ATTRIBUTES
//    private int                 _iCountId                 = 0;

    private Hashtable           _htDefaultAttributeValues = null;

    /**
     * Public Constructor
     */
    public TagScreenEnv()
    {
        //System.out.println("TagScreenEnv : Constructor " );
        _htDefaultAttributeValues = new Hashtable();
    }

    private String buildKey(String sTag, String sAttrib)
    {
        return (sTag + "." + sAttrib);
    }

    /**
     * Return the default attribute value for a given tag name
     * 
     * @param sTag
     * @param sAttrib
     * @return default value (or null if no default value)
     */
    public String getDefaultAttributeValue(String sTag, String sAttrib)
    {
        String sValue = (String) _htDefaultAttributeValues.get(buildKey(sTag, sAttrib));
        //System.out.println("getDefaultAttributeValue --> Value = '"+sValue +"' ");
        return sValue;
    }

    /**
     * Set the default attribute value for a given tag name
     * 
     * @param sTag
     * @param sAttrib
     * @param sValue
     */
    public void setDefaultAttributeValue(String sTag, String sAttrib, String sValue)
    {
        _htDefaultAttributeValues.put(buildKey(sTag, sAttrib), sValue);
    }

    /**
     * Reset the default attribute value for a given tag name
     * 
     * @param sTag
     * @param sAttrib
     */
    public void resetDefaultAttributeValue(String sTag, String sAttrib)
    {
        _htDefaultAttributeValues.remove(buildKey(sTag, sAttrib));
    }

//    /**
//     * Return a default built id
//     * 
//     * @return id built
//     */
//    public String generateId()
//    {
//        //System.out.println("TagScreenEnv : generateId() : iCountId = " + _iCountId);
//        _iCountId++;
//        return PREFIX_FOR_GENERATED_ID + _iCountId;
//    }
    
    public void printDefaults(JspWriter out)
    {
        String sKey = null ;
        Enumeration keys = _htDefaultAttributeValues.keys();
        try
        {
            if ( keys != null )
            {
                while ( keys.hasMoreElements() )
                {
                    sKey = (String) keys.nextElement();
                    out.println(" . '" + sKey + "' : '" + _htDefaultAttributeValues.get(sKey) + "'" );
                }            
            }
            else
            {
                out.println("void");
            }
        } catch (IOException e)
        {
            // Nothing to do 
        }
    }
}