package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.widget.data.TreeViewNode;

/**
 * @author Aurelien COUERON
 *  
 */
public class TreeNode extends GenericTag
{
    private static final String  TAG_NAME = "treenode";

    //----------------------------------------------------------------
    // TAG generators : NO TAG GENERATORS ( just use to buil the tree in memory )
    //----------------------------------------------------------------
    private final static IWidget $htmlGen = null;

    private final static IWidget $xulGen  = null;

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
    private String _sValue     = null;

    private String _sImg       = null;
    
    private String _sOnclick   = null;
    
    private String _sType      = null; // v 0.9.9

    private String _sOpen      = null; // v 0.9.9


    /**
     * Constructor
     */
    public TreeNode()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
    }

    //----------------------------------------------------------------
    // Opening TAG
    //----------------------------------------------------------------
    public int doStartTag()
    {
        // No output generation at the tag level, but startTag() must be called for tag init
        startTag(NO_OUTPUT);
        
        //--- Add a new node in the tree ...
        TreeViewNode node = new TreeViewNode ( 
                this.getId(), this.getCl(), this.getStyle(), 
                this.getImg(), this.getTxt(), this.getValue(), this.getOnclick() );
        
        if ( _sType != null ) // v 0.9.9
        {
            node.setType(_sType); // force node type
        }
        if ( _sOpen != null ) // v 0.9.9
        {
            node.setOpen(_sOpen); // force "branch" node state OPEN/CLOSED
        }
        
        //--- Add the node in the tree and set "current node" 
        TreeViewNode currentNode = Page.getCurrentNode(this.getPageContext());
        if ( currentNode != null )
        {
            currentNode.appendChild( node );
        }
        else
        {
            //out.println("<!-- ERROR : No Current Node -->");
        }
        Page.setCurrentNode(this.getPageContext(), node);
        //out.println("<!-- Current Node set : " + node + " -->");

        return (EVAL_BODY_INCLUDE);
    }

    //----------------------------------------------------------------
    // Closing TAG
    //----------------------------------------------------------------
    public int doEndTag()
    {
        // No output generation at the tag level
        // endTag();

        TreeViewNode currentNode = Page.getCurrentNode(this.getPageContext());
        if ( currentNode != null )
        {
            Page.setCurrentNode(this.getPageContext(), currentNode.getParentNode());
            //out.println("<!-- Current Node set (parent node) : " + currentNode.getParentNode() + " -->");
        }
        else
        {
            Page.setCurrentNode(this.getPageContext(), null);
            //out.println("<!-- Current Node set 'null' -->");
        }      
        
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }

    //-----------------------------------------------------------------------------
    public String getValue()
    {
        return _sValue;
    }

    public void setValue(String v)
    {
        _sValue = v;
    }

    //-----------------------------------------------------------------------------
    public String getImg()
    {
        return _sImg;
    }

    public void setImg(String v)
    {
        _sImg = v;
    }

    //-----------------------------------------------------------------------------
    public String getType()
    {
        return _sType;
    }

    public void setType(String v)
    {
        _sType = v;
    }

    //-----------------------------------------------------------------------------
    public String getOpen()
    {
        return _sOpen;
    }

    public void setOpen(String v)
    {
        _sOpen = v;
    }

    //-----------------------------------------------------------------------------
    public String getOnclick()
    {
        return _sOnclick;
    }

    public void setOnclick(String v)
    {
        _sOnclick = v;
    }
    //-----------------------------------------------------------------------------
    
}
