package org.objectweb.telosys.uil.taglib.widget;

import java.io.IOException;
import java.util.HashMap;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.util.StrUtil;

/**
 * @author T. Poingt, L. Guerin
 *  
 */
public class Variables
{
    private static final TelosysClassLogger log = new TelosysClassLogger(Variables.class);
    
    private static final String VARIABLES    = "SCREEN_MAP_VARIABLES";

    protected static final String X = "x";

    protected static final String Y = "y";
    
    protected static final String XGAP = "xgap";

    protected static final String YGAP = "ygap";
    
    protected static final String SCREEN_TYPE = "screen_type" ;

    private final HashMap       _hmVariables = new HashMap();

    /**
     * Return a variable storage space for the given page context (create it if necessary)
     * @param pageContext
     * @return
     */
    public final static Variables getVariables( PageContext pageContext )
    {
        return getVariables( pageContext, true );
    }
    
    /**
     * Return the variable storage space for the given page context if it exists,<br>
     * else return null.
     * 
     * @param pageContext
     * 
     * @param create true to create a new Variables instance if necessary;
     *  false to return null if there's no current Variables instance
     * 
     * @return
     */
    public final static Variables getVariables(PageContext pageContext, boolean create)
    {
        //return (HashMap) pageContext.getAttribute(VARIABLES);
        //--- Try to find an existing variables storage space in the page context
        Variables variables = (Variables) pageContext.getAttribute(VARIABLES);
        if (variables == null) // If not found => not yet created
        {
            if ( create )
            {
	            //--- Variables storage space creation
	            variables = new Variables();
	            //--- Keep in the page context for the next time
	            pageContext.setAttribute(VARIABLES, variables);
            }
        }
        return variables;
    }

    /**
     * Set a variable value
     * 
     * @param sVarName
     * @param sVarValue
     */
    public final void set(String sVarName, String sVarValue)
    {
        log.trace("set('" + sVarName + "', '" + sVarValue + "')");
        if (_hmVariables != null)
        {
            if ( sVarName != null && sVarValue != null )
            {
                _hmVariables.put(sVarName, sVarValue);
            }
        }
    }

//    public final void setX(String sVarValue)
//    {
//        set("x", sVarValue);
//    }
//
//    public final void setY(String sVarValue)
//    {
//        set("y", sVarValue);
//    }

    /**
     * Get a variable value
     * 
     * @param sVarName
     * @return
     */
    public final String get(String sVarName)
    {
        if (_hmVariables != null)
        {
            Object oValue = _hmVariables.get(sVarName);
            if (oValue != null)
            {
                if (oValue instanceof String)
                {
                    return (String) oValue;
                }
            }
        }
        return null;
    }
    
//    public final String getX()
//    {
//        return get(X);
//    }
//    
    //----------------------------------------------------------------------------------
    private final static boolean notVoid(String sTxt)
    {
        if (sTxt != null)
        {
            if (sTxt.trim().length() > 0)
            {
                return true; // not void
            }
        }
        return false;
    }
    
    //----------------------------------------------------------------------------------
    /**
     * Returns the value of the given variable or null if the variable is not defined.
     * @param pageContext
     * @param sVarName
     * @return
     */
    public final static String getVarValue( PageContext pageContext, String sVarName )
    {
        Variables variables = Variables.getVariables(pageContext, false);
        if ( variables != null )
        {
            return variables.get( sVarName );
        }
        return null ;
    }
    //----------------------------------------------------------------------------------
    /**
     * Returns the current X variable value or "0" if no X variable defined
     * @param pageContext
     * @return X value, never null ( returns "0" if undefined )
     */
    public final static String getX( PageContext pageContext )
    {
        String sRet = getXorY( pageContext, X );
        log.trace("getX() --> " + sRet );
        return sRet;
    }
    
    //----------------------------------------------------------------------------------
    /**
     * Returns the current Y variable value or "0" if no Y variable defined
     * @param pageContext
     * @return Y value, never null ( returns "0" if undefined )
     */
    public final static String getY( PageContext pageContext )
    {
        String sRet = getXorY( pageContext, Y );
        log.trace("getY() --> " + sRet );
        return sRet;
    }
    
    //----------------------------------------------------------------------------------
    private final static String getXorY( PageContext pageContext, String sValName )
    {
        //log.trace("getXorY('" + sValName + "')");
        String sRet = "0" ;
        Variables variables = Variables.getVariables(pageContext, false);
        if ( variables != null )
        {
            String sVar = variables.get( sValName );
            sRet = (notVoid(sVar) ? sVar : "0");
        }
        //log.trace("getXorY('" + sValName + "')  -> " + sRet );
        return sRet;
    }
    
    //----------------------------------------------------------------------------------
    /**
     * "x++" operation ( increment "x", but return its original value )
     * @param pageContext
     * @return X value, never null ( returns "0" if undefined )
     */
    public final static String getXPlusPlus( PageContext pageContext )
    {
        String sRet = getValuePlusPlus ( pageContext, X, XGAP );
        log.trace("getXPlusPlus -> " + sRet );
        return sRet;
    }
    //----------------------------------------------------------------------------------
    /**
     * "++x" operation ( increment "x" and return its new value )
     * @param pageContext
     * @return X value, never null ( returns "0" if undefined )
     */
    public final static String getPlusPlusX( PageContext pageContext )
    {
        String sRet = getPlusPlusValue ( pageContext, X, XGAP );
        log.trace("getPlusPlusX -> " + sRet );
        return sRet;
    }
    //----------------------------------------------------------------------------------
    /**
     * "y++" operation ( increment "y", but return its original value )
     * @param pageContext
     * @return Y value, never null ( returns "0" if undefined )
     */
    public final static String getYPlusPlus( PageContext pageContext )
    {
        String sRet = getValuePlusPlus ( pageContext, Y, YGAP );
        log.trace("getYPlusPlus -> " + sRet );
        return sRet;
    }
    //----------------------------------------------------------------------------------
    /**
     * "++y" operation ( increment "y" and return its new value )
     * @param pageContext
     * @return Y value, never null ( returns "0" if undefined )
     */
    public final static String getPlusPlusY( PageContext pageContext )
    {
        String sRet = getPlusPlusValue ( pageContext, Y, YGAP );
        log.trace("getPlusPlusY -> " + sRet );
        return sRet;
    }
    //----------------------------------------------------------------------------------
    /**
     * Generic method for "var++" operation<br>
     * Returns the current value, then increment it for future use
     * @param pageContext
     * @param sVal
     * @param sGap
     * @return
     */
    private final static String getValuePlusPlus( PageContext pageContext, String sVal, String sGap )
    {
        log.trace("getValuePlusPlus(..," + sVal + "," + sGap + ")"  );
        String sRetValue = "0" ;
        Variables variables = Variables.getVariables(pageContext);
        if (variables != null)
        {
            //log.trace("getValuePlusPlus :  variables.get( sVal ) = '" + variables.get( sVal ) + "'"  );
            //log.trace("getValuePlusPlus :  variables.get( sGap ) = '" + variables.get( sGap ) + "'"  );
            // get the current value and its gap
            int val = StrUtil.getInt( variables.get( sVal ), 0 );
            int gap = StrUtil.getInt( variables.get( sGap ), 0 );
            // 1) Set the value to return BEFORE increment
            sRetValue = "" + val ;  // return the same value  
            // 2) Increment the value and keep it
            val = val + gap ;
            variables.set(sVal, ""+val); // keep the new value
        }
//        else
//        {
//            log.trace("getValuePlusPlus -> NO VARIABLES"  );
//        }
        //log.trace("getValuePlusPlus(..," + sVal + "," + sGap + ") : return " + sRetValue );
        return sRetValue ;
    }

    //----------------------------------------------------------------------------------
    /**
     * Generic method for "++var" operation <br>
     * Increment the current value, then return its new value 
     * @param pageContext
     * @param sVal
     * @param sGap
     * @return
     */
    private final static String getPlusPlusValue( PageContext pageContext, String sVal, String sGap )
    {
        log.trace("getPlusPlusValue(..," + sVal + "," + sGap + ")"  );
        String sRetValue = "0" ;
        Variables variables = Variables.getVariables(pageContext);
        if (variables != null)
        {
            //log.trace("getPlusPlusValue :  variables.get( sVal ) = '" + variables.get( sVal ) + "'"  );
            //log.trace("getPlusPlusValue :  variables.get( sGap ) = '" + variables.get( sGap ) + "'"  );
            // get the current value and its gap
            int val = StrUtil.getInt( variables.get( sVal ), 0 );
            int gap = StrUtil.getInt( variables.get( sGap ), 0 );
            // 1) Increment the current value and keep it
            val = val + gap ;
            variables.set(sVal, ""+val); // keep the new value
            // 2) Set the value to return AFTER increment
            sRetValue = "" + val ; // return the new value  
        }
//        else
//        {
//            log.trace("getPlusPlusValue -> NO VARIABLES"  );
//        }
        //log.trace("getPlusPlusValue(..," + sVal + "," + sGap + ") : return " + sRetValue );
        return sRetValue ;
    }
    
    //----------------------------------------------------------------------------------
    public final String getScreenType()
    {
        return get( SCREEN_TYPE );
    }
    
    /**
     * Unset a variable
     * 
     * @param sVarName
     */
    public final void unset(String sVarName)
    {
        if (_hmVariables != null)
        {
            _hmVariables.remove(sVarName);
        }
    }

    /**
     * Print all the declared variables on the given output
     * 
     * @param out
     * @throws IOException
     */
    public final void print(JspWriter out) throws IOException
    {
        if (_hmVariables != null)
        {
            java.util.Set keys = _hmVariables.keySet();
            if (keys != null)
            {
                Object[] names = keys.toArray();
                String sName = null;
                String sValue = null;
                for ( int i = 0 ; i < names.length ; i++ )
                {
                    sName = (String) names[i];
                    sValue = (String) _hmVariables.get(sName);
                    out.println(" . " + sName + " = " + sValue);
                }
            }
        }
        else
        {
            out.println(" (no variables) ");
        }
    }

}