package org.objectweb.telosys.uil.taglib.widget.data;


/**
 * Generic Menu for tags generation
 * @author Laurent GUERIN
 * 
 */
public abstract class Menu 
{
    public final static int MENU_NO_TYPE   = 0 ;
    public final static int MENU_BAR       = 1 ;
    public final static int MENU_BOX       = 2 ;
    
    //--- Common attributes 
    private int    _iType   = MENU_NO_TYPE ;
    private String _sId     = null ;
    private String _sCl     = null ;
    private String _sStyle  = null ;
    private String _sWidth  = null ;
    private String _sHeight = null ;
    
    //--- MENU_BAR and MENU_POPUP 
    //private LinkedList _items = null ; // list of "MenuItem" objects 
    private MenuItems _items = null ; // list of "MenuItem" objects 

    //----------------------------------------------------------------------------------------
    // Constructors
    //----------------------------------------------------------------------------------------
    /**
     * Constructor
     */
    protected Menu (int iType, String sId, String sCl, String sStyle, 
            String sWidth, String sHeight,
            MenuItems children) 
    {
        super();
        _iType  = iType ;
        _sId    = sId ;
        _sCl    = sCl;
        _sStyle = sStyle;
        _sWidth  = sWidth ;
        _sHeight = sHeight ;
        _items = children;
    }
    
    //----------------------------------------------------------------------------------------
    // Getters & Setters
    //----------------------------------------------------------------------------------------    
    public int getType() 
    {
        return _iType;
    }
    public String getId()
    {
        return _sId;
    }
    public String getCl()
    {
        return _sCl;
    }
    public String getStyle()
    {
        return _sStyle;
    }
    public String getWidth()
    {
        return _sWidth;
    }
    public String getHeight()
    {
        return _sHeight;
    }
    
    //----------------------------------------------------------------------------------------
    // Items management
    //----------------------------------------------------------------------------------------    
    public MenuItems getItems() 
    {
        return _items;
    }
    //----------------------------------------------------------------------------------------
    public boolean hasItems() 
    {
        if ( _items != null )
        {
            return ( _items.size() > 0 ) ;
        }
        return false ;
    }
    //----------------------------------------------------------------------------------------    
    /**
     * Appends an item in the menu
     * @param item
     * @return
     */
    public boolean addItem( MenuItem item ) 
    {
        if ( _items == null )
        {
            _items = new MenuItems();
        }
        return _items.addItem(item);
    }
    
    //----------------------------------------------------------------------------------------    
    /**
     * Appends a separator in the menu
     * @return
     */
    public boolean addSeparator() 
    {
        if ( _items == null )
        {
            _items = new MenuItems();
        }
        return _items.addSeparator();
    }
    
//    //----------------------------------------------------------------------------------------    
//    /* (non-Javadoc)
//     * @see java.lang.Object#toString()
//     */
//    public String toString() 
//    {
//        int iChildrenCount = ( _items != null ? _items.size() : 0 ) ;
//        String sChilNodes = ( iChildrenCount > 0 ? iChildrenCount + " child node(s) " : "no child nodes" );
//        return "MenuNode [type:"+_iType +"|id:" +_sId +"|cl:"+_sCl + "|style:" +_sStyle
//        +"|img:"+_sImg +"|txt:"+_sTxt + "|onclick:"+_sOnClick + "|"+sChilNodes + "]";
//    }
    
}
