package org.objectweb.telosys.uil.taglib.widget.data;

/**
 * Menu item data bean for Menu tags generation
 * @author Laurent GUERIN
 * 
 */
public class MenuItem 
{
    public final static int MENU_NO_TYPE   = 0 ;
    public final static int MENU_ITEM      = 1 ;
    public final static int MENU_SEPARATOR = 2 ;
    
    //--- Common attributes 
    private int     _iType   = MENU_NO_TYPE ;
    private String  _sId     = null ;
//    private String _sCl    = null ;
//    private String _sStyle = null ;    
    
    //--- Image and Text 
    private String  _sImg     = null ;
    private String  _sTxt     = null ;
    
    //--- Actions or links
    private String  _sOnClick = null ;
    private String  _sHref    = null ;
    private String  _sCres    = null ;
    private String  _sScreen  = null ;
    private String  _sParam   = null ;
    
    //--- Link to MenuBox
    private String  _sMenuboxId = null ; // Relational reference (via MenuBox id )
    private MenuBox _menubox    = null ; // Object reference
    
    //----------------------------------------------------------------------------------------
    // Constructors
    //----------------------------------------------------------------------------------------
    /**
     * Constructor for "separator"
     */
    public MenuItem () 
    {
        super();
        _iType  = MENU_SEPARATOR ;
    }
    
    //----------------------------------------------------------------------------------------
    /**
     * Constructor for standard "menu item"  ( used by MenuItemTAG )
     * @param sId : the HTML id ( supposed to be NOT NULL )
     * @param sImg 
     * @param sTxt
     */
    public MenuItem (String sId, String sImg, String sTxt ) 
    {
        super();
        _iType  = MENU_ITEM ;
        _sId    = sId ;
//        _sCl    = sCl;
//        _sStyle = sStyle;

        _sImg    = sImg;
        _sTxt    = sTxt;
    }
    
    //----------------------------------------------------------------------------------------
    public boolean isSeparator() 
    {
        return ( _iType == MENU_SEPARATOR ) ;
    }
    
    //----------------------------------------------------------------------------------------
    // Getters & Setters
    //----------------------------------------------------------------------------------------    
    public int getNodeType() 
    {
        return _iType;
    }
    public String getId()
    {
        return _sId;
    }
//    public String getCl()
//    {
//        return _sCl;
//    }
//    public String getStyle()
//    {
//        return _sStyle;
//    }
    //----------------------------------------------------------------------------------------
    public void setImg( String v ) 
    {
        _sImg = v ;
    }
    public String getImg()
    {
        return _sImg;
    }
    //----------------------------------------------------------------------------------------
    public void setTxt( String v ) 
    {
        _sTxt = v ;
    }
    public String getTxt()
    {
        return _sTxt;
    }
    //----------------------------------------------------------------------------------------
    public void setOnclick( String v ) 
    {
        _sOnClick = v ;
    }
    public String getOnclick()
    {
        return _sOnClick;
    }
    
    //----------------------------------------------------------------------------------------
    public void setHref( String v ) 
    {
        _sHref = v ;
    }
    public String getHref()
    {
        return _sHref;
    }
    
    //----------------------------------------------------------------------------------------
    public void setCres( String v ) 
    {
        _sCres = v ;
    }
    public String getCres()
    {
        return _sCres;
    }
    
    //----------------------------------------------------------------------------------------
    public void setScreen( String v ) 
    {
        _sScreen = v ;
    }
    public String getScreen()
    {
        return _sScreen ;
    }
    //----------------------------------------------------------------------------------------
    public void setParam( String v ) 
    {
        _sParam = v ;
    }
    public String getParam()
    {
        return _sParam ;
    }
    
    //----------------------------------------------------------------------------------------
    public void setMenuBoxId( String v ) 
    {
        _sMenuboxId = v ;
    }
    public String getMenuBoxId()
    {
        return _sMenuboxId ;
    }
    //----------------------------------------------------------------------------------------
    public void setMenuBox( MenuBox v ) 
    {
        _menubox = v ;
    }
    public MenuBox getMenuBox()
    {
        return _menubox ;
    }
    /**
     * @return true if the item is a submenu (points to a MenuBox) , else false
     */
    public boolean isSubMenu() 
    {
        if ( _menubox != null ) 
        {
            return true ;
        }
        if ( _sMenuboxId != null ) 
        {
            return true ;
        }
        return false ;
    }
    //----------------------------------------------------------------------------------------
    
    
    //----------------------------------------------------------------------------------------    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() 
    {
        return "MenuItem [type:"+_iType 
        + "|id:" +_sId 
//        + "|cl:"+_sCl 
//        + "|style:" +_sStyle
        + "|img:"+_sImg 
        + "|txt:"+_sTxt 
        + "|onclick:"+_sOnClick 
        + "|href:"+ _sHref 
        + "|cres:"+ _sCres 
        + "|scree:"+ _sScreen 
        + "|param:"+ _sParam 
        + "|menubox:"+ ( _menubox != null ? "true" :"false" ) 
        + "]";
    }    
}
