package org.objectweb.telosys.uil.taglib.widget.data;

import java.util.LinkedList;

import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Aurelien COUERON 
 * Node bean to fill TreeView 
 */
public class TreeViewNode  extends TelosysObject
{
    public final static int TYPE_UNKNOWN = 0 ;
    public final static int TYPE_LEAF    = 1 ;
    public final static int TYPE_BRANCH  = 2 ;
    
    private String _sId      = null ;
    private String _sCl      = null ;
    private String _sStyle   = null ;

    private String _sValue   = null ;
    private String _sImg     = null ;
    private String _sTxt     = null ;
    
    private int     _iType    = TYPE_UNKNOWN ; // v 0.9.9
    private boolean _bOpen    = true ; // v 0.9.9
    
    private String _sOnclick  = null ;
    
    private TreeViewNode _parentNode   = null ;
    
    private LinkedList   _listChildren = null ;
    
    //----------------------------------------------------------------------------------------
    // Constructor(s)
    //----------------------------------------------------------------------------------------    
    public TreeViewNode ( String sTxt )
    {
        super();
        _sTxt = sTxt ;
    }

    public TreeViewNode ( String sTxt, String sOnclick )
    {
        super();
        _sTxt = sTxt ;
        _sOnclick = sOnclick;
    }
    
    public TreeViewNode ( String sId, String sCl, String sStyle, String sImg, String sTxt, String sValue, String sOnclick )
    {
        super();
        _sId = sId;
        _sCl = sCl;
        _sStyle = sStyle;
        _sImg = sImg;
        _sTxt = sTxt ;
        _sValue = sValue;        
        _sOnclick = sOnclick;
    }
    
    //----------------------------------------------------------------------------------------
    // Getters & Setters
    //----------------------------------------------------------------------------------------    
    public void setId(String v)
    {
        _sId = v;
    }
    public String getId()
    {
        return _sId;
    }
    //----------------------------------------------------------------------------------------
    public void setCl(String v)
    {
        _sCl = v;
    }
    public String getCl()
    {
        return _sCl;
    }
    //----------------------------------------------------------------------------------------
    public void setStyle(String v)
    {
        _sStyle = v;
    }
    public String getStyle()
    {
        return _sStyle;
    }
    //----------------------------------------------------------------------------------------
    public void setImg( String v ) 
    {
        _sImg = v ;
    }
    public String getImg()
    {
        return _sImg;
    }
    //----------------------------------------------------------------------------------------
    public void setTxt( String v ) 
    {
        _sTxt = v ;
    }
    public String getTxt()
    {
        return _sTxt;
    }
    //----------------------------------------------------------------------------------------
    public void setOnclick( String v ) 
    {
        _sOnclick = v ;
    }
    public String getOnclick()
    {
        return _sOnclick;
    }
    
    //----------------------------------------------------------------------------------------
    public void setValue( String v ) 
    {
        _sValue = v ;
    }
    public String getValue()
    {
        return _sValue;
    }
    //----------------------------------------------------------------------------------------
    public void setType( String v ) 
    {
        _iType = TYPE_UNKNOWN ;
        if ( v != null )
        {
            String s = v.trim().toLowerCase();
            if ( s.equals("branch") )
            {
                _iType = TYPE_BRANCH ;
            }
            else if ( s.equals("leaf") )
            {
                _iType = TYPE_LEAF ;
            }
        }
    }
    //----------------------------------------------------------------------------------------
    public void setOpen(String v)
    {
        if ( v != null )
        {
            _bOpen = v.trim().equalsIgnoreCase("true"); 
        }
    }

    public void setOpen(boolean v)
    {
        _bOpen = v;
    }

    public boolean isOpen()
    {
        return _bOpen;
    }

    
    //----------------------------------------------------------------------------------------
    // Child nodes & parent node management
    //----------------------------------------------------------------------------------------    
//    protected void setChildNodes( LinkedList list ) 
//    {
//        _listChildren = list ;
//    }
    //----------------------------------------------------------------------------------------
    public LinkedList getChildNodes() // like W3C DOM
    {
        return _listChildren;
    }
    //----------------------------------------------------------------------------------------
    public boolean hasChildNodes() // like W3C DOM
    {
        if ( _listChildren != null )
        {
            return ( _listChildren.size() > 0 ) ;
        }
        return false ;
    }
    //----------------------------------------------------------------------------------------
    /**
     * Returns the type of the node : <br>
     * "branch" ( TYPE_BRANCH ) : if hasChildNodes or the type was set to "branch" <br>
     * "leaf" ( TYPE_LEAF ) : if doesn't have ChildNodes and the type wasn't set to "branch"<br>
     * @return
     */
    public int getNodeType() // like W3C DOM
    {
        if ( hasChildNodes() ) // ChilNodes => BRANCH
        {
            return TYPE_BRANCH ;
        }
        else
        {
            return ( _iType == TYPE_BRANCH ? TYPE_BRANCH : TYPE_LEAF ) ;
        }
    }
    //----------------------------------------------------------------------------------------    
    /**
     * Appends a child node
     * @param node
     * @return
     */
    public boolean appendChild( TreeViewNode node ) // like W3C DOM
    {
        if ( _listChildren == null )
        {
            _listChildren = new LinkedList();
        }
        node.setParentNode(this);
        return _listChildren.add(node);
    }
    
    //----------------------------------------------------------------------------------------
    protected void setParentNode( TreeViewNode node ) // like W3C DOM
    {
        _parentNode = node ;
    }
    
    //----------------------------------------------------------------------------------------
    public TreeViewNode getParentNode() // like W3C DOM
    {
        return _parentNode ;
    }
    
    //----------------------------------------------------------------------------------------
    /**
     * ToString
     */
    public String toString() {
        int n = ( _listChildren != null ? _listChildren.size() : 0 ) ;
        return "TreeViewNode ["+_sId
	    	+"|"+_sCl
	    	+"|"+_sStyle
        	+"|"+_sTxt
        	+"|"+_sValue
        	+"|"+_sOnclick
        	+"|" + n + " child node(s)" ;
    }
    
}
