package org.objectweb.telosys.uil.taglib.widget.data.old;

import java.util.LinkedList;

/**
 * Menu node implementation for Menu tags generation
 * @author Laurent GUERIN
 * 
 */
public class MenuNode 
{
    public final static int MENU_NO_TYPE   = 0 ;
    public final static int MENU_BAR       = 1 ;
    public final static int MENU_POPUP     = 2 ;
    public final static int MENU_ITEM      = 3 ;
    public final static int MENU_SEPARATOR = 4 ;
    
    //--- Common attributes 
    private int    _iType  = MENU_NO_TYPE ;
    private String _sId    = null ;
    private String _sCl    = null ;
    private String _sStyle = null ;
    
    //--- MENU_ITEM 
    private String _sTxt     = null ;
    private String _sImg     = null ;
    private String _sOnClick = null ;
    
    //--- MENU_BAR and MENU_POPUP 
    private LinkedList _listChildren = null ; // list of "MenuNode" objects 

    //----------------------------------------------------------------------------------------
    // Constructors
    //----------------------------------------------------------------------------------------
    /**
     * Constructor for "MENU_BAR" node
     * @param iType
     * @param sId
     * @param sCl
     * @param sStyle
     * @param children
     */
//    public MenuNode (int iType, String sId, String sCl, String sStyle, 
//            LinkedList children) 
//    {
//        super();
//        _iType  = iType ;
//        _sId    = sId ;
//        _sCl    = sCl;
//        _sStyle = sStyle;
//        //--- Specific attributes
//        _listChildren = children;
//    }
    
    //----------------------------------------------------------------------------------------
    /**
     * Constructor for "MENU_POPUP" node
     * @param iType
     * @param sId
     * @param sCl
     * @param sStyle
     * @param children
     */
//    public MenuNode (int iType, String sId, String sCl, String sStyle, 
//            String sTxt, LinkedList children) 
//    {
//        super();
//        _iType  = iType ;
//        _sId    = sId ;
//        _sCl    = sCl;
//        _sStyle = sStyle;
//        //--- Specific attributes
//        _sTxt = sTxt ;
//        _listChildren = children;
//    }
    
    //----------------------------------------------------------------------------------------
    /**
     * Constructor for "MENU_ITEM" node
     * @param iType
     * @param sId
     * @param sCl
     * @param sStyle
     * @param sTxt
     * @param sImg
     * @param sOnClick
     */
//    public MenuNode (int iType, String sId, String sCl, String sStyle, 
//            String sImg, String sTxt, String sOnClick) 
//    {
//        super();
//        _iType  = iType ;
//        _sId    = sId ;
//        _sCl    = sCl;
//        _sStyle = sStyle;
//        //--- Specific attributes
//        _sImg = sImg ;
//        _sTxt = sTxt ;
//        _sOnClick = sOnClick ;
//    }
    
    //----------------------------------------------------------------------------------------
    /**
     * Constructor for "MENU_SEPARATOR" node
     * @param iType
     */
//    public MenuNode (int iType ) 
//    {
//        super();
//        _iType  = iType ;
//    }
    
    
    public MenuNode (int iType, String sId, String sCl, String sStyle ) 
    {
        super();
        _iType  = iType ;
        _sId    = sId ;
        _sCl    = sCl;
        _sStyle = sStyle;
    }
    
    //----------------------------------------------------------------------------------------
    // Getters & Setters
    //----------------------------------------------------------------------------------------    
    public int getNodeType() // like W3C DOM
    {
        return _iType;
    }
    public String getId()
    {
        return _sId;
    }
    public String getCl()
    {
        return _sCl;
    }
    public String getStyle()
    {
        return _sStyle;
    }
    //----------------------------------------------------------------------------------------
    protected void setImg( String v ) 
    {
        _sImg = v ;
    }
    public String getImg()
    {
        return _sImg;
    }
    //----------------------------------------------------------------------------------------
    protected void setTxt( String v ) 
    {
        _sTxt = v ;
    }
    public String getTxt()
    {
        return _sTxt;
    }
    //----------------------------------------------------------------------------------------
    protected void setOnclick( String v ) 
    {
        _sOnClick = v ;
    }
    public String getOnclick()
    {
        return _sOnClick;
    }
    
    //----------------------------------------------------------------------------------------
    // Child nodes management
    //----------------------------------------------------------------------------------------    
    protected void setChildNodes( LinkedList list ) 
    {
        _listChildren = list ;
    }
    //----------------------------------------------------------------------------------------
    public LinkedList getChildNodes() // like W3C DOM
    {
        return _listChildren;
    }
    //----------------------------------------------------------------------------------------
    public boolean hasChildNodes() // like W3C DOM
    {
        if ( _listChildren != null )
        {
            return ( _listChildren.size() > 0 ) ;
        }
        return false ;
    }
    //----------------------------------------------------------------------------------------    
    /**
     * Appends a child node ( only if the node is MENU_BAR or MENU_POPUP )
     * @param node
     * @return
     */
    protected boolean appendChild( MenuNode node ) // like W3C DOM
    {
        if ( _iType != MENU_BAR && _iType != MENU_POPUP )
        {
            return false ; // TODO : throw Exception ?
        }
        if ( _listChildren == null )
        {
            _listChildren = new LinkedList();
        }
        return _listChildren.add(node);
    }
    
    //----------------------------------------------------------------------------------------    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() 
    {
        int iChildrenCount = ( _listChildren != null ? _listChildren.size() : 0 ) ;
        String sChilNodes = ( iChildrenCount > 0 ? iChildrenCount + " child node(s) " : "no child nodes" );
        return "MenuNode [type:"+_iType +"|id:" +_sId +"|cl:"+_sCl + "|style:" +_sStyle
        +"|img:"+_sImg +"|txt:"+_sTxt + "|onclick:"+_sOnClick + "|"+sChilNodes + "]";
    }
    
}
