package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.widget.CheckBox;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;

/**
 * HTML generation for CheckBox widget
 * 
 * @author Laurent GUERIN
 *  
 */
public class CheckBoxHTML extends GenericHTML implements IWidget
{

    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        CheckBox tag = (CheckBox) genericTag;

        String sId = tag.getId();
        String sTxt = tag.getTxt();
        String sTxtPos = tag.getTxtpos();

        //--- If there's a text for this checkbox
        if (notVoid(sTxt))
        {
            String sLabelForTag = getHtmlLabelFor(sId, sTxt);
            //String sCheckBoxTag = getCheckBoxTag(tag, POS_NONE);

            out.print("<span style=\"" + getStyleFragPosition(tag) + "white-space:nowrap;display:block;\" >");
            if (sTxtPos.toLowerCase().startsWith("l"))
            {
                //--- Text position is "left"
                out.print(sLabelForTag);
                //out.print(sCheckBoxTag);
                printCheckBoxTag(out, tag, POS_NONE );
            }
            else
            {
                //--- Text position is "right" ( by default )
                //out.print(sCheckBoxTag);
                printCheckBoxTag(out, tag, POS_NONE );
                out.print(sLabelForTag);
            }
            out.print("</span>");
        }
        else
        {
            //--- No text => print only the checkbox ( no label )
            //out.print(getCheckBoxTag(tag, POS_XY));
            printCheckBoxTag(out, tag, POS_XY );
        }
    }

    private final void printCheckBoxTag(JspWriter out, CheckBox tag, int iPositionType) throws IOException
    {
        out.print( 
                "<input type=\"checkbox\" " 
        + getIdAttr(tag) + " " 
        + getClassAttr(tag) + " " 
        + getStyleAttr(tag, iPositionType) + " " 
        + getTitleAttr(tag) + " " 
        + getTabindexAttr(tag) + " "

        + getHtmlChecked(tag) 
        + getHtmlName(tag) 
        + getHtmlValue(tag)  
        + getHtmlValueOff(tag)  
        + getHtmlDisabled(tag) 
        );

        //--- Events set for this tag
        tag.printEvents(out);
        
        out.print( " />" );
    }
    
//    private final String getCheckBoxTag(CheckBox tag, int iPositionType)
//    {
//        return "<input type=\"checkbox\" " 
//        + getIdAttr(tag) + " " 
//        + getClassAttr(tag) + " " 
//        + getStyleAttr(tag, iPositionType) + " " 
//        + getTitleAttr(tag) + " " 
//        + getTabindexAttr(tag) + " "
//
//        + getHtmlChecked(tag) 
//        + getHtmlName(tag) 
//        + getHtmlValue(tag)  
//        + getHtmlValueOff(tag)  
//        + getHtmlDisabled(tag) 
//        + " />";
//    }

    private final String getHtmlChecked(CheckBox tag)
    {
        //if (tag.isChecked())
        if (tag.getCheckedAsBoolean() ) // v 0.9.9
        {
            return " checked=\"checked\" ";
        }
        else
        {
            return "";
        }
    }

    private final String getHtmlName(CheckBox tag)
    {
        if (tag.getName() != null)
        {
            return " name=\"" + tag.getName() + "\" " ;
        }
        else
        {
            return "";
        }
    }

    private final String getHtmlValue(CheckBox tag)
    {
        String sValue = tag.getValue();
        if (sValue != null)
        {
            return " value=\"" + sValue + "\" ";
        }
        else
        {
            return "";
        }
    }

    private final String getHtmlValueOff(CheckBox tag)
    {
        String sValueOff = tag.getValueoff();
        if (sValueOff != null)
        {
            return " valueoff=\"" + sValueOff + "\" ";
        }
        else
        {
            return "";
        }
    }

    private final String getHtmlDisabled(CheckBox tag)
    {
        if (tag.getDisabledAsBoolean())
        {
            return " disabled=\"disabled\" ";
        }
        else
        {
            return "";
        }
    }

    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        CheckBox tag = (CheckBox) genericTag;
        //--- Register the current field via Javascript ...
        registerField(tag);
    }
    //----------------------------------------------------------------
    private void registerField(CheckBox tag) throws IOException
    {
        if (needRegister(tag))
        {
            String sFirstArgs = firstArgsToRegister(tag);
            //--- Value ON
            String sValueOn = tag.getValue() ;
            if ( sValueOn == null )
            {
                sValueOn = "null";
            }
            else
            {
                sValueOn = "\"" + sValueOn + "\"";                    
            }
            //--- Value OFF 
            String sValueOff = tag.getValueoff() ;
            if ( sValueOff == null )
            {
                sValueOff = "null";
            }
            else
            {
                sValueOff = "\"" + sValueOff + "\"";                    
            }
            //--- Build the Javascript line ...
            String sJavascript = "RegisterCheckBox(" + sFirstArgs + ", " + sValueOn + ", " + sValueOff + ")";                    
            tag.addJavascriptLine(sJavascript);
        }        
    }
    //----------------------------------------------------------------

}