package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.widget.Event;
import org.objectweb.telosys.uil.taglib.widget.Field;
import org.objectweb.telosys.uil.taglib.widget.FieldCombo;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;

/**
 * HTML generation for FieldCombo widget
 * 
 * @author Aurelien COUERON
 *  
 */
public class FieldComboHTML extends GenericHTML implements IWidget
{
    //----------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        FieldCombo tag = (FieldCombo) genericTag;

        String sIdTag = tag.getId();
        // span id
        out.print("<span id=\"");
        out.print(sIdTag + "_main\" ");
        // span style
        out.print("style=\"white-space:nowrap; display:block; " + getStyleFragPosition(tag) + "\" >");
        // Input
        generateInputTag(out, tag);
        // Img
        generateImgTag(out, tag);
        out.println("</span>");
        // Select
        generateSelectTag(out, tag);
    }

    private void generateInputTag(JspWriter out, FieldCombo tag) throws IOException
    {
        int iType = tag.getInternaType();
        if (iType == Field.TYPE_INT || iType == Field.TYPE_NUM)
        {
            tag.defineStyle("text-align", "right");
        }
        //--- Open the tag <input type="text" ... />
        out.print("<input type=\"text\" ");
        //--- Generic widget attributes
        out.print(getIdAttr(tag) + " ");
        out.print(getClassAttr(tag) + " ");
        // Style
        out.print(" style=\"" + getStyleFragWidthHeight(tag) + "\" ");
        out.print(getTitleAttr(tag) + " ");
        out.print(getTabindexAttr(tag) + " ");
        out.print(getReadonlyAttr(tag) + " ");
        out.print(getDisabledAttr(tag) + " ");
        
        //--- Specific widget attributes
        if (tag.getMaxlength() != null)
        {
            out.print(" maxlength=\"" + tag.getMaxlength() + "\"");
        }
        //--- "size" attribute ( if not set use "maxlength" )
        String sSize = tag.getSize() != null ? tag.getSize() : tag.getMaxlength();
        if (sSize != null)
        {
            out.print(" size=\"" + sSize + "\"");
        }
        //--- "value" attribute
        String sValue = tag.getValue(); // use postback value if necessary
        if (sValue != null)
        {
            out.print(" value=\"" + sValue + "\"");
        }
        //--- "name" attribute
        if (tag.getName() != null)
        {
            out.print(" name=\"" + tag.getName() + "\"");
        }
//        //--- "disabled" attribute
//        if (tag.getDisabledAsBoolean())
//        {
//            out.print(" disabled=\"disabled\"");
//        }
//        //--- "readonly" attribute
//        if (tag.isReadonly())
//        {
//            out.print(" readonly=\"readonly\"");
//        }
        //--- Events set by user
        tag.printEvents(out);
        //--- Standard field events
        if (tag.isEventDefined(Event.ON_BLUR) != true)
        {
            //--- No specific event "ON BLUR => generate standard "ON BLUR"
            String sScript = tag.getOnBlurEvent();
            tag.printEvent(out, Event.ON_BLUR, sScript);
        }
        if (tag.isEventDefined(Event.ON_KEY_PRESS) != true)
        {
            //--- No specific event => generate standard event
            String sScript = tag.getOnKeyPressEvent();
            tag.printEvent(out, Event.ON_KEY_PRESS, sScript);
        }
        //--- Close the tag
        out.print("/>");
    }

    // Generate picto
    private void generateImgTag(JspWriter out, FieldCombo tag) throws IOException
    {
        //--- Get tag "img" attributes
        String sFieldId = tag.getId();
        String sImgId = null;
        if (notVoid(sFieldId))
        {
            sImgId = sFieldId.trim() + "_picto";
        }
        String sImgSrc = tag.getFullPictoImageSrc();
        String sImgClass = tag.getPictocl();

        //--- Generate tag <img ... />
        out.print("<img ");
        if (notVoid(sImgId))
        {
            out.print(" id=\"" + sImgId + "\"");
        }
        if (notVoid(sImgClass))
        {
            out.print(" class=\"" + sImgClass + "\"");
        }
        //--- Style
        String sStyle = "vertical-align:bottom; margin-bottom:2px; ";
        if (tag.getDisabledAsBoolean())
        {
            sStyle = sStyle + "visibility:hidden;";
        }
        out.print(" style=\"" + sStyle + "\" ");
        //--- Image source
        out.print(" src=\"" + sImgSrc + "\"");
        //--- "onclick" event
        out.print(" onclick=\"fwkComboDisplayList('" + sFieldId + "')\"");
        out.println(" />");
    }

    // Generate list
    private void generateSelectTag(JspWriter out, FieldCombo tag) throws IOException
    {
        //--- Get tag "img" attributes
        String sFieldId = tag.getId();
        String sSelectId = null;
        if (notVoid(sFieldId))
        {
            sSelectId = sFieldId.trim() + "_select";
        }
        String sSelectSize = tag.getListsize();
        String sSelectClass = tag.getListcl();

        //--- Generate tag <select ... />
        out.print("<select ");
        if (notVoid(sSelectId))
        {
            out.print(" id=\"" + sSelectId + "\"");
        }
        if (notVoid(sSelectClass))
        {
            out.print(" class=\"" + sSelectClass + "\"");
        }
        if (notVoid(sSelectSize))
        {
            out.print(" size=\"" + sSelectSize + "\"");
        }
        //--- Style
        String sStyle = "display:none; position:absolute; z-index:8000;";
//        if (tag.isDisabled())
//        {
//            sStyle = sStyle + "visibility:hidden;";
//        }
        out.print(" style=\"" + sStyle + "\" ");
        //--- Events
        out.print(" onblur=\"fwkComboSelectBlur('" + sFieldId + "')\" ");
        //--- "onclick" event
        out.print(" onchange=\"fwkComboSelectChange(this,'" + sFieldId + "')\" ");
        out.println(">");
    }

    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        Field tag = (Field) genericTag;
        //--- Register the current field via Javascript ...
        out.println("</select>");
        registerField(tag);
    }

    //----------------------------------------------------------------
    //    private void registerField(Field tag) throws IOException
    //    {
    //        if (needRegister(tag))
    //        {
    //            String sFirstArgs = firstArgsToRegister(tag);
    //            
    //            String sJavascript = null ;
    //            if ( tag.getInternaType() == Field.TYPE_DATE )
    //            {
    //                String sFormat = tag.getFormat() ;
    //                if ( sFormat == null )
    //                {
    //                    sFormat = "DD/MM/YY" ; // TODO : external tag parameter
    //                }
    //                sFormat = "\"" + sFormat + "\""; // no trim
    //                sJavascript = "RegisterFieldDate(" + sFirstArgs + ", " + sFormat + ")";
    //            }
    //            else
    //            {
    //                sJavascript = "RegisterField(" + sFirstArgs + ")";
    //            }
    //            tag.addJavascriptLine(sJavascript);
    //        }
    //    }
    //----------------------------------------------------------------
}