package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;
import java.util.Iterator;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;
import org.objectweb.telosys.uil.taglib.widget.MenuBarTAG;
import org.objectweb.telosys.uil.taglib.widget.data.MenuItem;
import org.objectweb.telosys.uil.taglib.widget.data.MenuItems;

/**
 * HTML generation for Menubar widget
 * 
 * @author Laurent GUERIN
 *  
 */
public class MenuBarHTML extends GenericHTML implements IWidget
{
    //----------------------------------------------------------------
    private String getSubMenuId( MenuItem item ) 
    {
        if ( item.getMenuBoxId() != null ) // menubox attribute 
        {
            String s = item.getMenuBoxId().trim();
            if ( s.length() > 0 )
            {
                return s ;
            }
        }
        if ( item.getMenuBox() != null ) // menubox object 
        {
            String s = item.getMenuBox().getId();
            if ( s != null )
            {
                s = s.trim();
                if ( s.length() > 0 )
                {
                    return s ;
                }
            }
        }
        return null ;
    }
    //----------------------------------------------------------------
    private String getOnMouseOver( MenuBarTAG tag, MenuItem item ) 
    {
        String s = getSubMenuId( item ) ;
        if ( s != null )
        {
            // call function with submenu ID parameter
            if ( tag.isVerticalLayout() )
            {
                return "fwkSelectMenuBarItem(this,'" + s + "','R')" ; // on right side 
            }
            else
            {
                return "fwkSelectMenuBarItem(this,'" + s + "')" ;
            }
        }
        // no submenu parameter
        return "fwkSelectMenuBarItem(this)" ;
    }
    //----------------------------------------------------------------
    private void generateItem(JspWriter out, MenuBarTAG tag, MenuItem item) throws IOException
    {
        if ( tag.isVerticalLayout() )
        {
            out.print("<tr class=\"telosys_menubar\" >" ); 
        }
        
        String sTxt = item.getTxt();
        if ( sTxt == null )
        {
            sTxt = "&nbsp;";
        }

        out.print( "<td" );
        out.print( " id=\"" + item.getId()+ "\"" ); // added in v 0.9.9
        out.print( " class=\"off\"" );
        out.print( " onmouseover=\"" + getOnMouseOver( tag, item ) + "\"" );  
        out.print( " onclick=\"" + MenuCommonsHTML.getOnclick(tag, item) + "\"" );
        out.print(" >" + sTxt + "</td>" );
        
        if ( tag.isVerticalLayout() )
        {
            out.print("</tr>");
        }
        
    }
    //----------------------------------------------------------------
    private void generateMenuItemsTags(JspWriter out, MenuBarTAG tag, MenuItems menuItems) throws IOException
    {
        MenuItem item = null ;
        Iterator it = menuItems.iterator();
        while ( it.hasNext() )
        {
            item = (MenuItem) it.next();
            if ( item.isSeparator() )
            {
                // No separator in menu bar
                //generateSeparator(out,item);
            }
            else
            {
                generateItem(out,tag,item);
            }
        }
    }
    //----------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
    }
    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        MenuBarTAG tag = (MenuBarTAG) genericTag;
        String sId = tag.getId() ;
        
        //--- (1) Open MenuBar
        out.print( "<div id=\"" + sId + "\" " ); // Id is generated if necessary 
        out.print(getClassAttr(tag, "telosys_menubar") + " ");        
        out.print(getStyleAttr(tag) + " "); 
        out.println(">");        
        
        if ( tag.isVerticalLayout() )
        {
            //--- VERTICAL
            out.println( "<table class=\"telosys_menubar\" style=\"width:100%;\" border=\"0\"  >");
        }
        else
        {
            //--- HORIZONTAL
            out.println( "<table class=\"telosys_menubar\" border=\"0\" >");
            out.println( "<tr class=\"telosys_menubar\" >"); 
        }
        
        //--- (2) Generate MenuBox Items
        MenuItems menuItems = tag.getMenuItems();
        if ( menuItems != null )
        {
            generateMenuItemsTags(out,tag, menuItems);
        }

        //--- (3) Close MenuBar
        if ( ! tag.isVerticalLayout() )
        {
            //--- HORIZONTAL
            out.println( "</tr>");
        }
        out.println( "</table>");
        out.println( "</div>");

        out.println("<script type=\"text/javascript\" language=\"JavaScript\">");
        out.println("fwkInitMenuBar(\"" + sId + "\");" );
        out.println("document.onclick=function(){fwkResetMenuBar();}");
        out.println("</script>");

    }
}