package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;
import java.util.Iterator;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;
import org.objectweb.telosys.uil.taglib.widget.MenuBoxTAG;
import org.objectweb.telosys.uil.taglib.widget.data.MenuBox;
import org.objectweb.telosys.uil.taglib.widget.data.MenuItem;
import org.objectweb.telosys.uil.taglib.widget.data.MenuItems;

/**
 * HTML generation for "menubox" widget
 * 
 * @author Laurent GUERIN
 *  
 */
public class MenuBoxHTML extends GenericHTML implements IWidget
{
    //----------------------------------------------------------------
    private String getImageTag( GenericTag tag, MenuItem item) 
    {
        String s = item.getImg();
        if ( s != null )
        {
            s = s.trim();
            if ( s.length() > 0 )
            {
                return "<img src=\"" + tag.getFullImageSrc(s) + "\" />" ;
            }
        }
        return "&nbsp;"; // No image tag 
    }
    //----------------------------------------------------------------
    private void generateItem(JspWriter out, GenericTag tag, MenuItem item) throws IOException
    {
        //--- New row
        out.print( "<tr" );
        out.print( " id=\"" + item.getId()+ "\"" );
        out.print( " class=\"off\"" );
        out.print( " onclick=\"" + MenuCommonsHTML.getOnclick(tag, item)  + "\"" );
        out.print(" >"); 
        
        //--- Col 1 : image (if any)
        out.print( "<td class=\"img\">" + getImageTag( tag, item)  + "</td>" );

        //--- Col 2 : text
        String sTxt = item.getTxt();
        if ( sTxt == null )
        {
            sTxt = "&nbsp;";
        }
        out.print( "<td class=\"txt\">" + sTxt + "</td>" );
        
        //--- Col 3 : arrow (if any)
        String sArrowClass = ( item.isSubMenu() ? "arrow" : "noarrow" ) ;
        out.print( "<td class=\"" + sArrowClass +"\">&nbsp;</td>" );
        
        //--- End of row
        out.println("</tr>");
    }
    //----------------------------------------------------------------
    private void generateSeparator(JspWriter out, MenuItem item) throws IOException
    {
        out.print("<tr class=\"off\" style=\"height:8px;\">");
        out.print("<td colspan=\"3\" style=\"height:4px\" ><hr></td>");
        out.println("</tr>");
    }
    //----------------------------------------------------------------
    private void generateMenuItemsTags(JspWriter out, GenericTag tag, MenuItems menuItems) throws IOException
    {
        MenuItem item = null ;
        Iterator it = menuItems.iterator();
        while ( it.hasNext() )
        {
            item = (MenuItem) it.next();
            if ( item.isSeparator() )
            {
                generateSeparator(out,item);
            }
            else
            {
                generateItem(out,tag,item);
            }
        }
    }
    //----------------------------------------------------------------
    private void generateMenuItemsJavascript(JspWriter out, MenuItems menuItems) throws IOException
    {
        out.println("<script type=\"text/javascript\" language=\"JavaScript\">");
        MenuItem item = null ;
        Iterator it = menuItems.iterator();
        while ( it.hasNext() )
        {
            item = (MenuItem) it.next();
            if ( ! item.isSeparator() )
            {
                String s = item.getId();
                if ( s != null )
                {
                    out.print("fwkInitMenuItem(\"" + s.trim() + "\"" );
                    if ( item.isSubMenu() )
                    {
                        //--- Add submenu id ( menu box id ) as function param
                        String sSubMenuId = item.getMenuBoxId();
                        if ( sSubMenuId == null )
                        {
                            MenuBox menuBox = item.getMenuBox();
                            if ( menuBox != null )
                            {
                                sSubMenuId = menuBox.getId();
                            }
                        }
                        if ( sSubMenuId != null )
                        {
                            out.print(",\"" + sSubMenuId.trim() + "\"" );                        
                        }                        
                    }
                    out.println ( ");" );
                }
            }
        }
        out.println("</script>");
    }
    
    //----------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
    }
    
    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        MenuBoxTAG tag = (MenuBoxTAG) genericTag;
        
        //--- (1) Open MenuBox
        out.print( "<div id=\"" + tag.getId() + "\" " ); // Id is generated if necessary 
        out.print(getClassAttr(tag, "telosys_menubox") + " ");        
        out.print(getStyleAttr(tag) + " "); 
        out.println(">");        
                
        out.print("<iframe frameborder=\"0\" " );
        out.print("style=\"width:20px; height:10px; position:absolute; top:0px; left:0px;\" >");
        out.println("</iframe>");
        
        out.println("<table class=\"telosys_menubox\" cellspacing=\"0\" >");
        
        //--- (2) Generate MenuBox Items
        MenuItems menuItems = tag.getMenuItems();
        if ( menuItems != null )
        {
            generateMenuItemsTags(out,tag, menuItems);
        }
        
        //--- (3) Close MenuBox
        out.println( "</table></div>");
        
        //--- (4) Init MenuBox items
        if ( menuItems != null )
        {
            generateMenuItemsJavascript(out, menuItems);
        }
        
        
    }
}