package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;
import org.objectweb.telosys.uil.taglib.widget.RichTextEditor;
import org.objectweb.telosys.util.StrUtil;

/**
 * HTML generation for TextEditor widget
 * 
 * @author Laurent GUERIN
 *  
 */

// 23/03/2008 : Remove instance variable "_sImgDir" because of potential URL bug 
// e.g. when Tomcat is used with Apache/mod_jk the URL is not always the same !

public class RichTextEditorHTML extends GenericHTML implements IWidget
{
    //private String _sImgDir = null; // Dangerous : do not keep in instance => removed
    
    private ColorPickerHTML _colorPicker = new ColorPickerHTML(); // Color Picker : always the same => Can be keep as instance variable
    

    /**
     *  
     */
    public RichTextEditorHTML()
    {
        super();
    }

    //----------------------------------------------------------------
//    /**
//     * Initialize the Tag Generator instance <br>
//     * NB : The HTML Generator is considered as thread safe => mono-instance ( static )
//     * 
//     * @param tag
//     * @throws IOException
//     */
//    private void init(RichTextEditor tag) throws IOException
//    {
//        //--- Init the image ROOT URL
//        if (_sImgDir == null) // Not yet initialized 
//        {
//            _sImgDir = tag.getFullTelosysImageSrc("");
//        }
//    }

    //----------------------------------------------------------------
    private int getWidth(RichTextEditor tag) throws IOException
    {
        int iWidth = -1;
        String sWidth = tag.getWidth();
        if (sWidth != null)
        {
            iWidth = StrUtil.getInt(sWidth, -1);
        }
        if (iWidth == -1) // Not set => default
        {
            iWidth = RichTextEditor.WITDH_DEFAULT;
        }
        else if (iWidth < RichTextEditor.WITDH_MIN) // Force to MIN WIDTH
        {
            iWidth = RichTextEditor.WITDH_MIN;
        }
        return iWidth;
    }

    //----------------------------------------------------------------
    private int getHeight(RichTextEditor tag) throws IOException
    {
        int iHeight = -1;
        String sHeight = tag.getHeight();
        if (sHeight != null)
        {
            iHeight = StrUtil.getInt(sHeight, -1);
        }
        if (iHeight == -1) // Not set => default
        {
            iHeight = RichTextEditor.HEIGHT_DEFAULT;
        }
        else if (iHeight < RichTextEditor.HEIGHT_MIN) // Force to MIN
        {
            iHeight = RichTextEditor.HEIGHT_MIN;
        }
        return iHeight;
    }

    //----------------------------------------------------------------
    private String img(RichTextEditor tag, String s)
    {
        // NB : rebuild the URL each time ( do not keep in instance )
        String sImgDir = tag.getFullTelosysImageSrc("");
        if (sImgDir != null)
        {
            return sImgDir + s;
        }
        else
        {
            return s;
        }
    }

    //----------------------------------------------------------------
    private void genFont(JspWriter out, String sId) throws IOException
    {
        String fonts[] = { "Arial", "Comic Sans MS", "Courier New", "Impact", "Tahoma", "Times New Roman", "Trebuchet MS", "Verdana" };

        out.print("<td style=\"width: 90px;\" >");
        out.print("<select style=\"width:100px; font-size: 9pt; font-family: Tahoma; height:20px; background-color: #E3E3E3;\"");
        out.println(" onchange=\"fwkRTESelectFontName(this, '" + sId + "');\" >");
        out.println("<option value=\"\" style=\"background-color: #E3E3E3; font-style: italic ; \" >Font</option>");

        String s = null;
        for ( int i = 0 ; i < fonts.length ; i++ )
        {
            s = fonts[i];
            out.println("<option value=\"" + s + "\" style=\"background-color: White; \" >" + s + "</option>");
        }
        out.println("</select></td>");
    }

    //----------------------------------------------------------------
    private void genSize(JspWriter out, String sId) throws IOException
    {
        out.print("<td style=\"width: 40px;\" >");
        out.print("<select style=\"width:50px; font-size: 9pt; font-family: Tahoma; height:20px; background-color: #E3E3E3;\"");
        out.println(" onchange=\"fwkRTESelectFontSize(this, '" + sId + "');\" >");
        out.println("<option value=\"\" style=\"background-color: #E3E3E3; font-style: italic ; \" >Size</option>");

        for ( int i = 1 ; i <= 7 ; i++ )
        {
            out.println("<option value=\"" + i + "\" style=\"background-color: White; \" >&nbsp;&nbsp;" + i + "</option>");
        }
        out.println("</select></td>");
    }

    //----------------------------------------------------------------
    private void genButton(JspWriter out, RichTextEditor tag, String sId, String sOnClick, String sImg) throws IOException
    {
        out.print("<td style=\"width: 22px;\" >");
        out.print("<img src=\"" + img(tag, sImg + ".gif") + "\" border=\"0\" alt=\"\" ");
        out.print(" style=\"width:20px; height:20px; border: 0px solid transparent; margin: 1px; padding: 0px; background: transparent;\"");
        out.print(" onclick=\"" + sOnClick + "\" ");
        out.print(" onmouseover=\"fwkRTEButtonMouseOver(this,'" + img(tag, sImg + "_on.gif") + "');\" ");
        out.print(" onmouseout=\"fwkRTEButtonMouseOut(this,'" + img(tag, sImg + ".gif") + "');\" ");
        out.println("></td>");
    }

    //----------------------------------------------------------------
    private int genButtonCommand(JspWriter out, RichTextEditor tag, boolean bFlag, String sId, String sCommand, String sImg) throws IOException
    {
        if ( ! bFlag ) return 0 ;
        String sOnClick = "fwkRTEFormatText('" + sCommand + "', '" + sId + "');";
        genButton(out, tag, sId, sOnClick, sImg);
        return 1;
    }

    //----------------------------------------------------------------
    private int genButtonPicker(JspWriter out, RichTextEditor tag, boolean bFlag, String sId, String sCommand, String sImg, String sColorPickerVar) throws IOException
    {
        if ( ! bFlag ) return 0 ;
        String sOnClick = sColorPickerVar + ".open('" + sCommand + "', this);" ;
        genButton(out, tag, sId, sOnClick, sImg);
        return 1;
    }

    //----------------------------------------------------------------
    private void genSeparator(JspWriter out, RichTextEditor tag) throws IOException
    {
        out.print("<td style=\"width: 8px;\" align=\"center\" >");
        out.print("<img src=\"" + img(tag, "separator.gif") + "\" border=\"0\" width=\"2\" height=\"18\" hspace=\"2\" >");
        out.println("</td>");
    }

    //----------------------------------------------------------------
    private void genToolBar(JspWriter out, RichTextEditor tag, String sId, String sColorPickerVar, int iWidth) throws IOException
    {
        boolean[] elements = tag.getToolbarElements();
        
        out.println("<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"  " + " style=\"" + "border: 1px solid #666666; height: 26px; " + "background-image: url(" + img(tag, "background_silver.jpg") + "); " + "width:" + iWidth + "px; " + "\" " + ">");
        out.println("<tr>");
        out.println("<td style=\"width: 6px;\"><img src=\"" + img(tag, "separator2.gif") + "\" alt=\"\" hspace=\"3\"></td>");

        if ( elements[RichTextEditor.BAR_FONT] )
        {
            genFont(out, sId);
            out.println("<td style=\"width: 4px;\"></td>");
        }

        if ( elements[RichTextEditor.BAR_SIZE] )
        {
	        genSize(out, sId);
	        out.println("<td style=\"width: 4px;\"></td>");
        }

        int i = 0 ;
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_BOLD],  sId, "Bold", "bold");
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_ITALIC], sId, "Italic", "italics");
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_UNDERLINE], sId, "Underline", "underline");
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_STRIKE], sId, "StrikeThrough", "strikethrough");
        if ( i > 0 ) genSeparator(out, tag);

        i = 0 ;
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_JLEFT],   sId, "Justifyleft", "justify_left");
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_JCENTER], sId, "Justifycenter", "justify_center");
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_JRIGHT],  sId, "Justifyright", "justify_right");
        if ( i > 0 ) genSeparator(out, tag);

        i = 0 ;
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_ULIST], sId, "InsertUnorderedList", "list_unordered");
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_OLIST], sId, "InsertOrderedList", "list_ordered");
        if ( i > 0 ) genSeparator(out, tag);

        i = 0 ;
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_OUTDENT], sId, "Outdent", "indent_left");
        i = i + genButtonCommand(out, tag, elements[RichTextEditor.BAR_INDENT],  sId, "Indent",  "indent_right");
        if ( i > 0 ) genSeparator(out, tag);

        i = 0 ;
        i = i + genButtonPicker(out, tag, elements[RichTextEditor.BAR_FCOLOR], sId, "ForeColor", "forecolor", sColorPickerVar);
        i = i + genButtonPicker(out, tag, elements[RichTextEditor.BAR_BCOLOR], sId, "BackColor", "backcolor", sColorPickerVar);
        if ( i > 0 ) genSeparator(out, tag);

        //i = 0 ;
        genButtonCommand(out, tag, elements[RichTextEditor.BAR_SUB], sId, "Subscript",   "subscript");
        genButtonCommand(out, tag, elements[RichTextEditor.BAR_SUP], sId, "Superscript", "superscript");

        out.println("<td>&nbsp;</td>");

        out.println("</tr>");
        out.println("</table>");
    }

    //----------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        RichTextEditor tag = (RichTextEditor) genericTag;

        //init(tag) ; // Init the generator (if first use of the instance)
        
        //--- Editor ID  ( IFRAME ID )
        String sEditorId = tag.getId();
        
        //--- Var name for the ColorPicker object  
        String sColorPickerVar   = "fwk" + sEditorId + "CPV" ;

        int iWidth = getWidth(tag);
        int iHeight = getHeight(tag);

        //--- Beginning ( DIV - just for absolute positioning )
        out.print("<div ");
        //out.print(getIdAttr(tag) + " ");
        //out.print(getClassAttr(tag) + " ");
        if (tag.isPositionAbsolute())
        {
            out.print(" style=\" " + getStyleFragPosition(tag) + "\" ");
        }
        else
        {
            out.print(" style=\" position:relative; \" ");
        }
        out.println(" >");

        //--- Tool bar ( <table> )
        genToolBar(out, tag, sEditorId, sColorPickerVar, iWidth);

        //--- Editable Text ( <iframe> )
        out.print("<iframe"); 
        out.print(" id=\"" + sEditorId + "\"" ); 
        out.print(" frameborder=\"0\"" ); 
        out.print(" style=\"width:" + (iWidth - 2) + "px; height:" + (iHeight - 26) + "px; border: 1px solid #666666 ; border-top: none ;\" " );
        out.print(" " + getTitleAttr(tag));
        out.print(" >"); 
        out.println( "</iframe>");

        //--- Init ( JavaScript )
        out.println("<script language=\"JavaScript\" type=\"text/javascript\" >");
        out.println("fwkRTEInit( \"" + sEditorId + "\")");
        out.println("</script>");

        //--- End
        out.println("</div>");
        
        _colorPicker.print(out, sEditorId, sColorPickerVar);
    }

    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        RichTextEditor tag = (RichTextEditor) genericTag;
        //--- Register the current field via Javascript ...
        registerFieldLongText(tag);
    }
    //----------------------------------------------------------------
    private void registerFieldLongText(RichTextEditor tag) 
    {
        if (needRegister(tag))
        {
            String sFirstArgs = firstArgsToRegister(tag);
            String sJavascript = "RegisterRichTextEditor(" + sFirstArgs + ")";
            tag.addJavascriptLine(sJavascript);
        }
    }
}