package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.widget.Event;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;
import org.objectweb.telosys.uil.taglib.widget.SpinEdit;
import org.objectweb.telosys.util.StrUtil;

/**
 * HTML generation for SpinEdit widget
 * 
 * @author Aurelien COUERON
 *  
 */
public class SpinEditHTML extends GenericHTML implements IWidget
{
    //----------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        SpinEdit tag = (SpinEdit) genericTag;
        
        //--- WIDTH ( Default = 50px , Minimum = 30px )
        String sDivWidth   = "width:50px;" ; // Default WIDTH
        String sTableWidth = "width:100%;" ;
        String sFieldWidth = "width:100%;" ;
        String sWidth = tag.getWidth() ;
        if ( sWidth != null )
        {
            int iWidth = tag.getWidthInt();
            if ( iWidth > 30 )
            {
                sDivWidth   = "width:" + sWidth + "px;" ;
//                sTableWidth = "width:100%;" ;
//                sFieldWidth = "width:100%;" ;
            }
            else
            {
                sDivWidth   = "width:30px;" ; // Minimum WIDTH
            }
        }        
        //--- HEIGHT ( No Default , Minimum = 10px )
        String sDivHeight   = "" ;
        String sTableHeight = "" ;
        String sFieldHeight = "" ;
        String sHeight = tag.getHeight() ;
        if ( sHeight != null )
        {
            int iHeight = tag.getHeightInt();
            if ( iHeight > 10 )
            {
                sDivHeight   = "height:" + sHeight + "px;" ;
            }
            else
            {
                sDivHeight   = "height:10px;" ; // Minimum HEIGHT
            }
            sTableHeight = "height:100%;" ;
            sFieldHeight = "height:100%;" ;
        }
        
        String sDivStyle = sDivWidth + sDivHeight + getStyleFragPosition(tag) ;
        String sFieldStyle = sFieldWidth + sFieldHeight ;
        
        // <div> 
        out.println("<div class=\"telosys_spinedit_box1\" style=\""+ sDivStyle + "\" > " ); 
        // <table> <tbody>
        out.print("<table class=\"telosys_spinedit_box2\"" );
        out.print(" style=\"position:relative;" + sTableWidth + sTableHeight + "\"" );
        out.print(" cellpadding=\"0\" cellspacing=\"0\" ");
        out.print(" >");
        out.println("<tbody>");
        // <tr>
        out.println("<tr>");
        
        // <td> input
        out.print("<td rowspan=\"2\">");
        //<input/>
        generateInputTag(out, tag, sFieldStyle );
        // </td>
        out.println("</td>");
        
        // <td> imgup
        generateArrowCell(out, tag, "+", "telosys_spinedit_up.gif");
        
//        out.print("<td class=\"telosys_spinedit_up\" ");
//        out.print("onclick=\"javascript:spin_move('"+tag.getId()+"',"+tag.getStep()+","+tag.getDigit()+")\">");
//        out.println("</td>");
        // </tr>
        out.println("</tr>");
        // <tr>
        out.println("<tr>");
        // <td> imgdown
        generateArrowCell(out, tag, "-", "telosys_spinedit_down.gif");
//        out.print("<td class=\"telosys_spinedit_down\" ");
//        out.print("onclick=\"javascript:spin_move('"+tag.getId()+"',-"+tag.getStep()+","+tag.getDigit()+")\">");
//        out.println("</td>");
        // </tr>
        out.println("</tr>");
        // </tbody></table>
        out.print("</tbody>");
        out.print("</table>");
        out.println("</div>");
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Generates the tag <input ... />
     * @param out
     * @param tag
     * @param sStyleContent
     * @throws IOException
     */
    private void generateInputTag(JspWriter out, SpinEdit tag, String sStyleContent ) throws IOException
    {
        String sFieldClass = "class=\"telosys_spinedit_field\"";
        String sClass = tag.getCl();
        if (sClass != null)
        {
            sFieldClass = "class=\"" + sClass + "\"";
        }

        String sFieldStyle = "";
        if ( sStyleContent.length() > 0 )
        {
            sFieldStyle = "style=\"" + sStyleContent + "\"";
        }
        
        //--- Open the tag <input type="text" ... />
        out.print("<input type=\"text\" ");
        //--- Generic widget attributes
        out.print(getIdAttr(tag) + " ");
        out.print(sFieldClass + " ");
        out.print(sFieldStyle + " ");
        out.print(getTitleAttr(tag) + " ");
        out.print(getTabindexAttr(tag) + " ");

        out.print(getReadonlyAttr(tag) + " ");
        out.print(getDisabledAttr(tag) + " ");

        //--- Specific widget attributes
        if (tag.getMaxlength() != null)
        {
            out.print(" maxlength=\"" + tag.getMaxlength() + "\"");
        }
        //--- "size" attribute ( if not set use "maxlength" )
        String sSize = tag.getSize() != null ? tag.getSize() : tag.getMaxlength();
        if (sSize != null)
        {
            out.print(" size=\"" + sSize + "\"");
        }
        String sValue = tag.getValue(); // use postback value if necessary
        if (sValue != null)
        {
            out.print(" value=\"" + sValue + "\"");
        }
        if (tag.getName() != null)
        {
            out.print(" name=\"" + tag.getName() + "\"");
        }
        
        //--- Events set by user
        tag.printEvents(out);
        //--- Standard field events
        if (tag.isEventDefined(Event.ON_BLUR) != true)
        {
            //--- No specific event "ON BLUR => generate standard "ON BLUR"
            tag.printEvent(out, Event.ON_BLUR, tag.getOnBlurEvent());
        }
        if (tag.isEventDefined(Event.ON_KEY_PRESS) != true)
        {
            //--- No specific event => generate standard event
            //tag.printEvent(out, Event.ON_KEY_PRESS, "return fwkFilter(event,this,'NUM')");
            tag.printEvent(out, Event.ON_KEY_PRESS, tag.getOnKeyPressEvent() );
        }
        
        //--- Close the tag
        out.print("/>");

    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Generates the arrow cell <td ..> <img .../> </td>
     * @param out
     * @param tag
     * @param sPlusMinus
     * @param sImage
     * @throws IOException
     */
    private void generateArrowCell(JspWriter out, SpinEdit tag, String sPlusMinus, String sImage ) throws IOException
    {
        String sMinMaxParameters = "" ;
        //--- MIN value
        String sMin = tag.getMin();
        double dMin = 0.0;
        if ( sMin != null )
        {
            dMin = StrUtil.getDouble(sMin);
        }
        //--- MAX value
        String sMax = tag.getMax();
        double dMax = 0.0;
        if ( sMax != null )
        {
            dMax = StrUtil.getDouble(sMax);
        }
        if ( sMin != null || sMax != null)
        {
            sMinMaxParameters += "," ; 
            sMinMaxParameters += ( sMin != null ? ""+dMin : "null" ) ; 
            sMinMaxParameters += "," ; 
            sMinMaxParameters += ( sMax != null ? ""+dMax : "null" ) ; 
        }
        
        String sFunctionCall = "fwkSpinEditClick("
            + "'"+tag.getId()+"'" 
            + ","
        	+ sPlusMinus + tag.getStep()
            + ","
        	+ tag.getDigit()
        	+ sMinMaxParameters 
        	+ ")";
        
        out.print("<td " );
        out.print(" class=\"telosys_spinedit_arrow\" ");
        out.print(" onselectstart=\"return false;\" ");
        //out.print(" onmouseup=\"fwkSpinEditClick('"+tag.getId()+"',"+ sPlusMinus + tag.getStep()+","+tag.getDigit()+")\" ");
        out.print(" onmouseup=\"" + sFunctionCall + "\" ");
        out.print(" >");
        
        String sImgSrc = tag.getFullTelosysImageSrc(sImage );
        out.print("<img onselectstart=\"return false;\" src=\"" + sImgSrc + "\"  align=\"middle\" />" ); 
        
        out.println("</td>");
        
    }
    
    //-----------------------------------------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        SpinEdit tag = (SpinEdit) genericTag;
        //--- Register the current field via Javascript ...
        registerField(tag);
    }
    //-----------------------------------------------------------------------------------------------
    private void registerField(SpinEdit tag) throws IOException
    {
        if (needRegister(tag))
        {
            String sFirstArgs = firstArgsToRegister(tag);
            String sJavascript = "RegisterField(" + sFirstArgs + ")";                    
            tag.addJavascriptLine(sJavascript);
        }
    }
    //-----------------------------------------------------------------------------------------------
}