package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;
import java.util.LinkedList;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;
import org.objectweb.telosys.uil.taglib.widget.TreeView;
import org.objectweb.telosys.uil.taglib.widget.data.TreeViewNode;

/**
 * HTML generation for Tree widget
 * 
 * @author Aurelien COUERON
 *  
 */
public class TreeViewHTML extends GenericHTML implements IWidget
{
    //------------------------------------------------------------------------------------------
    /**
     * Generates all the child nodes of the given root node
     * @param out
     * @param rootNode
     * @throws IOException
     */
    private void generateTree(JspWriter out, TreeViewNode rootNode ) throws IOException
    {
        //--- Generate all the child nodes 
        if ( rootNode.hasChildNodes() )
        {
            LinkedList list = rootNode.getChildNodes();
            if ( list!=null ) 
            {
                if ( list.size() > 0 ) 
                {
	                TreeViewNode childNode = null;
	                for(int i=0; i<list.size(); i++)
	                {
	                    childNode = (TreeViewNode)list.get(i);
	                    generateNode(out, childNode) ;
	                }
                }
            }
        }
    }
    
    //------------------------------------------------------------------------------------------
    /**
     * Recursive method to print a node and all its descendents 
     * @param out
     * @param node
     * @throws IOException
     */
    private void generateNode(JspWriter out, TreeViewNode node ) throws IOException
    {
        String sDefaultClass = "leaf"; 
        
        //if ( node.hasChildNodes() ) // v 0.9.9
        if ( node.getNodeType() == TreeViewNode.TYPE_BRANCH ) // it's a branch
        {            
            if ( node.isOpen() )
            {
                sDefaultClass = "opened" ; // Folder ( in "open" state )
            }
            else
            {
                sDefaultClass = "closed" ; // Folder ( in "closed" state )
            }
        }        
        
        out.print("<li ");
        //--- Id 
        if (node.getId()!=null) {
            out.print(" id=\"" + node.getId() + "\" " );
        }         
        //--- Class
        String sClass = sDefaultClass ;
        if (node.getCl()!=null) {
            sClass = node.getCl() ;
        }
        out.print(" class=\"" + sClass + "\" " );
        
        //--- Style 
        if (node.getStyle()!=null) {
            out.print(" style=\"" + node.getStyle() + "\" " );
        } 
        
        //--- Value 
        if (node.getValue()!=null) {
            out.print(" value=\"" + node.getValue() + "\" " );
        } 
        
        //--- OnClick
        if (node.getOnclick()!=null) {
            out.print(" action=\"" + node.getOnclick() + "\" ");
        } 
        out.print(">");
        
        /** ???
        if (node.getImg()!=null) {
            sbData.append("<img src=\"");
            sbData.append(node.getImg());
            sbData.append("\" border=\"0\"/>");
        } 
        ***/
        
        //--- Text
        out.print(node.getTxt());
	        
        //--- Generate child nodes 
        if ( node.hasChildNodes() )
        {
            LinkedList list = node.getChildNodes();
            if ( list!=null ) 
            {
                if ( list.size() > 0 ) 
                {
                    //--- v 0.9.9
	                //out.println("<ul>");
                    if ( node.isOpen() )
                    {
    	                out.println("<ul>");
                    }
                    else
                    {
    	                out.println("<ul style=\"display:none\" >");
                    }

                    TreeViewNode childNode = null;
	                for(int i=0; i<list.size(); i++)
	                {
	                    childNode = (TreeViewNode)list.get(i);
	                    generateNode(out, childNode) ;
	                }
	                out.println("</ul>");
                }
            }
        }
        
        //--- End of "LI" 
        out.println("</li>");
    }
    
    //------------------------------------------------------------------------------------------
//    /**
//     * Generates the Drag'n Drop DIV if not yet generated 
//     * @param out
//     * @param pageContext
//     * @throws IOException
//     */
//    private void generateDragAndDropDiv(JspWriter out, PageContext pageContext ) throws IOException
//    {
//        if ( Page.getTreeDnDDivFlag(pageContext) == false )
//        {
//        	String sStyle="filter:alpha(opacity=50); opacity: 0.5; -moz-opacity:0.5;" 
//        	    + "position:absolute; display:none;";
//
//            out.println( "<div id=\"fwkTreeDragAndDrop\" class=\"tree\" style=\"" + sStyle + "\" ></div>");
//            Page.setTreeDnDDivFlag(pageContext);
//        }
//    }
    
    //------------------------------------------------------------------------------------------
    // No generation in the "start tag" ( only root node creation )
    //------------------------------------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
//        TreeView tag = (TreeView) genericTag;
        
        //--- Create the ROOT NODE of the tree ( for "treenode" tags )
        TreeViewNode rootNode = new TreeViewNode ( "TREE-ROOT" ) ;
        
        //--- Set the "root node"  
        Page.setRootNode(genericTag.getPageContext(), rootNode);
        //out.println("<!-- Root Node set : " + rootNode + " -->");
        
        //--- Set the "current node"  
        Page.setCurrentNode(genericTag.getPageContext(), rootNode);
        //out.println("<!-- Current Node set : " + rootNode + " -->");
        
    }
    
    //------------------------------------------------------------------------------------------
    // ALL the TREE VIEW is generated here ( at the end of the tag )
    //------------------------------------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        TreeView tag = (TreeView) genericTag;
        
//        generateDragAndDropDiv(out, tag.getPageContext() );
        
        //--- Main "UL" tag
        
        out.print( "<ul id=\"");
        out.print(tag.getId());
        out.print("\" ");

        //--- Class
        String sClass = tag.getCl();
        if (sClass == null)
        {
            sClass = "tree" ; // Default class
        }
        out.print(" class=\"" + sClass + "\" ");
        
        //--- Style
        out.print(getStyleAttr(tag) + " "); 
        
        //--- No selection ( IE )
        out.print(" onselectstart=\"return false;\" ");
        
        out.println(" >");
        
        TreeViewNode rootNode = Page.getRootNode(genericTag.getPageContext());
        if ( rootNode != null )
        {
            generateTree(out, rootNode );
        }
        else
        {
            out.println( "<!-- No tree ( root node not found ) -->");
        }
        
        out.println( "</ul>");
        
        //--- Tree initialization
        String sFctInit = "fwkTreeInitTree" ; // With Drag and Drop 
        String sDnd = tag.getDnd() ;
        if ( sDnd != null )
        {
            if ( sDnd.equals("no") || sDnd.equals("none") || sDnd.equals("false"))
            {
                sFctInit = "fwkTreeInitTreeNoDND" ; // No Drag & Drop
            }
        }
        out.println("<script>" + sFctInit + "('"+tag.getId()+"');</script>");
    }
    //------------------------------------------------------------------------------------------
}