package org.objectweb.telosys.upload;

import java.io.IOException;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenContextManager;

/**
 * 
 */
public class FileManager
{
    /**
     * Retuns the FileUploaded object stored in the Http Session 
     * @param req
     * @param sSessionKey the session attribute key 
     * @return
     */
    public final static FileUploaded getFileFromHttpSession(HttpServletRequest req, String sSessionKey )
    {
        HttpSession session = req.getSession(false); // No session creation 
        if ( session != null )
        {
            Object obj = session.getAttribute(sSessionKey);
    		if ( obj != null )
    		{
        		if ( obj instanceof FileUploaded )
        		{
        		    return (FileUploaded) obj ;
        		}
    		}
        }
        return null ;
    }
    
    /**
     * Retuns the FileUploaded object stored as a Screen Context attribute 
     * @param request
     * @param iContextId
     * @param sContextAttribute
     * @return
     */
    public final static FileUploaded getFileFromScreenContext(HttpServletRequest request, int iContextId, String sContextAttribute )
    {
        ScreenContext screenContext = null ;
        try
        {
            screenContext = ScreenContextManager.getScreenContext(request, iContextId);
        } catch (TelosysException e)
        {
            return null ;
        }
        if (screenContext != null)
        {
            Object obj = screenContext.getAttribute(sContextAttribute);
    		if ( obj != null )
    		{
        		if ( obj instanceof FileUploaded )
        		{
        		    return (FileUploaded) obj ;
        		}
    		}
        }
        return null ;
    }
    
    /**
     * Removes the FileUploaded object from the Http Session if any 
     * @param req
     * @param sSessionKey the session attribute key
     * @return
     */
    public final static boolean removeFileFromHttpSession(HttpServletRequest req, String sSessionKey )
    {
        HttpSession session = req.getSession(false); // No session creation 
        if ( session != null )
        {
            Object obj = session.getAttribute(sSessionKey);
    		if ( obj != null )
    		{
        		if ( obj instanceof FileUploaded )
        		{
                    session.removeAttribute(sSessionKey);
                    return true ;
        		}
    		}
        }
        return false ;
    }
    
    /**
     * Generates the Http Response from the file content 
     * @param response
     * @param out
     * @param file
     * @return true if the response has been generated, false if nothing done
     * @throws IOException
     */
    public final static boolean generateFileContent(HttpServletResponse response, ServletOutputStream out, FileUploaded file ) throws IOException
    {
        if ( response == null ) return false ;
        if ( out == null ) return false ;
        if ( file == null ) return false ;
        byte[] data = file.getData() ;
        if ( data == null ) return false ;
        
        // Write the response  
        String sType = file.getFileType() ;
        response.setContentType(sType);
        response.setContentLength(data.length);
        //ServletOutputStream out = response.getOutputStream(); // Cannot be called more than 1 time !!
        out.write(data);
        
        return true ;
    }
}
