package org.objectweb.telosys.upload;

/**
 * A representation of a file uploaded on the server by a multpart http request 
 * 
 * @author Laurent GUERIN
 * 
 */
public class FileUploaded
{
    private String           _sFileName = null;

    private byte[]           _data      = null;

    private String           _sFileType = null;

    private long             _lMaxSizeUsed = 0;

    private String           _sDestDir  = null;

    private UploadParameters _uploadParam = null;

    /**
     * Constructor 
     * @param fileName
     * @param data
     * @param sFileType
     * @param maxSizeUsed
     * @param destDir
     * @param uploadParam
     */
    public FileUploaded(String fileName, byte[] data, String sFileType, long maxSizeUsed, String destDir, UploadParameters uploadParam)
    {
        super();
        _sFileName = fileName;
        _data = data;
        _sFileType = sFileType ;
        _lMaxSizeUsed = maxSizeUsed;
        _sDestDir = destDir;
        _uploadParam = uploadParam;
    }

    /**
     * Returns the data of the file uploaded
     * 
     * @return the file data ( bytes array )
     */
    public byte[] getData()
    {
        return _data;
    }

    /**
     * Returns the size of the data ( in bytes )
     * 
     * @return
     */
    public long getDataSize()
    {
        return _data != null ? _data.length : 0;
    }

    /**
     * Returns the name of the file uploaded
     * 
     * @return
     */
    public String getFileName()
    {
        return _sFileName;
    }

    /**
     * Returns the content type of the file uploaded ( i.e. "text/plain", ... )
     * 
     * @return
     */
    public String getFileType()
    {
        return _sFileType;
    }

    /**
     * Returns the maximum size used when the file data has been loaded in memory 
     * @return
     */
    public long getMaxSizeUsed()
    {
        return _lMaxSizeUsed ;
    }
    
    /**
     * Returns the destination directory ( where to save the file on the server side ) 
     * @return 
     */
    public String getDestDir()
    {
        return _sDestDir;
    }

    //---------------------------------------------------------------------------------
    // UPLOAD PARAMETERS
    //---------------------------------------------------------------------------------
    
    /**
     * Returns the "destdir" parameter ( see tag : destdir="..." )
     * 
     * @return the destination directory ( or null if not set )
     */
    public String getParamDestDir()
    {
        return _uploadParam != null ? _uploadParam.getDestDir() : null;
    }

    /**
     * Returns the "maxsize" parameter ( see tag : maxsize="..." )
     * 
     * @return the maximum size ( or -1 if not set )
     */
    public long getParamMaxSize()
    {
        return _uploadParam != null ? _uploadParam.getMaxSize() : 0;
    }
    
    /**
     * Returns the "sessionkey" parameter ( see tag : sessionkey="..." )
     * @return the session key ( or null if not set )
     */
    public String getParamSessionKey()
    {
        return _uploadParam != null ? _uploadParam.getSessionKey() : null ;
    }
    
    /**
     * Returns the "contextid" parameter ( see tag : contextid="..." )
     * @return the context id ( or -1 if not set )
     */
    public int getParamContextId()
    {
        return _uploadParam != null ? _uploadParam.getContextId() : -1 ;
    }
    
    /**
     * Returns the "contextattribute" parameter ( see tag : contextattribute="..." )
     * @return the context attribute ( or null if not set )
     */
    public String getParamContextAttribute()
    {
        return _uploadParam != null ? _uploadParam.getContextAttribute() : null ;
    }
    
}