package org.objectweb.telosys.upload;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

import javax.servlet.ServletInputStream;

/**
 * An upload part which represents a INPUT TYPE="file" form parameter
 */
public class Part
{

    /** Default encoding */
    private static final String ENCODING = "ISO-8859-1";

    /** name of the form element */
    private String              _sName;

    /** "file system" name of the file */
    private String              _sFileName;

    /** content type of the file */
    private String              _sContentType;

    /** contents of the parameter */
    private byte[]              _value;

    /**
     * Constructor
     */
    Part(String name, ServletInputStream in, String boundary, String contentType, String fileName) throws IOException
    {
        this._sName = name;
        this._sFileName = fileName;
        this._sContentType = contentType;
        // Fill value
        PartInputStream pis = new PartInputStream(in, boundary);
        ByteArrayOutputStream baos = new ByteArrayOutputStream(512);
        byte[] buf = new byte[128];
        int read;
        while ((read = pis.read(buf)) != -1)
        {
            baos.write(buf, 0, read);
        }
        pis.close();
        baos.close();
        this._value = baos.toByteArray();
    }

    /**
     * Returns the name of the form element that this Part corresponds to
     */
    public String getName()
    {
        return _sName;
    }

    /**
     * Returns the name of the file
     */
    public String getFileName()
    {
        return _sFileName;
    }

    /**
     * Returns the content type of the file data contained within.
     * 
     * @return content type of the file data.
     */
    public String getContentType()
    {
        return _sContentType;
    }

    /**
     * Returns an byte array which contains the contents of the file supplied.
     */
    public byte[] getValue()
    {
        return _value;
    }

    /**
     * Returns the value of the parameter in as a string
     */
    public String getStringValue() throws UnsupportedEncodingException
    {
        return getStringValue(ENCODING);
    }

    /**
     * Returns the value of the parameter in as a string
     */
    public String getStringValue(String encoding) throws UnsupportedEncodingException
    {
        return new String(_value, encoding);
    }

    /**
     * Returns true to indicate this part is a file.
     */
    public boolean isFile()
    {
        return (this._sFileName != null);
    }
}