package org.objectweb.telosys.util;

import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * Primitive types conversion with MIN/MAX values management
 * 
 */
public class Conv
{
    public final static boolean STRICT = true ;
    public final static boolean CUT = false ;
    
    private static long checkMinMax(long val, String sType, long min, long max, boolean strict )
    {
        if ( val < min )
        {
            if ( strict )
            {
                throw new TelosysRuntimeException("Cannot convert to " + sType + " : value lower than MIN ( " + val + " < " + min + " )" );
            }
            else
            {
                return min ;
            }
        }
        if ( val > max )
        {
            if ( strict )
            {
                throw new TelosysRuntimeException("Cannot convert to " + sType + " : value greater than MAX ( " + val + " > " + max + " )" );
            }
            else
            {
                return max ;
            }
        }
        return val;
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Converts to char strictly ( throws a TelosysRuntimeException if lower/greater than MIN/MAX )
     * @param l : value to convert
     * @return
     * @since v 1.0.2
     */
    public static char toChar(long l)
    {
        return (char) checkMinMax(l, "char", Character.MIN_VALUE, Character.MAX_VALUE, STRICT );
    }
    
    /**
     * Converts to char ( STRICT or CUT mode )
     * @param l : value to convert
     * @param b : mode :<br>
     * STRICT = throws a TelosysRuntimeException if lower/greater than MIN/MAX<br>
     * CUT = no exception ( just cut the value if necessary, returning MIN or MAX )
     * @return
     * @since v 1.0.2
     */
    public static char toChar(long l, boolean b)
    {
        return (char) checkMinMax(l, "char", Character.MIN_VALUE, Character.MAX_VALUE, b );
    }
        
    //-----------------------------------------------------------------------------------
    /**
     * Converts to byte strictly ( throws a TelosysRuntimeException if lower/greater than MIN/MAX )
     * @param l : value to convert
     * @return
     */
    public static byte toByte(long l)
    {
        return (byte) checkMinMax(l, "byte", Byte.MIN_VALUE, Byte.MAX_VALUE, STRICT );
    }
    
    /**
     * Converts to byte ( STRICT or CUT mode )
     * @param l : value to convert
     * @param b : mode :<br>
     * STRICT = throws a TelosysRuntimeException if lower/greater than MIN/MAX<br>
     * CUT = no exception ( just cut the value if necessary, returning MIN or MAX )
     * @return
     */
    public static byte toByte(long l, boolean b)
    {
        return (byte) checkMinMax(l, "byte", Byte.MIN_VALUE, Byte.MAX_VALUE, b );
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Converts to short strictly ( throws a TelosysRuntimeException if lower/greater than MIN/MAX )
     * @param l : value to convert
     * @return
     */
    public static short toShort(long l)
    {
        return (short) checkMinMax(l, "short", Short.MIN_VALUE, Short.MAX_VALUE, STRICT );
    }
    
    /**
     * Converts to short ( STRICT or CUT mode )
     * @param l : value to convert
     * @param b : mode :<br>
     * STRICT = throws a TelosysRuntimeException if lower/greater than MIN/MAX<br>
     * CUT = no exception ( just cut the value if necessary, returning MIN or MAX )
     * @return
     */
    public static short toShort(long l, boolean b)
    {
        return (short) checkMinMax(l, "short", Short.MIN_VALUE, Short.MAX_VALUE, b );
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Converts to int strictly ( throws a TelosysRuntimeException if lower/greater than MIN/MAX )
     * @param l : value to convert
     * @return
     */
    public static int toInt(long l)
    {
        return (int) checkMinMax(l, "int", Integer.MIN_VALUE, Integer.MAX_VALUE, STRICT );
    }
    
    /**
     * Converts to int ( STRICT or CUT mode )
     * @param l : value to convert
     * @param b : mode :<br>
     * STRICT = throws a TelosysRuntimeException if lower/greater than MIN/MAX<br>
     * CUT = no exception ( just cut the value if necessary, returning MIN or MAX )
     * @return
     */
    public static int toInt(long l, boolean b)
    {
        return (int) checkMinMax(l, "int", Integer.MIN_VALUE, Integer.MAX_VALUE, b );
    }

}
