package org.objectweb.telosys.util;

import java.io.File;

import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *
 */
public final class DirUtil
{
    /** */
    private DirUtil()
    {
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Check the existence of a directory
     * @param sDir
     * @return true if the given directory exists and is realy a directory (not a file)
     */
    public static boolean exists( String sDir )
    {
        File file = new File(sDir);
        return file.isDirectory(); // exists and is a directory 
    }
    
    /**
     * Creates the given directory, including any necessary but nonexistent parent directories 
     * @param sDir
     * @return true if the directory was created
     */
    public static boolean createDir( String sDir )
    {
        boolean bRet = false ;
        if ( sDir != null )
        {            
		    File dir = new File(sDir);
		    if ( ! dir.exists() )
		    {
			    if ( dir.mkdirs() )
		        {
			        bRet = true ;		            
		        }		
		        else
		        {
		            throw new TelosysRuntimeException("createDir() : Cannot create directory '" + sDir + "'" );
		        }
		    }
        }
        return bRet ;
    }

    /**
     * Creates the file directory if it doesn't exist 
     * @param sFile
     * @return true if the directory was created
     */
    public static boolean createFileDir( String sFile )
    {
        boolean bRet = false ;
        if ( sFile != null )
        {            
		    File file = new File(sFile);
		    bRet = createDir( file.getParent() ) ;
        }
        return bRet ;
    }



}