package org.objectweb.telosys.util;

import java.util.Date;

import org.objectweb.telosys.common.TelosysClassLogger;

/**
 * @author Laurent GUERIN
 *
 */
public final class JsonUtil
{
    private static final TelosysClassLogger $log = new TelosysClassLogger(JsonUtil.class);

    private static final String  NULL = "null";

	/**
	 * Comment for <code>$dateFormatter</code>
	 * The formatter to convert a Java Date to Xml String 
	 * International standard : ISO 8601 ( "yyyy-MM-dd" )
	 */
	//private static SimpleDateFormat     $dateISOFormatter  = new SimpleDateFormat ("yyyy-MM-dd" ) ;
    
    /** */
    private JsonUtil()
    {
    }
    
    /**
     * Convert "Standard String" to "JSON String value" <br>
     * A String value in JSON is just the string itself between quotes, <br>
     * with special characters protection ( \x ) 
     * 
     * @param sStdString :
     * @return String :
     */ 
    public static String value(final String sStdString)
    {
        if (sStdString == null)
        {
            return NULL ;
        }
        
        char c;
        StringBuffer sb = new StringBuffer(sStdString.length() + 40);
        sb.append('\"');
        for (int i = 0; i < sStdString.length(); i++)
        {
            c = sStdString.charAt(i);
            switch (c)
            {
//                case '<':
//                    sb.append("&lt;");
//                    break;
//                case '>':
//                    sb.append("&gt;");
//                    break;
//                case '&':
//                    sb.append("&amp;");
//                    break;
                case '\"':
                    sb.append("\"");
                    break;
                case '\\':
                    sb.append("\\");
                    break;
                case '/':
                    sb.append("\\/");
                    break;
                case '\'':
                    sb.append("\'");
                    break; 
                case '\b':
                    sb.append("\b");
                    break;
                case '\f':
                    sb.append("\f");
                    break;
                case '\n':
                    sb.append("\n");
                    break;
                case '\r':
                    sb.append("\r");
                    break;
                case '\t':
                    sb.append("\t");
                    break;
                default:
                    sb.append(c);
                    break;
            }
        }
        sb.append('\"');
        return sb.toString();
    }

    /**
     * There's no predefined format for Date objects in JSON <br>
     * Here, just use a string in ISO format
     * @param date
     * @return
     */
    public static String value(final Date date)
    {
        if (date == null)
        {
            return NULL;
        }
        return "\"" + DateUtil.dateISO(date) + "\"";
    }

    //----------------------------------------------------------------------------------------------
    // BOOLEANS : return ( without quotes ) or "null" if null
    //----------------------------------------------------------------------------------------------
    /**
     * 
     * @param b
     * @return "true" or "false"
     */
    public static String value(final boolean b)
    {
        if ( b )
        {
            return "true";
        }
        else
        {
            return "false";            
        }
    }
    //----------------------------------------------------------------------------------------------
    public static String value( Boolean v )
    {
        return ( v != null ? v.toString() : NULL );
    }
    
    //----------------------------------------------------------------------------------------------
    // NUMBERS : return value as is ( without quotes ) or "null" if null
    //----------------------------------------------------------------------------------------------
    /**
     * JSON value for 'Number' instances (Byte, Short, Integer, Long, Float, Double, BigInteger, BigDecimal)
     * @param v
     * @return the value as is ( without quotes ) or "null" if null
     */
    public static String value( Number v)
    {
        return ( v != null ? v.toString() : NULL ); 
    }
    //----------------------------------------------------------------------------------------------
    public static String value( byte v)
    {
        return "" + v ; 
    }
    public static String value( short v)
    {
        return "" + v ; 
    }
    public static String value( int v)
    {
        return "" + v ; 
    }
    public static String value( long v)
    {
        return "" + v ; 
    }
    public static String value( float v)
    {
        return "" + v ; 
    }
    public static String value( double v)
    {
        return "" + v ; 
    }
}