package org.objectweb.telosys.util;

import java.util.Map;

import org.objectweb.telosys.common.TelosysException;

/**
 * @author Laurent GUERIN
 *  
 */
public class MapUtil
{
    //private static final TelosysClassLogger $log = new TelosysClassLogger(MapUtil.class);

    private static int strToInt(String sParamName, String sParamValue) throws TelosysException
    {
        int iRet = 0;
        try
        {
            iRet = Integer.parseInt(sParamValue);
        } catch (NumberFormatException ex)
        {
            throw new TelosysException("MapUtil.getInt('" + sParamName + "') -> '" + sParamValue + "' : invalid number ");
        }
        return iRet;
    }

    /**
     * Returns the int value of the given parameter name
     * 
     * @param map :
     *            the map
     * @param sParamName :
     *            the parameter name
     * @return : the value found
     * @throws TelosysException
     *             if the parameter is unknown or if arguments are invalid
     */
    public static int getInt(Map map, String sParamName) throws TelosysException
    {
        int iRet = 0;
        if (map == null)
        {
            throw new TelosysException("getInt : map is null ");
        }
        if (sParamName == null)
        {
            throw new TelosysException("getInt : parameter name is null ");
        }
        Object obj = map.get(sParamName);
        if (obj != null)
        {
            if (obj instanceof String)
            {
                String sValue = (String) obj;
                iRet = strToInt(sParamName, sValue);
            }
            else if (obj instanceof String[]) // for HttpServletRequest parameters map
            {
                String[] values = (String[]) obj;
                if (values.length > 0)
                {
                    iRet = strToInt(sParamName, values[0]); // use the first item
                }
                else
                {
                    throw new TelosysException("getInt('" + sParamName + "') -> String[] length = 0 ! ");
                }
            }
            else if (obj instanceof Integer)
            {
                Integer integer = (Integer) obj;
                iRet = integer.intValue();
            }
            else
            {
                throw new TelosysException("getInt('" + sParamName + "') : invalid type ( class = " + obj.getClass().getName() + " )");
            }
        }
        else
        {
            throw new TelosysException("getInt('" + sParamName + "') : parameter not found ");
        }
        return iRet;
    }
}