package org.objectweb.telosys.util;

import java.text.ChoiceFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.objectweb.telosys.common.TelosysClassLogger;

/**
 * @author Laurent GUERIN
 *  
 */
public class ObjUtil
{
    private static final TelosysClassLogger $log = new TelosysClassLogger(ObjUtil.class);

    //---------------------------------------------------------------------------
    /**
     * Creates an instance of the given class name
     * 
     * @param sClassName
     * @return
     */
    public static Object createObjectInstance(String sClassName)
    {
        $log.trace("createObjectInstance(" + sClassName + ")... ");
        boolean b1 = false;
        boolean b2 = false;
        try
        {
            //--- Chargement de la classe avec le ClassLoader par dfaut
            Class theClass = Class.forName(sClassName);
            b1 = true;
            Object theInstance = theClass.newInstance();
            //b2 = true ;
            return theInstance;
        } catch (Throwable t)
        {
            $log.error("createObjectInstance : t = [" + t.getMessage() + "]");
            $log.error("b1 = " + b1);
            return null;
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Formats an object to produce a string
     * 
     * @param oValue
     * @param sFormat
     * @return
     */
    public static String format(Object oValue, String sFormat)
    {
        return format(oValue, sFormat, null);
    }

    //---------------------------------------------------------------------------
    /**
     * Formats an object to produce a string
     * 
     * @param oValue
     * @param sFormat
     * @param sLanguage :
     *            Locale language to use for numbers formatting ( eg "fr" )
     * @return
     */
    public static String format(Object oValue, String sFormat, String sLanguage)
    {
        if (oValue == null)
        {
            return "(null)";
        }
        if (sFormat == null)
        {
            return oValue.toString();
        }

//        System.out.print("Value = '" + oValue + "' - Format = '" + sFormat + "' ==> ");
        if (oValue instanceof Boolean)
        {
            //-----------------------------------------------------------------------------------
            //--- BOOLEAN
            //-----------------------------------------------------------------------------------
            String sTrueFormat = "(?)";
            String sFalseFormat = "(?)";
            if (sFormat.indexOf('|') >= 0) // Contains a Separator
            {
                String[] tokens = sFormat.split("\\|");
                if (tokens.length > 1)
                {
                    sTrueFormat = tokens[0];
                    sFalseFormat = tokens[1];
                }
            }
            if (((Boolean) oValue).booleanValue())
            {
                return sTrueFormat;
            }
            else
            {
                return sFalseFormat;
            }
        }
        else if (oValue instanceof Number)
        {
            //-----------------------------------------------------------------------------------
            //--- NUMBERS ( BigDecimal, BigInteger, Byte, Double, Float, Integer, Long, Short )
            //-----------------------------------------------------------------------------------
            if (sFormat.indexOf('|') >= 0) // Contains a CHOICE Separator
            {
                String sResult = "(?)";
                try
                {
                    ChoiceFormat choiceFormat = new ChoiceFormat(sFormat);
                    sResult = choiceFormat.format(oValue);
                } catch (Throwable e)
                {
                    sResult = "(error)";
                }
                return sResult;
            }
            else
            {
                DecimalFormat decimalFormat = null;
                if (sLanguage != null)
                {
                    //--- Create format with specific "Locale" symbols
                    Locale locale = new Locale( sLanguage );
                    DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
                    decimalFormat = new DecimalFormat(sFormat, symbols);

                }
                else
                {
                    //--- With default symbols
                    decimalFormat = new DecimalFormat(sFormat);
                }

                return decimalFormat.format(oValue);
            }
        }
        else if (oValue instanceof Date)
        {
            //-----------------------------------------------------------------------------------
            //--- DATE
            //-----------------------------------------------------------------------------------
            String sResult = "(?)";
            try
            {
                SimpleDateFormat dateFormat = new SimpleDateFormat(sFormat);
                return dateFormat.format((Date) oValue);
            } catch (Throwable e)
            {
                sResult = "(error)";
            }
            return sResult;
        }
        else
        {
            //-----------------------------------------------------------------------------------
            //--- OTHERS
            //-----------------------------------------------------------------------------------
            return oValue.toString();
        }
    }

    //----------------------------------------------------------------

}