package org.objectweb.telosys.util.serial;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.objectweb.telosys.common.TelosysException;

/**
 * @author L. GUERIN
 *
 */
public class MemorySerializer
{    
    private byte[] _lastSerializedObject = null ;  
                 
    public MemorySerializer()
    {
    }
    
    /**
     * Writes (serializes) the given object "in memory" and returns the result as an array of bytes.<br>
     * The result is kept, and can be retrieve with the 'read' method without parameter
     * @param obj
     * @return
     * @throws TelosysException
     */
    public byte[] write ( Object obj ) throws TelosysException
    {
        //byte[] serializedObject = null ;
        
        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        ObjectOutputStream oos = null ;
        try
        {
            oos = new ObjectOutputStream(baos);
        } catch (IOException e2)
        {
            throw new TelosysException("Cannot create ObjectOutputStream", e2);
        }

        try
        {
            //--- WRITE THE OBJECT
            oos.writeObject( obj );
            
            _lastSerializedObject = baos.toByteArray();
            
            //--- Close ALL
            oos.close();
            baos.close();
        } catch (IOException e3)
        {
            throw new TelosysException("ObjectOutputStream cannot write object ", e3);
        }
        
        return _lastSerializedObject ;
    }
    
    /**
     * Read the object from the bytes array produced by the last 'write' operation<br>
     * ( returns the last object serialized by this serializer) <br>
     * @return
     * @throws TelosysException
     */
    public Object read() throws TelosysException
    {
        if ( _lastSerializedObject != null )
        {
            return read ( _lastSerializedObject ) ;
        }
        return null ;
    }
    
    /**
     * Read the object from the given bytes array 
     * @param serializedObject
     * @return
     * @throws TelosysException
     */
    public Object read ( byte[] serializedObject ) throws TelosysException
    {
        ByteArrayInputStream bais = new ByteArrayInputStream( serializedObject );
        ObjectInputStream  ois = null ;
        Object obj = null ;

        try
        {
            ois = new ObjectInputStream(bais);
        } catch (IOException e2)
        {
            throw new TelosysException("Cannot create ObjectInputStream", e2);
        }

        try
        {
            //--- READ THE OBJECT
            obj = ois.readObject();
            //--- Close ALL
            ois.close();
            bais.close();
            
        } catch (IOException e3)
        {
            throw new TelosysException("ObjectOutputStream cannot read object (IOException)", e3);
        } catch (ClassNotFoundException e4)
        {
            throw new TelosysException("ObjectOutputStream cannot read object (ClassNotFoundException)", e4);
        }
        
        return obj;
    }
    
}
