package org.objectweb.telosys.util.web;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.util.web.WebUtil;

public abstract class GenericURI
{
    private String _sResourcePath = null ;
    
    protected void setResourcePath( String sResourcePath )
    {
        _sResourcePath = sResourcePath ;
    }
        
    /**
	 * Returns the current web app resource path  <br>
     * @return : the path without "/contextname" at the beginning ( ie : "/folder/file" )
     */
    public String getResourcePath()
    {
        return _sResourcePath ;
    }
        
    //------------------------------------------------------------------------------------
    // getContextURI()
    //------------------------------------------------------------------------------------
    /**
	 * Returns the context URI  <br>
	 * 
     * @param request
     * @return : the URI ( ie : "/contextname/folder/file" )
     */
    public String getContextURI(HttpServletRequest request)
    {
        return WebUtil.getContextResourceURI( request, _sResourcePath );
    }
    
    /**
	 * Returns the context URI  <br>
	 * 
     * @param request
     * @return : the URI ( ie : "/contextname/folder/file" )
     */
    public String getContextURI(ScreenRequest request)
    {
        if ( request != null )
        {
            return getContextURI ( request.getHttpRequest() );
        }
        return null ;
    }
    
    /**
	 * Returns the context URI  <br>
	 * 
     * @param request
     * @return : the URI ( ie : "/contextname/folder/file" )
     */
    public String getContextURI(ServiceRequest request)
    {
        if ( request != null )
        {
            return getContextURI ( request.getHttpRequest() );
        }
        return null ;
    }
    
    //------------------------------------------------------------------------------------
    // getFullURL()
    //------------------------------------------------------------------------------------
    /**
     * Returns the full "client side" http URL 
     * 
     * @param request
     * @return : the full URL ( ie : "http://myhost:8080/contextname/folder/file")
     */
    public String getFullURL(HttpServletRequest request)
    {
        return WebUtil.getContextResourceURL ( request, _sResourcePath );
    }
    
    /**
     * Returns the full "client side" http URL 
     * 
     * @param request
     * @return : the full URL ( ie : "http://myhost:8080/contextname/folder/file")
     */
    public String getFullURL(ScreenRequest request)
    {
        if ( request != null )
        {
            return getFullURL ( request.getHttpRequest() );
        }
        return null ;
    }
    
    /**
     * Returns the full "client side" http URL 
     * 
     * @param request
     * @return : the full URL ( ie : "http://myhost:8080/contextname/folder/file")
     */
    public String getFullURL(ServiceRequest request)
    {
        if ( request != null )
        {
            return getFullURL ( request.getHttpRequest() );
        }
        return null ;
    }
    
    //------------------------------------------------------------------------------------
    public String toString()
    {
        return _sResourcePath ;
    }
}
