package org.objectweb.telosys.util.web;

import org.objectweb.telosys.uil.screenmap.ScreenMap;


public class ScreenURI extends GenericURI
{
    /**
     * Contructs a ScreenMap URI definition
     * @param sScreenName : the ScreenMap name
     * @param sScreenType : the ScreenMap type ( "html" or "xul", null = default type )
     * @param sContextId : the Id of the ScreenContext associated with the ScreenMap ( null = default id ) 
     * @param sContextName : the name of the ScreenContext associated with the ScreenMap ( null = same as ScreenMap name )
     * @param sContextAction : the action to apply on the ScreenContext ( "open", "use", "none" ) ( null = default action )
     * @param sParams : the URL parameters to add ( i.e. "param1=aaa&param2=bbb" ) or null if none
     */
    public  ScreenURI( String sScreenName, String sScreenType, 
            String sContextId, String sContextName, String sContextAction, String sParams )
    {
        String s = ScreenMap.getContextURI(sScreenName, sScreenType, sContextId, sContextName, sContextAction, sParams);
        
        setResourcePath ( s ) ;
    }

    /**
     * Contructs a ScreenMap URI definition
     * @param sScreenName : the ScreenMap name
     * @param sScreenType : the ScreenMap type ( "html" or "xul", null = default type )
     * @param sContextId : the Id of the ScreenContext associated with the ScreenMap ( null = default id ) 
     * @param sContextName : the name of the ScreenContext associated with the ScreenMap ( null = same as ScreenMap name )
     * @param sContextAction : the action to apply on the ScreenContext ( "open", "use", "none" ) ( null = default action )
     */
    public  ScreenURI( String sScreenName, String sScreenType, 
            String sContextId, String sContextName, String sContextAction )
    {
        String s = ScreenMap.getContextURI(sScreenName, sScreenType, sContextId, sContextName, sContextAction, null);
        
        setResourcePath ( s ) ;
    }

    /**
     * Contructs a ScreenMap URI definition
     * @param sScreenName : the ScreenMap name
     * @param sScreenType : the ScreenMap type ( "html" or "xul", null = default type )
     * @param sContextId : the Id of the ScreenContext associated with the ScreenMap ( null = default id ) 
     * @param sContextName : the name of the ScreenContext associated with the ScreenMap ( null = same as ScreenMap name )
     */
    public  ScreenURI( String sScreenName, String sScreenType, 
            String sContextId, String sContextName)
    {
        String s = ScreenMap.getContextURI(sScreenName, sScreenType, sContextId, sContextName, null, null);
        
        setResourcePath ( s ) ;
    }
        
    /**
     * Contructs a ScreenMap URI definition
     * @param sScreenName : the ScreenMap name
     * @param sScreenType : the ScreenMap type ( "html" or "xul", null = default type )
     * @param sContextId : the Id of the ScreenContext associated with the ScreenMap ( null = default id ) 
     */
    public  ScreenURI( String sScreenName, String sScreenType, 
            String sContextId )
    {
        String s = ScreenMap.getContextURI(sScreenName, sScreenType, sContextId, null, null, null);
        
        setResourcePath ( s ) ;
    }
        
    /**
     * Contructs a ScreenMap URI definition
     * @param sScreenName : the ScreenMap name
     */
    public  ScreenURI( String sScreenName )
    {
        String s = ScreenMap.getContextURI(sScreenName, null, null, null, null, null);
        
        setResourcePath ( s ) ;
    }
        
}
