package org.objectweb.telosys.util.web;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ListIterator;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *  
 */
public final class WebUtil
{

	/** */
	private WebUtil ()
	{
	}

    //------------------------------------------------------------------------------------------------
	/**
	 * 
	 * @param sUrl :
	 * @param request :
	 * @param response :
	 * @return boolean :
	 * @throws ServletException :
	 * @throws IOException :
	 */
	public static boolean forward(final String sUrl, final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException
	{
		RequestDispatcher rd = request.getRequestDispatcher(sUrl);
		if (rd != null)
		{
			rd.forward(request, response);
			return true;
		}
		return false;
	}

    //------------------------------------------------------------------------------------------------
	/**
	 * @param sUrl : "HTTP redirection" to an other URL ( HTTP code "302" )
	 * @param response :
	 * @return boolean : true = OK, false = ERR
	 */
	public static boolean redirect(final String sUrl, final HttpServletResponse response)
	{
		if (sUrl != null)
		{
			try
			{
				response.sendRedirect(sUrl);
				return true ;
			}
			catch (Exception ex)
			{
			    Telosys.error( "redirect : Exception : " + ex.getMessage());
			}
		}
		else
		{
		    Telosys.error( "redirect : URL is null !");
		}
		return false ;
	}

    //------------------------------------------------------------------------------------------------
	/**
	 * Set the HTTP headers to avoid caching 
	 * @param response
	 */
	public static void noCache( final HttpServletResponse response )
	{
        response.setHeader ("Pragma", "no-cache"); //HTTP 1.0
        
        response.setHeader ("Cache-Control", "no-cache"); //HTTP 1.1
        // response.setHeader("Cache-Control","no-store"); 
        // "no-store" is used sometimes, but "no-cache" is the more polite keyword
        
        response.setDateHeader ("Expires", 0); //prevents caching at the proxy server
	}
	
    //------------------------------------------------------------------------------------------------
	/**
	 * 
	 * @param response :
	 * @param sMsg :
	 * @throws ServletException :
	 * @throws IOException :
	 */
	public static void info(final HttpServletResponse response, final String sMsg) throws ServletException, IOException
	{
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		out.println("<html><body><span style='font: 8pt Tahoma;'>");
		out.println("<b>" + sMsg + "</b>");
		out.println("</span></body></html>");
	}

    //------------------------------------------------------------------------------------------------
	/**
	 * 
	 * @param response :
	 * @param sMsg :
	 * @throws ServletException :
	 * @throws IOException :
	 */
	public static void error(final HttpServletResponse response, final String sMsg) throws ServletException, IOException
	{
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		out.println("<html><body><span style='font: 8pt Tahoma;'>");
		out.println("<b>ERROR</b><br>");
		out.println("<b>" + sMsg + "</b>");
		out.println("</span></body></html>");
	}

    //------------------------------------------------------------------------------------------------
	/**
	 * @param response :
	 * @param sMsg :
	 * @param iter :
	 * @throws ServletException :
	 * @throws IOException :
	 */
	public static void error(final HttpServletResponse response, final String sMsg, final ListIterator iter) throws ServletException, IOException
	{
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		out.println("<html><body><span style='font: 8pt Tahoma;'>");
		//out.println("<b>ERREUR</b><br>");
		out.println("<b>" + sMsg + "</b><br>");
		String s = null;
		if (iter != null)
		{
			while (iter.hasNext())
			{
				s = (String) iter.next();
				out.println(" - " + s + "<br>");
			}
		}
		out.println("</span></body></html>");
	}

	//------------------------------------------------------------------------------------------------
	public static void msgPage(HttpServletResponse response, String sTitle, String sMsg)
	{
		try
		{
			response.setContentType("text/html");
			PrintWriter out = response.getWriter();
			out.println("<html><body>");
			out.println("<h2>"+sTitle+"</h2>");
			out.println("<h4>"+sMsg+"</h4>");
			out.println("</body></html>");
			out.close();
		}
		catch (IOException e)
		{
		    Telosys.error( "Cannot get Writer from HttpResponse !");
		    Telosys.error( "Exception : " + e.getMessage());
		}
	}

    //------------------------------------------------------------------------------------------------
	/**
	 * Returns the full context root URL for the given request with "/" at the end <br>
	 * ( e.g. : "http://server:8080/WebAppContext/" )
	 * @deprecated  : use getContextRootURL() instead
	 * @param request
	 * @return
	 */
	public static String getRootUrl(HttpServletRequest request)
	{
		/*
		return  "http://" + request.getLocalName() 
		+ ":" + request.getLocalPort() 
		+ request.getContextPath() + "/";
		*/
		
		// "context/dir/file.xx"
		String sUri = request.getRequestURI() ; 
		// "http://xxx:80/context/dir/file.xx"
		String sUrl = request.getRequestURL().toString() ; 
		//int i = sUrl.indexOf(sUri) ;
		int d = sUrl.length() - sUri.length() ;
		return sUrl.substring(0,d) + request.getContextPath() + "/";
	}

    //------------------------------------------------------------------------------------------------
	/**
	 * Returns the full context root URL for the given request <br>
	 * ( e.g. : "http://server:8080/WebAppContext" )
	 * @param request
	 * @return "http://server:8080/WebAppContext"
	 */
	public static String getContextRootURL ( HttpServletRequest request )
	{
	    if ( request == null )
	    {
	        throw new TelosysRuntimeException ("getContextRootURL(request) : request parameter is null") ;
	    }

	    String sProtocol = request.getScheme(); // "http"
        if (sProtocol == null)
        {
	        throw new TelosysRuntimeException ("getContextRootURL(request) : cannot get protocol") ;
        }
        
        String sHostName = request.getServerName(); // the server hostname
        if (sHostName == null)
        {
	        throw new TelosysRuntimeException ("getContextRootURL(request) : cannot get server name") ;
        }
        
        int iHostPort = request.getServerPort(); // 80, 8080 etc...

        String sContexPath = request.getContextPath(); // "/WebAppContext"
        if (sContexPath == null)
        {
	        throw new TelosysRuntimeException ("getContextRootURL(request) : cannot get context path") ;
        }
        
        //--- Build the full http URL
        return sProtocol + "://" + sHostName + ":" + iHostPort + sContexPath;

	}
	
    //------------------------------------------------------------------------------------------------
	/**
	 * Returns the context root URI for the given request <br>
	 * ( e.g. : "/WebAppContext" )
	 * @param request
	 * @return "/WebAppContext"
	 */
	public static String getContextRootURI ( HttpServletRequest request )
	{
	    if ( request == null )
	    {
	        throw new TelosysRuntimeException ("getContextRootURI(request) : request parameter is null") ;
	    }

        String sContexPath = request.getContextPath(); // "/WebAppContext"
        if (sContexPath == null)
        {
	        throw new TelosysRuntimeException ("getContextRootURI(request) : cannot get context path") ;
        }
        return sContexPath ;
	}
	
    //------------------------------------------------------------------------------------------------
	/**
     * Returns the full "client side" http URL for the given resource path
     * 
	 * @param request
	 * @param sResourcePath : the server side context resource path <br>
	 *  ( ie : "/folder/file", "folder/file", "/file", "file" )
	 * @return : the full URL ( ie : "http://myhost:8080/contextname/folder/file")
	 */
	public static String getContextResourceURL ( HttpServletRequest request, String sResourcePath )
	{
        return concatRootAndResource(getContextRootURL(request), sResourcePath );
	}
	
    //------------------------------------------------------------------------------------------------
	/**
	 * Returns the context URI for the given resource path <br>
	 * 
	 * @param request
	 * @param sResourcePath : the server side context resource path <br>
	 *  ( ie : "/folder/file", "folder/file", "/file", "file" )
	 * @return : the URI ( ie : "/contextname/folder/file" )
	 */
	public static String getContextResourceURI ( HttpServletRequest request, String sResourcePath )
	{
        return concatRootAndResource(getContextRootURI(request), sResourcePath );
	}

    //------------------------------------------------------------------------------------------------
	private static String concatRootAndResource ( String sRoot, String sResourcePath )
	{
        String sRoot2 = sRoot;
        if (sRoot.endsWith("/"))
        {
            //--- Cut the last char
            sRoot2 = sRoot.substring(0, sRoot.length() - 1);
        }

        if (sResourcePath != null)
        {
            if (sResourcePath.startsWith("/"))
            {
                return sRoot2 + sResourcePath ; // "root" + "/resource"
            }
            else
            {
                return sRoot2 + "/" + sResourcePath; // 
            }
        }
        else
        {
            return sRoot2 + "/";
        }
	}
}